/****************************************************************************
** Meta object code from reading C++ file 'qareaseries.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../qtgraphs/src/graphs2d/areachart/qareaseries.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qareaseries.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN11QAreaSeriesE_t {};
} // unnamed namespace

template <> constexpr inline auto QAreaSeries::qt_create_metaobjectdata<qt_meta_tag_ZN11QAreaSeriesE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QAreaSeries",
        "QML.Element",
        "AreaSeries",
        "colorChanged",
        "",
        "QColor",
        "newColor",
        "selectedColorChanged",
        "newSelectedColor",
        "borderColorChanged",
        "newBorderColor",
        "selectedBorderColorChanged",
        "newSelectedBorderColor",
        "borderWidthChanged",
        "selectedChanged",
        "upperSeriesChanged",
        "lowerSeriesChanged",
        "gradientChanged",
        "QQuickShapeGradient*",
        "newGradient",
        "selectedGradientChanged",
        "clicked",
        "QPoint",
        "point",
        "doubleClicked",
        "pressed",
        "released",
        "color",
        "selectedColor",
        "borderColor",
        "selectedBorderColor",
        "borderWidth",
        "selected",
        "upperSeries",
        "QXYSeries*",
        "lowerSeries",
        "gradient",
        "selectedGradient"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'colorChanged'
        QtMocHelpers::SignalData<void(QColor)>(3, 4, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 6 },
        }}),
        // Signal 'selectedColorChanged'
        QtMocHelpers::SignalData<void(QColor)>(7, 4, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 8 },
        }}),
        // Signal 'borderColorChanged'
        QtMocHelpers::SignalData<void(QColor)>(9, 4, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 10 },
        }}),
        // Signal 'selectedBorderColorChanged'
        QtMocHelpers::SignalData<void(QColor)>(11, 4, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 12 },
        }}),
        // Signal 'borderWidthChanged'
        QtMocHelpers::SignalData<void()>(13, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'selectedChanged'
        QtMocHelpers::SignalData<void()>(14, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'upperSeriesChanged'
        QtMocHelpers::SignalData<void()>(15, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'lowerSeriesChanged'
        QtMocHelpers::SignalData<void()>(16, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'gradientChanged'
        QtMocHelpers::RevisionedSignalData<void(QQuickShapeGradient *)>(17, 4, QMC::AccessPublic, 0x60b, QMetaType::Void, {{
            { 0x80000000 | 18, 19 },
        }}),
        // Signal 'selectedGradientChanged'
        QtMocHelpers::RevisionedSignalData<void(QQuickShapeGradient *)>(20, 4, QMC::AccessPublic, 0x60b, QMetaType::Void, {{
            { 0x80000000 | 18, 19 },
        }}),
        // Signal 'clicked'
        QtMocHelpers::RevisionedSignalData<void(QPoint)>(21, 4, QMC::AccessPublic, 0x609, QMetaType::Void, {{
            { 0x80000000 | 22, 23 },
        }}),
        // Signal 'doubleClicked'
        QtMocHelpers::RevisionedSignalData<void(QPoint)>(24, 4, QMC::AccessPublic, 0x609, QMetaType::Void, {{
            { 0x80000000 | 22, 23 },
        }}),
        // Signal 'pressed'
        QtMocHelpers::RevisionedSignalData<void(QPoint)>(25, 4, QMC::AccessPublic, 0x609, QMetaType::Void, {{
            { 0x80000000 | 22, 23 },
        }}),
        // Signal 'released'
        QtMocHelpers::RevisionedSignalData<void(QPoint)>(26, 4, QMC::AccessPublic, 0x609, QMetaType::Void, {{
            { 0x80000000 | 22, 23 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'color'
        QtMocHelpers::PropertyData<QColor>(27, 0x80000000 | 5, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet | QMC::Final, 0),
        // property 'selectedColor'
        QtMocHelpers::PropertyData<QColor>(28, 0x80000000 | 5, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet | QMC::Final, 1),
        // property 'borderColor'
        QtMocHelpers::PropertyData<QColor>(29, 0x80000000 | 5, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet | QMC::Final, 2),
        // property 'selectedBorderColor'
        QtMocHelpers::PropertyData<QColor>(30, 0x80000000 | 5, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet | QMC::Final, 3),
        // property 'borderWidth'
        QtMocHelpers::PropertyData<qreal>(31, QMetaType::QReal, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 4),
        // property 'selected'
        QtMocHelpers::PropertyData<bool>(32, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 5),
        // property 'upperSeries'
        QtMocHelpers::PropertyData<QXYSeries*>(33, 0x80000000 | 34, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet | QMC::Final, 6),
        // property 'lowerSeries'
        QtMocHelpers::PropertyData<QXYSeries*>(35, 0x80000000 | 34, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet | QMC::Final, 7),
        // property 'gradient'
        QtMocHelpers::PropertyData<QQuickShapeGradient*>(36, 0x80000000 | 18, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 8, 0x60b),
        // property 'selectedGradient'
        QtMocHelpers::PropertyData<QQuickShapeGradient*>(37, 0x80000000 | 18, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 9, 0x60b),
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
    });
    return QtMocHelpers::metaObjectData<QAreaSeries, void>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject QAreaSeries::staticMetaObject = { {
    QMetaObject::SuperData::link<QAbstractSeries::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11QAreaSeriesE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11QAreaSeriesE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN11QAreaSeriesE_t>.metaTypes,
    nullptr
} };

void QAreaSeries::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QAreaSeries *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->colorChanged((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        case 1: _t->selectedColorChanged((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        case 2: _t->borderColorChanged((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        case 3: _t->selectedBorderColorChanged((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        case 4: _t->borderWidthChanged(); break;
        case 5: _t->selectedChanged(); break;
        case 6: _t->upperSeriesChanged(); break;
        case 7: _t->lowerSeriesChanged(); break;
        case 8: _t->gradientChanged((*reinterpret_cast<std::add_pointer_t<QQuickShapeGradient*>>(_a[1]))); break;
        case 9: _t->selectedGradientChanged((*reinterpret_cast<std::add_pointer_t<QQuickShapeGradient*>>(_a[1]))); break;
        case 10: _t->clicked((*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[1]))); break;
        case 11: _t->doubleClicked((*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[1]))); break;
        case 12: _t->pressed((*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[1]))); break;
        case 13: _t->released((*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (QAreaSeries::*)(QColor )>(_a, &QAreaSeries::colorChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAreaSeries::*)(QColor )>(_a, &QAreaSeries::selectedColorChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAreaSeries::*)(QColor )>(_a, &QAreaSeries::borderColorChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAreaSeries::*)(QColor )>(_a, &QAreaSeries::selectedBorderColorChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAreaSeries::*)()>(_a, &QAreaSeries::borderWidthChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAreaSeries::*)()>(_a, &QAreaSeries::selectedChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAreaSeries::*)()>(_a, &QAreaSeries::upperSeriesChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAreaSeries::*)()>(_a, &QAreaSeries::lowerSeriesChanged, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAreaSeries::*)(QQuickShapeGradient * )>(_a, &QAreaSeries::gradientChanged, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAreaSeries::*)(QQuickShapeGradient * )>(_a, &QAreaSeries::selectedGradientChanged, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAreaSeries::*)(QPoint )>(_a, &QAreaSeries::clicked, 10))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAreaSeries::*)(QPoint )>(_a, &QAreaSeries::doubleClicked, 11))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAreaSeries::*)(QPoint )>(_a, &QAreaSeries::pressed, 12))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAreaSeries::*)(QPoint )>(_a, &QAreaSeries::released, 13))
            return;
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 7:
        case 6:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QXYSeries* >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QColor*>(_v) = _t->color(); break;
        case 1: *reinterpret_cast<QColor*>(_v) = _t->selectedColor(); break;
        case 2: *reinterpret_cast<QColor*>(_v) = _t->borderColor(); break;
        case 3: *reinterpret_cast<QColor*>(_v) = _t->selectedBorderColor(); break;
        case 4: *reinterpret_cast<qreal*>(_v) = _t->borderWidth(); break;
        case 5: *reinterpret_cast<bool*>(_v) = _t->isSelected(); break;
        case 6: *reinterpret_cast<QXYSeries**>(_v) = _t->upperSeries(); break;
        case 7: *reinterpret_cast<QXYSeries**>(_v) = _t->lowerSeries(); break;
        case 8: *reinterpret_cast<QQuickShapeGradient**>(_v) = _t->gradient(); break;
        case 9: *reinterpret_cast<QQuickShapeGradient**>(_v) = _t->selectedGradient(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setColor(*reinterpret_cast<QColor*>(_v)); break;
        case 1: _t->setSelectedColor(*reinterpret_cast<QColor*>(_v)); break;
        case 2: _t->setBorderColor(*reinterpret_cast<QColor*>(_v)); break;
        case 3: _t->setSelectedBorderColor(*reinterpret_cast<QColor*>(_v)); break;
        case 4: _t->setBorderWidth(*reinterpret_cast<qreal*>(_v)); break;
        case 5: _t->setSelected(*reinterpret_cast<bool*>(_v)); break;
        case 6: _t->setUpperSeries(*reinterpret_cast<QXYSeries**>(_v)); break;
        case 7: _t->setLowerSeries(*reinterpret_cast<QXYSeries**>(_v)); break;
        case 8: _t->setGradient(*reinterpret_cast<QQuickShapeGradient**>(_v)); break;
        case 9: _t->setSelectedGradient(*reinterpret_cast<QQuickShapeGradient**>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *QAreaSeries::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QAreaSeries::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11QAreaSeriesE_t>.strings))
        return static_cast<void*>(this);
    return QAbstractSeries::qt_metacast(_clname);
}

int QAreaSeries::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QAbstractSeries::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 14)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 14;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 14)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 14;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    }
    return _id;
}

// SIGNAL 0
void QAreaSeries::colorChanged(QColor _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void QAreaSeries::selectedColorChanged(QColor _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void QAreaSeries::borderColorChanged(QColor _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void QAreaSeries::selectedBorderColorChanged(QColor _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void QAreaSeries::borderWidthChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void QAreaSeries::selectedChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void QAreaSeries::upperSeriesChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void QAreaSeries::lowerSeriesChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void QAreaSeries::gradientChanged(QQuickShapeGradient * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 8, nullptr, _t1);
}

// SIGNAL 9
void QAreaSeries::selectedGradientChanged(QQuickShapeGradient * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 9, nullptr, _t1);
}

// SIGNAL 10
void QAreaSeries::clicked(QPoint _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 10, nullptr, _t1);
}

// SIGNAL 11
void QAreaSeries::doubleClicked(QPoint _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 11, nullptr, _t1);
}

// SIGNAL 12
void QAreaSeries::pressed(QPoint _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 12, nullptr, _t1);
}

// SIGNAL 13
void QAreaSeries::released(QPoint _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 13, nullptr, _t1);
}
QT_WARNING_POP
