/****************************************************************************
** Meta object code from reading C++ file 'qxyseries.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../qtgraphs/src/graphs2d/xychart/qxyseries.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qxyseries.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN9QXYSeriesE_t {};
} // unnamed namespace

template <> constexpr inline auto QXYSeries::qt_create_metaobjectdata<qt_meta_tag_ZN9QXYSeriesE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QXYSeries",
        "QML.Element",
        "auto",
        "QML.Creatable",
        "false",
        "QML.UncreatableReason",
        "XYSeries is an abstract base class.",
        "pointReplaced",
        "",
        "qsizetype",
        "index",
        "pointRemoved",
        "pointAdded",
        "pointsAdded",
        "start",
        "end",
        "colorChanged",
        "QColor",
        "color",
        "selectedColorChanged",
        "pointsReplaced",
        "pointsRemoved",
        "count",
        "selectedPointsChanged",
        "pointDelegateChanged",
        "draggableChanged",
        "seriesUpdated",
        "countChanged",
        "clicked",
        "QPoint",
        "point",
        "doubleClicked",
        "pressed",
        "released",
        "declarativePointsChanged",
        "QVariantList",
        "declarativePoints",
        "declarativePointHintChanged",
        "QXYSeries::DeclarativePointHint",
        "newHint",
        "declarativeMinChanged",
        "newMin",
        "stepSizeChanged",
        "newStepSize",
        "append",
        "x",
        "y",
        "QPointF",
        "QList<QPointF>",
        "points",
        "replace",
        "oldX",
        "oldY",
        "newX",
        "newY",
        "oldPoint",
        "newPoint",
        "remove",
        "insert",
        "clear",
        "at",
        "find",
        "removeMultiple",
        "take",
        "isPointSelected",
        "selectPoint",
        "deselectPoint",
        "setPointSelected",
        "selected",
        "selectAllPoints",
        "deselectAllPoints",
        "selectPoints",
        "QList<qsizetype>",
        "indexes",
        "deselectPoints",
        "toggleSelection",
        "selectedColor",
        "pointDelegate",
        "QQmlComponent*",
        "draggable",
        "selectedPoints",
        "declarativePointHint",
        "DeclarativePointHint",
        "declarativeMin",
        "stepSize",
        "X",
        "Y"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'pointReplaced'
        QtMocHelpers::SignalData<void(qsizetype)>(7, 8, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 10 },
        }}),
        // Signal 'pointRemoved'
        QtMocHelpers::SignalData<void(qsizetype)>(11, 8, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 10 },
        }}),
        // Signal 'pointAdded'
        QtMocHelpers::SignalData<void(qsizetype)>(12, 8, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 10 },
        }}),
        // Signal 'pointsAdded'
        QtMocHelpers::RevisionedSignalData<void(qsizetype, qsizetype)>(13, 8, QMC::AccessPublic, 0x609, QMetaType::Void, {{
            { 0x80000000 | 9, 14 }, { 0x80000000 | 9, 15 },
        }}),
        // Signal 'colorChanged'
        QtMocHelpers::SignalData<void(QColor)>(16, 8, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 17, 18 },
        }}),
        // Signal 'selectedColorChanged'
        QtMocHelpers::SignalData<void(QColor)>(19, 8, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 17, 18 },
        }}),
        // Signal 'pointsReplaced'
        QtMocHelpers::SignalData<void()>(20, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'pointsRemoved'
        QtMocHelpers::SignalData<void(qsizetype, qsizetype)>(21, 8, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 10 }, { 0x80000000 | 9, 22 },
        }}),
        // Signal 'selectedPointsChanged'
        QtMocHelpers::SignalData<void()>(23, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'pointDelegateChanged'
        QtMocHelpers::SignalData<void()>(24, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'draggableChanged'
        QtMocHelpers::SignalData<void()>(25, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'seriesUpdated'
        QtMocHelpers::SignalData<void()>(26, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'countChanged'
        QtMocHelpers::SignalData<void()>(27, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'clicked'
        QtMocHelpers::RevisionedSignalData<void(QPoint)>(28, 8, QMC::AccessPublic, 0x609, QMetaType::Void, {{
            { 0x80000000 | 29, 30 },
        }}),
        // Signal 'doubleClicked'
        QtMocHelpers::RevisionedSignalData<void(QPoint)>(31, 8, QMC::AccessPublic, 0x609, QMetaType::Void, {{
            { 0x80000000 | 29, 30 },
        }}),
        // Signal 'pressed'
        QtMocHelpers::RevisionedSignalData<void(QPoint)>(32, 8, QMC::AccessPublic, 0x609, QMetaType::Void, {{
            { 0x80000000 | 29, 30 },
        }}),
        // Signal 'released'
        QtMocHelpers::RevisionedSignalData<void(QPoint)>(33, 8, QMC::AccessPublic, 0x609, QMetaType::Void, {{
            { 0x80000000 | 29, 30 },
        }}),
        // Signal 'declarativePointsChanged'
        QtMocHelpers::RevisionedSignalData<void(const QVariantList &)>(34, 8, QMC::AccessPublic, 0x60c, QMetaType::Void, {{
            { 0x80000000 | 35, 36 },
        }}),
        // Signal 'declarativePointHintChanged'
        QtMocHelpers::RevisionedSignalData<void(QXYSeries::DeclarativePointHint)>(37, 8, QMC::AccessPublic, 0x60c, QMetaType::Void, {{
            { 0x80000000 | 38, 39 },
        }}),
        // Signal 'declarativeMinChanged'
        QtMocHelpers::RevisionedSignalData<void(qreal)>(40, 8, QMC::AccessPublic, 0x60c, QMetaType::Void, {{
            { QMetaType::QReal, 41 },
        }}),
        // Signal 'stepSizeChanged'
        QtMocHelpers::RevisionedSignalData<void(qreal)>(42, 8, QMC::AccessPublic, 0x60c, QMetaType::Void, {{
            { QMetaType::QReal, 43 },
        }}),
        // Method 'append'
        QtMocHelpers::MethodData<void(qreal, qreal)>(44, 8, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QReal, 45 }, { QMetaType::QReal, 46 },
        }}),
        // Method 'append'
        QtMocHelpers::MethodData<void(QPointF)>(44, 8, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 47, 30 },
        }}),
        // Method 'append'
        QtMocHelpers::MethodData<void(const QList<QPointF> &)>(44, 8, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 48, 49 },
        }}),
        // Method 'replace'
        QtMocHelpers::MethodData<void(qreal, qreal, qreal, qreal)>(50, 8, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QReal, 51 }, { QMetaType::QReal, 52 }, { QMetaType::QReal, 53 }, { QMetaType::QReal, 54 },
        }}),
        // Method 'replace'
        QtMocHelpers::MethodData<void(QPointF, QPointF)>(50, 8, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 47, 55 }, { 0x80000000 | 47, 56 },
        }}),
        // Method 'replace'
        QtMocHelpers::MethodData<void(qsizetype, qreal, qreal)>(50, 8, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 10 }, { QMetaType::QReal, 53 }, { QMetaType::QReal, 54 },
        }}),
        // Method 'replace'
        QtMocHelpers::MethodData<void(qsizetype, QPointF)>(50, 8, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 10 }, { 0x80000000 | 47, 56 },
        }}),
        // Method 'replace'
        QtMocHelpers::MethodData<void(const QList<QPointF> &)>(50, 8, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 48, 49 },
        }}),
        // Method 'remove'
        QtMocHelpers::MethodData<void(qreal, qreal)>(57, 8, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QReal, 45 }, { QMetaType::QReal, 46 },
        }}),
        // Method 'remove'
        QtMocHelpers::MethodData<void(QPointF)>(57, 8, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 47, 30 },
        }}),
        // Method 'remove'
        QtMocHelpers::MethodData<void(qsizetype)>(57, 8, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 10 },
        }}),
        // Method 'insert'
        QtMocHelpers::MethodData<void(qsizetype, QPointF)>(58, 8, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 10 }, { 0x80000000 | 47, 30 },
        }}),
        // Method 'clear'
        QtMocHelpers::MethodData<void()>(59, 8, QMC::AccessPublic, QMetaType::Void),
        // Method 'at'
        QtMocHelpers::MethodData<QPointF(qsizetype) const>(60, 8, QMC::AccessPublic, 0x80000000 | 47, {{
            { 0x80000000 | 9, 10 },
        }}),
        // Method 'find'
        QtMocHelpers::MethodData<qsizetype(QPointF) const>(61, 8, QMC::AccessPublic, 0x80000000 | 9, {{
            { 0x80000000 | 47, 30 },
        }}),
        // Method 'removeMultiple'
        QtMocHelpers::MethodData<void(qsizetype, qsizetype)>(62, 8, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 10 }, { 0x80000000 | 9, 22 },
        }}),
        // Method 'take'
        QtMocHelpers::MethodData<bool(QPointF)>(63, 8, QMC::AccessPublic, QMetaType::Bool, {{
            { 0x80000000 | 47, 30 },
        }}),
        // Method 'isPointSelected'
        QtMocHelpers::MethodData<bool(qsizetype) const>(64, 8, QMC::AccessPublic, QMetaType::Bool, {{
            { 0x80000000 | 9, 10 },
        }}),
        // Method 'selectPoint'
        QtMocHelpers::MethodData<void(qsizetype)>(65, 8, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 10 },
        }}),
        // Method 'deselectPoint'
        QtMocHelpers::MethodData<void(qsizetype)>(66, 8, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 10 },
        }}),
        // Method 'setPointSelected'
        QtMocHelpers::MethodData<void(qsizetype, bool)>(67, 8, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 10 }, { QMetaType::Bool, 68 },
        }}),
        // Method 'selectAllPoints'
        QtMocHelpers::MethodData<void()>(69, 8, QMC::AccessPublic, QMetaType::Void),
        // Method 'deselectAllPoints'
        QtMocHelpers::MethodData<void()>(70, 8, QMC::AccessPublic, QMetaType::Void),
        // Method 'selectPoints'
        QtMocHelpers::MethodData<void(const QList<qsizetype> &)>(71, 8, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 72, 73 },
        }}),
        // Method 'deselectPoints'
        QtMocHelpers::MethodData<void(const QList<qsizetype> &)>(74, 8, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 72, 73 },
        }}),
        // Method 'toggleSelection'
        QtMocHelpers::MethodData<void(const QList<qsizetype> &)>(75, 8, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 72, 73 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'color'
        QtMocHelpers::PropertyData<QColor>(18, 0x80000000 | 17, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet | QMC::Final, 4),
        // property 'selectedColor'
        QtMocHelpers::PropertyData<QColor>(76, 0x80000000 | 17, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet | QMC::Final, 5),
        // property 'pointDelegate'
        QtMocHelpers::PropertyData<QQmlComponent*>(77, 0x80000000 | 78, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet | QMC::Final, 9),
        // property 'draggable'
        QtMocHelpers::PropertyData<bool>(79, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 10),
        // property 'selectedPoints'
        QtMocHelpers::PropertyData<QList<qsizetype>>(80, 0x80000000 | 72, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Final, 8),
        // property 'count'
        QtMocHelpers::PropertyData<qsizetype>(22, 0x80000000 | 9, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Final, 12),
        // property 'declarativePoints'
        QtMocHelpers::PropertyData<QVariantList>(36, 0x80000000 | 35, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 17, 0x60c),
        // property 'declarativePointHint'
        QtMocHelpers::PropertyData<enum DeclarativePointHint>(81, 0x80000000 | 82, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 18, 0x60c),
        // property 'declarativeMin'
        QtMocHelpers::PropertyData<qreal>(83, QMetaType::QReal, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 19, 0x60c),
        // property 'stepSize'
        QtMocHelpers::PropertyData<qreal>(84, QMetaType::QReal, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 20, 0x60c),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'DeclarativePointHint'
        QtMocHelpers::EnumData<enum DeclarativePointHint>(82, 82, QMC::EnumIsScoped).add({
            {   85, DeclarativePointHint::X },
            {   86, DeclarativePointHint::Y },
        }),
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
            {    5,    6 },
    });
    return QtMocHelpers::metaObjectData<QXYSeries, void>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject QXYSeries::staticMetaObject = { {
    QMetaObject::SuperData::link<QAbstractSeries::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9QXYSeriesE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9QXYSeriesE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN9QXYSeriesE_t>.metaTypes,
    nullptr
} };

void QXYSeries::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QXYSeries *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->pointReplaced((*reinterpret_cast<std::add_pointer_t<qsizetype>>(_a[1]))); break;
        case 1: _t->pointRemoved((*reinterpret_cast<std::add_pointer_t<qsizetype>>(_a[1]))); break;
        case 2: _t->pointAdded((*reinterpret_cast<std::add_pointer_t<qsizetype>>(_a[1]))); break;
        case 3: _t->pointsAdded((*reinterpret_cast<std::add_pointer_t<qsizetype>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<qsizetype>>(_a[2]))); break;
        case 4: _t->colorChanged((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        case 5: _t->selectedColorChanged((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        case 6: _t->pointsReplaced(); break;
        case 7: _t->pointsRemoved((*reinterpret_cast<std::add_pointer_t<qsizetype>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<qsizetype>>(_a[2]))); break;
        case 8: _t->selectedPointsChanged(); break;
        case 9: _t->pointDelegateChanged(); break;
        case 10: _t->draggableChanged(); break;
        case 11: _t->seriesUpdated(); break;
        case 12: _t->countChanged(); break;
        case 13: _t->clicked((*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[1]))); break;
        case 14: _t->doubleClicked((*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[1]))); break;
        case 15: _t->pressed((*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[1]))); break;
        case 16: _t->released((*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[1]))); break;
        case 17: _t->declarativePointsChanged((*reinterpret_cast<std::add_pointer_t<QVariantList>>(_a[1]))); break;
        case 18: _t->declarativePointHintChanged((*reinterpret_cast<std::add_pointer_t<QXYSeries::DeclarativePointHint>>(_a[1]))); break;
        case 19: _t->declarativeMinChanged((*reinterpret_cast<std::add_pointer_t<qreal>>(_a[1]))); break;
        case 20: _t->stepSizeChanged((*reinterpret_cast<std::add_pointer_t<qreal>>(_a[1]))); break;
        case 21: _t->append((*reinterpret_cast<std::add_pointer_t<qreal>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<qreal>>(_a[2]))); break;
        case 22: _t->append((*reinterpret_cast<std::add_pointer_t<QPointF>>(_a[1]))); break;
        case 23: _t->append((*reinterpret_cast<std::add_pointer_t<QList<QPointF>>>(_a[1]))); break;
        case 24: _t->replace((*reinterpret_cast<std::add_pointer_t<qreal>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<qreal>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<qreal>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<qreal>>(_a[4]))); break;
        case 25: _t->replace((*reinterpret_cast<std::add_pointer_t<QPointF>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QPointF>>(_a[2]))); break;
        case 26: _t->replace((*reinterpret_cast<std::add_pointer_t<qsizetype>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<qreal>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<qreal>>(_a[3]))); break;
        case 27: _t->replace((*reinterpret_cast<std::add_pointer_t<qsizetype>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QPointF>>(_a[2]))); break;
        case 28: _t->replace((*reinterpret_cast<std::add_pointer_t<QList<QPointF>>>(_a[1]))); break;
        case 29: _t->remove((*reinterpret_cast<std::add_pointer_t<qreal>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<qreal>>(_a[2]))); break;
        case 30: _t->remove((*reinterpret_cast<std::add_pointer_t<QPointF>>(_a[1]))); break;
        case 31: _t->remove((*reinterpret_cast<std::add_pointer_t<qsizetype>>(_a[1]))); break;
        case 32: _t->insert((*reinterpret_cast<std::add_pointer_t<qsizetype>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QPointF>>(_a[2]))); break;
        case 33: _t->clear(); break;
        case 34: { QPointF _r = _t->at((*reinterpret_cast<std::add_pointer_t<qsizetype>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QPointF*>(_a[0]) = std::move(_r); }  break;
        case 35: { qsizetype _r = _t->find((*reinterpret_cast<std::add_pointer_t<QPointF>>(_a[1])));
            if (_a[0]) *reinterpret_cast<qsizetype*>(_a[0]) = std::move(_r); }  break;
        case 36: _t->removeMultiple((*reinterpret_cast<std::add_pointer_t<qsizetype>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<qsizetype>>(_a[2]))); break;
        case 37: { bool _r = _t->take((*reinterpret_cast<std::add_pointer_t<QPointF>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 38: { bool _r = _t->isPointSelected((*reinterpret_cast<std::add_pointer_t<qsizetype>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 39: _t->selectPoint((*reinterpret_cast<std::add_pointer_t<qsizetype>>(_a[1]))); break;
        case 40: _t->deselectPoint((*reinterpret_cast<std::add_pointer_t<qsizetype>>(_a[1]))); break;
        case 41: _t->setPointSelected((*reinterpret_cast<std::add_pointer_t<qsizetype>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 42: _t->selectAllPoints(); break;
        case 43: _t->deselectAllPoints(); break;
        case 44: _t->selectPoints((*reinterpret_cast<std::add_pointer_t<QList<qsizetype>>>(_a[1]))); break;
        case 45: _t->deselectPoints((*reinterpret_cast<std::add_pointer_t<QList<qsizetype>>>(_a[1]))); break;
        case 46: _t->toggleSelection((*reinterpret_cast<std::add_pointer_t<QList<qsizetype>>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (QXYSeries::*)(qsizetype )>(_a, &QXYSeries::pointReplaced, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (QXYSeries::*)(qsizetype )>(_a, &QXYSeries::pointRemoved, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (QXYSeries::*)(qsizetype )>(_a, &QXYSeries::pointAdded, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (QXYSeries::*)(qsizetype , qsizetype )>(_a, &QXYSeries::pointsAdded, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (QXYSeries::*)(QColor )>(_a, &QXYSeries::colorChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (QXYSeries::*)(QColor )>(_a, &QXYSeries::selectedColorChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (QXYSeries::*)()>(_a, &QXYSeries::pointsReplaced, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (QXYSeries::*)(qsizetype , qsizetype )>(_a, &QXYSeries::pointsRemoved, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (QXYSeries::*)()>(_a, &QXYSeries::selectedPointsChanged, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (QXYSeries::*)()>(_a, &QXYSeries::pointDelegateChanged, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (QXYSeries::*)()>(_a, &QXYSeries::draggableChanged, 10))
            return;
        if (QtMocHelpers::indexOfMethod<void (QXYSeries::*)()>(_a, &QXYSeries::seriesUpdated, 11))
            return;
        if (QtMocHelpers::indexOfMethod<void (QXYSeries::*)()>(_a, &QXYSeries::countChanged, 12))
            return;
        if (QtMocHelpers::indexOfMethod<void (QXYSeries::*)(QPoint )>(_a, &QXYSeries::clicked, 13))
            return;
        if (QtMocHelpers::indexOfMethod<void (QXYSeries::*)(QPoint )>(_a, &QXYSeries::doubleClicked, 14))
            return;
        if (QtMocHelpers::indexOfMethod<void (QXYSeries::*)(QPoint )>(_a, &QXYSeries::pressed, 15))
            return;
        if (QtMocHelpers::indexOfMethod<void (QXYSeries::*)(QPoint )>(_a, &QXYSeries::released, 16))
            return;
        if (QtMocHelpers::indexOfMethod<void (QXYSeries::*)(const QVariantList & )>(_a, &QXYSeries::declarativePointsChanged, 17))
            return;
        if (QtMocHelpers::indexOfMethod<void (QXYSeries::*)(QXYSeries::DeclarativePointHint )>(_a, &QXYSeries::declarativePointHintChanged, 18))
            return;
        if (QtMocHelpers::indexOfMethod<void (QXYSeries::*)(qreal )>(_a, &QXYSeries::declarativeMinChanged, 19))
            return;
        if (QtMocHelpers::indexOfMethod<void (QXYSeries::*)(qreal )>(_a, &QXYSeries::stepSizeChanged, 20))
            return;
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 2:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQmlComponent* >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QColor*>(_v) = _t->color(); break;
        case 1: *reinterpret_cast<QColor*>(_v) = _t->selectedColor(); break;
        case 2: *reinterpret_cast<QQmlComponent**>(_v) = _t->pointDelegate(); break;
        case 3: *reinterpret_cast<bool*>(_v) = _t->isDraggable(); break;
        case 4: *reinterpret_cast<QList<qsizetype>*>(_v) = _t->selectedPoints(); break;
        case 5: *reinterpret_cast<qsizetype*>(_v) = _t->count(); break;
        case 6: *reinterpret_cast<QVariantList*>(_v) = _t->declarativePoints(); break;
        case 7: *reinterpret_cast<enum DeclarativePointHint*>(_v) = _t->declarativePointHint(); break;
        case 8: *reinterpret_cast<qreal*>(_v) = _t->declarativeMin(); break;
        case 9: *reinterpret_cast<qreal*>(_v) = _t->stepSize(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setColor(*reinterpret_cast<QColor*>(_v)); break;
        case 1: _t->setSelectedColor(*reinterpret_cast<QColor*>(_v)); break;
        case 2: _t->setPointDelegate(*reinterpret_cast<QQmlComponent**>(_v)); break;
        case 3: _t->setDraggable(*reinterpret_cast<bool*>(_v)); break;
        case 6: _t->setDeclarativePoints(*reinterpret_cast<QVariantList*>(_v)); break;
        case 7: _t->setDeclarativePointHint(*reinterpret_cast<enum DeclarativePointHint*>(_v)); break;
        case 8: _t->setDeclarativeMin(*reinterpret_cast<qreal*>(_v)); break;
        case 9: _t->setStepSize(*reinterpret_cast<qreal*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *QXYSeries::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QXYSeries::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9QXYSeriesE_t>.strings))
        return static_cast<void*>(this);
    return QAbstractSeries::qt_metacast(_clname);
}

int QXYSeries::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QAbstractSeries::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 47)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 47;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 47)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 47;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    }
    return _id;
}

// SIGNAL 0
void QXYSeries::pointReplaced(qsizetype _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void QXYSeries::pointRemoved(qsizetype _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void QXYSeries::pointAdded(qsizetype _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void QXYSeries::pointsAdded(qsizetype _t1, qsizetype _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1, _t2);
}

// SIGNAL 4
void QXYSeries::colorChanged(QColor _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void QXYSeries::selectedColorChanged(QColor _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void QXYSeries::pointsReplaced()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void QXYSeries::pointsRemoved(qsizetype _t1, qsizetype _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1, _t2);
}

// SIGNAL 8
void QXYSeries::selectedPointsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 8, nullptr);
}

// SIGNAL 9
void QXYSeries::pointDelegateChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 9, nullptr);
}

// SIGNAL 10
void QXYSeries::draggableChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 10, nullptr);
}

// SIGNAL 11
void QXYSeries::seriesUpdated()
{
    QMetaObject::activate(this, &staticMetaObject, 11, nullptr);
}

// SIGNAL 12
void QXYSeries::countChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 12, nullptr);
}

// SIGNAL 13
void QXYSeries::clicked(QPoint _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 13, nullptr, _t1);
}

// SIGNAL 14
void QXYSeries::doubleClicked(QPoint _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 14, nullptr, _t1);
}

// SIGNAL 15
void QXYSeries::pressed(QPoint _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 15, nullptr, _t1);
}

// SIGNAL 16
void QXYSeries::released(QPoint _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 16, nullptr, _t1);
}

// SIGNAL 17
void QXYSeries::declarativePointsChanged(const QVariantList & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 17, nullptr, _t1);
}

// SIGNAL 18
void QXYSeries::declarativePointHintChanged(QXYSeries::DeclarativePointHint _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 18, nullptr, _t1);
}

// SIGNAL 19
void QXYSeries::declarativeMinChanged(qreal _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 19, nullptr, _t1);
}

// SIGNAL 20
void QXYSeries::stepSizeChanged(qreal _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 20, nullptr, _t1);
}
QT_WARNING_POP
