/****************************************************************************
** Meta object code from reading C++ file 'qbarseries.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../qtgraphs/src/graphs2d/barchart/qbarseries.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qbarseries.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN10QBarSeriesE_t {};
} // unnamed namespace

template <> constexpr inline auto QBarSeries::qt_create_metaobjectdata<qt_meta_tag_ZN10QBarSeriesE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QBarSeries",
        "RegisterEnumClassesUnscoped",
        "false",
        "QML.Element",
        "BarSeries",
        "updatedBars",
        "",
        "seriesColorsChanged",
        "borderColorsChanged",
        "countChanged",
        "barWidthChanged",
        "labelsVisibleChanged",
        "visible",
        "labelsFormatChanged",
        "format",
        "labelsPositionChanged",
        "QBarSeries::LabelsPosition",
        "position",
        "barsTypeChanged",
        "QBarSeries::BarsType",
        "type",
        "labelsMarginChanged",
        "margin",
        "labelsAngleChanged",
        "angle",
        "labelsPrecisionChanged",
        "precision",
        "barDelegateChanged",
        "barsetsAdded",
        "QList<QBarSet*>",
        "sets",
        "barsetsReplaced",
        "barsetsRemoved",
        "setValueChanged",
        "qsizetype",
        "index",
        "QBarSet*",
        "barset",
        "setValueAdded",
        "count",
        "setValueRemoved",
        "barSetsChanged",
        "clicked",
        "doubleClicked",
        "pressed",
        "released",
        "selectAll",
        "deselectAll",
        "handleSetValueChange",
        "handleSetValueAdd",
        "handleSetValueRemove",
        "append",
        "set",
        "take",
        "remove",
        "insert",
        "clear",
        "replace",
        "at",
        "find",
        "removeMultiple",
        "oldValue",
        "newValue",
        "seriesColors",
        "QList<QColor>",
        "borderColors",
        "barsType",
        "BarsType",
        "barWidth",
        "labelsVisible",
        "labelsFormat",
        "labelsPosition",
        "LabelsPosition",
        "labelsMargin",
        "labelsAngle",
        "labelsPrecision",
        "barDelegate",
        "QQmlComponent*",
        "barSets",
        "Center",
        "InsideEnd",
        "InsideBase",
        "OutsideEnd",
        "Groups",
        "Stacked",
        "StackedPercent"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'updatedBars'
        QtMocHelpers::SignalData<void()>(5, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'seriesColorsChanged'
        QtMocHelpers::SignalData<void()>(7, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'borderColorsChanged'
        QtMocHelpers::SignalData<void()>(8, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'countChanged'
        QtMocHelpers::SignalData<void()>(9, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'barWidthChanged'
        QtMocHelpers::SignalData<void()>(10, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'labelsVisibleChanged'
        QtMocHelpers::SignalData<void(bool)>(11, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 12 },
        }}),
        // Signal 'labelsFormatChanged'
        QtMocHelpers::SignalData<void(const QString &)>(13, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 14 },
        }}),
        // Signal 'labelsPositionChanged'
        QtMocHelpers::SignalData<void(QBarSeries::LabelsPosition)>(15, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 16, 17 },
        }}),
        // Signal 'barsTypeChanged'
        QtMocHelpers::SignalData<void(QBarSeries::BarsType)>(18, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 19, 20 },
        }}),
        // Signal 'labelsMarginChanged'
        QtMocHelpers::SignalData<void(qreal)>(21, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QReal, 22 },
        }}),
        // Signal 'labelsAngleChanged'
        QtMocHelpers::SignalData<void(qreal)>(23, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QReal, 24 },
        }}),
        // Signal 'labelsPrecisionChanged'
        QtMocHelpers::SignalData<void(int)>(25, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 26 },
        }}),
        // Signal 'barDelegateChanged'
        QtMocHelpers::SignalData<void()>(27, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'barsetsAdded'
        QtMocHelpers::SignalData<void(const QList<QBarSet*> &)>(28, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 29, 30 },
        }}),
        // Signal 'barsetsReplaced'
        QtMocHelpers::SignalData<void(const QList<QBarSet*> &)>(31, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 29, 30 },
        }}),
        // Signal 'barsetsRemoved'
        QtMocHelpers::SignalData<void(const QList<QBarSet*> &)>(32, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 29, 30 },
        }}),
        // Signal 'setValueChanged'
        QtMocHelpers::SignalData<void(qsizetype, QBarSet *)>(33, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 34, 35 }, { 0x80000000 | 36, 37 },
        }}),
        // Signal 'setValueAdded'
        QtMocHelpers::SignalData<void(qsizetype, qsizetype, QBarSet *)>(38, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 34, 35 }, { 0x80000000 | 34, 39 }, { 0x80000000 | 36, 37 },
        }}),
        // Signal 'setValueRemoved'
        QtMocHelpers::SignalData<void(qsizetype, qsizetype, QBarSet *)>(40, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 34, 35 }, { 0x80000000 | 34, 39 }, { 0x80000000 | 36, 37 },
        }}),
        // Signal 'barSetsChanged'
        QtMocHelpers::SignalData<void()>(41, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'clicked'
        QtMocHelpers::RevisionedSignalData<void(qsizetype, QBarSet *)>(42, 6, QMC::AccessPublic, 0x609, QMetaType::Void, {{
            { 0x80000000 | 34, 35 }, { 0x80000000 | 36, 37 },
        }}),
        // Signal 'doubleClicked'
        QtMocHelpers::RevisionedSignalData<void(qsizetype, QBarSet *)>(43, 6, QMC::AccessPublic, 0x609, QMetaType::Void, {{
            { 0x80000000 | 34, 35 }, { 0x80000000 | 36, 37 },
        }}),
        // Signal 'pressed'
        QtMocHelpers::RevisionedSignalData<void(qsizetype, QBarSet *)>(44, 6, QMC::AccessPublic, 0x609, QMetaType::Void, {{
            { 0x80000000 | 34, 35 }, { 0x80000000 | 36, 37 },
        }}),
        // Signal 'released'
        QtMocHelpers::RevisionedSignalData<void(qsizetype, QBarSet *)>(45, 6, QMC::AccessPublic, 0x609, QMetaType::Void, {{
            { 0x80000000 | 34, 35 }, { 0x80000000 | 36, 37 },
        }}),
        // Slot 'selectAll'
        QtMocHelpers::SlotData<void()>(46, 6, QMC::AccessPublic, QMetaType::Void),
        // Slot 'deselectAll'
        QtMocHelpers::SlotData<void()>(47, 6, QMC::AccessPublic, QMetaType::Void),
        // Slot 'handleSetValueChange'
        QtMocHelpers::SlotData<void(qsizetype)>(48, 6, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 34, 35 },
        }}),
        // Slot 'handleSetValueAdd'
        QtMocHelpers::SlotData<void(qsizetype, qsizetype)>(49, 6, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 34, 35 }, { 0x80000000 | 34, 39 },
        }}),
        // Slot 'handleSetValueRemove'
        QtMocHelpers::SlotData<void(qsizetype, qsizetype)>(50, 6, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 34, 35 }, { 0x80000000 | 34, 39 },
        }}),
        // Method 'append'
        QtMocHelpers::MethodData<bool(QBarSet *)>(51, 6, QMC::AccessPublic, QMetaType::Bool, {{
            { 0x80000000 | 36, 52 },
        }}),
        // Method 'take'
        QtMocHelpers::MethodData<bool(QBarSet *)>(53, 6, QMC::AccessPublic, QMetaType::Bool, {{
            { 0x80000000 | 36, 52 },
        }}),
        // Method 'count'
        QtMocHelpers::MethodData<qsizetype() const>(39, 6, QMC::AccessPublic, 0x80000000 | 34),
        // Method 'append'
        QtMocHelpers::MethodData<bool(const QList<QBarSet*> &)>(51, 6, QMC::AccessPublic, QMetaType::Bool, {{
            { 0x80000000 | 29, 30 },
        }}),
        // Method 'remove'
        QtMocHelpers::MethodData<bool(QBarSet *)>(54, 6, QMC::AccessPublic, QMetaType::Bool, {{
            { 0x80000000 | 36, 52 },
        }}),
        // Method 'insert'
        QtMocHelpers::MethodData<bool(qsizetype, QBarSet *)>(55, 6, QMC::AccessPublic, QMetaType::Bool, {{
            { 0x80000000 | 34, 35 }, { 0x80000000 | 36, 52 },
        }}),
        // Method 'clear'
        QtMocHelpers::MethodData<void()>(56, 6, QMC::AccessPublic, QMetaType::Void),
        // Method 'replace'
        QtMocHelpers::MethodData<void(qsizetype, QBarSet *)>(57, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 34, 35 }, { 0x80000000 | 36, 52 },
        }}),
        // Method 'at'
        QtMocHelpers::MethodData<QBarSet *(qsizetype)>(58, 6, QMC::AccessPublic, 0x80000000 | 36, {{
            { 0x80000000 | 34, 35 },
        }}),
        // Method 'find'
        QtMocHelpers::MethodData<qsizetype(QBarSet *) const>(59, 6, QMC::AccessPublic, 0x80000000 | 34, {{
            { 0x80000000 | 36, 52 },
        }}),
        // Method 'removeMultiple'
        QtMocHelpers::MethodData<void(qsizetype, qsizetype)>(60, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 34, 35 }, { 0x80000000 | 34, 39 },
        }}),
        // Method 'remove'
        QtMocHelpers::MethodData<bool(qsizetype)>(54, 6, QMC::AccessPublic, QMetaType::Bool, {{
            { 0x80000000 | 34, 35 },
        }}),
        // Method 'replace'
        QtMocHelpers::MethodData<bool(QBarSet *, QBarSet *)>(57, 6, QMC::AccessPublic, QMetaType::Bool, {{
            { 0x80000000 | 36, 61 }, { 0x80000000 | 36, 62 },
        }}),
        // Method 'replace'
        QtMocHelpers::MethodData<bool(const QList<QBarSet*> &)>(57, 6, QMC::AccessPublic, QMetaType::Bool, {{
            { 0x80000000 | 29, 30 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'seriesColors'
        QtMocHelpers::PropertyData<QList<QColor>>(63, 0x80000000 | 64, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet | QMC::Final, 1),
        // property 'borderColors'
        QtMocHelpers::PropertyData<QList<QColor>>(65, 0x80000000 | 64, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet | QMC::Final, 2),
        // property 'barsType'
        QtMocHelpers::PropertyData<enum BarsType>(66, 0x80000000 | 67, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet | QMC::Final, 8),
        // property 'barWidth'
        QtMocHelpers::PropertyData<qreal>(68, QMetaType::QReal, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 4),
        // property 'count'
        QtMocHelpers::PropertyData<qsizetype>(39, 0x80000000 | 34, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Final, 3),
        // property 'labelsVisible'
        QtMocHelpers::PropertyData<bool>(69, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 5),
        // property 'labelsFormat'
        QtMocHelpers::PropertyData<QString>(70, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 6),
        // property 'labelsPosition'
        QtMocHelpers::PropertyData<enum LabelsPosition>(71, 0x80000000 | 72, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet | QMC::Final, 7),
        // property 'labelsMargin'
        QtMocHelpers::PropertyData<qreal>(73, QMetaType::QReal, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 9),
        // property 'labelsAngle'
        QtMocHelpers::PropertyData<qreal>(74, QMetaType::QReal, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 10),
        // property 'labelsPrecision'
        QtMocHelpers::PropertyData<int>(75, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 11),
        // property 'barDelegate'
        QtMocHelpers::PropertyData<QQmlComponent*>(76, 0x80000000 | 77, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet | QMC::Final, 12),
        // property 'barSets'
        QtMocHelpers::PropertyData<QList<QBarSet*>>(78, 0x80000000 | 29, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Final, 19),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'LabelsPosition'
        QtMocHelpers::EnumData<enum LabelsPosition>(72, 72, QMC::EnumIsScoped).add({
            {   79, LabelsPosition::Center },
            {   80, LabelsPosition::InsideEnd },
            {   81, LabelsPosition::InsideBase },
            {   82, LabelsPosition::OutsideEnd },
        }),
        // enum 'BarsType'
        QtMocHelpers::EnumData<enum BarsType>(67, 67, QMC::EnumIsScoped).add({
            {   83, BarsType::Groups },
            {   84, BarsType::Stacked },
            {   85, BarsType::StackedPercent },
        }),
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
    });
    return QtMocHelpers::metaObjectData<QBarSeries, void>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject QBarSeries::staticMetaObject = { {
    QMetaObject::SuperData::link<QAbstractSeries::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10QBarSeriesE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10QBarSeriesE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10QBarSeriesE_t>.metaTypes,
    nullptr
} };

void QBarSeries::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QBarSeries *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->updatedBars(); break;
        case 1: _t->seriesColorsChanged(); break;
        case 2: _t->borderColorsChanged(); break;
        case 3: _t->countChanged(); break;
        case 4: _t->barWidthChanged(); break;
        case 5: _t->labelsVisibleChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 6: _t->labelsFormatChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 7: _t->labelsPositionChanged((*reinterpret_cast<std::add_pointer_t<QBarSeries::LabelsPosition>>(_a[1]))); break;
        case 8: _t->barsTypeChanged((*reinterpret_cast<std::add_pointer_t<QBarSeries::BarsType>>(_a[1]))); break;
        case 9: _t->labelsMarginChanged((*reinterpret_cast<std::add_pointer_t<qreal>>(_a[1]))); break;
        case 10: _t->labelsAngleChanged((*reinterpret_cast<std::add_pointer_t<qreal>>(_a[1]))); break;
        case 11: _t->labelsPrecisionChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 12: _t->barDelegateChanged(); break;
        case 13: _t->barsetsAdded((*reinterpret_cast<std::add_pointer_t<QList<QBarSet*>>>(_a[1]))); break;
        case 14: _t->barsetsReplaced((*reinterpret_cast<std::add_pointer_t<QList<QBarSet*>>>(_a[1]))); break;
        case 15: _t->barsetsRemoved((*reinterpret_cast<std::add_pointer_t<QList<QBarSet*>>>(_a[1]))); break;
        case 16: _t->setValueChanged((*reinterpret_cast<std::add_pointer_t<qsizetype>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QBarSet*>>(_a[2]))); break;
        case 17: _t->setValueAdded((*reinterpret_cast<std::add_pointer_t<qsizetype>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<qsizetype>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QBarSet*>>(_a[3]))); break;
        case 18: _t->setValueRemoved((*reinterpret_cast<std::add_pointer_t<qsizetype>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<qsizetype>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QBarSet*>>(_a[3]))); break;
        case 19: _t->barSetsChanged(); break;
        case 20: _t->clicked((*reinterpret_cast<std::add_pointer_t<qsizetype>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QBarSet*>>(_a[2]))); break;
        case 21: _t->doubleClicked((*reinterpret_cast<std::add_pointer_t<qsizetype>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QBarSet*>>(_a[2]))); break;
        case 22: _t->pressed((*reinterpret_cast<std::add_pointer_t<qsizetype>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QBarSet*>>(_a[2]))); break;
        case 23: _t->released((*reinterpret_cast<std::add_pointer_t<qsizetype>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QBarSet*>>(_a[2]))); break;
        case 24: _t->selectAll(); break;
        case 25: _t->deselectAll(); break;
        case 26: _t->handleSetValueChange((*reinterpret_cast<std::add_pointer_t<qsizetype>>(_a[1]))); break;
        case 27: _t->handleSetValueAdd((*reinterpret_cast<std::add_pointer_t<qsizetype>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<qsizetype>>(_a[2]))); break;
        case 28: _t->handleSetValueRemove((*reinterpret_cast<std::add_pointer_t<qsizetype>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<qsizetype>>(_a[2]))); break;
        case 29: { bool _r = _t->append((*reinterpret_cast<std::add_pointer_t<QBarSet*>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 30: { bool _r = _t->take((*reinterpret_cast<std::add_pointer_t<QBarSet*>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 31: { qsizetype _r = _t->count();
            if (_a[0]) *reinterpret_cast<qsizetype*>(_a[0]) = std::move(_r); }  break;
        case 32: { bool _r = _t->append((*reinterpret_cast<std::add_pointer_t<QList<QBarSet*>>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 33: { bool _r = _t->remove((*reinterpret_cast<std::add_pointer_t<QBarSet*>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 34: { bool _r = _t->insert((*reinterpret_cast<std::add_pointer_t<qsizetype>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QBarSet*>>(_a[2])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 35: _t->clear(); break;
        case 36: _t->replace((*reinterpret_cast<std::add_pointer_t<qsizetype>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QBarSet*>>(_a[2]))); break;
        case 37: { QBarSet* _r = _t->at((*reinterpret_cast<std::add_pointer_t<qsizetype>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QBarSet**>(_a[0]) = std::move(_r); }  break;
        case 38: { qsizetype _r = _t->find((*reinterpret_cast<std::add_pointer_t<QBarSet*>>(_a[1])));
            if (_a[0]) *reinterpret_cast<qsizetype*>(_a[0]) = std::move(_r); }  break;
        case 39: _t->removeMultiple((*reinterpret_cast<std::add_pointer_t<qsizetype>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<qsizetype>>(_a[2]))); break;
        case 40: { bool _r = _t->remove((*reinterpret_cast<std::add_pointer_t<qsizetype>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 41: { bool _r = _t->replace((*reinterpret_cast<std::add_pointer_t<QBarSet*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QBarSet*>>(_a[2])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 42: { bool _r = _t->replace((*reinterpret_cast<std::add_pointer_t<QList<QBarSet*>>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (QBarSeries::*)()>(_a, &QBarSeries::updatedBars, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (QBarSeries::*)()>(_a, &QBarSeries::seriesColorsChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (QBarSeries::*)()>(_a, &QBarSeries::borderColorsChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (QBarSeries::*)()>(_a, &QBarSeries::countChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (QBarSeries::*)()>(_a, &QBarSeries::barWidthChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (QBarSeries::*)(bool )>(_a, &QBarSeries::labelsVisibleChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (QBarSeries::*)(const QString & )>(_a, &QBarSeries::labelsFormatChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (QBarSeries::*)(QBarSeries::LabelsPosition )>(_a, &QBarSeries::labelsPositionChanged, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (QBarSeries::*)(QBarSeries::BarsType )>(_a, &QBarSeries::barsTypeChanged, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (QBarSeries::*)(qreal )>(_a, &QBarSeries::labelsMarginChanged, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (QBarSeries::*)(qreal )>(_a, &QBarSeries::labelsAngleChanged, 10))
            return;
        if (QtMocHelpers::indexOfMethod<void (QBarSeries::*)(int )>(_a, &QBarSeries::labelsPrecisionChanged, 11))
            return;
        if (QtMocHelpers::indexOfMethod<void (QBarSeries::*)()>(_a, &QBarSeries::barDelegateChanged, 12))
            return;
        if (QtMocHelpers::indexOfMethod<void (QBarSeries::*)(const QList<QBarSet*> & )>(_a, &QBarSeries::barsetsAdded, 13))
            return;
        if (QtMocHelpers::indexOfMethod<void (QBarSeries::*)(const QList<QBarSet*> & )>(_a, &QBarSeries::barsetsReplaced, 14))
            return;
        if (QtMocHelpers::indexOfMethod<void (QBarSeries::*)(const QList<QBarSet*> & )>(_a, &QBarSeries::barsetsRemoved, 15))
            return;
        if (QtMocHelpers::indexOfMethod<void (QBarSeries::*)(qsizetype , QBarSet * )>(_a, &QBarSeries::setValueChanged, 16))
            return;
        if (QtMocHelpers::indexOfMethod<void (QBarSeries::*)(qsizetype , qsizetype , QBarSet * )>(_a, &QBarSeries::setValueAdded, 17))
            return;
        if (QtMocHelpers::indexOfMethod<void (QBarSeries::*)(qsizetype , qsizetype , QBarSet * )>(_a, &QBarSeries::setValueRemoved, 18))
            return;
        if (QtMocHelpers::indexOfMethod<void (QBarSeries::*)()>(_a, &QBarSeries::barSetsChanged, 19))
            return;
        if (QtMocHelpers::indexOfMethod<void (QBarSeries::*)(qsizetype , QBarSet * )>(_a, &QBarSeries::clicked, 20))
            return;
        if (QtMocHelpers::indexOfMethod<void (QBarSeries::*)(qsizetype , QBarSet * )>(_a, &QBarSeries::doubleClicked, 21))
            return;
        if (QtMocHelpers::indexOfMethod<void (QBarSeries::*)(qsizetype , QBarSet * )>(_a, &QBarSeries::pressed, 22))
            return;
        if (QtMocHelpers::indexOfMethod<void (QBarSeries::*)(qsizetype , QBarSet * )>(_a, &QBarSeries::released, 23))
            return;
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 11:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQmlComponent* >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QList<QColor>*>(_v) = _t->seriesColors(); break;
        case 1: *reinterpret_cast<QList<QColor>*>(_v) = _t->borderColors(); break;
        case 2: *reinterpret_cast<enum BarsType*>(_v) = _t->barsType(); break;
        case 3: *reinterpret_cast<qreal*>(_v) = _t->barWidth(); break;
        case 4: *reinterpret_cast<qsizetype*>(_v) = _t->count(); break;
        case 5: *reinterpret_cast<bool*>(_v) = _t->labelsVisible(); break;
        case 6: *reinterpret_cast<QString*>(_v) = _t->labelsFormat(); break;
        case 7: *reinterpret_cast<enum LabelsPosition*>(_v) = _t->labelsPosition(); break;
        case 8: *reinterpret_cast<qreal*>(_v) = _t->labelsMargin(); break;
        case 9: *reinterpret_cast<qreal*>(_v) = _t->labelsAngle(); break;
        case 10: *reinterpret_cast<int*>(_v) = _t->labelsPrecision(); break;
        case 11: *reinterpret_cast<QQmlComponent**>(_v) = _t->barDelegate(); break;
        case 12: *reinterpret_cast<QList<QBarSet*>*>(_v) = _t->barSets(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setSeriesColors(*reinterpret_cast<QList<QColor>*>(_v)); break;
        case 1: _t->setBorderColors(*reinterpret_cast<QList<QColor>*>(_v)); break;
        case 2: _t->setBarsType(*reinterpret_cast<enum BarsType*>(_v)); break;
        case 3: _t->setBarWidth(*reinterpret_cast<qreal*>(_v)); break;
        case 5: _t->setLabelsVisible(*reinterpret_cast<bool*>(_v)); break;
        case 6: _t->setLabelsFormat(*reinterpret_cast<QString*>(_v)); break;
        case 7: _t->setLabelsPosition(*reinterpret_cast<enum LabelsPosition*>(_v)); break;
        case 8: _t->setLabelsMargin(*reinterpret_cast<qreal*>(_v)); break;
        case 9: _t->setLabelsAngle(*reinterpret_cast<qreal*>(_v)); break;
        case 10: _t->setLabelsPrecision(*reinterpret_cast<int*>(_v)); break;
        case 11: _t->setBarDelegate(*reinterpret_cast<QQmlComponent**>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *QBarSeries::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QBarSeries::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10QBarSeriesE_t>.strings))
        return static_cast<void*>(this);
    return QAbstractSeries::qt_metacast(_clname);
}

int QBarSeries::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QAbstractSeries::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 43)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 43;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 43)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 43;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 13;
    }
    return _id;
}

// SIGNAL 0
void QBarSeries::updatedBars()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void QBarSeries::seriesColorsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void QBarSeries::borderColorsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void QBarSeries::countChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void QBarSeries::barWidthChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void QBarSeries::labelsVisibleChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void QBarSeries::labelsFormatChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}

// SIGNAL 7
void QBarSeries::labelsPositionChanged(QBarSeries::LabelsPosition _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1);
}

// SIGNAL 8
void QBarSeries::barsTypeChanged(QBarSeries::BarsType _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 8, nullptr, _t1);
}

// SIGNAL 9
void QBarSeries::labelsMarginChanged(qreal _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 9, nullptr, _t1);
}

// SIGNAL 10
void QBarSeries::labelsAngleChanged(qreal _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 10, nullptr, _t1);
}

// SIGNAL 11
void QBarSeries::labelsPrecisionChanged(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 11, nullptr, _t1);
}

// SIGNAL 12
void QBarSeries::barDelegateChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 12, nullptr);
}

// SIGNAL 13
void QBarSeries::barsetsAdded(const QList<QBarSet*> & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 13, nullptr, _t1);
}

// SIGNAL 14
void QBarSeries::barsetsReplaced(const QList<QBarSet*> & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 14, nullptr, _t1);
}

// SIGNAL 15
void QBarSeries::barsetsRemoved(const QList<QBarSet*> & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 15, nullptr, _t1);
}

// SIGNAL 16
void QBarSeries::setValueChanged(qsizetype _t1, QBarSet * _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 16, nullptr, _t1, _t2);
}

// SIGNAL 17
void QBarSeries::setValueAdded(qsizetype _t1, qsizetype _t2, QBarSet * _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 17, nullptr, _t1, _t2, _t3);
}

// SIGNAL 18
void QBarSeries::setValueRemoved(qsizetype _t1, qsizetype _t2, QBarSet * _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 18, nullptr, _t1, _t2, _t3);
}

// SIGNAL 19
void QBarSeries::barSetsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 19, nullptr);
}

// SIGNAL 20
void QBarSeries::clicked(qsizetype _t1, QBarSet * _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 20, nullptr, _t1, _t2);
}

// SIGNAL 21
void QBarSeries::doubleClicked(qsizetype _t1, QBarSet * _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 21, nullptr, _t1, _t2);
}

// SIGNAL 22
void QBarSeries::pressed(qsizetype _t1, QBarSet * _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 22, nullptr, _t1, _t2);
}

// SIGNAL 23
void QBarSeries::released(qsizetype _t1, QBarSet * _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 23, nullptr, _t1, _t2);
}
QT_WARNING_POP
