/****************************************************************************
** Meta object code from reading C++ file 'qsurface3dseries.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../qtgraphs/src/graphs3d/data/qsurface3dseries.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qsurface3dseries.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN16QSurface3DSeriesE_t {};
} // unnamed namespace

template <> constexpr inline auto QSurface3DSeries::qt_create_metaobjectdata<qt_meta_tag_ZN16QSurface3DSeriesE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QSurface3DSeries",
        "QML.Element",
        "auto",
        "QML.Creatable",
        "false",
        "QML.UncreatableReason",
        "Trying to create uncreatable: QSurface3DSeries, use Surface3DSerie"
        "s instead.",
        "dataProxyChanged",
        "",
        "QSurfaceDataProxy*",
        "proxy",
        "selectedPointChanged",
        "QPoint",
        "position",
        "flatShadingSupportedChanged",
        "enabled",
        "drawModeChanged",
        "QSurface3DSeries::DrawFlags",
        "mode",
        "textureChanged",
        "QImage",
        "image",
        "textureFileChanged",
        "filename",
        "wireframeColorChanged",
        "QColor",
        "color",
        "dataArrayChanged",
        "QSurfaceDataArray",
        "array",
        "shadingChanged",
        "Shading",
        "shading",
        "rowsSanitizedChanged",
        "newRowsSanitized",
        "axisXChanged",
        "QValue3DAxis*",
        "axis",
        "axisYChanged",
        "axisZChanged",
        "dataProxy",
        "selectedPoint",
        "flatShadingSupported",
        "drawMode",
        "QSurface3DSeries::Shading",
        "texture",
        "textureFile",
        "wireframeColor",
        "dataArray",
        "rowsSanitized",
        "axisX",
        "axisY",
        "axisZ",
        "DrawFlag",
        "DrawWireframe",
        "DrawSurface",
        "DrawSurfaceAndWireframe",
        "DrawFilledSurface",
        "DrawFlags",
        "Smooth",
        "Flat"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'dataProxyChanged'
        QtMocHelpers::SignalData<void(QSurfaceDataProxy *)>(7, 8, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 10 },
        }}),
        // Signal 'selectedPointChanged'
        QtMocHelpers::SignalData<void(QPoint)>(11, 8, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 12, 13 },
        }}),
        // Signal 'flatShadingSupportedChanged'
        QtMocHelpers::SignalData<void(bool)>(14, 8, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 15 },
        }}),
        // Signal 'drawModeChanged'
        QtMocHelpers::SignalData<void(QSurface3DSeries::DrawFlags)>(16, 8, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 17, 18 },
        }}),
        // Signal 'textureChanged'
        QtMocHelpers::SignalData<void(const QImage &)>(19, 8, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 20, 21 },
        }}),
        // Signal 'textureFileChanged'
        QtMocHelpers::SignalData<void(const QString &)>(22, 8, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 23 },
        }}),
        // Signal 'wireframeColorChanged'
        QtMocHelpers::SignalData<void(QColor)>(24, 8, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 25, 26 },
        }}),
        // Signal 'dataArrayChanged'
        QtMocHelpers::SignalData<void(const QSurfaceDataArray &)>(27, 8, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 28, 29 },
        }}),
        // Signal 'shadingChanged'
        QtMocHelpers::SignalData<void(const Shading)>(30, 8, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 31, 32 },
        }}),
        // Signal 'rowsSanitizedChanged'
        QtMocHelpers::RevisionedSignalData<void(bool)>(33, 8, QMC::AccessPublic, 0x60b, QMetaType::Void, {{
            { QMetaType::Bool, 34 },
        }}),
        // Signal 'axisXChanged'
        QtMocHelpers::RevisionedSignalData<void(QValue3DAxis *)>(35, 8, QMC::AccessPublic, 0x60b, QMetaType::Void, {{
            { 0x80000000 | 36, 37 },
        }}),
        // Signal 'axisYChanged'
        QtMocHelpers::RevisionedSignalData<void(QValue3DAxis *)>(38, 8, QMC::AccessPublic, 0x60b, QMetaType::Void, {{
            { 0x80000000 | 36, 37 },
        }}),
        // Signal 'axisZChanged'
        QtMocHelpers::RevisionedSignalData<void(QValue3DAxis *)>(39, 8, QMC::AccessPublic, 0x60b, QMetaType::Void, {{
            { 0x80000000 | 36, 37 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'dataProxy'
        QtMocHelpers::PropertyData<QSurfaceDataProxy*>(40, 0x80000000 | 9, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet | QMC::Final, 0),
        // property 'selectedPoint'
        QtMocHelpers::PropertyData<QPoint>(41, 0x80000000 | 12, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 1),
        // property 'flatShadingSupported'
        QtMocHelpers::PropertyData<bool>(42, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Final, 2),
        // property 'drawMode'
        QtMocHelpers::PropertyData<QSurface3DSeries::DrawFlags>(43, 0x80000000 | 17, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet | QMC::Final, 3),
        // property 'shading'
        QtMocHelpers::PropertyData<QSurface3DSeries::Shading>(32, 0x80000000 | 44, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 8),
        // property 'texture'
        QtMocHelpers::PropertyData<QImage>(45, 0x80000000 | 20, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet | QMC::Final, 4),
        // property 'textureFile'
        QtMocHelpers::PropertyData<QString>(46, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 5),
        // property 'wireframeColor'
        QtMocHelpers::PropertyData<QColor>(47, 0x80000000 | 25, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet | QMC::Final, 6),
        // property 'dataArray'
        QtMocHelpers::PropertyData<QSurfaceDataArray>(48, 0x80000000 | 28, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet | QMC::Final, 7),
        // property 'rowsSanitized'
        QtMocHelpers::PropertyData<bool>(49, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 9, 0x60b),
        // property 'axisX'
        QtMocHelpers::PropertyData<QValue3DAxis*>(50, 0x80000000 | 36, QMC::DefaultPropertyFlags | QMC::Writable | QMC::Resettable | QMC::EnumOrFlag | QMC::StdCppSet, 10, 0x60b),
        // property 'axisY'
        QtMocHelpers::PropertyData<QValue3DAxis*>(51, 0x80000000 | 36, QMC::DefaultPropertyFlags | QMC::Writable | QMC::Resettable | QMC::EnumOrFlag | QMC::StdCppSet, 11, 0x60b),
        // property 'axisZ'
        QtMocHelpers::PropertyData<QValue3DAxis*>(52, 0x80000000 | 36, QMC::DefaultPropertyFlags | QMC::Writable | QMC::Resettable | QMC::EnumOrFlag | QMC::StdCppSet, 12, 0x60b),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'DrawFlag'
        QtMocHelpers::EnumData<enum DrawFlag>(53, 53, QMC::EnumFlags{}).add({
            {   54, DrawFlag::DrawWireframe },
            {   55, DrawFlag::DrawSurface },
            {   56, DrawFlag::DrawSurfaceAndWireframe },
            {   57, DrawFlag::DrawFilledSurface },
        }),
        // flag 'DrawFlags'
        QtMocHelpers::EnumData<DrawFlags>(58, 53, QMC::EnumIsFlag).add({
            {   54, DrawFlag::DrawWireframe },
            {   55, DrawFlag::DrawSurface },
            {   56, DrawFlag::DrawSurfaceAndWireframe },
            {   57, DrawFlag::DrawFilledSurface },
        }),
        // enum 'Shading'
        QtMocHelpers::EnumData<enum Shading>(31, 31, QMC::EnumIsScoped).add({
            {   59, Shading::Smooth },
            {   60, Shading::Flat },
        }),
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
            {    5,    6 },
    });
    return QtMocHelpers::metaObjectData<QSurface3DSeries, void>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject QSurface3DSeries::staticMetaObject = { {
    QMetaObject::SuperData::link<QAbstract3DSeries::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16QSurface3DSeriesE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16QSurface3DSeriesE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN16QSurface3DSeriesE_t>.metaTypes,
    nullptr
} };

void QSurface3DSeries::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QSurface3DSeries *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->dataProxyChanged((*reinterpret_cast<std::add_pointer_t<QSurfaceDataProxy*>>(_a[1]))); break;
        case 1: _t->selectedPointChanged((*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[1]))); break;
        case 2: _t->flatShadingSupportedChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 3: _t->drawModeChanged((*reinterpret_cast<std::add_pointer_t<QSurface3DSeries::DrawFlags>>(_a[1]))); break;
        case 4: _t->textureChanged((*reinterpret_cast<std::add_pointer_t<QImage>>(_a[1]))); break;
        case 5: _t->textureFileChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 6: _t->wireframeColorChanged((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        case 7: _t->dataArrayChanged((*reinterpret_cast<std::add_pointer_t<QSurfaceDataArray>>(_a[1]))); break;
        case 8: _t->shadingChanged((*reinterpret_cast<std::add_pointer_t<enum Shading>>(_a[1]))); break;
        case 9: _t->rowsSanitizedChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 10: _t->axisXChanged((*reinterpret_cast<std::add_pointer_t<QValue3DAxis*>>(_a[1]))); break;
        case 11: _t->axisYChanged((*reinterpret_cast<std::add_pointer_t<QValue3DAxis*>>(_a[1]))); break;
        case 12: _t->axisZChanged((*reinterpret_cast<std::add_pointer_t<QValue3DAxis*>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QSurfaceDataProxy* >(); break;
            }
            break;
        case 10:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QValue3DAxis* >(); break;
            }
            break;
        case 11:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QValue3DAxis* >(); break;
            }
            break;
        case 12:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QValue3DAxis* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (QSurface3DSeries::*)(QSurfaceDataProxy * )>(_a, &QSurface3DSeries::dataProxyChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (QSurface3DSeries::*)(QPoint )>(_a, &QSurface3DSeries::selectedPointChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (QSurface3DSeries::*)(bool )>(_a, &QSurface3DSeries::flatShadingSupportedChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (QSurface3DSeries::*)(QSurface3DSeries::DrawFlags )>(_a, &QSurface3DSeries::drawModeChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (QSurface3DSeries::*)(const QImage & )>(_a, &QSurface3DSeries::textureChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (QSurface3DSeries::*)(const QString & )>(_a, &QSurface3DSeries::textureFileChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (QSurface3DSeries::*)(QColor )>(_a, &QSurface3DSeries::wireframeColorChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (QSurface3DSeries::*)(const QSurfaceDataArray & )>(_a, &QSurface3DSeries::dataArrayChanged, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (QSurface3DSeries::*)(const Shading )>(_a, &QSurface3DSeries::shadingChanged, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (QSurface3DSeries::*)(bool )>(_a, &QSurface3DSeries::rowsSanitizedChanged, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (QSurface3DSeries::*)(QValue3DAxis * )>(_a, &QSurface3DSeries::axisXChanged, 10))
            return;
        if (QtMocHelpers::indexOfMethod<void (QSurface3DSeries::*)(QValue3DAxis * )>(_a, &QSurface3DSeries::axisYChanged, 11))
            return;
        if (QtMocHelpers::indexOfMethod<void (QSurface3DSeries::*)(QValue3DAxis * )>(_a, &QSurface3DSeries::axisZChanged, 12))
            return;
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 0:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QSurfaceDataProxy* >(); break;
        case 12:
        case 11:
        case 10:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QValue3DAxis* >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QSurfaceDataProxy**>(_v) = _t->dataProxy(); break;
        case 1: *reinterpret_cast<QPoint*>(_v) = _t->selectedPoint(); break;
        case 2: *reinterpret_cast<bool*>(_v) = _t->isFlatShadingSupported(); break;
        case 3: *reinterpret_cast<QSurface3DSeries::DrawFlags*>(_v) = _t->drawMode(); break;
        case 4: *reinterpret_cast<QSurface3DSeries::Shading*>(_v) = _t->shading(); break;
        case 5: *reinterpret_cast<QImage*>(_v) = _t->texture(); break;
        case 6: *reinterpret_cast<QString*>(_v) = _t->textureFile(); break;
        case 7: *reinterpret_cast<QColor*>(_v) = _t->wireframeColor(); break;
        case 8: *reinterpret_cast<QSurfaceDataArray*>(_v) = _t->dataArray(); break;
        case 9: *reinterpret_cast<bool*>(_v) = _t->rowsSanitized(); break;
        case 10: *reinterpret_cast<QValue3DAxis**>(_v) = _t->axisX(); break;
        case 11: *reinterpret_cast<QValue3DAxis**>(_v) = _t->axisY(); break;
        case 12: *reinterpret_cast<QValue3DAxis**>(_v) = _t->axisZ(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setDataProxy(*reinterpret_cast<QSurfaceDataProxy**>(_v)); break;
        case 1: _t->setSelectedPoint(*reinterpret_cast<QPoint*>(_v)); break;
        case 3: _t->setDrawMode(*reinterpret_cast<QSurface3DSeries::DrawFlags*>(_v)); break;
        case 4: _t->setShading(*reinterpret_cast<QSurface3DSeries::Shading*>(_v)); break;
        case 5: _t->setTexture(*reinterpret_cast<QImage*>(_v)); break;
        case 6: _t->setTextureFile(*reinterpret_cast<QString*>(_v)); break;
        case 7: _t->setWireframeColor(*reinterpret_cast<QColor*>(_v)); break;
        case 8: _t->setDataArray(*reinterpret_cast<QSurfaceDataArray*>(_v)); break;
        case 9: _t->setRowsSanitized(*reinterpret_cast<bool*>(_v)); break;
        case 10: _t->setAxisX(*reinterpret_cast<QValue3DAxis**>(_v)); break;
        case 11: _t->setAxisY(*reinterpret_cast<QValue3DAxis**>(_v)); break;
        case 12: _t->setAxisZ(*reinterpret_cast<QValue3DAxis**>(_v)); break;
        default: break;
        }
    }
    if (_c == QMetaObject::ResetProperty) {
        switch (_id) {
        case 10: _t->resetAxisX(); break;
        case 11: _t->resetAxisY(); break;
        case 12: _t->resetAxisZ(); break;
        default: break;
        }
    }
}

const QMetaObject *QSurface3DSeries::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QSurface3DSeries::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16QSurface3DSeriesE_t>.strings))
        return static_cast<void*>(this);
    return QAbstract3DSeries::qt_metacast(_clname);
}

int QSurface3DSeries::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QAbstract3DSeries::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 13)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 13;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 13)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 13;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 13;
    }
    return _id;
}

// SIGNAL 0
void QSurface3DSeries::dataProxyChanged(QSurfaceDataProxy * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void QSurface3DSeries::selectedPointChanged(QPoint _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void QSurface3DSeries::flatShadingSupportedChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void QSurface3DSeries::drawModeChanged(QSurface3DSeries::DrawFlags _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void QSurface3DSeries::textureChanged(const QImage & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void QSurface3DSeries::textureFileChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void QSurface3DSeries::wireframeColorChanged(QColor _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}

// SIGNAL 7
void QSurface3DSeries::dataArrayChanged(const QSurfaceDataArray & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1);
}

// SIGNAL 8
void QSurface3DSeries::shadingChanged(const Shading _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 8, nullptr, _t1);
}

// SIGNAL 9
void QSurface3DSeries::rowsSanitizedChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 9, nullptr, _t1);
}

// SIGNAL 10
void QSurface3DSeries::axisXChanged(QValue3DAxis * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 10, nullptr, _t1);
}

// SIGNAL 11
void QSurface3DSeries::axisYChanged(QValue3DAxis * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 11, nullptr, _t1);
}

// SIGNAL 12
void QSurface3DSeries::axisZChanged(QValue3DAxis * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 12, nullptr, _t1);
}
QT_WARNING_POP
