/****************************************************************************
** Meta object code from reading C++ file 'qabstractseries.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../qtgraphs/src/graphs2d/qabstractseries.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qabstractseries.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN11QLegendDataE_t {};
} // unnamed namespace

template <> constexpr inline auto QLegendData::qt_create_metaobjectdata<qt_meta_tag_ZN11QLegendDataE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QLegendData",
        "QML.Element",
        "legendData",
        "QML.AddedInVersion",
        "1546",
        "color",
        "QColor",
        "borderColor",
        "label"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
        // property 'color'
        QtMocHelpers::PropertyData<QColor>(5, 0x80000000 | 6, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::Final),
        // property 'borderColor'
        QtMocHelpers::PropertyData<QColor>(7, 0x80000000 | 6, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::Final),
        // property 'label'
        QtMocHelpers::PropertyData<QString>(8, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::Final),
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
    });
    return QtMocHelpers::metaObjectData<QLegendData, void>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject QLegendData::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11QLegendDataE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11QLegendDataE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN11QLegendDataE_t>.metaTypes,
    nullptr
} };

void QLegendData::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = reinterpret_cast<QLegendData *>(_o);
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QColor*>(_v) = _t->color; break;
        case 1: *reinterpret_cast<QColor*>(_v) = _t->borderColor; break;
        case 2: *reinterpret_cast<QString*>(_v) = _t->label; break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: QtMocHelpers::setProperty(_t->color, *reinterpret_cast<QColor*>(_v)); break;
        case 1: QtMocHelpers::setProperty(_t->borderColor, *reinterpret_cast<QColor*>(_v)); break;
        case 2: QtMocHelpers::setProperty(_t->label, *reinterpret_cast<QString*>(_v)); break;
        default: break;
        }
    }
}
namespace {
struct qt_meta_tag_ZN15QAbstractSeriesE_t {};
} // unnamed namespace

template <> constexpr inline auto QAbstractSeries::qt_create_metaobjectdata<qt_meta_tag_ZN15QAbstractSeriesE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QAbstractSeries",
        "RegisterEnumClassesUnscoped",
        "false",
        "DefaultProperty",
        "seriesChildren",
        "QML.Element",
        "anonymous",
        "update",
        "",
        "nameChanged",
        "visibleChanged",
        "selectableChanged",
        "hoverableChanged",
        "opacityChanged",
        "valuesMultiplierChanged",
        "legendDataChanged",
        "hoverEnter",
        "seriesName",
        "QPointF",
        "position",
        "value",
        "hoverExit",
        "hover",
        "hoveredChanged",
        "hovered",
        "axisXChanged",
        "QAbstractAxis*",
        "newAxis",
        "axisYChanged",
        "zValueChanged",
        "z",
        "name",
        "visible",
        "selectable",
        "hoverable",
        "opacity",
        "valuesMultiplier",
        "type",
        "SeriesType",
        "QQmlListProperty<QObject>",
        "legendData",
        "QList<QLegendData>",
        "axisX",
        "axisY",
        "zValue",
        "Line",
        "Area",
        "Bar",
        "Pie",
        "Scatter",
        "Spline",
        "Custom"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'update'
        QtMocHelpers::SignalData<void()>(7, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'nameChanged'
        QtMocHelpers::SignalData<void()>(9, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'visibleChanged'
        QtMocHelpers::SignalData<void()>(10, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'selectableChanged'
        QtMocHelpers::SignalData<void()>(11, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'hoverableChanged'
        QtMocHelpers::SignalData<void()>(12, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'opacityChanged'
        QtMocHelpers::SignalData<void()>(13, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'valuesMultiplierChanged'
        QtMocHelpers::SignalData<void()>(14, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'legendDataChanged'
        QtMocHelpers::SignalData<void()>(15, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'hoverEnter'
        QtMocHelpers::SignalData<void(const QString &, QPointF, QPointF)>(16, 8, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 17 }, { 0x80000000 | 18, 19 }, { 0x80000000 | 18, 20 },
        }}),
        // Signal 'hoverExit'
        QtMocHelpers::SignalData<void(const QString &, QPointF)>(21, 8, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 17 }, { 0x80000000 | 18, 19 },
        }}),
        // Signal 'hover'
        QtMocHelpers::SignalData<void(const QString &, QPointF, QPointF)>(22, 8, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 17 }, { 0x80000000 | 18, 19 }, { 0x80000000 | 18, 20 },
        }}),
        // Signal 'hoveredChanged'
        QtMocHelpers::RevisionedSignalData<void(bool)>(23, 8, QMC::AccessPublic, 0x60a, QMetaType::Void, {{
            { QMetaType::Bool, 24 },
        }}),
        // Signal 'axisXChanged'
        QtMocHelpers::RevisionedSignalData<void(QAbstractAxis *)>(25, 8, QMC::AccessPublic, 0x60a, QMetaType::Void, {{
            { 0x80000000 | 26, 27 },
        }}),
        // Signal 'axisYChanged'
        QtMocHelpers::RevisionedSignalData<void(QAbstractAxis *)>(28, 8, QMC::AccessPublic, 0x60a, QMetaType::Void, {{
            { 0x80000000 | 26, 27 },
        }}),
        // Signal 'zValueChanged'
        QtMocHelpers::RevisionedSignalData<void(int)>(29, 8, QMC::AccessPublic, 0x60a, QMetaType::Void, {{
            { QMetaType::Int, 30 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'name'
        QtMocHelpers::PropertyData<QString>(31, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 1),
        // property 'visible'
        QtMocHelpers::PropertyData<bool>(32, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 2),
        // property 'selectable'
        QtMocHelpers::PropertyData<bool>(33, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 3),
        // property 'hoverable'
        QtMocHelpers::PropertyData<bool>(34, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 4),
        // property 'opacity'
        QtMocHelpers::PropertyData<qreal>(35, QMetaType::QReal, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 5),
        // property 'valuesMultiplier'
        QtMocHelpers::PropertyData<qreal>(36, QMetaType::QReal, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 6),
        // property 'type'
        QtMocHelpers::PropertyData<enum SeriesType>(37, 0x80000000 | 38, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant | QMC::Final),
        // property 'seriesChildren'
        QtMocHelpers::PropertyData<QQmlListProperty<QObject>>(4, 0x80000000 | 39, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant | QMC::Final),
        // property 'legendData'
        QtMocHelpers::PropertyData<QList<QLegendData>>(40, 0x80000000 | 41, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Final, 7),
        // property 'hovered'
        QtMocHelpers::PropertyData<bool>(24, QMetaType::Bool, QMC::DefaultPropertyFlags, 11, 0x60a),
        // property 'axisX'
        QtMocHelpers::PropertyData<QAbstractAxis*>(42, 0x80000000 | 26, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 12, 0x60a),
        // property 'axisY'
        QtMocHelpers::PropertyData<QAbstractAxis*>(43, 0x80000000 | 26, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 13, 0x60a),
        // property 'zValue'
        QtMocHelpers::PropertyData<int>(44, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 14, 0x60a),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'SeriesType'
        QtMocHelpers::EnumData<enum SeriesType>(38, 38, QMC::EnumIsScoped).add({
            {   45, SeriesType::Line },
            {   46, SeriesType::Area },
            {   47, SeriesType::Bar },
            {   48, SeriesType::Pie },
            {   49, SeriesType::Scatter },
            {   50, SeriesType::Spline },
            {   51, SeriesType::Custom },
        }),
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
            {    5,    6 },
    });
    return QtMocHelpers::metaObjectData<QAbstractSeries, void>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject QAbstractSeries::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15QAbstractSeriesE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15QAbstractSeriesE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN15QAbstractSeriesE_t>.metaTypes,
    nullptr
} };

void QAbstractSeries::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QAbstractSeries *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->update(); break;
        case 1: _t->nameChanged(); break;
        case 2: _t->visibleChanged(); break;
        case 3: _t->selectableChanged(); break;
        case 4: _t->hoverableChanged(); break;
        case 5: _t->opacityChanged(); break;
        case 6: _t->valuesMultiplierChanged(); break;
        case 7: _t->legendDataChanged(); break;
        case 8: _t->hoverEnter((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QPointF>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QPointF>>(_a[3]))); break;
        case 9: _t->hoverExit((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QPointF>>(_a[2]))); break;
        case 10: _t->hover((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QPointF>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QPointF>>(_a[3]))); break;
        case 11: _t->hoveredChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 12: _t->axisXChanged((*reinterpret_cast<std::add_pointer_t<QAbstractAxis*>>(_a[1]))); break;
        case 13: _t->axisYChanged((*reinterpret_cast<std::add_pointer_t<QAbstractAxis*>>(_a[1]))); break;
        case 14: _t->zValueChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 12:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QAbstractAxis* >(); break;
            }
            break;
        case 13:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QAbstractAxis* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (QAbstractSeries::*)()>(_a, &QAbstractSeries::update, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAbstractSeries::*)()>(_a, &QAbstractSeries::nameChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAbstractSeries::*)()>(_a, &QAbstractSeries::visibleChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAbstractSeries::*)()>(_a, &QAbstractSeries::selectableChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAbstractSeries::*)()>(_a, &QAbstractSeries::hoverableChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAbstractSeries::*)()>(_a, &QAbstractSeries::opacityChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAbstractSeries::*)()>(_a, &QAbstractSeries::valuesMultiplierChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAbstractSeries::*)()>(_a, &QAbstractSeries::legendDataChanged, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAbstractSeries::*)(const QString & , QPointF , QPointF )>(_a, &QAbstractSeries::hoverEnter, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAbstractSeries::*)(const QString & , QPointF )>(_a, &QAbstractSeries::hoverExit, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAbstractSeries::*)(const QString & , QPointF , QPointF )>(_a, &QAbstractSeries::hover, 10))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAbstractSeries::*)(bool )>(_a, &QAbstractSeries::hoveredChanged, 11))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAbstractSeries::*)(QAbstractAxis * )>(_a, &QAbstractSeries::axisXChanged, 12))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAbstractSeries::*)(QAbstractAxis * )>(_a, &QAbstractSeries::axisYChanged, 13))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAbstractSeries::*)(int )>(_a, &QAbstractSeries::zValueChanged, 14))
            return;
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 11:
        case 10:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QAbstractAxis* >(); break;
        case 8:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<QLegendData> >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QString*>(_v) = _t->name(); break;
        case 1: *reinterpret_cast<bool*>(_v) = _t->isVisible(); break;
        case 2: *reinterpret_cast<bool*>(_v) = _t->isSelectable(); break;
        case 3: *reinterpret_cast<bool*>(_v) = _t->isHoverable(); break;
        case 4: *reinterpret_cast<qreal*>(_v) = _t->opacity(); break;
        case 5: *reinterpret_cast<qreal*>(_v) = _t->valuesMultiplier(); break;
        case 6: *reinterpret_cast<enum SeriesType*>(_v) = _t->type(); break;
        case 7: *reinterpret_cast<QQmlListProperty<QObject>*>(_v) = _t->seriesChildren(); break;
        case 8: *reinterpret_cast<QList<QLegendData>*>(_v) = _t->legendData(); break;
        case 9: *reinterpret_cast<bool*>(_v) = _t->isHovered(); break;
        case 10: *reinterpret_cast<QAbstractAxis**>(_v) = _t->axisX(); break;
        case 11: *reinterpret_cast<QAbstractAxis**>(_v) = _t->axisY(); break;
        case 12: *reinterpret_cast<int*>(_v) = _t->zValue(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setName(*reinterpret_cast<QString*>(_v)); break;
        case 1: _t->setVisible(*reinterpret_cast<bool*>(_v)); break;
        case 2: _t->setSelectable(*reinterpret_cast<bool*>(_v)); break;
        case 3: _t->setHoverable(*reinterpret_cast<bool*>(_v)); break;
        case 4: _t->setOpacity(*reinterpret_cast<qreal*>(_v)); break;
        case 5: _t->setValuesMultiplier(*reinterpret_cast<qreal*>(_v)); break;
        case 10: _t->setAxisX(*reinterpret_cast<QAbstractAxis**>(_v)); break;
        case 11: _t->setAxisY(*reinterpret_cast<QAbstractAxis**>(_v)); break;
        case 12: _t->setZValue(*reinterpret_cast<int*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *QAbstractSeries::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QAbstractSeries::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15QAbstractSeriesE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "QQmlParserStatus"))
        return static_cast< QQmlParserStatus*>(this);
    if (!strcmp(_clname, "org.qt-project.Qt.QQmlParserStatus"))
        return static_cast< QQmlParserStatus*>(this);
    return QObject::qt_metacast(_clname);
}

int QAbstractSeries::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 15)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 15;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 15)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 15;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 13;
    }
    return _id;
}

// SIGNAL 0
void QAbstractSeries::update()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void QAbstractSeries::nameChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void QAbstractSeries::visibleChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void QAbstractSeries::selectableChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void QAbstractSeries::hoverableChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void QAbstractSeries::opacityChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void QAbstractSeries::valuesMultiplierChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void QAbstractSeries::legendDataChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void QAbstractSeries::hoverEnter(const QString & _t1, QPointF _t2, QPointF _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 8, nullptr, _t1, _t2, _t3);
}

// SIGNAL 9
void QAbstractSeries::hoverExit(const QString & _t1, QPointF _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 9, nullptr, _t1, _t2);
}

// SIGNAL 10
void QAbstractSeries::hover(const QString & _t1, QPointF _t2, QPointF _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 10, nullptr, _t1, _t2, _t3);
}

// SIGNAL 11
void QAbstractSeries::hoveredChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 11, nullptr, _t1);
}

// SIGNAL 12
void QAbstractSeries::axisXChanged(QAbstractAxis * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 12, nullptr, _t1);
}

// SIGNAL 13
void QAbstractSeries::axisYChanged(QAbstractAxis * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 13, nullptr, _t1);
}

// SIGNAL 14
void QAbstractSeries::zValueChanged(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 14, nullptr, _t1);
}
QT_WARNING_POP
