/****************************************************************************
** Meta object code from reading C++ file 'qgraphsview_p.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../qtgraphs/src/graphs2d/qgraphsview_p.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qgraphsview_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN11QGraphsViewE_t {};
} // unnamed namespace

template <> constexpr inline auto QGraphsView::qt_create_metaobjectdata<qt_meta_tag_ZN11QGraphsViewE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QGraphsView",
        "DefaultProperty",
        "seriesList",
        "QML.Element",
        "GraphsView",
        "themeChanged",
        "",
        "marginTopChanged",
        "marginBottomChanged",
        "marginLeftChanged",
        "marginRightChanged",
        "clipPlotAreaChanged",
        "plotAreaChanged",
        "hoverEnter",
        "seriesName",
        "QPointF",
        "position",
        "value",
        "hoverExit",
        "hover",
        "axisXSmoothingChanged",
        "axisYSmoothingChanged",
        "gridSmoothingChanged",
        "shadowVisibleChanged",
        "shadowColorChanged",
        "shadowBarWidthChanged",
        "shadowXOffsetChanged",
        "shadowYOffsetChanged",
        "shadowSmoothingChanged",
        "axisXChanged",
        "axisYChanged",
        "orientationChanged",
        "zoomStyleChanged",
        "panStyleChanged",
        "zoomAreaEnabledChanged",
        "zoomAreaDelegateChanged",
        "zoomSensitivityChanged",
        "useCanvasPainterChanged",
        "dynamicLabelMarginsChanged",
        "addSeries",
        "series",
        "insertSeries",
        "qsizetype",
        "index",
        "removeSeries",
        "hasSeries",
        "theme",
        "QGraphsTheme*",
        "QQmlListProperty<QObject>",
        "marginTop",
        "marginBottom",
        "marginLeft",
        "marginRight",
        "clipPlotArea",
        "plotArea",
        "QRectF",
        "axisXSmoothing",
        "axisYSmoothing",
        "gridSmoothing",
        "shadowVisible",
        "shadowColor",
        "QColor",
        "shadowBarWidth",
        "shadowXOffset",
        "shadowYOffset",
        "shadowSmoothing",
        "axisX",
        "QAbstractAxis*",
        "axisY",
        "orientation",
        "Qt::Orientation",
        "zoomStyle",
        "ZoomStyle",
        "panStyle",
        "PanStyle",
        "zoomSensitivity",
        "zoomAreaEnabled",
        "zoomAreaDelegate",
        "QQmlComponent*",
        "useCanvasPainter",
        "dynamicLabelMargins",
        "None",
        "Center",
        "Drag"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'themeChanged'
        QtMocHelpers::SignalData<void()>(5, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'marginTopChanged'
        QtMocHelpers::SignalData<void()>(7, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'marginBottomChanged'
        QtMocHelpers::SignalData<void()>(8, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'marginLeftChanged'
        QtMocHelpers::SignalData<void()>(9, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'marginRightChanged'
        QtMocHelpers::SignalData<void()>(10, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'clipPlotAreaChanged'
        QtMocHelpers::RevisionedSignalData<void()>(11, 6, QMC::AccessPublic, 0x60a, QMetaType::Void),
        // Signal 'plotAreaChanged'
        QtMocHelpers::RevisionedSignalData<void()>(12, 6, QMC::AccessPublic, 0x609, QMetaType::Void),
        // Signal 'hoverEnter'
        QtMocHelpers::SignalData<void(const QString &, QPointF, QPointF)>(13, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 14 }, { 0x80000000 | 15, 16 }, { 0x80000000 | 15, 17 },
        }}),
        // Signal 'hoverExit'
        QtMocHelpers::SignalData<void(const QString &, QPointF)>(18, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 14 }, { 0x80000000 | 15, 16 },
        }}),
        // Signal 'hover'
        QtMocHelpers::SignalData<void(const QString &, QPointF, QPointF)>(19, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 14 }, { 0x80000000 | 15, 16 }, { 0x80000000 | 15, 17 },
        }}),
        // Signal 'axisXSmoothingChanged'
        QtMocHelpers::SignalData<void()>(20, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'axisYSmoothingChanged'
        QtMocHelpers::SignalData<void()>(21, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'gridSmoothingChanged'
        QtMocHelpers::SignalData<void()>(22, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'shadowVisibleChanged'
        QtMocHelpers::SignalData<void()>(23, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'shadowColorChanged'
        QtMocHelpers::SignalData<void()>(24, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'shadowBarWidthChanged'
        QtMocHelpers::SignalData<void()>(25, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'shadowXOffsetChanged'
        QtMocHelpers::SignalData<void()>(26, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'shadowYOffsetChanged'
        QtMocHelpers::SignalData<void()>(27, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'shadowSmoothingChanged'
        QtMocHelpers::SignalData<void()>(28, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'axisXChanged'
        QtMocHelpers::SignalData<void()>(29, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'axisYChanged'
        QtMocHelpers::SignalData<void()>(30, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'orientationChanged'
        QtMocHelpers::SignalData<void()>(31, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'zoomStyleChanged'
        QtMocHelpers::RevisionedSignalData<void()>(32, 6, QMC::AccessPublic, 0x609, QMetaType::Void),
        // Signal 'panStyleChanged'
        QtMocHelpers::RevisionedSignalData<void()>(33, 6, QMC::AccessPublic, 0x609, QMetaType::Void),
        // Signal 'zoomAreaEnabledChanged'
        QtMocHelpers::RevisionedSignalData<void()>(34, 6, QMC::AccessPublic, 0x609, QMetaType::Void),
        // Signal 'zoomAreaDelegateChanged'
        QtMocHelpers::RevisionedSignalData<void()>(35, 6, QMC::AccessPublic, 0x609, QMetaType::Void),
        // Signal 'zoomSensitivityChanged'
        QtMocHelpers::RevisionedSignalData<void()>(36, 6, QMC::AccessPublic, 0x609, QMetaType::Void),
        // Signal 'useCanvasPainterChanged'
        QtMocHelpers::RevisionedSignalData<void()>(37, 6, QMC::AccessPublic, 0x60c, QMetaType::Void),
        // Signal 'dynamicLabelMarginsChanged'
        QtMocHelpers::RevisionedSignalData<void()>(38, 6, QMC::AccessPublic, 0x60c, QMetaType::Void),
        // Method 'addSeries'
        QtMocHelpers::MethodData<void(QObject *)>(39, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QObjectStar, 40 },
        }}),
        // Method 'insertSeries'
        QtMocHelpers::MethodData<void(qsizetype, QObject *)>(41, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 42, 43 }, { QMetaType::QObjectStar, 40 },
        }}),
        // Method 'removeSeries'
        QtMocHelpers::MethodData<void(QObject *)>(44, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QObjectStar, 40 },
        }}),
        // Method 'removeSeries'
        QtMocHelpers::MethodData<void(qsizetype)>(44, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 42, 43 },
        }}),
        // Method 'hasSeries'
        QtMocHelpers::MethodData<bool(QObject *)>(45, 6, QMC::AccessPublic, QMetaType::Bool, {{
            { QMetaType::QObjectStar, 40 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'theme'
        QtMocHelpers::PropertyData<QGraphsTheme*>(46, 0x80000000 | 47, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet | QMC::Final, 0),
        // property 'seriesList'
        QtMocHelpers::PropertyData<QQmlListProperty<QObject>>(2, 0x80000000 | 48, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'marginTop'
        QtMocHelpers::PropertyData<qreal>(49, QMetaType::QReal, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 1),
        // property 'marginBottom'
        QtMocHelpers::PropertyData<qreal>(50, QMetaType::QReal, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 2),
        // property 'marginLeft'
        QtMocHelpers::PropertyData<qreal>(51, QMetaType::QReal, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 3),
        // property 'marginRight'
        QtMocHelpers::PropertyData<qreal>(52, QMetaType::QReal, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 4),
        // property 'clipPlotArea'
        QtMocHelpers::PropertyData<bool>(53, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 5, 0x60a),
        // property 'plotArea'
        QtMocHelpers::PropertyData<QRectF>(54, 0x80000000 | 55, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 6, 0x609),
        // property 'axisXSmoothing'
        QtMocHelpers::PropertyData<qreal>(56, QMetaType::QReal, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 10),
        // property 'axisYSmoothing'
        QtMocHelpers::PropertyData<qreal>(57, QMetaType::QReal, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 11),
        // property 'gridSmoothing'
        QtMocHelpers::PropertyData<qreal>(58, QMetaType::QReal, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 12),
        // property 'shadowVisible'
        QtMocHelpers::PropertyData<bool>(59, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 13),
        // property 'shadowColor'
        QtMocHelpers::PropertyData<QColor>(60, 0x80000000 | 61, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet | QMC::Final, 14),
        // property 'shadowBarWidth'
        QtMocHelpers::PropertyData<qreal>(62, QMetaType::QReal, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 15),
        // property 'shadowXOffset'
        QtMocHelpers::PropertyData<qreal>(63, QMetaType::QReal, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 16),
        // property 'shadowYOffset'
        QtMocHelpers::PropertyData<qreal>(64, QMetaType::QReal, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 17),
        // property 'shadowSmoothing'
        QtMocHelpers::PropertyData<qreal>(65, QMetaType::QReal, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 18),
        // property 'axisX'
        QtMocHelpers::PropertyData<QAbstractAxis*>(66, 0x80000000 | 67, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet | QMC::Final, 19),
        // property 'axisY'
        QtMocHelpers::PropertyData<QAbstractAxis*>(68, 0x80000000 | 67, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet | QMC::Final, 20),
        // property 'orientation'
        QtMocHelpers::PropertyData<Qt::Orientation>(69, 0x80000000 | 70, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet | QMC::Final, 21),
        // property 'zoomStyle'
        QtMocHelpers::PropertyData<enum ZoomStyle>(71, 0x80000000 | 72, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 22, 0x609),
        // property 'panStyle'
        QtMocHelpers::PropertyData<enum PanStyle>(73, 0x80000000 | 74, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 23, 0x609),
        // property 'zoomSensitivity'
        QtMocHelpers::PropertyData<qreal>(75, QMetaType::QReal, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 26, 0x609),
        // property 'zoomAreaEnabled'
        QtMocHelpers::PropertyData<bool>(76, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 24, 0x609),
        // property 'zoomAreaDelegate'
        QtMocHelpers::PropertyData<QQmlComponent*>(77, 0x80000000 | 78, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 25, 0x609),
        // property 'useCanvasPainter'
        QtMocHelpers::PropertyData<bool>(79, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 27, 0x60c),
        // property 'dynamicLabelMargins'
        QtMocHelpers::PropertyData<bool>(80, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 28, 0x60c),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'ZoomStyle'
        QtMocHelpers::EnumData<enum ZoomStyle>(72, 72, QMC::EnumIsScoped).add({
            {   81, ZoomStyle::None },
            {   82, ZoomStyle::Center },
        }),
        // enum 'PanStyle'
        QtMocHelpers::EnumData<enum PanStyle>(74, 74, QMC::EnumIsScoped).add({
            {   81, PanStyle::None },
            {   83, PanStyle::Drag },
        }),
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
    });
    return QtMocHelpers::metaObjectData<QGraphsView, void>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject QGraphsView::staticMetaObject = { {
    QMetaObject::SuperData::link<QQuickItem::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11QGraphsViewE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11QGraphsViewE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN11QGraphsViewE_t>.metaTypes,
    nullptr
} };

void QGraphsView::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QGraphsView *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->themeChanged(); break;
        case 1: _t->marginTopChanged(); break;
        case 2: _t->marginBottomChanged(); break;
        case 3: _t->marginLeftChanged(); break;
        case 4: _t->marginRightChanged(); break;
        case 5: _t->clipPlotAreaChanged(); break;
        case 6: _t->plotAreaChanged(); break;
        case 7: _t->hoverEnter((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QPointF>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QPointF>>(_a[3]))); break;
        case 8: _t->hoverExit((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QPointF>>(_a[2]))); break;
        case 9: _t->hover((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QPointF>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QPointF>>(_a[3]))); break;
        case 10: _t->axisXSmoothingChanged(); break;
        case 11: _t->axisYSmoothingChanged(); break;
        case 12: _t->gridSmoothingChanged(); break;
        case 13: _t->shadowVisibleChanged(); break;
        case 14: _t->shadowColorChanged(); break;
        case 15: _t->shadowBarWidthChanged(); break;
        case 16: _t->shadowXOffsetChanged(); break;
        case 17: _t->shadowYOffsetChanged(); break;
        case 18: _t->shadowSmoothingChanged(); break;
        case 19: _t->axisXChanged(); break;
        case 20: _t->axisYChanged(); break;
        case 21: _t->orientationChanged(); break;
        case 22: _t->zoomStyleChanged(); break;
        case 23: _t->panStyleChanged(); break;
        case 24: _t->zoomAreaEnabledChanged(); break;
        case 25: _t->zoomAreaDelegateChanged(); break;
        case 26: _t->zoomSensitivityChanged(); break;
        case 27: _t->useCanvasPainterChanged(); break;
        case 28: _t->dynamicLabelMarginsChanged(); break;
        case 29: _t->addSeries((*reinterpret_cast<std::add_pointer_t<QObject*>>(_a[1]))); break;
        case 30: _t->insertSeries((*reinterpret_cast<std::add_pointer_t<qsizetype>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QObject*>>(_a[2]))); break;
        case 31: _t->removeSeries((*reinterpret_cast<std::add_pointer_t<QObject*>>(_a[1]))); break;
        case 32: _t->removeSeries((*reinterpret_cast<std::add_pointer_t<qsizetype>>(_a[1]))); break;
        case 33: { bool _r = _t->hasSeries((*reinterpret_cast<std::add_pointer_t<QObject*>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (QGraphsView::*)()>(_a, &QGraphsView::themeChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (QGraphsView::*)()>(_a, &QGraphsView::marginTopChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (QGraphsView::*)()>(_a, &QGraphsView::marginBottomChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (QGraphsView::*)()>(_a, &QGraphsView::marginLeftChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (QGraphsView::*)()>(_a, &QGraphsView::marginRightChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (QGraphsView::*)()>(_a, &QGraphsView::clipPlotAreaChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (QGraphsView::*)()>(_a, &QGraphsView::plotAreaChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (QGraphsView::*)(const QString & , QPointF , QPointF )>(_a, &QGraphsView::hoverEnter, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (QGraphsView::*)(const QString & , QPointF )>(_a, &QGraphsView::hoverExit, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (QGraphsView::*)(const QString & , QPointF , QPointF )>(_a, &QGraphsView::hover, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (QGraphsView::*)()>(_a, &QGraphsView::axisXSmoothingChanged, 10))
            return;
        if (QtMocHelpers::indexOfMethod<void (QGraphsView::*)()>(_a, &QGraphsView::axisYSmoothingChanged, 11))
            return;
        if (QtMocHelpers::indexOfMethod<void (QGraphsView::*)()>(_a, &QGraphsView::gridSmoothingChanged, 12))
            return;
        if (QtMocHelpers::indexOfMethod<void (QGraphsView::*)()>(_a, &QGraphsView::shadowVisibleChanged, 13))
            return;
        if (QtMocHelpers::indexOfMethod<void (QGraphsView::*)()>(_a, &QGraphsView::shadowColorChanged, 14))
            return;
        if (QtMocHelpers::indexOfMethod<void (QGraphsView::*)()>(_a, &QGraphsView::shadowBarWidthChanged, 15))
            return;
        if (QtMocHelpers::indexOfMethod<void (QGraphsView::*)()>(_a, &QGraphsView::shadowXOffsetChanged, 16))
            return;
        if (QtMocHelpers::indexOfMethod<void (QGraphsView::*)()>(_a, &QGraphsView::shadowYOffsetChanged, 17))
            return;
        if (QtMocHelpers::indexOfMethod<void (QGraphsView::*)()>(_a, &QGraphsView::shadowSmoothingChanged, 18))
            return;
        if (QtMocHelpers::indexOfMethod<void (QGraphsView::*)()>(_a, &QGraphsView::axisXChanged, 19))
            return;
        if (QtMocHelpers::indexOfMethod<void (QGraphsView::*)()>(_a, &QGraphsView::axisYChanged, 20))
            return;
        if (QtMocHelpers::indexOfMethod<void (QGraphsView::*)()>(_a, &QGraphsView::orientationChanged, 21))
            return;
        if (QtMocHelpers::indexOfMethod<void (QGraphsView::*)()>(_a, &QGraphsView::zoomStyleChanged, 22))
            return;
        if (QtMocHelpers::indexOfMethod<void (QGraphsView::*)()>(_a, &QGraphsView::panStyleChanged, 23))
            return;
        if (QtMocHelpers::indexOfMethod<void (QGraphsView::*)()>(_a, &QGraphsView::zoomAreaEnabledChanged, 24))
            return;
        if (QtMocHelpers::indexOfMethod<void (QGraphsView::*)()>(_a, &QGraphsView::zoomAreaDelegateChanged, 25))
            return;
        if (QtMocHelpers::indexOfMethod<void (QGraphsView::*)()>(_a, &QGraphsView::zoomSensitivityChanged, 26))
            return;
        if (QtMocHelpers::indexOfMethod<void (QGraphsView::*)()>(_a, &QGraphsView::useCanvasPainterChanged, 27))
            return;
        if (QtMocHelpers::indexOfMethod<void (QGraphsView::*)()>(_a, &QGraphsView::dynamicLabelMarginsChanged, 28))
            return;
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 18:
        case 17:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QAbstractAxis* >(); break;
        case 0:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QGraphsTheme* >(); break;
        case 24:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQmlComponent* >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QGraphsTheme**>(_v) = _t->theme(); break;
        case 1: *reinterpret_cast<QQmlListProperty<QObject>*>(_v) = _t->seriesList(); break;
        case 2: *reinterpret_cast<qreal*>(_v) = _t->marginTop(); break;
        case 3: *reinterpret_cast<qreal*>(_v) = _t->marginBottom(); break;
        case 4: *reinterpret_cast<qreal*>(_v) = _t->marginLeft(); break;
        case 5: *reinterpret_cast<qreal*>(_v) = _t->marginRight(); break;
        case 6: *reinterpret_cast<bool*>(_v) = _t->clipPlotArea(); break;
        case 7: *reinterpret_cast<QRectF*>(_v) = _t->plotArea(); break;
        case 8: *reinterpret_cast<qreal*>(_v) = _t->axisXSmoothing(); break;
        case 9: *reinterpret_cast<qreal*>(_v) = _t->axisYSmoothing(); break;
        case 10: *reinterpret_cast<qreal*>(_v) = _t->gridSmoothing(); break;
        case 11: *reinterpret_cast<bool*>(_v) = _t->isShadowVisible(); break;
        case 12: *reinterpret_cast<QColor*>(_v) = _t->shadowColor(); break;
        case 13: *reinterpret_cast<qreal*>(_v) = _t->shadowBarWidth(); break;
        case 14: *reinterpret_cast<qreal*>(_v) = _t->shadowXOffset(); break;
        case 15: *reinterpret_cast<qreal*>(_v) = _t->shadowYOffset(); break;
        case 16: *reinterpret_cast<qreal*>(_v) = _t->shadowSmoothing(); break;
        case 17: *reinterpret_cast<QAbstractAxis**>(_v) = _t->axisX(); break;
        case 18: *reinterpret_cast<QAbstractAxis**>(_v) = _t->axisY(); break;
        case 19: *reinterpret_cast<Qt::Orientation*>(_v) = _t->orientation(); break;
        case 20: *reinterpret_cast<enum ZoomStyle*>(_v) = _t->zoomStyle(); break;
        case 21: *reinterpret_cast<enum PanStyle*>(_v) = _t->panStyle(); break;
        case 22: *reinterpret_cast<qreal*>(_v) = _t->zoomSensitivity(); break;
        case 23: *reinterpret_cast<bool*>(_v) = _t->zoomAreaEnabled(); break;
        case 24: *reinterpret_cast<QQmlComponent**>(_v) = _t->zoomAreaDelegate(); break;
        case 25: *reinterpret_cast<bool*>(_v) = _t->useCanvasPainter(); break;
        case 26: *reinterpret_cast<bool*>(_v) = _t->dynamicLabelMargins(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setTheme(*reinterpret_cast<QGraphsTheme**>(_v)); break;
        case 2: _t->setMarginTop(*reinterpret_cast<qreal*>(_v)); break;
        case 3: _t->setMarginBottom(*reinterpret_cast<qreal*>(_v)); break;
        case 4: _t->setMarginLeft(*reinterpret_cast<qreal*>(_v)); break;
        case 5: _t->setMarginRight(*reinterpret_cast<qreal*>(_v)); break;
        case 6: _t->setClipPlotArea(*reinterpret_cast<bool*>(_v)); break;
        case 8: _t->setAxisXSmoothing(*reinterpret_cast<qreal*>(_v)); break;
        case 9: _t->setAxisYSmoothing(*reinterpret_cast<qreal*>(_v)); break;
        case 10: _t->setGridSmoothing(*reinterpret_cast<qreal*>(_v)); break;
        case 11: _t->setShadowVisible(*reinterpret_cast<bool*>(_v)); break;
        case 12: _t->setShadowColor(*reinterpret_cast<QColor*>(_v)); break;
        case 13: _t->setShadowBarWidth(*reinterpret_cast<qreal*>(_v)); break;
        case 14: _t->setShadowXOffset(*reinterpret_cast<qreal*>(_v)); break;
        case 15: _t->setShadowYOffset(*reinterpret_cast<qreal*>(_v)); break;
        case 16: _t->setShadowSmoothing(*reinterpret_cast<qreal*>(_v)); break;
        case 17: _t->setAxisX(*reinterpret_cast<QAbstractAxis**>(_v)); break;
        case 18: _t->setAxisY(*reinterpret_cast<QAbstractAxis**>(_v)); break;
        case 19: _t->setOrientation(*reinterpret_cast<Qt::Orientation*>(_v)); break;
        case 20: _t->setZoomStyle(*reinterpret_cast<enum ZoomStyle*>(_v)); break;
        case 21: _t->setPanStyle(*reinterpret_cast<enum PanStyle*>(_v)); break;
        case 22: _t->setZoomSensitivity(*reinterpret_cast<qreal*>(_v)); break;
        case 23: _t->setZoomAreaEnabled(*reinterpret_cast<bool*>(_v)); break;
        case 24: _t->setZoomAreaDelegate(*reinterpret_cast<QQmlComponent**>(_v)); break;
        case 25: _t->setUseCanvasPainter(*reinterpret_cast<bool*>(_v)); break;
        case 26: _t->setDynamicLabelMargins(*reinterpret_cast<bool*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *QGraphsView::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QGraphsView::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11QGraphsViewE_t>.strings))
        return static_cast<void*>(this);
    return QQuickItem::qt_metacast(_clname);
}

int QGraphsView::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QQuickItem::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 34)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 34;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 34)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 34;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 27;
    }
    return _id;
}

// SIGNAL 0
void QGraphsView::themeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void QGraphsView::marginTopChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void QGraphsView::marginBottomChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void QGraphsView::marginLeftChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void QGraphsView::marginRightChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void QGraphsView::clipPlotAreaChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void QGraphsView::plotAreaChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void QGraphsView::hoverEnter(const QString & _t1, QPointF _t2, QPointF _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1, _t2, _t3);
}

// SIGNAL 8
void QGraphsView::hoverExit(const QString & _t1, QPointF _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 8, nullptr, _t1, _t2);
}

// SIGNAL 9
void QGraphsView::hover(const QString & _t1, QPointF _t2, QPointF _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 9, nullptr, _t1, _t2, _t3);
}

// SIGNAL 10
void QGraphsView::axisXSmoothingChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 10, nullptr);
}

// SIGNAL 11
void QGraphsView::axisYSmoothingChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 11, nullptr);
}

// SIGNAL 12
void QGraphsView::gridSmoothingChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 12, nullptr);
}

// SIGNAL 13
void QGraphsView::shadowVisibleChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 13, nullptr);
}

// SIGNAL 14
void QGraphsView::shadowColorChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 14, nullptr);
}

// SIGNAL 15
void QGraphsView::shadowBarWidthChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 15, nullptr);
}

// SIGNAL 16
void QGraphsView::shadowXOffsetChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 16, nullptr);
}

// SIGNAL 17
void QGraphsView::shadowYOffsetChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 17, nullptr);
}

// SIGNAL 18
void QGraphsView::shadowSmoothingChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 18, nullptr);
}

// SIGNAL 19
void QGraphsView::axisXChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 19, nullptr);
}

// SIGNAL 20
void QGraphsView::axisYChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 20, nullptr);
}

// SIGNAL 21
void QGraphsView::orientationChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 21, nullptr);
}

// SIGNAL 22
void QGraphsView::zoomStyleChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 22, nullptr);
}

// SIGNAL 23
void QGraphsView::panStyleChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 23, nullptr);
}

// SIGNAL 24
void QGraphsView::zoomAreaEnabledChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 24, nullptr);
}

// SIGNAL 25
void QGraphsView::zoomAreaDelegateChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 25, nullptr);
}

// SIGNAL 26
void QGraphsView::zoomSensitivityChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 26, nullptr);
}

// SIGNAL 27
void QGraphsView::useCanvasPainterChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 27, nullptr);
}

// SIGNAL 28
void QGraphsView::dynamicLabelMarginsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 28, nullptr);
}
QT_WARNING_POP
