/****************************************************************************
** Meta object code from reading C++ file 'qitemmodelscatterdataproxy.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../qtgraphs/src/graphs3d/data/qitemmodelscatterdataproxy.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qitemmodelscatterdataproxy.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN26QItemModelScatterDataProxyE_t {};
} // unnamed namespace

template <> constexpr inline auto QItemModelScatterDataProxy::qt_create_metaobjectdata<qt_meta_tag_ZN26QItemModelScatterDataProxyE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QItemModelScatterDataProxy",
        "QML.Element",
        "ItemModelScatterDataProxy",
        "itemModelChanged",
        "",
        "const QAbstractItemModel*",
        "itemModel",
        "xPosRoleChanged",
        "role",
        "yPosRoleChanged",
        "zPosRoleChanged",
        "rotationRoleChanged",
        "scaleRoleChanged",
        "xPosRolePatternChanged",
        "pattern",
        "yPosRolePatternChanged",
        "zPosRolePatternChanged",
        "rotationRolePatternChanged",
        "scaleRolePatternChanged",
        "rotationRoleReplaceChanged",
        "replace",
        "xPosRoleReplaceChanged",
        "yPosRoleReplaceChanged",
        "zPosRoleReplaceChanged",
        "scaleRoleReplaceChanged",
        "QAbstractItemModel*",
        "xPosRole",
        "yPosRole",
        "zPosRole",
        "rotationRole",
        "scaleRole",
        "xPosRolePattern",
        "yPosRolePattern",
        "zPosRolePattern",
        "rotationRolePattern",
        "scaleRolePattern",
        "xPosRoleReplace",
        "yPosRoleReplace",
        "zPosRoleReplace",
        "rotationRoleReplace",
        "scaleRoleReplace"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'itemModelChanged'
        QtMocHelpers::SignalData<void(const QAbstractItemModel *)>(3, 4, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 6 },
        }}),
        // Signal 'xPosRoleChanged'
        QtMocHelpers::SignalData<void(const QString &)>(7, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 8 },
        }}),
        // Signal 'yPosRoleChanged'
        QtMocHelpers::SignalData<void(const QString &)>(9, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 8 },
        }}),
        // Signal 'zPosRoleChanged'
        QtMocHelpers::SignalData<void(const QString &)>(10, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 8 },
        }}),
        // Signal 'rotationRoleChanged'
        QtMocHelpers::SignalData<void(const QString &)>(11, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 8 },
        }}),
        // Signal 'scaleRoleChanged'
        QtMocHelpers::SignalData<void(const QString &)>(12, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 8 },
        }}),
        // Signal 'xPosRolePatternChanged'
        QtMocHelpers::SignalData<void(const QRegularExpression &)>(13, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QRegularExpression, 14 },
        }}),
        // Signal 'yPosRolePatternChanged'
        QtMocHelpers::SignalData<void(const QRegularExpression &)>(15, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QRegularExpression, 14 },
        }}),
        // Signal 'zPosRolePatternChanged'
        QtMocHelpers::SignalData<void(const QRegularExpression &)>(16, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QRegularExpression, 14 },
        }}),
        // Signal 'rotationRolePatternChanged'
        QtMocHelpers::SignalData<void(const QRegularExpression &)>(17, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QRegularExpression, 14 },
        }}),
        // Signal 'scaleRolePatternChanged'
        QtMocHelpers::SignalData<void(const QRegularExpression &)>(18, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QRegularExpression, 14 },
        }}),
        // Signal 'rotationRoleReplaceChanged'
        QtMocHelpers::SignalData<void(const QString &)>(19, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 20 },
        }}),
        // Signal 'xPosRoleReplaceChanged'
        QtMocHelpers::SignalData<void(const QString &)>(21, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 20 },
        }}),
        // Signal 'yPosRoleReplaceChanged'
        QtMocHelpers::SignalData<void(const QString &)>(22, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 20 },
        }}),
        // Signal 'zPosRoleReplaceChanged'
        QtMocHelpers::SignalData<void(const QString &)>(23, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 20 },
        }}),
        // Signal 'scaleRoleReplaceChanged'
        QtMocHelpers::SignalData<void(const QString &)>(24, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 20 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'itemModel'
        QtMocHelpers::PropertyData<QAbstractItemModel*>(6, 0x80000000 | 25, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet | QMC::Final, 0),
        // property 'xPosRole'
        QtMocHelpers::PropertyData<QString>(26, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 1),
        // property 'yPosRole'
        QtMocHelpers::PropertyData<QString>(27, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 2),
        // property 'zPosRole'
        QtMocHelpers::PropertyData<QString>(28, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 3),
        // property 'rotationRole'
        QtMocHelpers::PropertyData<QString>(29, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 4),
        // property 'scaleRole'
        QtMocHelpers::PropertyData<QString>(30, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 5, 0x60a),
        // property 'xPosRolePattern'
        QtMocHelpers::PropertyData<QRegularExpression>(31, QMetaType::QRegularExpression, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 6),
        // property 'yPosRolePattern'
        QtMocHelpers::PropertyData<QRegularExpression>(32, QMetaType::QRegularExpression, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 7),
        // property 'zPosRolePattern'
        QtMocHelpers::PropertyData<QRegularExpression>(33, QMetaType::QRegularExpression, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 8),
        // property 'rotationRolePattern'
        QtMocHelpers::PropertyData<QRegularExpression>(34, QMetaType::QRegularExpression, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 9),
        // property 'scaleRolePattern'
        QtMocHelpers::PropertyData<QRegularExpression>(35, QMetaType::QRegularExpression, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 10, 0x60a),
        // property 'xPosRoleReplace'
        QtMocHelpers::PropertyData<QString>(36, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 12),
        // property 'yPosRoleReplace'
        QtMocHelpers::PropertyData<QString>(37, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 13),
        // property 'zPosRoleReplace'
        QtMocHelpers::PropertyData<QString>(38, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 14),
        // property 'rotationRoleReplace'
        QtMocHelpers::PropertyData<QString>(39, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 11),
        // property 'scaleRoleReplace'
        QtMocHelpers::PropertyData<QString>(40, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 15, 0x60a),
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
    });
    return QtMocHelpers::metaObjectData<QItemModelScatterDataProxy, void>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject QItemModelScatterDataProxy::staticMetaObject = { {
    QMetaObject::SuperData::link<QScatterDataProxy::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN26QItemModelScatterDataProxyE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN26QItemModelScatterDataProxyE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN26QItemModelScatterDataProxyE_t>.metaTypes,
    nullptr
} };

void QItemModelScatterDataProxy::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QItemModelScatterDataProxy *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->itemModelChanged((*reinterpret_cast<std::add_pointer_t<const QAbstractItemModel*>>(_a[1]))); break;
        case 1: _t->xPosRoleChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 2: _t->yPosRoleChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 3: _t->zPosRoleChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 4: _t->rotationRoleChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 5: _t->scaleRoleChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 6: _t->xPosRolePatternChanged((*reinterpret_cast<std::add_pointer_t<QRegularExpression>>(_a[1]))); break;
        case 7: _t->yPosRolePatternChanged((*reinterpret_cast<std::add_pointer_t<QRegularExpression>>(_a[1]))); break;
        case 8: _t->zPosRolePatternChanged((*reinterpret_cast<std::add_pointer_t<QRegularExpression>>(_a[1]))); break;
        case 9: _t->rotationRolePatternChanged((*reinterpret_cast<std::add_pointer_t<QRegularExpression>>(_a[1]))); break;
        case 10: _t->scaleRolePatternChanged((*reinterpret_cast<std::add_pointer_t<QRegularExpression>>(_a[1]))); break;
        case 11: _t->rotationRoleReplaceChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 12: _t->xPosRoleReplaceChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 13: _t->yPosRoleReplaceChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 14: _t->zPosRoleReplaceChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 15: _t->scaleRoleReplaceChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (QItemModelScatterDataProxy::*)(const QAbstractItemModel * )>(_a, &QItemModelScatterDataProxy::itemModelChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (QItemModelScatterDataProxy::*)(const QString & )>(_a, &QItemModelScatterDataProxy::xPosRoleChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (QItemModelScatterDataProxy::*)(const QString & )>(_a, &QItemModelScatterDataProxy::yPosRoleChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (QItemModelScatterDataProxy::*)(const QString & )>(_a, &QItemModelScatterDataProxy::zPosRoleChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (QItemModelScatterDataProxy::*)(const QString & )>(_a, &QItemModelScatterDataProxy::rotationRoleChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (QItemModelScatterDataProxy::*)(const QString & )>(_a, &QItemModelScatterDataProxy::scaleRoleChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (QItemModelScatterDataProxy::*)(const QRegularExpression & )>(_a, &QItemModelScatterDataProxy::xPosRolePatternChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (QItemModelScatterDataProxy::*)(const QRegularExpression & )>(_a, &QItemModelScatterDataProxy::yPosRolePatternChanged, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (QItemModelScatterDataProxy::*)(const QRegularExpression & )>(_a, &QItemModelScatterDataProxy::zPosRolePatternChanged, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (QItemModelScatterDataProxy::*)(const QRegularExpression & )>(_a, &QItemModelScatterDataProxy::rotationRolePatternChanged, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (QItemModelScatterDataProxy::*)(const QRegularExpression & )>(_a, &QItemModelScatterDataProxy::scaleRolePatternChanged, 10))
            return;
        if (QtMocHelpers::indexOfMethod<void (QItemModelScatterDataProxy::*)(const QString & )>(_a, &QItemModelScatterDataProxy::rotationRoleReplaceChanged, 11))
            return;
        if (QtMocHelpers::indexOfMethod<void (QItemModelScatterDataProxy::*)(const QString & )>(_a, &QItemModelScatterDataProxy::xPosRoleReplaceChanged, 12))
            return;
        if (QtMocHelpers::indexOfMethod<void (QItemModelScatterDataProxy::*)(const QString & )>(_a, &QItemModelScatterDataProxy::yPosRoleReplaceChanged, 13))
            return;
        if (QtMocHelpers::indexOfMethod<void (QItemModelScatterDataProxy::*)(const QString & )>(_a, &QItemModelScatterDataProxy::zPosRoleReplaceChanged, 14))
            return;
        if (QtMocHelpers::indexOfMethod<void (QItemModelScatterDataProxy::*)(const QString & )>(_a, &QItemModelScatterDataProxy::scaleRoleReplaceChanged, 15))
            return;
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 0:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QAbstractItemModel* >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QAbstractItemModel**>(_v) = _t->itemModel(); break;
        case 1: *reinterpret_cast<QString*>(_v) = _t->xPosRole(); break;
        case 2: *reinterpret_cast<QString*>(_v) = _t->yPosRole(); break;
        case 3: *reinterpret_cast<QString*>(_v) = _t->zPosRole(); break;
        case 4: *reinterpret_cast<QString*>(_v) = _t->rotationRole(); break;
        case 5: *reinterpret_cast<QString*>(_v) = _t->scaleRole(); break;
        case 6: *reinterpret_cast<QRegularExpression*>(_v) = _t->xPosRolePattern(); break;
        case 7: *reinterpret_cast<QRegularExpression*>(_v) = _t->yPosRolePattern(); break;
        case 8: *reinterpret_cast<QRegularExpression*>(_v) = _t->zPosRolePattern(); break;
        case 9: *reinterpret_cast<QRegularExpression*>(_v) = _t->rotationRolePattern(); break;
        case 10: *reinterpret_cast<QRegularExpression*>(_v) = _t->scaleRolePattern(); break;
        case 11: *reinterpret_cast<QString*>(_v) = _t->xPosRoleReplace(); break;
        case 12: *reinterpret_cast<QString*>(_v) = _t->yPosRoleReplace(); break;
        case 13: *reinterpret_cast<QString*>(_v) = _t->zPosRoleReplace(); break;
        case 14: *reinterpret_cast<QString*>(_v) = _t->rotationRoleReplace(); break;
        case 15: *reinterpret_cast<QString*>(_v) = _t->scaleRoleReplace(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setItemModel(*reinterpret_cast<QAbstractItemModel**>(_v)); break;
        case 1: _t->setXPosRole(*reinterpret_cast<QString*>(_v)); break;
        case 2: _t->setYPosRole(*reinterpret_cast<QString*>(_v)); break;
        case 3: _t->setZPosRole(*reinterpret_cast<QString*>(_v)); break;
        case 4: _t->setRotationRole(*reinterpret_cast<QString*>(_v)); break;
        case 5: _t->setScaleRole(*reinterpret_cast<QString*>(_v)); break;
        case 6: _t->setXPosRolePattern(*reinterpret_cast<QRegularExpression*>(_v)); break;
        case 7: _t->setYPosRolePattern(*reinterpret_cast<QRegularExpression*>(_v)); break;
        case 8: _t->setZPosRolePattern(*reinterpret_cast<QRegularExpression*>(_v)); break;
        case 9: _t->setRotationRolePattern(*reinterpret_cast<QRegularExpression*>(_v)); break;
        case 10: _t->setScaleRolePattern(*reinterpret_cast<QRegularExpression*>(_v)); break;
        case 11: _t->setXPosRoleReplace(*reinterpret_cast<QString*>(_v)); break;
        case 12: _t->setYPosRoleReplace(*reinterpret_cast<QString*>(_v)); break;
        case 13: _t->setZPosRoleReplace(*reinterpret_cast<QString*>(_v)); break;
        case 14: _t->setRotationRoleReplace(*reinterpret_cast<QString*>(_v)); break;
        case 15: _t->setScaleRoleReplace(*reinterpret_cast<QString*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *QItemModelScatterDataProxy::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QItemModelScatterDataProxy::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN26QItemModelScatterDataProxyE_t>.strings))
        return static_cast<void*>(this);
    return QScatterDataProxy::qt_metacast(_clname);
}

int QItemModelScatterDataProxy::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QScatterDataProxy::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 16)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 16;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 16)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 16;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 16;
    }
    return _id;
}

// SIGNAL 0
void QItemModelScatterDataProxy::itemModelChanged(const QAbstractItemModel * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void QItemModelScatterDataProxy::xPosRoleChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void QItemModelScatterDataProxy::yPosRoleChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void QItemModelScatterDataProxy::zPosRoleChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void QItemModelScatterDataProxy::rotationRoleChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void QItemModelScatterDataProxy::scaleRoleChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void QItemModelScatterDataProxy::xPosRolePatternChanged(const QRegularExpression & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}

// SIGNAL 7
void QItemModelScatterDataProxy::yPosRolePatternChanged(const QRegularExpression & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1);
}

// SIGNAL 8
void QItemModelScatterDataProxy::zPosRolePatternChanged(const QRegularExpression & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 8, nullptr, _t1);
}

// SIGNAL 9
void QItemModelScatterDataProxy::rotationRolePatternChanged(const QRegularExpression & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 9, nullptr, _t1);
}

// SIGNAL 10
void QItemModelScatterDataProxy::scaleRolePatternChanged(const QRegularExpression & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 10, nullptr, _t1);
}

// SIGNAL 11
void QItemModelScatterDataProxy::rotationRoleReplaceChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 11, nullptr, _t1);
}

// SIGNAL 12
void QItemModelScatterDataProxy::xPosRoleReplaceChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 12, nullptr, _t1);
}

// SIGNAL 13
void QItemModelScatterDataProxy::yPosRoleReplaceChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 13, nullptr, _t1);
}

// SIGNAL 14
void QItemModelScatterDataProxy::zPosRoleReplaceChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 14, nullptr, _t1);
}

// SIGNAL 15
void QItemModelScatterDataProxy::scaleRoleReplaceChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 15, nullptr, _t1);
}
QT_WARNING_POP
