/****************************************************************************
** Meta object code from reading C++ file 'qpieslice.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../qtgraphs/src/graphs2d/piechart/qpieslice.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qpieslice.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN9QPieSliceE_t {};
} // unnamed namespace

template <> constexpr inline auto QPieSlice::qt_create_metaobjectdata<qt_meta_tag_ZN9QPieSliceE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QPieSlice",
        "RegisterEnumClassesUnscoped",
        "false",
        "DefaultProperty",
        "sliceChildren",
        "QML.Element",
        "PieSlice",
        "labelChanged",
        "",
        "labelVisibleChanged",
        "labelFontChanged",
        "labelColorChanged",
        "valueChanged",
        "explodedChanged",
        "explodeDistanceFactorChanged",
        "percentageChanged",
        "startAngleChanged",
        "angleSpanChanged",
        "sliceChanged",
        "labelPositionChanged",
        "labelArmLengthFactorChanged",
        "colorChanged",
        "borderColorChanged",
        "borderWidthChanged",
        "subSlicesAdded",
        "QList<QPieSlice*>",
        "slices",
        "subSlicesRemoved",
        "subSlicesReplaced",
        "subSlicesCountChanged",
        "qsizetype",
        "count",
        "subSlicesSumChanged",
        "sum",
        "subSlicesRatioChanged",
        "newSubSlicesRatio",
        "clear",
        "append",
        "QPieSlice*",
        "slice",
        "insert",
        "index",
        "remove",
        "label",
        "value",
        "at",
        "find",
        "replace",
        "removeMultiple",
        "oldSlice",
        "newSlice",
        "replaceAll",
        "take",
        "labelVisible",
        "labelPosition",
        "LabelPosition",
        "labelColor",
        "QColor",
        "labelFont",
        "QFont",
        "labelArmLengthFactor",
        "color",
        "borderColor",
        "borderWidth",
        "exploded",
        "explodeDistanceFactor",
        "percentage",
        "startAngle",
        "angleSpan",
        "subSlicesCount",
        "subSlicesSum",
        "subSlicesRatio",
        "QQmlListProperty<QPieSlice>",
        "Outside",
        "InsideHorizontal",
        "InsideTangential",
        "InsideNormal"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'labelChanged'
        QtMocHelpers::SignalData<void()>(7, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'labelVisibleChanged'
        QtMocHelpers::SignalData<void()>(9, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'labelFontChanged'
        QtMocHelpers::SignalData<void()>(10, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'labelColorChanged'
        QtMocHelpers::SignalData<void()>(11, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'valueChanged'
        QtMocHelpers::SignalData<void()>(12, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'explodedChanged'
        QtMocHelpers::SignalData<void()>(13, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'explodeDistanceFactorChanged'
        QtMocHelpers::SignalData<void()>(14, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'percentageChanged'
        QtMocHelpers::SignalData<void()>(15, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'startAngleChanged'
        QtMocHelpers::SignalData<void()>(16, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'angleSpanChanged'
        QtMocHelpers::SignalData<void()>(17, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'sliceChanged'
        QtMocHelpers::SignalData<void()>(18, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'labelPositionChanged'
        QtMocHelpers::SignalData<void()>(19, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'labelArmLengthFactorChanged'
        QtMocHelpers::SignalData<void()>(20, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'colorChanged'
        QtMocHelpers::SignalData<void()>(21, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'borderColorChanged'
        QtMocHelpers::SignalData<void()>(22, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'borderWidthChanged'
        QtMocHelpers::SignalData<void()>(23, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'subSlicesAdded'
        QtMocHelpers::RevisionedSignalData<void(const QList<QPieSlice*> &)>(24, 8, QMC::AccessPublic, 0x60b, QMetaType::Void, {{
            { 0x80000000 | 25, 26 },
        }}),
        // Signal 'subSlicesRemoved'
        QtMocHelpers::RevisionedSignalData<void(const QList<QPieSlice*> &)>(27, 8, QMC::AccessPublic, 0x60b, QMetaType::Void, {{
            { 0x80000000 | 25, 26 },
        }}),
        // Signal 'subSlicesReplaced'
        QtMocHelpers::RevisionedSignalData<void(const QList<QPieSlice*> &)>(28, 8, QMC::AccessPublic, 0x60b, QMetaType::Void, {{
            { 0x80000000 | 25, 26 },
        }}),
        // Signal 'subSlicesCountChanged'
        QtMocHelpers::RevisionedSignalData<void(qsizetype)>(29, 8, QMC::AccessPublic, 0x60b, QMetaType::Void, {{
            { 0x80000000 | 30, 31 },
        }}),
        // Signal 'subSlicesSumChanged'
        QtMocHelpers::RevisionedSignalData<void(qreal)>(32, 8, QMC::AccessPublic, 0x60b, QMetaType::Void, {{
            { QMetaType::QReal, 33 },
        }}),
        // Signal 'subSlicesRatioChanged'
        QtMocHelpers::RevisionedSignalData<void(qreal)>(34, 8, QMC::AccessPublic, 0x60b, QMetaType::Void, {{
            { QMetaType::QReal, 35 },
        }}),
        // Slot 'clear'
        QtMocHelpers::RevisionedSlotData<void()>(36, 8, QMC::AccessPublic, 0x60b, QMetaType::Void),
        // Method 'append'
        QtMocHelpers::RevisionedMethodData<bool(QPieSlice *)>(37, 8, QMC::AccessPublic, 0x60b, QMetaType::Bool, {{
            { 0x80000000 | 38, 39 },
        }}),
        // Method 'append'
        QtMocHelpers::RevisionedMethodData<bool(const QList<QPieSlice*> &)>(37, 8, QMC::AccessPublic, 0x60b, QMetaType::Bool, {{
            { 0x80000000 | 25, 26 },
        }}),
        // Method 'insert'
        QtMocHelpers::RevisionedMethodData<bool(qsizetype, QPieSlice *)>(40, 8, QMC::AccessPublic, 0x60b, QMetaType::Bool, {{
            { 0x80000000 | 30, 41 }, { 0x80000000 | 38, 39 },
        }}),
        // Method 'remove'
        QtMocHelpers::RevisionedMethodData<bool(QPieSlice *)>(42, 8, QMC::AccessPublic, 0x60b, QMetaType::Bool, {{
            { 0x80000000 | 38, 39 },
        }}),
        // Method 'append'
        QtMocHelpers::RevisionedMethodData<QPieSlice *(const QString &, qreal)>(37, 8, QMC::AccessPublic, 0x60b, 0x80000000 | 38, {{
            { QMetaType::QString, 43 }, { QMetaType::QReal, 44 },
        }}),
        // Method 'at'
        QtMocHelpers::RevisionedMethodData<QPieSlice *(qsizetype) const>(45, 8, QMC::AccessPublic, 0x60b, 0x80000000 | 38, {{
            { 0x80000000 | 30, 41 },
        }}),
        // Method 'find'
        QtMocHelpers::RevisionedMethodData<QPieSlice *(const QString &) const>(46, 8, QMC::AccessPublic, 0x60b, 0x80000000 | 38, {{
            { QMetaType::QString, 43 },
        }}),
        // Method 'replace'
        QtMocHelpers::RevisionedMethodData<bool(qsizetype, QPieSlice *)>(47, 8, QMC::AccessPublic, 0x60b, QMetaType::Bool, {{
            { 0x80000000 | 30, 41 }, { 0x80000000 | 38, 39 },
        }}),
        // Method 'removeMultiple'
        QtMocHelpers::RevisionedMethodData<void(qsizetype, int)>(48, 8, QMC::AccessPublic, 0x60b, QMetaType::Void, {{
            { 0x80000000 | 30, 41 }, { QMetaType::Int, 31 },
        }}),
        // Method 'remove'
        QtMocHelpers::RevisionedMethodData<bool(qsizetype)>(42, 8, QMC::AccessPublic, 0x60b, QMetaType::Bool, {{
            { 0x80000000 | 30, 41 },
        }}),
        // Method 'replace'
        QtMocHelpers::RevisionedMethodData<bool(QPieSlice *, QPieSlice *)>(47, 8, QMC::AccessPublic, 0x60b, QMetaType::Bool, {{
            { 0x80000000 | 38, 49 }, { 0x80000000 | 38, 50 },
        }}),
        // Method 'replaceAll'
        QtMocHelpers::RevisionedMethodData<bool(const QList<QPieSlice*> &)>(51, 8, QMC::AccessPublic, 0x60b, QMetaType::Bool, {{
            { 0x80000000 | 25, 26 },
        }}),
        // Method 'take'
        QtMocHelpers::RevisionedMethodData<bool(QPieSlice *)>(52, 8, QMC::AccessPublic, 0x60b, QMetaType::Bool, {{
            { 0x80000000 | 38, 39 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'label'
        QtMocHelpers::PropertyData<QString>(43, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 0),
        // property 'labelVisible'
        QtMocHelpers::PropertyData<bool>(53, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 1),
        // property 'labelPosition'
        QtMocHelpers::PropertyData<enum LabelPosition>(54, 0x80000000 | 55, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet | QMC::Final, 11),
        // property 'labelColor'
        QtMocHelpers::PropertyData<QColor>(56, 0x80000000 | 57, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet | QMC::Final, 3),
        // property 'labelFont'
        QtMocHelpers::PropertyData<QFont>(58, 0x80000000 | 59, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet | QMC::Final, 2),
        // property 'labelArmLengthFactor'
        QtMocHelpers::PropertyData<qreal>(60, QMetaType::QReal, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 12),
        // property 'color'
        QtMocHelpers::PropertyData<QColor>(61, 0x80000000 | 57, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet | QMC::Final, 13),
        // property 'borderColor'
        QtMocHelpers::PropertyData<QColor>(62, 0x80000000 | 57, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet | QMC::Final, 14),
        // property 'borderWidth'
        QtMocHelpers::PropertyData<qreal>(63, QMetaType::QReal, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 15),
        // property 'value'
        QtMocHelpers::PropertyData<qreal>(44, QMetaType::QReal, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 4),
        // property 'exploded'
        QtMocHelpers::PropertyData<bool>(64, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 5),
        // property 'explodeDistanceFactor'
        QtMocHelpers::PropertyData<qreal>(65, QMetaType::QReal, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 6),
        // property 'percentage'
        QtMocHelpers::PropertyData<qreal>(66, QMetaType::QReal, QMC::DefaultPropertyFlags | QMC::Final, 7),
        // property 'startAngle'
        QtMocHelpers::PropertyData<qreal>(67, QMetaType::QReal, QMC::DefaultPropertyFlags | QMC::Final, 8),
        // property 'angleSpan'
        QtMocHelpers::PropertyData<qreal>(68, QMetaType::QReal, QMC::DefaultPropertyFlags | QMC::Final, 9),
        // property 'subSlicesCount'
        QtMocHelpers::PropertyData<qsizetype>(69, 0x80000000 | 30, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 19, 0x60b),
        // property 'subSlicesSum'
        QtMocHelpers::PropertyData<qreal>(70, QMetaType::QReal, QMC::DefaultPropertyFlags, 20, 0x60b),
        // property 'subSlicesRatio'
        QtMocHelpers::PropertyData<qreal>(71, QMetaType::QReal, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 21, 0x60b),
        // property 'sliceChildren'
        QtMocHelpers::PropertyData<QQmlListProperty<QPieSlice>>(4, 0x80000000 | 72, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, -1, 0x60b),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'LabelPosition'
        QtMocHelpers::EnumData<enum LabelPosition>(55, 55, QMC::EnumIsScoped).add({
            {   73, LabelPosition::Outside },
            {   74, LabelPosition::InsideHorizontal },
            {   75, LabelPosition::InsideTangential },
            {   76, LabelPosition::InsideNormal },
        }),
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
            {    5,    6 },
    });
    return QtMocHelpers::metaObjectData<QPieSlice, void>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject QPieSlice::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9QPieSliceE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9QPieSliceE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN9QPieSliceE_t>.metaTypes,
    nullptr
} };

void QPieSlice::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QPieSlice *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->labelChanged(); break;
        case 1: _t->labelVisibleChanged(); break;
        case 2: _t->labelFontChanged(); break;
        case 3: _t->labelColorChanged(); break;
        case 4: _t->valueChanged(); break;
        case 5: _t->explodedChanged(); break;
        case 6: _t->explodeDistanceFactorChanged(); break;
        case 7: _t->percentageChanged(); break;
        case 8: _t->startAngleChanged(); break;
        case 9: _t->angleSpanChanged(); break;
        case 10: _t->sliceChanged(); break;
        case 11: _t->labelPositionChanged(); break;
        case 12: _t->labelArmLengthFactorChanged(); break;
        case 13: _t->colorChanged(); break;
        case 14: _t->borderColorChanged(); break;
        case 15: _t->borderWidthChanged(); break;
        case 16: _t->subSlicesAdded((*reinterpret_cast<std::add_pointer_t<QList<QPieSlice*>>>(_a[1]))); break;
        case 17: _t->subSlicesRemoved((*reinterpret_cast<std::add_pointer_t<QList<QPieSlice*>>>(_a[1]))); break;
        case 18: _t->subSlicesReplaced((*reinterpret_cast<std::add_pointer_t<QList<QPieSlice*>>>(_a[1]))); break;
        case 19: _t->subSlicesCountChanged((*reinterpret_cast<std::add_pointer_t<qsizetype>>(_a[1]))); break;
        case 20: _t->subSlicesSumChanged((*reinterpret_cast<std::add_pointer_t<qreal>>(_a[1]))); break;
        case 21: _t->subSlicesRatioChanged((*reinterpret_cast<std::add_pointer_t<qreal>>(_a[1]))); break;
        case 22: _t->clear(); break;
        case 23: { bool _r = _t->append((*reinterpret_cast<std::add_pointer_t<QPieSlice*>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 24: { bool _r = _t->append((*reinterpret_cast<std::add_pointer_t<QList<QPieSlice*>>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 25: { bool _r = _t->insert((*reinterpret_cast<std::add_pointer_t<qsizetype>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QPieSlice*>>(_a[2])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 26: { bool _r = _t->remove((*reinterpret_cast<std::add_pointer_t<QPieSlice*>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 27: { QPieSlice* _r = _t->append((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<qreal>>(_a[2])));
            if (_a[0]) *reinterpret_cast<QPieSlice**>(_a[0]) = std::move(_r); }  break;
        case 28: { QPieSlice* _r = _t->at((*reinterpret_cast<std::add_pointer_t<qsizetype>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QPieSlice**>(_a[0]) = std::move(_r); }  break;
        case 29: { QPieSlice* _r = _t->find((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QPieSlice**>(_a[0]) = std::move(_r); }  break;
        case 30: { bool _r = _t->replace((*reinterpret_cast<std::add_pointer_t<qsizetype>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QPieSlice*>>(_a[2])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 31: _t->removeMultiple((*reinterpret_cast<std::add_pointer_t<qsizetype>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 32: { bool _r = _t->remove((*reinterpret_cast<std::add_pointer_t<qsizetype>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 33: { bool _r = _t->replace((*reinterpret_cast<std::add_pointer_t<QPieSlice*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QPieSlice*>>(_a[2])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 34: { bool _r = _t->replaceAll((*reinterpret_cast<std::add_pointer_t<QList<QPieSlice*>>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 35: { bool _r = _t->take((*reinterpret_cast<std::add_pointer_t<QPieSlice*>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 16:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QList<QPieSlice*> >(); break;
            }
            break;
        case 17:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QList<QPieSlice*> >(); break;
            }
            break;
        case 18:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QList<QPieSlice*> >(); break;
            }
            break;
        case 23:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QPieSlice* >(); break;
            }
            break;
        case 24:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QList<QPieSlice*> >(); break;
            }
            break;
        case 25:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QPieSlice* >(); break;
            }
            break;
        case 26:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QPieSlice* >(); break;
            }
            break;
        case 30:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QPieSlice* >(); break;
            }
            break;
        case 33:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QPieSlice* >(); break;
            }
            break;
        case 34:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QList<QPieSlice*> >(); break;
            }
            break;
        case 35:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QPieSlice* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (QPieSlice::*)()>(_a, &QPieSlice::labelChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (QPieSlice::*)()>(_a, &QPieSlice::labelVisibleChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (QPieSlice::*)()>(_a, &QPieSlice::labelFontChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (QPieSlice::*)()>(_a, &QPieSlice::labelColorChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (QPieSlice::*)()>(_a, &QPieSlice::valueChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (QPieSlice::*)()>(_a, &QPieSlice::explodedChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (QPieSlice::*)()>(_a, &QPieSlice::explodeDistanceFactorChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (QPieSlice::*)()>(_a, &QPieSlice::percentageChanged, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (QPieSlice::*)()>(_a, &QPieSlice::startAngleChanged, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (QPieSlice::*)()>(_a, &QPieSlice::angleSpanChanged, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (QPieSlice::*)()>(_a, &QPieSlice::sliceChanged, 10))
            return;
        if (QtMocHelpers::indexOfMethod<void (QPieSlice::*)()>(_a, &QPieSlice::labelPositionChanged, 11))
            return;
        if (QtMocHelpers::indexOfMethod<void (QPieSlice::*)()>(_a, &QPieSlice::labelArmLengthFactorChanged, 12))
            return;
        if (QtMocHelpers::indexOfMethod<void (QPieSlice::*)()>(_a, &QPieSlice::colorChanged, 13))
            return;
        if (QtMocHelpers::indexOfMethod<void (QPieSlice::*)()>(_a, &QPieSlice::borderColorChanged, 14))
            return;
        if (QtMocHelpers::indexOfMethod<void (QPieSlice::*)()>(_a, &QPieSlice::borderWidthChanged, 15))
            return;
        if (QtMocHelpers::indexOfMethod<void (QPieSlice::*)(const QList<QPieSlice*> & )>(_a, &QPieSlice::subSlicesAdded, 16))
            return;
        if (QtMocHelpers::indexOfMethod<void (QPieSlice::*)(const QList<QPieSlice*> & )>(_a, &QPieSlice::subSlicesRemoved, 17))
            return;
        if (QtMocHelpers::indexOfMethod<void (QPieSlice::*)(const QList<QPieSlice*> & )>(_a, &QPieSlice::subSlicesReplaced, 18))
            return;
        if (QtMocHelpers::indexOfMethod<void (QPieSlice::*)(qsizetype )>(_a, &QPieSlice::subSlicesCountChanged, 19))
            return;
        if (QtMocHelpers::indexOfMethod<void (QPieSlice::*)(qreal )>(_a, &QPieSlice::subSlicesSumChanged, 20))
            return;
        if (QtMocHelpers::indexOfMethod<void (QPieSlice::*)(qreal )>(_a, &QPieSlice::subSlicesRatioChanged, 21))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QString*>(_v) = _t->label(); break;
        case 1: *reinterpret_cast<bool*>(_v) = _t->isLabelVisible(); break;
        case 2: *reinterpret_cast<enum LabelPosition*>(_v) = _t->labelPosition(); break;
        case 3: *reinterpret_cast<QColor*>(_v) = _t->labelColor(); break;
        case 4: *reinterpret_cast<QFont*>(_v) = _t->labelFont(); break;
        case 5: *reinterpret_cast<qreal*>(_v) = _t->labelArmLengthFactor(); break;
        case 6: *reinterpret_cast<QColor*>(_v) = _t->color(); break;
        case 7: *reinterpret_cast<QColor*>(_v) = _t->borderColor(); break;
        case 8: *reinterpret_cast<qreal*>(_v) = _t->borderWidth(); break;
        case 9: *reinterpret_cast<qreal*>(_v) = _t->value(); break;
        case 10: *reinterpret_cast<bool*>(_v) = _t->isExploded(); break;
        case 11: *reinterpret_cast<qreal*>(_v) = _t->explodeDistanceFactor(); break;
        case 12: *reinterpret_cast<qreal*>(_v) = _t->percentage(); break;
        case 13: *reinterpret_cast<qreal*>(_v) = _t->startAngle(); break;
        case 14: *reinterpret_cast<qreal*>(_v) = _t->angleSpan(); break;
        case 15: *reinterpret_cast<qsizetype*>(_v) = _t->subSlicesCount(); break;
        case 16: *reinterpret_cast<qreal*>(_v) = _t->subSlicesSum(); break;
        case 17: *reinterpret_cast<qreal*>(_v) = _t->subSlicesRatio(); break;
        case 18: *reinterpret_cast<QQmlListProperty<QPieSlice>*>(_v) = _t->sliceChildren(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setLabel(*reinterpret_cast<QString*>(_v)); break;
        case 1: _t->setLabelVisible(*reinterpret_cast<bool*>(_v)); break;
        case 2: _t->setLabelPosition(*reinterpret_cast<enum LabelPosition*>(_v)); break;
        case 3: _t->setLabelColor(*reinterpret_cast<QColor*>(_v)); break;
        case 4: _t->setLabelFont(*reinterpret_cast<QFont*>(_v)); break;
        case 5: _t->setLabelArmLengthFactor(*reinterpret_cast<qreal*>(_v)); break;
        case 6: _t->setColor(*reinterpret_cast<QColor*>(_v)); break;
        case 7: _t->setBorderColor(*reinterpret_cast<QColor*>(_v)); break;
        case 8: _t->setBorderWidth(*reinterpret_cast<qreal*>(_v)); break;
        case 9: _t->setValue(*reinterpret_cast<qreal*>(_v)); break;
        case 10: _t->setExploded(*reinterpret_cast<bool*>(_v)); break;
        case 11: _t->setExplodeDistanceFactor(*reinterpret_cast<qreal*>(_v)); break;
        case 17: _t->setSubSlicesRatio(*reinterpret_cast<qreal*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *QPieSlice::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QPieSlice::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9QPieSliceE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int QPieSlice::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 36)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 36;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 36)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 36;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 19;
    }
    return _id;
}

// SIGNAL 0
void QPieSlice::labelChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void QPieSlice::labelVisibleChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void QPieSlice::labelFontChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void QPieSlice::labelColorChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void QPieSlice::valueChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void QPieSlice::explodedChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void QPieSlice::explodeDistanceFactorChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void QPieSlice::percentageChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void QPieSlice::startAngleChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 8, nullptr);
}

// SIGNAL 9
void QPieSlice::angleSpanChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 9, nullptr);
}

// SIGNAL 10
void QPieSlice::sliceChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 10, nullptr);
}

// SIGNAL 11
void QPieSlice::labelPositionChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 11, nullptr);
}

// SIGNAL 12
void QPieSlice::labelArmLengthFactorChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 12, nullptr);
}

// SIGNAL 13
void QPieSlice::colorChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 13, nullptr);
}

// SIGNAL 14
void QPieSlice::borderColorChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 14, nullptr);
}

// SIGNAL 15
void QPieSlice::borderWidthChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 15, nullptr);
}

// SIGNAL 16
void QPieSlice::subSlicesAdded(const QList<QPieSlice*> & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 16, nullptr, _t1);
}

// SIGNAL 17
void QPieSlice::subSlicesRemoved(const QList<QPieSlice*> & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 17, nullptr, _t1);
}

// SIGNAL 18
void QPieSlice::subSlicesReplaced(const QList<QPieSlice*> & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 18, nullptr, _t1);
}

// SIGNAL 19
void QPieSlice::subSlicesCountChanged(qsizetype _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 19, nullptr, _t1);
}

// SIGNAL 20
void QPieSlice::subSlicesSumChanged(qreal _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 20, nullptr, _t1);
}

// SIGNAL 21
void QPieSlice::subSlicesRatioChanged(qreal _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 21, nullptr, _t1);
}
QT_WARNING_POP
