/****************************************************************************
** Meta object code from reading C++ file 'qquickgraphssplineseries_p.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../qtgraphs/src/graphs3d/qml/qquickgraphssplineseries_p.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qquickgraphssplineseries_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN26QQuickGraphsSpline3DSeriesE_t {};
} // unnamed namespace

template <> constexpr inline auto QQuickGraphsSpline3DSeries::qt_create_metaobjectdata<qt_meta_tag_ZN26QQuickGraphsSpline3DSeriesE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QQuickGraphsSpline3DSeries",
        "DefaultProperty",
        "seriesChildren",
        "QML.AddedInVersion",
        "1545",
        "QML.Element",
        "Spline3DSeries",
        "baseGradientChanged",
        "",
        "QQuickGradient*",
        "gradient",
        "singleHighlightGradientChanged",
        "multiHighlightGradientChanged",
        "handleBaseGradientUpdate",
        "handleSingleHighlightGradientUpdate",
        "handleMultiHighlightGradientUpdate",
        "QQmlListProperty<QObject>",
        "baseGradient",
        "singleHighlightGradient",
        "multiHighlightGradient",
        "invalidSelectionIndex"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'baseGradientChanged'
        QtMocHelpers::SignalData<void(QQuickGradient *)>(7, 8, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 10 },
        }}),
        // Signal 'singleHighlightGradientChanged'
        QtMocHelpers::SignalData<void(QQuickGradient *)>(11, 8, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 10 },
        }}),
        // Signal 'multiHighlightGradientChanged'
        QtMocHelpers::SignalData<void(QQuickGradient *)>(12, 8, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 10 },
        }}),
        // Slot 'handleBaseGradientUpdate'
        QtMocHelpers::SlotData<void()>(13, 8, QMC::AccessPublic, QMetaType::Void),
        // Slot 'handleSingleHighlightGradientUpdate'
        QtMocHelpers::SlotData<void()>(14, 8, QMC::AccessPublic, QMetaType::Void),
        // Slot 'handleMultiHighlightGradientUpdate'
        QtMocHelpers::SlotData<void()>(15, 8, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'seriesChildren'
        QtMocHelpers::PropertyData<QQmlListProperty<QObject>>(2, 0x80000000 | 16, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'baseGradient'
        QtMocHelpers::PropertyData<QQuickGradient*>(17, 0x80000000 | 9, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet | QMC::Final, 0),
        // property 'singleHighlightGradient'
        QtMocHelpers::PropertyData<QQuickGradient*>(18, 0x80000000 | 9, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet | QMC::Final, 1),
        // property 'multiHighlightGradient'
        QtMocHelpers::PropertyData<QQuickGradient*>(19, 0x80000000 | 9, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet | QMC::Final, 2),
        // property 'invalidSelectionIndex'
        QtMocHelpers::PropertyData<int>(20, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
            {    5,    6 },
    });
    return QtMocHelpers::metaObjectData<QQuickGraphsSpline3DSeries, void>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject QQuickGraphsSpline3DSeries::staticMetaObject = { {
    QMetaObject::SuperData::link<QSpline3DSeries::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN26QQuickGraphsSpline3DSeriesE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN26QQuickGraphsSpline3DSeriesE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN26QQuickGraphsSpline3DSeriesE_t>.metaTypes,
    nullptr
} };

void QQuickGraphsSpline3DSeries::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QQuickGraphsSpline3DSeries *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->baseGradientChanged((*reinterpret_cast<std::add_pointer_t<QQuickGradient*>>(_a[1]))); break;
        case 1: _t->singleHighlightGradientChanged((*reinterpret_cast<std::add_pointer_t<QQuickGradient*>>(_a[1]))); break;
        case 2: _t->multiHighlightGradientChanged((*reinterpret_cast<std::add_pointer_t<QQuickGradient*>>(_a[1]))); break;
        case 3: _t->handleBaseGradientUpdate(); break;
        case 4: _t->handleSingleHighlightGradientUpdate(); break;
        case 5: _t->handleMultiHighlightGradientUpdate(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QQuickGradient* >(); break;
            }
            break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QQuickGradient* >(); break;
            }
            break;
        case 2:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QQuickGradient* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (QQuickGraphsSpline3DSeries::*)(QQuickGradient * )>(_a, &QQuickGraphsSpline3DSeries::baseGradientChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (QQuickGraphsSpline3DSeries::*)(QQuickGradient * )>(_a, &QQuickGraphsSpline3DSeries::singleHighlightGradientChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (QQuickGraphsSpline3DSeries::*)(QQuickGradient * )>(_a, &QQuickGraphsSpline3DSeries::multiHighlightGradientChanged, 2))
            return;
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 3:
        case 2:
        case 1:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQuickGradient* >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QQmlListProperty<QObject>*>(_v) = _t->seriesChildren(); break;
        case 1: *reinterpret_cast<QQuickGradient**>(_v) = _t->baseGradient(); break;
        case 2: *reinterpret_cast<QQuickGradient**>(_v) = _t->singleHighlightGradient(); break;
        case 3: *reinterpret_cast<QQuickGradient**>(_v) = _t->multiHighlightGradient(); break;
        case 4: *reinterpret_cast<int*>(_v) = _t->invalidSelectionIndex(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 1: _t->setBaseGradient(*reinterpret_cast<QQuickGradient**>(_v)); break;
        case 2: _t->setSingleHighlightGradient(*reinterpret_cast<QQuickGradient**>(_v)); break;
        case 3: _t->setMultiHighlightGradient(*reinterpret_cast<QQuickGradient**>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *QQuickGraphsSpline3DSeries::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QQuickGraphsSpline3DSeries::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN26QQuickGraphsSpline3DSeriesE_t>.strings))
        return static_cast<void*>(this);
    return QSpline3DSeries::qt_metacast(_clname);
}

int QQuickGraphsSpline3DSeries::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QSpline3DSeries::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 6)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 6)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    }
    return _id;
}

// SIGNAL 0
void QQuickGraphsSpline3DSeries::baseGradientChanged(QQuickGradient * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void QQuickGraphsSpline3DSeries::singleHighlightGradientChanged(QQuickGradient * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void QQuickGraphsSpline3DSeries::multiHighlightGradientChanged(QQuickGradient * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}
QT_WARNING_POP
