/****************************************************************************
** Meta object code from reading C++ file 'qquickgraphsitem_p.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../qtgraphs/src/graphs3d/qml/qquickgraphsitem_p.h"
#include <QtGui/qtextcursor.h>
#include <QtNetwork/QSslError>
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qquickgraphsitem_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN16QQuickGraphsItemE_t {};
} // unnamed namespace

template <> constexpr inline auto QQuickGraphsItem::qt_create_metaobjectdata<qt_meta_tag_ZN16QQuickGraphsItemE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QQuickGraphsItem",
        "QML.Element",
        "GraphsItem3D",
        "QML.Creatable",
        "false",
        "QML.UncreatableReason",
        "",
        "selectionModeChanged",
        "QtGraphs3D::SelectionFlags",
        "mode",
        "shadowQualityChanged",
        "QtGraphs3D::ShadowQuality",
        "quality",
        "shadowsSupportedChanged",
        "supported",
        "msaaSamplesChanged",
        "samples",
        "themeChanged",
        "QGraphsTheme*",
        "theme",
        "renderingModeChanged",
        "QtGraphs3D::RenderingMode",
        "transparencyTechniqueChanged",
        "QtGraphs3D::TransparencyTechnique",
        "technique",
        "measureFpsChanged",
        "enabled",
        "currentFpsChanged",
        "fps",
        "selectedElementChanged",
        "QtGraphs3D::ElementType",
        "type",
        "orthoProjectionChanged",
        "aspectRatioChanged",
        "ratio",
        "optimizationHintChanged",
        "QtGraphs3D::OptimizationHint",
        "hint",
        "polarChanged",
        "labelMarginChanged",
        "margin",
        "radialLabelOffsetChanged",
        "offset",
        "horizontalAspectRatioChanged",
        "localeChanged",
        "locale",
        "queriedGraphPositionChanged",
        "QVector3D",
        "data",
        "marginChanged",
        "cutoffMarginChanged",
        "newMargin",
        "cameraPresetChanged",
        "QtGraphs3D::CameraPreset",
        "preset",
        "cameraXRotationChanged",
        "rotation",
        "cameraYRotationChanged",
        "cameraZoomLevelChanged",
        "zoomLevel",
        "cameraTargetPositionChanged",
        "target",
        "minCameraZoomLevelChanged",
        "maxCameraZoomLevelChanged",
        "minCameraXRotationChanged",
        "minCameraYRotationChanged",
        "maxCameraXRotationChanged",
        "maxCameraYRotationChanged",
        "wrapCameraXRotationChanged",
        "wrap",
        "wrapCameraYRotationChanged",
        "needRender",
        "themeTypeChanged",
        "axisXChanged",
        "QAbstract3DAxis*",
        "axis",
        "axisYChanged",
        "axisZChanged",
        "activeThemeChanged",
        "activeTheme",
        "tapped",
        "QEventPoint",
        "eventPoint",
        "Qt::MouseButton",
        "button",
        "doubleTapped",
        "longPressed",
        "dragged",
        "QVector2D",
        "delta",
        "wheel",
        "QQuickWheelEvent*",
        "event",
        "pinch",
        "mouseMove",
        "QPoint",
        "mousePos",
        "zoomEnabledChanged",
        "enable",
        "zoomAtTargetEnabledChanged",
        "rotationEnabledChanged",
        "panEnabledChanged",
        "panModeEnabledChanged",
        "selectionEnabledChanged",
        "ambientLightStrengthChanged",
        "lightStrengthChanged",
        "shadowStrengthChanged",
        "lightColorChanged",
        "gridLineTypeChanged",
        "handleAxisXChanged",
        "handleAxisYChanged",
        "handleAxisZChanged",
        "handleFpsChanged",
        "windowDestroyed",
        "obj",
        "handleAxisTitleChanged",
        "title",
        "handleAxisLabelsChanged",
        "handleAxisRangeChanged",
        "min",
        "max",
        "handleAxisSegmentCountChanged",
        "qsizetype",
        "count",
        "handleAxisSubSegmentCountChanged",
        "handleAxisAutoAdjustRangeChanged",
        "autoAdjust",
        "handleAxisScaleLabelsByCountChanged",
        "adjust",
        "handleAxisLabelSizeChanged",
        "size",
        "handleAxisLabelFormatChanged",
        "format",
        "handleAxisReversedChanged",
        "handleAxisFormatterDirty",
        "handleAxisLabelAutoRotationChanged",
        "angle",
        "handleAxisTitleVisibilityChanged",
        "visible",
        "handleAxisLabelVisibilityChanged",
        "handleAxisTitleFixedChanged",
        "fixed",
        "handleAxisTitleOffsetChanged",
        "handleInputPositionChanged",
        "position",
        "handleSeriesVisibilityChanged",
        "handleItemLabelVisibleChanged",
        "handleItemLabelVisibleChangedBySender",
        "sender",
        "handleThemeColorStyleChanged",
        "QGraphsTheme::ColorStyle",
        "style",
        "handleThemeBaseColorsChanged",
        "QList<QColor>",
        "color",
        "handleThemeBaseGradientsChanged",
        "QList<QLinearGradient>",
        "gradient",
        "handleThemeSingleHighlightColorChanged",
        "QColor",
        "handleThemeSingleHighlightGradientChanged",
        "QLinearGradient",
        "handleThemeMultiHighlightColorChanged",
        "handleThemeMultiHighlightGradientChanged",
        "handleThemeTypeChanged",
        "QGraphsTheme::Theme",
        "handleRequestShadowQuality",
        "updateCustomItem",
        "hasSeries",
        "QAbstract3DSeries*",
        "series",
        "clearSelection",
        "addCustomItem",
        "QCustom3DItem*",
        "item",
        "removeCustomItems",
        "removeCustomItem",
        "removeCustomItemAt",
        "releaseCustomItem",
        "selectedLabelIndex",
        "selectedAxis",
        "selectedCustomItemIndex",
        "selectedCustomItem",
        "setDefaultInputHandler",
        "unsetDefaultInputHandler",
        "unsetDefaultTapHandler",
        "unsetDefaultDragHandler",
        "unsetDefaultWheelHandler",
        "unsetDefaultPinchHandler",
        "setDragButton",
        "Qt::MouseButtons",
        "setPanButton",
        "doPicking",
        "QPointF",
        "point",
        "doRayPicking",
        "origin",
        "direction",
        "sceneChanged",
        "rootNode",
        "QQuick3DNode*",
        "selectionMode",
        "shadowQuality",
        "msaaSamples",
        "scene",
        "Q3DScene*",
        "renderingMode",
        "transparencyTechnique",
        "measureFps",
        "currentFps",
        "customItemList",
        "QQmlListProperty<QCustom3DItem>",
        "orthoProjection",
        "selectedElement",
        "aspectRatio",
        "optimizationHint",
        "polar",
        "labelMargin",
        "radialLabelOffset",
        "horizontalAspectRatio",
        "queriedGraphPosition",
        "cutoffMargin",
        "cameraXRotation",
        "cameraYRotation",
        "minCameraXRotation",
        "maxCameraXRotation",
        "minCameraYRotation",
        "maxCameraYRotation",
        "cameraZoomLevel",
        "cameraPreset",
        "cameraTargetPosition",
        "minCameraZoomLevel",
        "maxCameraZoomLevel",
        "wrapCameraXRotation",
        "wrapCameraYRotation",
        "rotationEnabled",
        "panEnabled",
        "panModeEnabled",
        "zoomAtTargetEnabled",
        "selectionEnabled",
        "zoomEnabled",
        "lightColor",
        "ambientLightStrength",
        "lightStrength",
        "shadowStrength",
        "gridLineType",
        "QtGraphs3D::GridLineType"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'selectionModeChanged'
        QtMocHelpers::SignalData<void(QtGraphs3D::SelectionFlags)>(7, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 8, 9 },
        }}),
        // Signal 'shadowQualityChanged'
        QtMocHelpers::SignalData<void(QtGraphs3D::ShadowQuality)>(10, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 11, 12 },
        }}),
        // Signal 'shadowsSupportedChanged'
        QtMocHelpers::SignalData<void(bool)>(13, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 14 },
        }}),
        // Signal 'msaaSamplesChanged'
        QtMocHelpers::SignalData<void(int)>(15, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 16 },
        }}),
        // Signal 'themeChanged'
        QtMocHelpers::SignalData<void(QGraphsTheme *)>(17, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 18, 19 },
        }}),
        // Signal 'renderingModeChanged'
        QtMocHelpers::SignalData<void(QtGraphs3D::RenderingMode)>(20, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 21, 9 },
        }}),
        // Signal 'transparencyTechniqueChanged'
        QtMocHelpers::RevisionedSignalData<void(QtGraphs3D::TransparencyTechnique)>(22, 6, QMC::AccessPublic, 0x609, QMetaType::Void, {{
            { 0x80000000 | 23, 24 },
        }}),
        // Signal 'measureFpsChanged'
        QtMocHelpers::SignalData<void(bool)>(25, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 26 },
        }}),
        // Signal 'currentFpsChanged'
        QtMocHelpers::SignalData<void(int)>(27, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 28 },
        }}),
        // Signal 'selectedElementChanged'
        QtMocHelpers::SignalData<void(QtGraphs3D::ElementType)>(29, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 30, 31 },
        }}),
        // Signal 'orthoProjectionChanged'
        QtMocHelpers::SignalData<void(bool)>(32, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 26 },
        }}),
        // Signal 'aspectRatioChanged'
        QtMocHelpers::SignalData<void(qreal)>(33, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QReal, 34 },
        }}),
        // Signal 'optimizationHintChanged'
        QtMocHelpers::SignalData<void(QtGraphs3D::OptimizationHint)>(35, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 36, 37 },
        }}),
        // Signal 'polarChanged'
        QtMocHelpers::SignalData<void(bool)>(38, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 26 },
        }}),
        // Signal 'labelMarginChanged'
        QtMocHelpers::SignalData<void(float)>(39, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Float, 40 },
        }}),
        // Signal 'radialLabelOffsetChanged'
        QtMocHelpers::SignalData<void(float)>(41, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Float, 42 },
        }}),
        // Signal 'horizontalAspectRatioChanged'
        QtMocHelpers::SignalData<void(qreal)>(43, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QReal, 34 },
        }}),
        // Signal 'localeChanged'
        QtMocHelpers::SignalData<void(const QLocale &)>(44, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QLocale, 45 },
        }}),
        // Signal 'queriedGraphPositionChanged'
        QtMocHelpers::SignalData<void(QVector3D)>(46, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 47, 48 },
        }}),
        // Signal 'marginChanged'
        QtMocHelpers::SignalData<void(qreal)>(49, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QReal, 40 },
        }}),
        // Signal 'cutoffMarginChanged'
        QtMocHelpers::RevisionedSignalData<void(qreal)>(50, 6, QMC::AccessPublic, 0x60b, QMetaType::Void, {{
            { QMetaType::QReal, 51 },
        }}),
        // Signal 'cameraPresetChanged'
        QtMocHelpers::SignalData<void(QtGraphs3D::CameraPreset)>(52, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 53, 54 },
        }}),
        // Signal 'cameraXRotationChanged'
        QtMocHelpers::SignalData<void(float)>(55, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Float, 56 },
        }}),
        // Signal 'cameraYRotationChanged'
        QtMocHelpers::SignalData<void(float)>(57, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Float, 56 },
        }}),
        // Signal 'cameraZoomLevelChanged'
        QtMocHelpers::SignalData<void(float)>(58, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Float, 59 },
        }}),
        // Signal 'cameraTargetPositionChanged'
        QtMocHelpers::SignalData<void(QVector3D)>(60, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 47, 61 },
        }}),
        // Signal 'minCameraZoomLevelChanged'
        QtMocHelpers::SignalData<void(float)>(62, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Float, 59 },
        }}),
        // Signal 'maxCameraZoomLevelChanged'
        QtMocHelpers::SignalData<void(float)>(63, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Float, 59 },
        }}),
        // Signal 'minCameraXRotationChanged'
        QtMocHelpers::RevisionedSignalData<void(float)>(64, 6, QMC::AccessPublic, 0x609, QMetaType::Void, {{
            { QMetaType::Float, 56 },
        }}),
        // Signal 'minCameraYRotationChanged'
        QtMocHelpers::RevisionedSignalData<void(float)>(65, 6, QMC::AccessPublic, 0x609, QMetaType::Void, {{
            { QMetaType::Float, 56 },
        }}),
        // Signal 'maxCameraXRotationChanged'
        QtMocHelpers::RevisionedSignalData<void(float)>(66, 6, QMC::AccessPublic, 0x609, QMetaType::Void, {{
            { QMetaType::Float, 56 },
        }}),
        // Signal 'maxCameraYRotationChanged'
        QtMocHelpers::RevisionedSignalData<void(float)>(67, 6, QMC::AccessPublic, 0x609, QMetaType::Void, {{
            { QMetaType::Float, 56 },
        }}),
        // Signal 'wrapCameraXRotationChanged'
        QtMocHelpers::SignalData<void(bool)>(68, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 69 },
        }}),
        // Signal 'wrapCameraYRotationChanged'
        QtMocHelpers::SignalData<void(bool)>(70, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 69 },
        }}),
        // Signal 'needRender'
        QtMocHelpers::SignalData<void()>(71, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'themeTypeChanged'
        QtMocHelpers::SignalData<void()>(72, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'axisXChanged'
        QtMocHelpers::SignalData<void(QAbstract3DAxis *)>(73, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 74, 75 },
        }}),
        // Signal 'axisYChanged'
        QtMocHelpers::SignalData<void(QAbstract3DAxis *)>(76, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 74, 75 },
        }}),
        // Signal 'axisZChanged'
        QtMocHelpers::SignalData<void(QAbstract3DAxis *)>(77, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 74, 75 },
        }}),
        // Signal 'activeThemeChanged'
        QtMocHelpers::SignalData<void(QGraphsTheme *)>(78, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 18, 79 },
        }}),
        // Signal 'tapped'
        QtMocHelpers::SignalData<void(QEventPoint, Qt::MouseButton)>(80, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 81, 82 }, { 0x80000000 | 83, 84 },
        }}),
        // Signal 'doubleTapped'
        QtMocHelpers::SignalData<void(QEventPoint, Qt::MouseButton)>(85, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 81, 82 }, { 0x80000000 | 83, 84 },
        }}),
        // Signal 'longPressed'
        QtMocHelpers::SignalData<void()>(86, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'dragged'
        QtMocHelpers::SignalData<void(QVector2D)>(87, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 88, 89 },
        }}),
        // Signal 'wheel'
        QtMocHelpers::SignalData<void(QQuickWheelEvent *)>(90, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 91, 92 },
        }}),
        // Signal 'pinch'
        QtMocHelpers::SignalData<void(qreal)>(93, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QReal, 89 },
        }}),
        // Signal 'mouseMove'
        QtMocHelpers::SignalData<void(QPoint)>(94, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 95, 96 },
        }}),
        // Signal 'zoomEnabledChanged'
        QtMocHelpers::SignalData<void(bool)>(97, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 98 },
        }}),
        // Signal 'zoomAtTargetEnabledChanged'
        QtMocHelpers::SignalData<void(bool)>(99, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 98 },
        }}),
        // Signal 'rotationEnabledChanged'
        QtMocHelpers::SignalData<void(bool)>(100, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 98 },
        }}),
        // Signal 'panEnabledChanged'
        QtMocHelpers::RevisionedSignalData<void(bool)>(101, 6, QMC::AccessPublic, 0x60c, QMetaType::Void, {{
            { QMetaType::Bool, 98 },
        }}),
        // Signal 'panModeEnabledChanged'
        QtMocHelpers::RevisionedSignalData<void(bool)>(102, 6, QMC::AccessPublic, 0x60c, QMetaType::Void, {{
            { QMetaType::Bool, 98 },
        }}),
        // Signal 'selectionEnabledChanged'
        QtMocHelpers::SignalData<void(bool)>(103, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 98 },
        }}),
        // Signal 'ambientLightStrengthChanged'
        QtMocHelpers::SignalData<void()>(104, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'lightStrengthChanged'
        QtMocHelpers::SignalData<void()>(105, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'shadowStrengthChanged'
        QtMocHelpers::SignalData<void()>(106, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'lightColorChanged'
        QtMocHelpers::SignalData<void()>(107, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'gridLineTypeChanged'
        QtMocHelpers::SignalData<void()>(108, 6, QMC::AccessPublic, QMetaType::Void),
        // Slot 'handleAxisXChanged'
        QtMocHelpers::SlotData<void(QAbstract3DAxis *)>(109, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 74, 75 },
        }}),
        // Slot 'handleAxisYChanged'
        QtMocHelpers::SlotData<void(QAbstract3DAxis *)>(110, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 74, 75 },
        }}),
        // Slot 'handleAxisZChanged'
        QtMocHelpers::SlotData<void(QAbstract3DAxis *)>(111, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 74, 75 },
        }}),
        // Slot 'handleFpsChanged'
        QtMocHelpers::SlotData<void()>(112, 6, QMC::AccessPublic, QMetaType::Void),
        // Slot 'windowDestroyed'
        QtMocHelpers::SlotData<void(QObject *)>(113, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QObjectStar, 114 },
        }}),
        // Slot 'handleAxisTitleChanged'
        QtMocHelpers::SlotData<void(const QString &)>(115, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 116 },
        }}),
        // Slot 'handleAxisLabelsChanged'
        QtMocHelpers::SlotData<void()>(117, 6, QMC::AccessPublic, QMetaType::Void),
        // Slot 'handleAxisRangeChanged'
        QtMocHelpers::SlotData<void(float, float)>(118, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Float, 119 }, { QMetaType::Float, 120 },
        }}),
        // Slot 'handleAxisSegmentCountChanged'
        QtMocHelpers::SlotData<void(qsizetype)>(121, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 122, 123 },
        }}),
        // Slot 'handleAxisSubSegmentCountChanged'
        QtMocHelpers::SlotData<void(qsizetype)>(124, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 122, 123 },
        }}),
        // Slot 'handleAxisAutoAdjustRangeChanged'
        QtMocHelpers::SlotData<void(bool)>(125, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 126 },
        }}),
        // Slot 'handleAxisScaleLabelsByCountChanged'
        QtMocHelpers::SlotData<void(bool)>(127, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 128 },
        }}),
        // Slot 'handleAxisLabelSizeChanged'
        QtMocHelpers::SlotData<void(qreal)>(129, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QReal, 130 },
        }}),
        // Slot 'handleAxisLabelFormatChanged'
        QtMocHelpers::SlotData<void(const QString &)>(131, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 132 },
        }}),
        // Slot 'handleAxisReversedChanged'
        QtMocHelpers::SlotData<void(bool)>(133, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 98 },
        }}),
        // Slot 'handleAxisFormatterDirty'
        QtMocHelpers::SlotData<void()>(134, 6, QMC::AccessPublic, QMetaType::Void),
        // Slot 'handleAxisLabelAutoRotationChanged'
        QtMocHelpers::SlotData<void(float)>(135, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Float, 136 },
        }}),
        // Slot 'handleAxisTitleVisibilityChanged'
        QtMocHelpers::SlotData<void(bool)>(137, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 138 },
        }}),
        // Slot 'handleAxisLabelVisibilityChanged'
        QtMocHelpers::SlotData<void(bool)>(139, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 138 },
        }}),
        // Slot 'handleAxisTitleFixedChanged'
        QtMocHelpers::SlotData<void(bool)>(140, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 141 },
        }}),
        // Slot 'handleAxisTitleOffsetChanged'
        QtMocHelpers::SlotData<void(float)>(142, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Float, 42 },
        }}),
        // Slot 'handleInputPositionChanged'
        QtMocHelpers::SlotData<void(QPoint)>(143, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 95, 144 },
        }}),
        // Slot 'handleSeriesVisibilityChanged'
        QtMocHelpers::SlotData<void(bool)>(145, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 138 },
        }}),
        // Slot 'handleItemLabelVisibleChanged'
        QtMocHelpers::SlotData<void(bool)>(146, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 138 },
        }}),
        // Slot 'handleItemLabelVisibleChangedBySender'
        QtMocHelpers::SlotData<void(bool, QObject *)>(147, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 138 }, { QMetaType::QObjectStar, 148 },
        }}),
        // Slot 'handleThemeColorStyleChanged'
        QtMocHelpers::SlotData<void(QGraphsTheme::ColorStyle)>(149, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 150, 151 },
        }}),
        // Slot 'handleThemeBaseColorsChanged'
        QtMocHelpers::SlotData<void(const QList<QColor> &)>(152, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 153, 154 },
        }}),
        // Slot 'handleThemeBaseGradientsChanged'
        QtMocHelpers::SlotData<void(const QList<QLinearGradient> &)>(155, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 156, 157 },
        }}),
        // Slot 'handleThemeSingleHighlightColorChanged'
        QtMocHelpers::SlotData<void(QColor)>(158, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 159, 154 },
        }}),
        // Slot 'handleThemeSingleHighlightGradientChanged'
        QtMocHelpers::SlotData<void(const QLinearGradient &)>(160, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 161, 157 },
        }}),
        // Slot 'handleThemeMultiHighlightColorChanged'
        QtMocHelpers::SlotData<void(QColor)>(162, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 159, 154 },
        }}),
        // Slot 'handleThemeMultiHighlightGradientChanged'
        QtMocHelpers::SlotData<void(const QLinearGradient &)>(163, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 161, 157 },
        }}),
        // Slot 'handleThemeTypeChanged'
        QtMocHelpers::SlotData<void(QGraphsTheme::Theme)>(164, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 165, 19 },
        }}),
        // Slot 'handleRequestShadowQuality'
        QtMocHelpers::SlotData<void(QtGraphs3D::ShadowQuality)>(166, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 11, 12 },
        }}),
        // Slot 'updateCustomItem'
        QtMocHelpers::SlotData<void()>(167, 6, QMC::AccessPublic, QMetaType::Void),
        // Method 'hasSeries'
        QtMocHelpers::MethodData<bool(QAbstract3DSeries *)>(168, 6, QMC::AccessPublic, QMetaType::Bool, {{
            { 0x80000000 | 169, 170 },
        }}),
        // Method 'clearSelection'
        QtMocHelpers::MethodData<void()>(171, 6, QMC::AccessPublic, QMetaType::Void),
        // Method 'addCustomItem'
        QtMocHelpers::MethodData<qsizetype(QCustom3DItem *)>(172, 6, QMC::AccessPublic, 0x80000000 | 122, {{
            { 0x80000000 | 173, 174 },
        }}),
        // Method 'removeCustomItems'
        QtMocHelpers::MethodData<void()>(175, 6, QMC::AccessPublic, QMetaType::Void),
        // Method 'removeCustomItem'
        QtMocHelpers::MethodData<void(QCustom3DItem *)>(176, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 173, 174 },
        }}),
        // Method 'removeCustomItemAt'
        QtMocHelpers::MethodData<void(QVector3D)>(177, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 47, 144 },
        }}),
        // Method 'releaseCustomItem'
        QtMocHelpers::MethodData<void(QCustom3DItem *)>(178, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 173, 174 },
        }}),
        // Method 'selectedLabelIndex'
        QtMocHelpers::MethodData<int() const>(179, 6, QMC::AccessPublic, QMetaType::Int),
        // Method 'selectedAxis'
        QtMocHelpers::MethodData<QAbstract3DAxis *() const>(180, 6, QMC::AccessPublic, 0x80000000 | 74),
        // Method 'selectedCustomItemIndex'
        QtMocHelpers::MethodData<qsizetype() const>(181, 6, QMC::AccessPublic, 0x80000000 | 122),
        // Method 'selectedCustomItem'
        QtMocHelpers::MethodData<QCustom3DItem *() const>(182, 6, QMC::AccessPublic, 0x80000000 | 173),
        // Method 'setDefaultInputHandler'
        QtMocHelpers::MethodData<void()>(183, 6, QMC::AccessPublic, QMetaType::Void),
        // Method 'unsetDefaultInputHandler'
        QtMocHelpers::MethodData<void()>(184, 6, QMC::AccessPublic, QMetaType::Void),
        // Method 'unsetDefaultTapHandler'
        QtMocHelpers::MethodData<void()>(185, 6, QMC::AccessPublic, QMetaType::Void),
        // Method 'unsetDefaultDragHandler'
        QtMocHelpers::MethodData<void()>(186, 6, QMC::AccessPublic, QMetaType::Void),
        // Method 'unsetDefaultWheelHandler'
        QtMocHelpers::MethodData<void()>(187, 6, QMC::AccessPublic, QMetaType::Void),
        // Method 'unsetDefaultPinchHandler'
        QtMocHelpers::MethodData<void()>(188, 6, QMC::AccessPublic, QMetaType::Void),
        // Method 'setDragButton'
        QtMocHelpers::MethodData<void(Qt::MouseButtons)>(189, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 190, 84 },
        }}),
        // Method 'setPanButton'
        QtMocHelpers::MethodData<void(Qt::MouseButtons)>(191, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 190, 84 },
        }}),
        // Method 'doPicking'
        QtMocHelpers::MethodData<bool(QPointF)>(192, 6, QMC::AccessPublic, QMetaType::Bool, {{
            { 0x80000000 | 193, 194 },
        }}),
        // Method 'doRayPicking'
        QtMocHelpers::RevisionedMethodData<bool(QVector3D, QVector3D)>(195, 6, QMC::AccessPublic, 0x609, QMetaType::Bool, {{
            { 0x80000000 | 47, 196 }, { 0x80000000 | 47, 197 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'rootNode'
        QtMocHelpers::PropertyData<QQuick3DNode*>(199, 0x80000000 | 200, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant, -1, 0x609),
        // property 'selectionMode'
        QtMocHelpers::PropertyData<QtGraphs3D::SelectionFlags>(201, 0x80000000 | 8, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 0),
        // property 'shadowQuality'
        QtMocHelpers::PropertyData<QtGraphs3D::ShadowQuality>(202, 0x80000000 | 11, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 1),
        // property 'msaaSamples'
        QtMocHelpers::PropertyData<int>(203, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 3),
        // property 'scene'
        QtMocHelpers::PropertyData<Q3DScene*>(204, 0x80000000 | 205, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 0x70000000 | 198),
        // property 'theme'
        QtMocHelpers::PropertyData<QGraphsTheme*>(19, 0x80000000 | 18, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 4),
        // property 'renderingMode'
        QtMocHelpers::PropertyData<QtGraphs3D::RenderingMode>(206, 0x80000000 | 21, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 5),
        // property 'transparencyTechnique'
        QtMocHelpers::PropertyData<QtGraphs3D::TransparencyTechnique>(207, 0x80000000 | 23, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 6, 0x609),
        // property 'measureFps'
        QtMocHelpers::PropertyData<bool>(208, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 7),
        // property 'currentFps'
        QtMocHelpers::PropertyData<int>(209, QMetaType::Int, QMC::DefaultPropertyFlags, 8),
        // property 'customItemList'
        QtMocHelpers::PropertyData<QQmlListProperty<QCustom3DItem>>(210, 0x80000000 | 211, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'orthoProjection'
        QtMocHelpers::PropertyData<bool>(212, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 10),
        // property 'selectedElement'
        QtMocHelpers::PropertyData<QtGraphs3D::ElementType>(213, 0x80000000 | 30, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 9),
        // property 'aspectRatio'
        QtMocHelpers::PropertyData<qreal>(214, QMetaType::QReal, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 11),
        // property 'optimizationHint'
        QtMocHelpers::PropertyData<QtGraphs3D::OptimizationHint>(215, 0x80000000 | 36, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 12),
        // property 'polar'
        QtMocHelpers::PropertyData<bool>(216, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 13),
        // property 'labelMargin'
        QtMocHelpers::PropertyData<float>(217, QMetaType::Float, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 14),
        // property 'radialLabelOffset'
        QtMocHelpers::PropertyData<float>(218, QMetaType::Float, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 15),
        // property 'horizontalAspectRatio'
        QtMocHelpers::PropertyData<qreal>(219, QMetaType::QReal, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 16),
        // property 'locale'
        QtMocHelpers::PropertyData<QLocale>(45, QMetaType::QLocale, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 17),
        // property 'queriedGraphPosition'
        QtMocHelpers::PropertyData<QVector3D>(220, 0x80000000 | 47, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 18),
        // property 'margin'
        QtMocHelpers::PropertyData<qreal>(40, QMetaType::QReal, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 19),
        // property 'cutoffMargin'
        QtMocHelpers::PropertyData<qreal>(221, QMetaType::QReal, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 20, 0x60b),
        // property 'cameraXRotation'
        QtMocHelpers::PropertyData<float>(222, QMetaType::Float, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 22),
        // property 'cameraYRotation'
        QtMocHelpers::PropertyData<float>(223, QMetaType::Float, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 23),
        // property 'minCameraXRotation'
        QtMocHelpers::PropertyData<float>(224, QMetaType::Float, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 28, 0x609),
        // property 'maxCameraXRotation'
        QtMocHelpers::PropertyData<float>(225, QMetaType::Float, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 30, 0x609),
        // property 'minCameraYRotation'
        QtMocHelpers::PropertyData<float>(226, QMetaType::Float, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 29, 0x609),
        // property 'maxCameraYRotation'
        QtMocHelpers::PropertyData<float>(227, QMetaType::Float, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 31, 0x609),
        // property 'cameraZoomLevel'
        QtMocHelpers::PropertyData<float>(228, QMetaType::Float, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 24),
        // property 'cameraPreset'
        QtMocHelpers::PropertyData<QtGraphs3D::CameraPreset>(229, 0x80000000 | 53, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 21),
        // property 'cameraTargetPosition'
        QtMocHelpers::PropertyData<QVector3D>(230, 0x80000000 | 47, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 25),
        // property 'minCameraZoomLevel'
        QtMocHelpers::PropertyData<float>(231, QMetaType::Float, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 26),
        // property 'maxCameraZoomLevel'
        QtMocHelpers::PropertyData<float>(232, QMetaType::Float, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 27),
        // property 'wrapCameraXRotation'
        QtMocHelpers::PropertyData<bool>(233, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 32),
        // property 'wrapCameraYRotation'
        QtMocHelpers::PropertyData<bool>(234, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 33),
        // property 'rotationEnabled'
        QtMocHelpers::PropertyData<bool>(235, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 49),
        // property 'panEnabled'
        QtMocHelpers::PropertyData<bool>(236, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 50, 0x60c),
        // property 'panModeEnabled'
        QtMocHelpers::PropertyData<bool>(237, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 51, 0x60c),
        // property 'zoomAtTargetEnabled'
        QtMocHelpers::PropertyData<bool>(238, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 48),
        // property 'selectionEnabled'
        QtMocHelpers::PropertyData<bool>(239, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 52),
        // property 'zoomEnabled'
        QtMocHelpers::PropertyData<bool>(240, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 47),
        // property 'lightColor'
        QtMocHelpers::PropertyData<QColor>(241, 0x80000000 | 159, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 56),
        // property 'ambientLightStrength'
        QtMocHelpers::PropertyData<float>(242, QMetaType::Float, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 53),
        // property 'lightStrength'
        QtMocHelpers::PropertyData<float>(243, QMetaType::Float, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 54),
        // property 'shadowStrength'
        QtMocHelpers::PropertyData<float>(244, QMetaType::Float, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 55),
        // property 'gridLineType'
        QtMocHelpers::PropertyData<QtGraphs3D::GridLineType>(245, 0x80000000 | 246, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet | QMC::Final, 57),
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
            {    5,    6 },
    });
    return QtMocHelpers::metaObjectData<QQuickGraphsItem, void>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT static const QMetaObject::SuperData qt_meta_extradata_ZN16QQuickGraphsItemE[] = {
    QMetaObject::SuperData::link<QtGraphs3D::staticMetaObject>(),
    nullptr
};

Q_CONSTINIT const QMetaObject QQuickGraphsItem::staticMetaObject = { {
    QMetaObject::SuperData::link<QQuick3DViewport::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16QQuickGraphsItemE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16QQuickGraphsItemE_t>.data,
    qt_static_metacall,
    qt_meta_extradata_ZN16QQuickGraphsItemE,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN16QQuickGraphsItemE_t>.metaTypes,
    nullptr
} };

void QQuickGraphsItem::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QQuickGraphsItem *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->selectionModeChanged((*reinterpret_cast<std::add_pointer_t<QtGraphs3D::SelectionFlags>>(_a[1]))); break;
        case 1: _t->shadowQualityChanged((*reinterpret_cast<std::add_pointer_t<QtGraphs3D::ShadowQuality>>(_a[1]))); break;
        case 2: _t->shadowsSupportedChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 3: _t->msaaSamplesChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 4: _t->themeChanged((*reinterpret_cast<std::add_pointer_t<QGraphsTheme*>>(_a[1]))); break;
        case 5: _t->renderingModeChanged((*reinterpret_cast<std::add_pointer_t<QtGraphs3D::RenderingMode>>(_a[1]))); break;
        case 6: _t->transparencyTechniqueChanged((*reinterpret_cast<std::add_pointer_t<QtGraphs3D::TransparencyTechnique>>(_a[1]))); break;
        case 7: _t->measureFpsChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 8: _t->currentFpsChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 9: _t->selectedElementChanged((*reinterpret_cast<std::add_pointer_t<QtGraphs3D::ElementType>>(_a[1]))); break;
        case 10: _t->orthoProjectionChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 11: _t->aspectRatioChanged((*reinterpret_cast<std::add_pointer_t<qreal>>(_a[1]))); break;
        case 12: _t->optimizationHintChanged((*reinterpret_cast<std::add_pointer_t<QtGraphs3D::OptimizationHint>>(_a[1]))); break;
        case 13: _t->polarChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 14: _t->labelMarginChanged((*reinterpret_cast<std::add_pointer_t<float>>(_a[1]))); break;
        case 15: _t->radialLabelOffsetChanged((*reinterpret_cast<std::add_pointer_t<float>>(_a[1]))); break;
        case 16: _t->horizontalAspectRatioChanged((*reinterpret_cast<std::add_pointer_t<qreal>>(_a[1]))); break;
        case 17: _t->localeChanged((*reinterpret_cast<std::add_pointer_t<QLocale>>(_a[1]))); break;
        case 18: _t->queriedGraphPositionChanged((*reinterpret_cast<std::add_pointer_t<QVector3D>>(_a[1]))); break;
        case 19: _t->marginChanged((*reinterpret_cast<std::add_pointer_t<qreal>>(_a[1]))); break;
        case 20: _t->cutoffMarginChanged((*reinterpret_cast<std::add_pointer_t<qreal>>(_a[1]))); break;
        case 21: _t->cameraPresetChanged((*reinterpret_cast<std::add_pointer_t<QtGraphs3D::CameraPreset>>(_a[1]))); break;
        case 22: _t->cameraXRotationChanged((*reinterpret_cast<std::add_pointer_t<float>>(_a[1]))); break;
        case 23: _t->cameraYRotationChanged((*reinterpret_cast<std::add_pointer_t<float>>(_a[1]))); break;
        case 24: _t->cameraZoomLevelChanged((*reinterpret_cast<std::add_pointer_t<float>>(_a[1]))); break;
        case 25: _t->cameraTargetPositionChanged((*reinterpret_cast<std::add_pointer_t<QVector3D>>(_a[1]))); break;
        case 26: _t->minCameraZoomLevelChanged((*reinterpret_cast<std::add_pointer_t<float>>(_a[1]))); break;
        case 27: _t->maxCameraZoomLevelChanged((*reinterpret_cast<std::add_pointer_t<float>>(_a[1]))); break;
        case 28: _t->minCameraXRotationChanged((*reinterpret_cast<std::add_pointer_t<float>>(_a[1]))); break;
        case 29: _t->minCameraYRotationChanged((*reinterpret_cast<std::add_pointer_t<float>>(_a[1]))); break;
        case 30: _t->maxCameraXRotationChanged((*reinterpret_cast<std::add_pointer_t<float>>(_a[1]))); break;
        case 31: _t->maxCameraYRotationChanged((*reinterpret_cast<std::add_pointer_t<float>>(_a[1]))); break;
        case 32: _t->wrapCameraXRotationChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 33: _t->wrapCameraYRotationChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 34: _t->needRender(); break;
        case 35: _t->themeTypeChanged(); break;
        case 36: _t->axisXChanged((*reinterpret_cast<std::add_pointer_t<QAbstract3DAxis*>>(_a[1]))); break;
        case 37: _t->axisYChanged((*reinterpret_cast<std::add_pointer_t<QAbstract3DAxis*>>(_a[1]))); break;
        case 38: _t->axisZChanged((*reinterpret_cast<std::add_pointer_t<QAbstract3DAxis*>>(_a[1]))); break;
        case 39: _t->activeThemeChanged((*reinterpret_cast<std::add_pointer_t<QGraphsTheme*>>(_a[1]))); break;
        case 40: _t->tapped((*reinterpret_cast<std::add_pointer_t<QEventPoint>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<Qt::MouseButton>>(_a[2]))); break;
        case 41: _t->doubleTapped((*reinterpret_cast<std::add_pointer_t<QEventPoint>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<Qt::MouseButton>>(_a[2]))); break;
        case 42: _t->longPressed(); break;
        case 43: _t->dragged((*reinterpret_cast<std::add_pointer_t<QVector2D>>(_a[1]))); break;
        case 44: _t->wheel((*reinterpret_cast<std::add_pointer_t<QQuickWheelEvent*>>(_a[1]))); break;
        case 45: _t->pinch((*reinterpret_cast<std::add_pointer_t<qreal>>(_a[1]))); break;
        case 46: _t->mouseMove((*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[1]))); break;
        case 47: _t->zoomEnabledChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 48: _t->zoomAtTargetEnabledChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 49: _t->rotationEnabledChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 50: _t->panEnabledChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 51: _t->panModeEnabledChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 52: _t->selectionEnabledChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 53: _t->ambientLightStrengthChanged(); break;
        case 54: _t->lightStrengthChanged(); break;
        case 55: _t->shadowStrengthChanged(); break;
        case 56: _t->lightColorChanged(); break;
        case 57: _t->gridLineTypeChanged(); break;
        case 58: _t->handleAxisXChanged((*reinterpret_cast<std::add_pointer_t<QAbstract3DAxis*>>(_a[1]))); break;
        case 59: _t->handleAxisYChanged((*reinterpret_cast<std::add_pointer_t<QAbstract3DAxis*>>(_a[1]))); break;
        case 60: _t->handleAxisZChanged((*reinterpret_cast<std::add_pointer_t<QAbstract3DAxis*>>(_a[1]))); break;
        case 61: _t->handleFpsChanged(); break;
        case 62: _t->windowDestroyed((*reinterpret_cast<std::add_pointer_t<QObject*>>(_a[1]))); break;
        case 63: _t->handleAxisTitleChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 64: _t->handleAxisLabelsChanged(); break;
        case 65: _t->handleAxisRangeChanged((*reinterpret_cast<std::add_pointer_t<float>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[2]))); break;
        case 66: _t->handleAxisSegmentCountChanged((*reinterpret_cast<std::add_pointer_t<qsizetype>>(_a[1]))); break;
        case 67: _t->handleAxisSubSegmentCountChanged((*reinterpret_cast<std::add_pointer_t<qsizetype>>(_a[1]))); break;
        case 68: _t->handleAxisAutoAdjustRangeChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 69: _t->handleAxisScaleLabelsByCountChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 70: _t->handleAxisLabelSizeChanged((*reinterpret_cast<std::add_pointer_t<qreal>>(_a[1]))); break;
        case 71: _t->handleAxisLabelFormatChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 72: _t->handleAxisReversedChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 73: _t->handleAxisFormatterDirty(); break;
        case 74: _t->handleAxisLabelAutoRotationChanged((*reinterpret_cast<std::add_pointer_t<float>>(_a[1]))); break;
        case 75: _t->handleAxisTitleVisibilityChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 76: _t->handleAxisLabelVisibilityChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 77: _t->handleAxisTitleFixedChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 78: _t->handleAxisTitleOffsetChanged((*reinterpret_cast<std::add_pointer_t<float>>(_a[1]))); break;
        case 79: _t->handleInputPositionChanged((*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[1]))); break;
        case 80: _t->handleSeriesVisibilityChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 81: _t->handleItemLabelVisibleChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 82: _t->handleItemLabelVisibleChangedBySender((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QObject*>>(_a[2]))); break;
        case 83: _t->handleThemeColorStyleChanged((*reinterpret_cast<std::add_pointer_t<QGraphsTheme::ColorStyle>>(_a[1]))); break;
        case 84: _t->handleThemeBaseColorsChanged((*reinterpret_cast<std::add_pointer_t<QList<QColor>>>(_a[1]))); break;
        case 85: _t->handleThemeBaseGradientsChanged((*reinterpret_cast<std::add_pointer_t<QList<QLinearGradient>>>(_a[1]))); break;
        case 86: _t->handleThemeSingleHighlightColorChanged((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        case 87: _t->handleThemeSingleHighlightGradientChanged((*reinterpret_cast<std::add_pointer_t<QLinearGradient>>(_a[1]))); break;
        case 88: _t->handleThemeMultiHighlightColorChanged((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        case 89: _t->handleThemeMultiHighlightGradientChanged((*reinterpret_cast<std::add_pointer_t<QLinearGradient>>(_a[1]))); break;
        case 90: _t->handleThemeTypeChanged((*reinterpret_cast<std::add_pointer_t<QGraphsTheme::Theme>>(_a[1]))); break;
        case 91: _t->handleRequestShadowQuality((*reinterpret_cast<std::add_pointer_t<QtGraphs3D::ShadowQuality>>(_a[1]))); break;
        case 92: _t->updateCustomItem(); break;
        case 93: { bool _r = _t->hasSeries((*reinterpret_cast<std::add_pointer_t<QAbstract3DSeries*>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 94: _t->clearSelection(); break;
        case 95: { qsizetype _r = _t->addCustomItem((*reinterpret_cast<std::add_pointer_t<QCustom3DItem*>>(_a[1])));
            if (_a[0]) *reinterpret_cast<qsizetype*>(_a[0]) = std::move(_r); }  break;
        case 96: _t->removeCustomItems(); break;
        case 97: _t->removeCustomItem((*reinterpret_cast<std::add_pointer_t<QCustom3DItem*>>(_a[1]))); break;
        case 98: _t->removeCustomItemAt((*reinterpret_cast<std::add_pointer_t<QVector3D>>(_a[1]))); break;
        case 99: _t->releaseCustomItem((*reinterpret_cast<std::add_pointer_t<QCustom3DItem*>>(_a[1]))); break;
        case 100: { int _r = _t->selectedLabelIndex();
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 101: { QAbstract3DAxis* _r = _t->selectedAxis();
            if (_a[0]) *reinterpret_cast<QAbstract3DAxis**>(_a[0]) = std::move(_r); }  break;
        case 102: { qsizetype _r = _t->selectedCustomItemIndex();
            if (_a[0]) *reinterpret_cast<qsizetype*>(_a[0]) = std::move(_r); }  break;
        case 103: { QCustom3DItem* _r = _t->selectedCustomItem();
            if (_a[0]) *reinterpret_cast<QCustom3DItem**>(_a[0]) = std::move(_r); }  break;
        case 104: _t->setDefaultInputHandler(); break;
        case 105: _t->unsetDefaultInputHandler(); break;
        case 106: _t->unsetDefaultTapHandler(); break;
        case 107: _t->unsetDefaultDragHandler(); break;
        case 108: _t->unsetDefaultWheelHandler(); break;
        case 109: _t->unsetDefaultPinchHandler(); break;
        case 110: _t->setDragButton((*reinterpret_cast<std::add_pointer_t<Qt::MouseButtons>>(_a[1]))); break;
        case 111: _t->setPanButton((*reinterpret_cast<std::add_pointer_t<Qt::MouseButtons>>(_a[1]))); break;
        case 112: { bool _r = _t->doPicking((*reinterpret_cast<std::add_pointer_t<QPointF>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 113: { bool _r = _t->doRayPicking((*reinterpret_cast<std::add_pointer_t<QVector3D>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QVector3D>>(_a[2])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 4:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QGraphsTheme* >(); break;
            }
            break;
        case 36:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QAbstract3DAxis* >(); break;
            }
            break;
        case 37:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QAbstract3DAxis* >(); break;
            }
            break;
        case 38:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QAbstract3DAxis* >(); break;
            }
            break;
        case 39:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QGraphsTheme* >(); break;
            }
            break;
        case 44:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QQuickWheelEvent* >(); break;
            }
            break;
        case 58:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QAbstract3DAxis* >(); break;
            }
            break;
        case 59:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QAbstract3DAxis* >(); break;
            }
            break;
        case 60:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QAbstract3DAxis* >(); break;
            }
            break;
        case 93:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QAbstract3DSeries* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (QQuickGraphsItem::*)(QtGraphs3D::SelectionFlags )>(_a, &QQuickGraphsItem::selectionModeChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (QQuickGraphsItem::*)(QtGraphs3D::ShadowQuality )>(_a, &QQuickGraphsItem::shadowQualityChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (QQuickGraphsItem::*)(bool )>(_a, &QQuickGraphsItem::shadowsSupportedChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (QQuickGraphsItem::*)(int )>(_a, &QQuickGraphsItem::msaaSamplesChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (QQuickGraphsItem::*)(QGraphsTheme * )>(_a, &QQuickGraphsItem::themeChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (QQuickGraphsItem::*)(QtGraphs3D::RenderingMode )>(_a, &QQuickGraphsItem::renderingModeChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (QQuickGraphsItem::*)(QtGraphs3D::TransparencyTechnique )>(_a, &QQuickGraphsItem::transparencyTechniqueChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (QQuickGraphsItem::*)(bool )>(_a, &QQuickGraphsItem::measureFpsChanged, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (QQuickGraphsItem::*)(int )>(_a, &QQuickGraphsItem::currentFpsChanged, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (QQuickGraphsItem::*)(QtGraphs3D::ElementType )>(_a, &QQuickGraphsItem::selectedElementChanged, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (QQuickGraphsItem::*)(bool )>(_a, &QQuickGraphsItem::orthoProjectionChanged, 10))
            return;
        if (QtMocHelpers::indexOfMethod<void (QQuickGraphsItem::*)(qreal )>(_a, &QQuickGraphsItem::aspectRatioChanged, 11))
            return;
        if (QtMocHelpers::indexOfMethod<void (QQuickGraphsItem::*)(QtGraphs3D::OptimizationHint )>(_a, &QQuickGraphsItem::optimizationHintChanged, 12))
            return;
        if (QtMocHelpers::indexOfMethod<void (QQuickGraphsItem::*)(bool )>(_a, &QQuickGraphsItem::polarChanged, 13))
            return;
        if (QtMocHelpers::indexOfMethod<void (QQuickGraphsItem::*)(float )>(_a, &QQuickGraphsItem::labelMarginChanged, 14))
            return;
        if (QtMocHelpers::indexOfMethod<void (QQuickGraphsItem::*)(float )>(_a, &QQuickGraphsItem::radialLabelOffsetChanged, 15))
            return;
        if (QtMocHelpers::indexOfMethod<void (QQuickGraphsItem::*)(qreal )>(_a, &QQuickGraphsItem::horizontalAspectRatioChanged, 16))
            return;
        if (QtMocHelpers::indexOfMethod<void (QQuickGraphsItem::*)(const QLocale & )>(_a, &QQuickGraphsItem::localeChanged, 17))
            return;
        if (QtMocHelpers::indexOfMethod<void (QQuickGraphsItem::*)(QVector3D )>(_a, &QQuickGraphsItem::queriedGraphPositionChanged, 18))
            return;
        if (QtMocHelpers::indexOfMethod<void (QQuickGraphsItem::*)(qreal )>(_a, &QQuickGraphsItem::marginChanged, 19))
            return;
        if (QtMocHelpers::indexOfMethod<void (QQuickGraphsItem::*)(qreal )>(_a, &QQuickGraphsItem::cutoffMarginChanged, 20))
            return;
        if (QtMocHelpers::indexOfMethod<void (QQuickGraphsItem::*)(QtGraphs3D::CameraPreset )>(_a, &QQuickGraphsItem::cameraPresetChanged, 21))
            return;
        if (QtMocHelpers::indexOfMethod<void (QQuickGraphsItem::*)(float )>(_a, &QQuickGraphsItem::cameraXRotationChanged, 22))
            return;
        if (QtMocHelpers::indexOfMethod<void (QQuickGraphsItem::*)(float )>(_a, &QQuickGraphsItem::cameraYRotationChanged, 23))
            return;
        if (QtMocHelpers::indexOfMethod<void (QQuickGraphsItem::*)(float )>(_a, &QQuickGraphsItem::cameraZoomLevelChanged, 24))
            return;
        if (QtMocHelpers::indexOfMethod<void (QQuickGraphsItem::*)(QVector3D )>(_a, &QQuickGraphsItem::cameraTargetPositionChanged, 25))
            return;
        if (QtMocHelpers::indexOfMethod<void (QQuickGraphsItem::*)(float )>(_a, &QQuickGraphsItem::minCameraZoomLevelChanged, 26))
            return;
        if (QtMocHelpers::indexOfMethod<void (QQuickGraphsItem::*)(float )>(_a, &QQuickGraphsItem::maxCameraZoomLevelChanged, 27))
            return;
        if (QtMocHelpers::indexOfMethod<void (QQuickGraphsItem::*)(float )>(_a, &QQuickGraphsItem::minCameraXRotationChanged, 28))
            return;
        if (QtMocHelpers::indexOfMethod<void (QQuickGraphsItem::*)(float )>(_a, &QQuickGraphsItem::minCameraYRotationChanged, 29))
            return;
        if (QtMocHelpers::indexOfMethod<void (QQuickGraphsItem::*)(float )>(_a, &QQuickGraphsItem::maxCameraXRotationChanged, 30))
            return;
        if (QtMocHelpers::indexOfMethod<void (QQuickGraphsItem::*)(float )>(_a, &QQuickGraphsItem::maxCameraYRotationChanged, 31))
            return;
        if (QtMocHelpers::indexOfMethod<void (QQuickGraphsItem::*)(bool )>(_a, &QQuickGraphsItem::wrapCameraXRotationChanged, 32))
            return;
        if (QtMocHelpers::indexOfMethod<void (QQuickGraphsItem::*)(bool )>(_a, &QQuickGraphsItem::wrapCameraYRotationChanged, 33))
            return;
        if (QtMocHelpers::indexOfMethod<void (QQuickGraphsItem::*)()>(_a, &QQuickGraphsItem::needRender, 34))
            return;
        if (QtMocHelpers::indexOfMethod<void (QQuickGraphsItem::*)()>(_a, &QQuickGraphsItem::themeTypeChanged, 35))
            return;
        if (QtMocHelpers::indexOfMethod<void (QQuickGraphsItem::*)(QAbstract3DAxis * )>(_a, &QQuickGraphsItem::axisXChanged, 36))
            return;
        if (QtMocHelpers::indexOfMethod<void (QQuickGraphsItem::*)(QAbstract3DAxis * )>(_a, &QQuickGraphsItem::axisYChanged, 37))
            return;
        if (QtMocHelpers::indexOfMethod<void (QQuickGraphsItem::*)(QAbstract3DAxis * )>(_a, &QQuickGraphsItem::axisZChanged, 38))
            return;
        if (QtMocHelpers::indexOfMethod<void (QQuickGraphsItem::*)(QGraphsTheme * )>(_a, &QQuickGraphsItem::activeThemeChanged, 39))
            return;
        if (QtMocHelpers::indexOfMethod<void (QQuickGraphsItem::*)(QEventPoint , Qt::MouseButton )>(_a, &QQuickGraphsItem::tapped, 40))
            return;
        if (QtMocHelpers::indexOfMethod<void (QQuickGraphsItem::*)(QEventPoint , Qt::MouseButton )>(_a, &QQuickGraphsItem::doubleTapped, 41))
            return;
        if (QtMocHelpers::indexOfMethod<void (QQuickGraphsItem::*)()>(_a, &QQuickGraphsItem::longPressed, 42))
            return;
        if (QtMocHelpers::indexOfMethod<void (QQuickGraphsItem::*)(QVector2D )>(_a, &QQuickGraphsItem::dragged, 43))
            return;
        if (QtMocHelpers::indexOfMethod<void (QQuickGraphsItem::*)(QQuickWheelEvent * )>(_a, &QQuickGraphsItem::wheel, 44))
            return;
        if (QtMocHelpers::indexOfMethod<void (QQuickGraphsItem::*)(qreal )>(_a, &QQuickGraphsItem::pinch, 45))
            return;
        if (QtMocHelpers::indexOfMethod<void (QQuickGraphsItem::*)(QPoint )>(_a, &QQuickGraphsItem::mouseMove, 46))
            return;
        if (QtMocHelpers::indexOfMethod<void (QQuickGraphsItem::*)(bool )>(_a, &QQuickGraphsItem::zoomEnabledChanged, 47))
            return;
        if (QtMocHelpers::indexOfMethod<void (QQuickGraphsItem::*)(bool )>(_a, &QQuickGraphsItem::zoomAtTargetEnabledChanged, 48))
            return;
        if (QtMocHelpers::indexOfMethod<void (QQuickGraphsItem::*)(bool )>(_a, &QQuickGraphsItem::rotationEnabledChanged, 49))
            return;
        if (QtMocHelpers::indexOfMethod<void (QQuickGraphsItem::*)(bool )>(_a, &QQuickGraphsItem::panEnabledChanged, 50))
            return;
        if (QtMocHelpers::indexOfMethod<void (QQuickGraphsItem::*)(bool )>(_a, &QQuickGraphsItem::panModeEnabledChanged, 51))
            return;
        if (QtMocHelpers::indexOfMethod<void (QQuickGraphsItem::*)(bool )>(_a, &QQuickGraphsItem::selectionEnabledChanged, 52))
            return;
        if (QtMocHelpers::indexOfMethod<void (QQuickGraphsItem::*)()>(_a, &QQuickGraphsItem::ambientLightStrengthChanged, 53))
            return;
        if (QtMocHelpers::indexOfMethod<void (QQuickGraphsItem::*)()>(_a, &QQuickGraphsItem::lightStrengthChanged, 54))
            return;
        if (QtMocHelpers::indexOfMethod<void (QQuickGraphsItem::*)()>(_a, &QQuickGraphsItem::shadowStrengthChanged, 55))
            return;
        if (QtMocHelpers::indexOfMethod<void (QQuickGraphsItem::*)()>(_a, &QQuickGraphsItem::lightColorChanged, 56))
            return;
        if (QtMocHelpers::indexOfMethod<void (QQuickGraphsItem::*)()>(_a, &QQuickGraphsItem::gridLineTypeChanged, 57))
            return;
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 5:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QGraphsTheme* >(); break;
        case 0:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQuick3DNode* >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QQuick3DNode**>(_v) = _t->rootNode(); break;
        case 1: *reinterpret_cast<QtGraphs3D::SelectionFlags*>(_v) = _t->selectionMode(); break;
        case 2: *reinterpret_cast<QtGraphs3D::ShadowQuality*>(_v) = _t->shadowQuality(); break;
        case 3: *reinterpret_cast<int*>(_v) = _t->msaaSamples(); break;
        case 4: *reinterpret_cast<Q3DScene**>(_v) = _t->scene(); break;
        case 5: *reinterpret_cast<QGraphsTheme**>(_v) = _t->theme(); break;
        case 6: *reinterpret_cast<QtGraphs3D::RenderingMode*>(_v) = _t->renderingMode(); break;
        case 7: *reinterpret_cast<QtGraphs3D::TransparencyTechnique*>(_v) = _t->transparencyTechnique(); break;
        case 8: *reinterpret_cast<bool*>(_v) = _t->measureFps(); break;
        case 9: *reinterpret_cast<int*>(_v) = _t->currentFps(); break;
        case 10: *reinterpret_cast<QQmlListProperty<QCustom3DItem>*>(_v) = _t->customItemList(); break;
        case 11: *reinterpret_cast<bool*>(_v) = _t->isOrthoProjection(); break;
        case 12: *reinterpret_cast<QtGraphs3D::ElementType*>(_v) = _t->selectedElement(); break;
        case 13: *reinterpret_cast<qreal*>(_v) = _t->aspectRatio(); break;
        case 14: *reinterpret_cast<QtGraphs3D::OptimizationHint*>(_v) = _t->optimizationHint(); break;
        case 15: *reinterpret_cast<bool*>(_v) = _t->isPolar(); break;
        case 16: *reinterpret_cast<float*>(_v) = _t->labelMargin(); break;
        case 17: *reinterpret_cast<float*>(_v) = _t->radialLabelOffset(); break;
        case 18: *reinterpret_cast<qreal*>(_v) = _t->horizontalAspectRatio(); break;
        case 19: *reinterpret_cast<QLocale*>(_v) = _t->locale(); break;
        case 20: *reinterpret_cast<QVector3D*>(_v) = _t->queriedGraphPosition(); break;
        case 21: *reinterpret_cast<qreal*>(_v) = _t->margin(); break;
        case 22: *reinterpret_cast<qreal*>(_v) = _t->cutoffMargin(); break;
        case 23: *reinterpret_cast<float*>(_v) = _t->cameraXRotation(); break;
        case 24: *reinterpret_cast<float*>(_v) = _t->cameraYRotation(); break;
        case 25: *reinterpret_cast<float*>(_v) = _t->minCameraXRotation(); break;
        case 26: *reinterpret_cast<float*>(_v) = _t->maxCameraXRotation(); break;
        case 27: *reinterpret_cast<float*>(_v) = _t->minCameraYRotation(); break;
        case 28: *reinterpret_cast<float*>(_v) = _t->maxCameraYRotation(); break;
        case 29: *reinterpret_cast<float*>(_v) = _t->cameraZoomLevel(); break;
        case 30: *reinterpret_cast<QtGraphs3D::CameraPreset*>(_v) = _t->cameraPreset(); break;
        case 31: *reinterpret_cast<QVector3D*>(_v) = _t->cameraTargetPosition(); break;
        case 32: *reinterpret_cast<float*>(_v) = _t->minCameraZoomLevel(); break;
        case 33: *reinterpret_cast<float*>(_v) = _t->maxCameraZoomLevel(); break;
        case 34: *reinterpret_cast<bool*>(_v) = _t->wrapCameraXRotation(); break;
        case 35: *reinterpret_cast<bool*>(_v) = _t->wrapCameraYRotation(); break;
        case 36: *reinterpret_cast<bool*>(_v) = _t->rotationEnabled(); break;
        case 37: *reinterpret_cast<bool*>(_v) = _t->panEnabled(); break;
        case 38: *reinterpret_cast<bool*>(_v) = _t->isPanModeEnabled(); break;
        case 39: *reinterpret_cast<bool*>(_v) = _t->zoomAtTargetEnabled(); break;
        case 40: *reinterpret_cast<bool*>(_v) = _t->selectionEnabled(); break;
        case 41: *reinterpret_cast<bool*>(_v) = _t->zoomEnabled(); break;
        case 42: *reinterpret_cast<QColor*>(_v) = _t->lightColor(); break;
        case 43: *reinterpret_cast<float*>(_v) = _t->ambientLightStrength(); break;
        case 44: *reinterpret_cast<float*>(_v) = _t->lightStrength(); break;
        case 45: *reinterpret_cast<float*>(_v) = _t->shadowStrength(); break;
        case 46: *reinterpret_cast<QtGraphs3D::GridLineType*>(_v) = _t->gridLineType(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 1: _t->setSelectionMode(*reinterpret_cast<QtGraphs3D::SelectionFlags*>(_v)); break;
        case 2: _t->setShadowQuality(*reinterpret_cast<QtGraphs3D::ShadowQuality*>(_v)); break;
        case 3: _t->setMsaaSamples(*reinterpret_cast<int*>(_v)); break;
        case 5: _t->setTheme(*reinterpret_cast<QGraphsTheme**>(_v)); break;
        case 6: _t->setRenderingMode(*reinterpret_cast<QtGraphs3D::RenderingMode*>(_v)); break;
        case 7: _t->setTransparencyTechnique(*reinterpret_cast<QtGraphs3D::TransparencyTechnique*>(_v)); break;
        case 8: _t->setMeasureFps(*reinterpret_cast<bool*>(_v)); break;
        case 11: _t->setOrthoProjection(*reinterpret_cast<bool*>(_v)); break;
        case 13: _t->setAspectRatio(*reinterpret_cast<qreal*>(_v)); break;
        case 14: _t->setOptimizationHint(*reinterpret_cast<QtGraphs3D::OptimizationHint*>(_v)); break;
        case 15: _t->setPolar(*reinterpret_cast<bool*>(_v)); break;
        case 16: _t->setLabelMargin(*reinterpret_cast<float*>(_v)); break;
        case 17: _t->setRadialLabelOffset(*reinterpret_cast<float*>(_v)); break;
        case 18: _t->setHorizontalAspectRatio(*reinterpret_cast<qreal*>(_v)); break;
        case 19: _t->setLocale(*reinterpret_cast<QLocale*>(_v)); break;
        case 21: _t->setMargin(*reinterpret_cast<qreal*>(_v)); break;
        case 22: _t->setCutoffMargin(*reinterpret_cast<qreal*>(_v)); break;
        case 23: _t->setCameraXRotation(*reinterpret_cast<float*>(_v)); break;
        case 24: _t->setCameraYRotation(*reinterpret_cast<float*>(_v)); break;
        case 25: _t->setMinCameraXRotation(*reinterpret_cast<float*>(_v)); break;
        case 26: _t->setMaxCameraXRotation(*reinterpret_cast<float*>(_v)); break;
        case 27: _t->setMinCameraYRotation(*reinterpret_cast<float*>(_v)); break;
        case 28: _t->setMaxCameraYRotation(*reinterpret_cast<float*>(_v)); break;
        case 29: _t->setCameraZoomLevel(*reinterpret_cast<float*>(_v)); break;
        case 30: _t->setCameraPreset(*reinterpret_cast<QtGraphs3D::CameraPreset*>(_v)); break;
        case 31: _t->setCameraTargetPosition(*reinterpret_cast<QVector3D*>(_v)); break;
        case 32: _t->setMinCameraZoomLevel(*reinterpret_cast<float*>(_v)); break;
        case 33: _t->setMaxCameraZoomLevel(*reinterpret_cast<float*>(_v)); break;
        case 34: _t->setWrapCameraXRotation(*reinterpret_cast<bool*>(_v)); break;
        case 35: _t->setWrapCameraYRotation(*reinterpret_cast<bool*>(_v)); break;
        case 36: _t->setRotationEnabled(*reinterpret_cast<bool*>(_v)); break;
        case 37: _t->setPanEnabled(*reinterpret_cast<bool*>(_v)); break;
        case 38: _t->setPanModeEnabled(*reinterpret_cast<bool*>(_v)); break;
        case 39: _t->setZoomAtTargetEnabled(*reinterpret_cast<bool*>(_v)); break;
        case 40: _t->setSelectionEnabled(*reinterpret_cast<bool*>(_v)); break;
        case 41: _t->setZoomEnabled(*reinterpret_cast<bool*>(_v)); break;
        case 42: _t->setLightColor(*reinterpret_cast<QColor*>(_v)); break;
        case 43: _t->setAmbientLightStrength(*reinterpret_cast<float*>(_v)); break;
        case 44: _t->setLightStrength(*reinterpret_cast<float*>(_v)); break;
        case 45: _t->setShadowStrength(*reinterpret_cast<float*>(_v)); break;
        case 46: _t->setGridLineType(*reinterpret_cast<QtGraphs3D::GridLineType*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *QQuickGraphsItem::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QQuickGraphsItem::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16QQuickGraphsItemE_t>.strings))
        return static_cast<void*>(this);
    return QQuick3DViewport::qt_metacast(_clname);
}

int QQuickGraphsItem::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QQuick3DViewport::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 114)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 114;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 114)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 114;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 47;
    }
    return _id;
}

// SIGNAL 0
void QQuickGraphsItem::selectionModeChanged(QtGraphs3D::SelectionFlags _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void QQuickGraphsItem::shadowQualityChanged(QtGraphs3D::ShadowQuality _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void QQuickGraphsItem::shadowsSupportedChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void QQuickGraphsItem::msaaSamplesChanged(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void QQuickGraphsItem::themeChanged(QGraphsTheme * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void QQuickGraphsItem::renderingModeChanged(QtGraphs3D::RenderingMode _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void QQuickGraphsItem::transparencyTechniqueChanged(QtGraphs3D::TransparencyTechnique _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}

// SIGNAL 7
void QQuickGraphsItem::measureFpsChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1);
}

// SIGNAL 8
void QQuickGraphsItem::currentFpsChanged(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 8, nullptr, _t1);
}

// SIGNAL 9
void QQuickGraphsItem::selectedElementChanged(QtGraphs3D::ElementType _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 9, nullptr, _t1);
}

// SIGNAL 10
void QQuickGraphsItem::orthoProjectionChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 10, nullptr, _t1);
}

// SIGNAL 11
void QQuickGraphsItem::aspectRatioChanged(qreal _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 11, nullptr, _t1);
}

// SIGNAL 12
void QQuickGraphsItem::optimizationHintChanged(QtGraphs3D::OptimizationHint _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 12, nullptr, _t1);
}

// SIGNAL 13
void QQuickGraphsItem::polarChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 13, nullptr, _t1);
}

// SIGNAL 14
void QQuickGraphsItem::labelMarginChanged(float _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 14, nullptr, _t1);
}

// SIGNAL 15
void QQuickGraphsItem::radialLabelOffsetChanged(float _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 15, nullptr, _t1);
}

// SIGNAL 16
void QQuickGraphsItem::horizontalAspectRatioChanged(qreal _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 16, nullptr, _t1);
}

// SIGNAL 17
void QQuickGraphsItem::localeChanged(const QLocale & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 17, nullptr, _t1);
}

// SIGNAL 18
void QQuickGraphsItem::queriedGraphPositionChanged(QVector3D _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 18, nullptr, _t1);
}

// SIGNAL 19
void QQuickGraphsItem::marginChanged(qreal _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 19, nullptr, _t1);
}

// SIGNAL 20
void QQuickGraphsItem::cutoffMarginChanged(qreal _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 20, nullptr, _t1);
}

// SIGNAL 21
void QQuickGraphsItem::cameraPresetChanged(QtGraphs3D::CameraPreset _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 21, nullptr, _t1);
}

// SIGNAL 22
void QQuickGraphsItem::cameraXRotationChanged(float _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 22, nullptr, _t1);
}

// SIGNAL 23
void QQuickGraphsItem::cameraYRotationChanged(float _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 23, nullptr, _t1);
}

// SIGNAL 24
void QQuickGraphsItem::cameraZoomLevelChanged(float _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 24, nullptr, _t1);
}

// SIGNAL 25
void QQuickGraphsItem::cameraTargetPositionChanged(QVector3D _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 25, nullptr, _t1);
}

// SIGNAL 26
void QQuickGraphsItem::minCameraZoomLevelChanged(float _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 26, nullptr, _t1);
}

// SIGNAL 27
void QQuickGraphsItem::maxCameraZoomLevelChanged(float _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 27, nullptr, _t1);
}

// SIGNAL 28
void QQuickGraphsItem::minCameraXRotationChanged(float _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 28, nullptr, _t1);
}

// SIGNAL 29
void QQuickGraphsItem::minCameraYRotationChanged(float _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 29, nullptr, _t1);
}

// SIGNAL 30
void QQuickGraphsItem::maxCameraXRotationChanged(float _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 30, nullptr, _t1);
}

// SIGNAL 31
void QQuickGraphsItem::maxCameraYRotationChanged(float _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 31, nullptr, _t1);
}

// SIGNAL 32
void QQuickGraphsItem::wrapCameraXRotationChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 32, nullptr, _t1);
}

// SIGNAL 33
void QQuickGraphsItem::wrapCameraYRotationChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 33, nullptr, _t1);
}

// SIGNAL 34
void QQuickGraphsItem::needRender()
{
    QMetaObject::activate(this, &staticMetaObject, 34, nullptr);
}

// SIGNAL 35
void QQuickGraphsItem::themeTypeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 35, nullptr);
}

// SIGNAL 36
void QQuickGraphsItem::axisXChanged(QAbstract3DAxis * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 36, nullptr, _t1);
}

// SIGNAL 37
void QQuickGraphsItem::axisYChanged(QAbstract3DAxis * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 37, nullptr, _t1);
}

// SIGNAL 38
void QQuickGraphsItem::axisZChanged(QAbstract3DAxis * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 38, nullptr, _t1);
}

// SIGNAL 39
void QQuickGraphsItem::activeThemeChanged(QGraphsTheme * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 39, nullptr, _t1);
}

// SIGNAL 40
void QQuickGraphsItem::tapped(QEventPoint _t1, Qt::MouseButton _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 40, nullptr, _t1, _t2);
}

// SIGNAL 41
void QQuickGraphsItem::doubleTapped(QEventPoint _t1, Qt::MouseButton _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 41, nullptr, _t1, _t2);
}

// SIGNAL 42
void QQuickGraphsItem::longPressed()
{
    QMetaObject::activate(this, &staticMetaObject, 42, nullptr);
}

// SIGNAL 43
void QQuickGraphsItem::dragged(QVector2D _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 43, nullptr, _t1);
}

// SIGNAL 44
void QQuickGraphsItem::wheel(QQuickWheelEvent * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 44, nullptr, _t1);
}

// SIGNAL 45
void QQuickGraphsItem::pinch(qreal _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 45, nullptr, _t1);
}

// SIGNAL 46
void QQuickGraphsItem::mouseMove(QPoint _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 46, nullptr, _t1);
}

// SIGNAL 47
void QQuickGraphsItem::zoomEnabledChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 47, nullptr, _t1);
}

// SIGNAL 48
void QQuickGraphsItem::zoomAtTargetEnabledChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 48, nullptr, _t1);
}

// SIGNAL 49
void QQuickGraphsItem::rotationEnabledChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 49, nullptr, _t1);
}

// SIGNAL 50
void QQuickGraphsItem::panEnabledChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 50, nullptr, _t1);
}

// SIGNAL 51
void QQuickGraphsItem::panModeEnabledChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 51, nullptr, _t1);
}

// SIGNAL 52
void QQuickGraphsItem::selectionEnabledChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 52, nullptr, _t1);
}

// SIGNAL 53
void QQuickGraphsItem::ambientLightStrengthChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 53, nullptr);
}

// SIGNAL 54
void QQuickGraphsItem::lightStrengthChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 54, nullptr);
}

// SIGNAL 55
void QQuickGraphsItem::shadowStrengthChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 55, nullptr);
}

// SIGNAL 56
void QQuickGraphsItem::lightColorChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 56, nullptr);
}

// SIGNAL 57
void QQuickGraphsItem::gridLineTypeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 57, nullptr);
}
namespace CheckNotifySignalValidity_ZN16QQuickGraphsItemE {
template<typename T> using has_nullary_sceneChanged = decltype(std::declval<T>().sceneChanged());
template<typename T> using has_unary_sceneChanged = decltype(std::declval<T>().sceneChanged(std::declval<Q3DScene*>()));
static_assert(qxp::is_detected_v<has_nullary_sceneChanged, QQuickGraphsItem> || qxp::is_detected_v<has_unary_sceneChanged, QQuickGraphsItem>,
              "NOTIFY signal sceneChanged does not exist in class (or is private in its parent)");
}
QT_WARNING_POP
