/****************************************************************************
** Meta object code from reading C++ file 'qbarmodelmapper.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../qtgraphs/src/graphs2d/barchart/qbarmodelmapper.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qbarmodelmapper.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN15QBarModelMapperE_t {};
} // unnamed namespace

template <> constexpr inline auto QBarModelMapper::qt_create_metaobjectdata<qt_meta_tag_ZN15QBarModelMapperE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QBarModelMapper",
        "QML.Element",
        "BarModelMapper",
        "seriesChanged",
        "",
        "modelChanged",
        "firstBarSetSectionChanged",
        "lastBarSetSectionChanged",
        "firstChanged",
        "countChanged",
        "orientationChanged",
        "onValuesAdded",
        "qsizetype",
        "index",
        "count",
        "onBarLabelChanged",
        "onBarValueChanged",
        "series",
        "QBarSeries*",
        "model",
        "QAbstractItemModel*",
        "firstBarSetSection",
        "lastBarSetSection",
        "first",
        "orientation",
        "Qt::Orientation"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'seriesChanged'
        QtMocHelpers::SignalData<void()>(3, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'modelChanged'
        QtMocHelpers::SignalData<void()>(5, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'firstBarSetSectionChanged'
        QtMocHelpers::SignalData<void()>(6, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'lastBarSetSectionChanged'
        QtMocHelpers::SignalData<void()>(7, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'firstChanged'
        QtMocHelpers::SignalData<void()>(8, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'countChanged'
        QtMocHelpers::SignalData<void()>(9, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'orientationChanged'
        QtMocHelpers::SignalData<void()>(10, 4, QMC::AccessPublic, QMetaType::Void),
        // Slot 'onValuesAdded'
        QtMocHelpers::SlotData<void(qsizetype, qsizetype)>(11, 4, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 12, 13 }, { 0x80000000 | 12, 14 },
        }}),
        // Slot 'onBarLabelChanged'
        QtMocHelpers::SlotData<void()>(15, 4, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'onBarValueChanged'
        QtMocHelpers::SlotData<void(qsizetype)>(16, 4, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 12, 13 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'series'
        QtMocHelpers::PropertyData<QBarSeries*>(17, 0x80000000 | 18, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet | QMC::Final, 0),
        // property 'model'
        QtMocHelpers::PropertyData<QAbstractItemModel*>(19, 0x80000000 | 20, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet | QMC::Final, 1),
        // property 'firstBarSetSection'
        QtMocHelpers::PropertyData<qsizetype>(21, 0x80000000 | 12, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet | QMC::Final, 2),
        // property 'lastBarSetSection'
        QtMocHelpers::PropertyData<qsizetype>(22, 0x80000000 | 12, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet | QMC::Final, 3),
        // property 'first'
        QtMocHelpers::PropertyData<qsizetype>(23, 0x80000000 | 12, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet | QMC::Final, 4),
        // property 'count'
        QtMocHelpers::PropertyData<qsizetype>(14, 0x80000000 | 12, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet | QMC::Final, 5),
        // property 'orientation'
        QtMocHelpers::PropertyData<Qt::Orientation>(24, 0x80000000 | 25, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet | QMC::Final, 6),
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
    });
    return QtMocHelpers::metaObjectData<QBarModelMapper, void>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject QBarModelMapper::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15QBarModelMapperE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15QBarModelMapperE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN15QBarModelMapperE_t>.metaTypes,
    nullptr
} };

void QBarModelMapper::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QBarModelMapper *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->seriesChanged(); break;
        case 1: _t->modelChanged(); break;
        case 2: _t->firstBarSetSectionChanged(); break;
        case 3: _t->lastBarSetSectionChanged(); break;
        case 4: _t->firstChanged(); break;
        case 5: _t->countChanged(); break;
        case 6: _t->orientationChanged(); break;
        case 7: _t->onValuesAdded((*reinterpret_cast<std::add_pointer_t<qsizetype>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<qsizetype>>(_a[2]))); break;
        case 8: _t->onBarLabelChanged(); break;
        case 9: _t->onBarValueChanged((*reinterpret_cast<std::add_pointer_t<qsizetype>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (QBarModelMapper::*)()>(_a, &QBarModelMapper::seriesChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (QBarModelMapper::*)()>(_a, &QBarModelMapper::modelChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (QBarModelMapper::*)()>(_a, &QBarModelMapper::firstBarSetSectionChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (QBarModelMapper::*)()>(_a, &QBarModelMapper::lastBarSetSectionChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (QBarModelMapper::*)()>(_a, &QBarModelMapper::firstChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (QBarModelMapper::*)()>(_a, &QBarModelMapper::countChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (QBarModelMapper::*)()>(_a, &QBarModelMapper::orientationChanged, 6))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QBarSeries**>(_v) = _t->series(); break;
        case 1: *reinterpret_cast<QAbstractItemModel**>(_v) = _t->model(); break;
        case 2: *reinterpret_cast<qsizetype*>(_v) = _t->firstBarSetSection(); break;
        case 3: *reinterpret_cast<qsizetype*>(_v) = _t->lastBarSetSection(); break;
        case 4: *reinterpret_cast<qsizetype*>(_v) = _t->first(); break;
        case 5: *reinterpret_cast<qsizetype*>(_v) = _t->count(); break;
        case 6: *reinterpret_cast<Qt::Orientation*>(_v) = _t->orientation(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setSeries(*reinterpret_cast<QBarSeries**>(_v)); break;
        case 1: _t->setModel(*reinterpret_cast<QAbstractItemModel**>(_v)); break;
        case 2: _t->setFirstBarSetSection(*reinterpret_cast<qsizetype*>(_v)); break;
        case 3: _t->setLastBarSetSection(*reinterpret_cast<qsizetype*>(_v)); break;
        case 4: _t->setFirst(*reinterpret_cast<qsizetype*>(_v)); break;
        case 5: _t->setCount(*reinterpret_cast<qsizetype*>(_v)); break;
        case 6: _t->setOrientation(*reinterpret_cast<Qt::Orientation*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *QBarModelMapper::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QBarModelMapper::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15QBarModelMapperE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int QBarModelMapper::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 10)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 10)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 10;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    }
    return _id;
}

// SIGNAL 0
void QBarModelMapper::seriesChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void QBarModelMapper::modelChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void QBarModelMapper::firstBarSetSectionChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void QBarModelMapper::lastBarSetSectionChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void QBarModelMapper::firstChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void QBarModelMapper::countChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void QBarModelMapper::orientationChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}
QT_WARNING_POP
