/****************************************************************************
** Generated QML type registration code
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <QtQml/qqml.h>
#include <QtQml/qqmlmoduleregistration.h>

#if __has_include(<private/qcustomseriesdata_p.h>)
#  include <private/qcustomseriesdata_p.h>
#endif
#if __has_include(<private/qgraphpointanimation_p.h>)
#  include <private/qgraphpointanimation_p.h>
#endif
#if __has_include(<private/qgraphsview_p.h>)
#  include <private/qgraphsview_p.h>
#endif
#if __has_include(<private/qgraphtransition_p.h>)
#  include <private/qgraphtransition_p.h>
#endif
#if __has_include(<private/qquickgraphsbars_p.h>)
#  include <private/qquickgraphsbars_p.h>
#endif
#if __has_include(<private/qquickgraphsbarsnode_p.h>)
#  include <private/qquickgraphsbarsnode_p.h>
#endif
#if __has_include(<private/qquickgraphsbarsseries_p.h>)
#  include <private/qquickgraphsbarsseries_p.h>
#endif
#if __has_include(<private/qquickgraphscolor_p.h>)
#  include <private/qquickgraphscolor_p.h>
#endif
#if __has_include(<private/qquickgraphsitem_p.h>)
#  include <private/qquickgraphsitem_p.h>
#endif
#if __has_include(<private/qquickgraphsnode_p.h>)
#  include <private/qquickgraphsnode_p.h>
#endif
#if __has_include(<private/qquickgraphsscatter_p.h>)
#  include <private/qquickgraphsscatter_p.h>
#endif
#if __has_include(<private/qquickgraphsscatternode_p.h>)
#  include <private/qquickgraphsscatternode_p.h>
#endif
#if __has_include(<private/qquickgraphsscatterseries_p.h>)
#  include <private/qquickgraphsscatterseries_p.h>
#endif
#if __has_include(<private/qquickgraphssplineseries_p.h>)
#  include <private/qquickgraphssplineseries_p.h>
#endif
#if __has_include(<private/qquickgraphssurface_p.h>)
#  include <private/qquickgraphssurface_p.h>
#endif
#if __has_include(<private/qquickgraphssurfacenode_p.h>)
#  include <private/qquickgraphssurfacenode_p.h>
#endif
#if __has_include(<private/qquickgraphssurfaceseries_p.h>)
#  include <private/qquickgraphssurfaceseries_p.h>
#endif
#if __has_include(<private/qsplinecontrolanimation_p.h>)
#  include <private/qsplinecontrolanimation_p.h>
#endif
#if __has_include(<private/qxypoint_p.h>)
#  include <private/qxypoint_p.h>
#endif
#if __has_include(<q3dscene.h>)
#  include <q3dscene.h>
#endif
#if __has_include(<qabstract3daxis.h>)
#  include <qabstract3daxis.h>
#endif
#if __has_include(<qabstract3dseries.h>)
#  include <qabstract3dseries.h>
#endif
#if __has_include(<qabstractaxis.h>)
#  include <qabstractaxis.h>
#endif
#if __has_include(<qabstractdataproxy.h>)
#  include <qabstractdataproxy.h>
#endif
#if __has_include(<qabstractseries.h>)
#  include <qabstractseries.h>
#endif
#if __has_include(<qareaseries.h>)
#  include <qareaseries.h>
#endif
#if __has_include(<qbar3dseries.h>)
#  include <qbar3dseries.h>
#endif
#if __has_include(<qbarcategoryaxis.h>)
#  include <qbarcategoryaxis.h>
#endif
#if __has_include(<qbardataproxy.h>)
#  include <qbardataproxy.h>
#endif
#if __has_include(<qbarmodelmapper.h>)
#  include <qbarmodelmapper.h>
#endif
#if __has_include(<qbarseries.h>)
#  include <qbarseries.h>
#endif
#if __has_include(<qbarset.h>)
#  include <qbarset.h>
#endif
#if __has_include(<qcategory3daxis.h>)
#  include <qcategory3daxis.h>
#endif
#if __has_include(<qcustom3ditem.h>)
#  include <qcustom3ditem.h>
#endif
#if __has_include(<qcustom3dlabel.h>)
#  include <qcustom3dlabel.h>
#endif
#if __has_include(<qcustom3dvolume.h>)
#  include <qcustom3dvolume.h>
#endif
#if __has_include(<qcustomseries.h>)
#  include <qcustomseries.h>
#endif
#if __has_include(<qdatetimeaxis.h>)
#  include <qdatetimeaxis.h>
#endif
#if __has_include(<qgraphs3dnamespace.h>)
#  include <qgraphs3dnamespace.h>
#endif
#if __has_include(<qgraphstheme.h>)
#  include <qgraphstheme.h>
#endif
#if __has_include(<qheightmapsurfacedataproxy.h>)
#  include <qheightmapsurfacedataproxy.h>
#endif
#if __has_include(<qitemmodelbardataproxy.h>)
#  include <qitemmodelbardataproxy.h>
#endif
#if __has_include(<qitemmodelscatterdataproxy.h>)
#  include <qitemmodelscatterdataproxy.h>
#endif
#if __has_include(<qitemmodelsurfacedataproxy.h>)
#  include <qitemmodelsurfacedataproxy.h>
#endif
#if __has_include(<qlineseries.h>)
#  include <qlineseries.h>
#endif
#if __has_include(<qlogvalue3daxisformatter.h>)
#  include <qlogvalue3daxisformatter.h>
#endif
#if __has_include(<qlogvalueaxis.h>)
#  include <qlogvalueaxis.h>
#endif
#if __has_include(<qpiemodelmapper.h>)
#  include <qpiemodelmapper.h>
#endif
#if __has_include(<qpieseries.h>)
#  include <qpieseries.h>
#endif
#if __has_include(<qpieslice.h>)
#  include <qpieslice.h>
#endif
#if __has_include(<qscatter3dseries.h>)
#  include <qscatter3dseries.h>
#endif
#if __has_include(<qscatterdataproxy.h>)
#  include <qscatterdataproxy.h>
#endif
#if __has_include(<qscatterseries.h>)
#  include <qscatterseries.h>
#endif
#if __has_include(<qspline3dseries.h>)
#  include <qspline3dseries.h>
#endif
#if __has_include(<qsplineseries.h>)
#  include <qsplineseries.h>
#endif
#if __has_include(<qsurface3dseries.h>)
#  include <qsurface3dseries.h>
#endif
#if __has_include(<qsurfacedataproxy.h>)
#  include <qsurfacedataproxy.h>
#endif
#if __has_include(<qvalue3daxis.h>)
#  include <qvalue3daxis.h>
#endif
#if __has_include(<qvalue3daxisformatter.h>)
#  include <qvalue3daxisformatter.h>
#endif
#if __has_include(<qvalueaxis.h>)
#  include <qvalueaxis.h>
#endif
#if __has_include(<qxymodelmapper.h>)
#  include <qxymodelmapper.h>
#endif
#if __has_include(<qxyseries.h>)
#  include <qxyseries.h>
#endif


#if !defined(QT_STATIC)
#define Q_QMLTYPE_EXPORT Q_DECL_EXPORT
#else
#define Q_QMLTYPE_EXPORT
#endif
Q_QMLTYPE_EXPORT void qml_register_types_QtGraphs()
{
    qmlRegisterModule("QtGraphs", 1, 0);
    qmlRegisterModule("QtGraphs", 1, 254);
    qmlRegisterModule("QtGraphs", 6, 0);
    QT_WARNING_PUSH QT_WARNING_DISABLE_DEPRECATED
    qmlRegisterTypesAndRevisions<Q3DScene>("QtGraphs", 6);
    qmlRegisterTypesAndRevisions<QAbstract3DAxis>("QtGraphs", 6);
    qmlRegisterEnum<QAbstract3DAxis::AxisOrientation>("QAbstract3DAxis::AxisOrientation");
    qmlRegisterEnum<QAbstract3DAxis::AxisType>("QAbstract3DAxis::AxisType");
    qmlRegisterTypesAndRevisions<QAbstract3DSeries>("QtGraphs", 6);
    qmlRegisterEnum<QAbstract3DSeries::SeriesType>("QAbstract3DSeries::SeriesType");
    qmlRegisterEnum<QAbstract3DSeries::Mesh>("QAbstract3DSeries::Mesh");
    qmlRegisterEnum<QAbstract3DSeries::LightingMode>("QAbstract3DSeries::LightingMode");
    QMetaType::fromType<QAbstractAnimation *>().id();
    qmlRegisterEnum<QAbstractAnimation::Direction>("QAbstractAnimation::Direction");
    qmlRegisterEnum<QAbstractAnimation::State>("QAbstractAnimation::State");
    qmlRegisterTypesAndRevisions<QAbstractAxis>("QtGraphs", 6);
    qmlRegisterEnum<QAbstractAxis::AxisType>("QAbstractAxis::AxisType");
    qmlRegisterTypesAndRevisions<QAbstractDataProxy>("QtGraphs", 6);
    qmlRegisterEnum<QAbstractDataProxy::DataType>("QAbstractDataProxy::DataType");
    qmlRegisterTypesAndRevisions<QAbstractSeries>("QtGraphs", 6);
    qmlRegisterEnum<QAbstractSeries::SeriesType>("QAbstractSeries::SeriesType");
    qmlRegisterTypesAndRevisions<QAreaSeries>("QtGraphs", 6);
    qmlRegisterTypesAndRevisions<QBar3DSeries>("QtGraphs", 6);
    qmlRegisterTypesAndRevisions<QBarCategoryAxis>("QtGraphs", 6);
    qmlRegisterEnum<QBarCategoryAxis::LabelPosition>("QBarCategoryAxis::LabelPosition");
    qmlRegisterTypesAndRevisions<QBarDataProxy>("QtGraphs", 6);
    qmlRegisterEnum<QBarDataProxy::RemoveLabels>("QBarDataProxy::RemoveLabels");
    qmlRegisterTypesAndRevisions<QBarModelMapper>("QtGraphs", 6);
    qmlRegisterTypesAndRevisions<QBarSeries>("QtGraphs", 6);
    qmlRegisterEnum<QBarSeries::LabelsPosition>("QBarSeries::LabelsPosition");
    qmlRegisterEnum<QBarSeries::BarsType>("QBarSeries::BarsType");
    qmlRegisterTypesAndRevisions<QBarSet>("QtGraphs", 6);
    qmlRegisterTypesAndRevisions<QCategory3DAxis>("QtGraphs", 6);
    qmlRegisterTypesAndRevisions<QCustom3DItem>("QtGraphs", 6);
    qmlRegisterTypesAndRevisions<QCustom3DLabel>("QtGraphs", 6);
    qmlRegisterTypesAndRevisions<QCustom3DVolume>("QtGraphs", 6);
    qmlRegisterTypesAndRevisions<QCustomSeries>("QtGraphs", 6);
    qmlRegisterTypesAndRevisions<QCustomSeriesData>("QtGraphs", 6);
    qmlRegisterTypesAndRevisions<QDateTimeAxis>("QtGraphs", 6);
    QMetaType::fromType<QGraphAnimation *>().id();
    qmlRegisterEnum<QGraphAnimation::AnimationState>("QGraphAnimation::AnimationState");
    qmlRegisterEnum<QGraphAnimation::GraphAnimationType>("QGraphAnimation::GraphAnimationType");
    qmlRegisterTypesAndRevisions<QGraphPointAnimation>("QtGraphs", 6);
    qmlRegisterTypesAndRevisions<QGraphTransition>("QtGraphs", 6);
    qmlRegisterEnum<QGraphTransition::TransitionType>("QGraphTransition::TransitionType");
    qmlRegisterTypesAndRevisions<QGraphsLine>("QtGraphs", 6);
    QMetaType::fromType<QGraphsLine *>().id();
    qmlRegisterTypesAndRevisions<QGraphsTheme>("QtGraphs", 6);
    qmlRegisterEnum<QGraphsTheme::Theme>("QGraphsTheme::Theme");
    qmlRegisterEnum<QGraphsTheme::ColorStyle>("QGraphsTheme::ColorStyle");
    qmlRegisterEnum<QGraphsTheme::ForceTheme>("QGraphsTheme::ForceTheme");
    qmlRegisterEnum<QGraphsTheme::ColorScheme>("QGraphsTheme::ColorScheme");
    qmlRegisterTypesAndRevisions<QGraphsView>("QtGraphs", 6);
    qmlRegisterAnonymousTypesAndRevisions<QQuickItem>("QtGraphs", 6);
    qmlRegisterEnum<QGraphsView::ZoomStyle>("QGraphsView::ZoomStyle");
    qmlRegisterEnum<QGraphsView::PanStyle>("QGraphsView::PanStyle");
    qmlRegisterTypesAndRevisions<QHeightMapSurfaceDataProxy>("QtGraphs", 6);
    qmlRegisterTypesAndRevisions<QItemModelBarDataProxy>("QtGraphs", 6);
    qmlRegisterEnum<QItemModelBarDataProxy::MultiMatchBehavior>("QItemModelBarDataProxy::MultiMatchBehavior");
    qmlRegisterTypesAndRevisions<QItemModelScatterDataProxy>("QtGraphs", 6);
    qmlRegisterTypesAndRevisions<QItemModelSurfaceDataProxy>("QtGraphs", 6);
    qmlRegisterEnum<QItemModelSurfaceDataProxy::MultiMatchBehavior>("QItemModelSurfaceDataProxy::MultiMatchBehavior");
    qmlRegisterTypesAndRevisions<QLegendData>("QtGraphs", 6);
    QMetaType::fromType<QLegendData *>().id();
    qmlRegisterTypesAndRevisions<QLineSeries>("QtGraphs", 6);
    qmlRegisterEnum<QLineSeries::LineStyle>("QLineSeries::LineStyle");
    qmlRegisterEnum<QLineSeries::StrokeStyle>("QLineSeries::StrokeStyle");
    qmlRegisterTypesAndRevisions<QLogValue3DAxisFormatter>("QtGraphs", 6);
    qmlRegisterTypesAndRevisions<QLogValueAxis>("QtGraphs", 6);
    qmlRegisterTypesAndRevisions<QPieModelMapper>("QtGraphs", 6);
    qmlRegisterTypesAndRevisions<QPieSeries>("QtGraphs", 6);
    qmlRegisterEnum<QPieSeries::LabelVisibility>("QPieSeries::LabelVisibility");
    qmlRegisterTypesAndRevisions<QPieSlice>("QtGraphs", 6);
    qmlRegisterEnum<QPieSlice::LabelPosition>("QPieSlice::LabelPosition");
    qmlRegisterTypesAndRevisions<QQuickGraphsBar3DSeries>("QtGraphs", 6);
    qmlRegisterTypesAndRevisions<QQuickGraphsBars>("QtGraphs", 6);
    qmlRegisterAnonymousTypesAndRevisions<QQuick3DViewport>("QtGraphs", 6);
    qmlRegisterTypesAndRevisions<QQuickGraphsBarsNode>("QtGraphs", 6);
    qmlRegisterAnonymousTypesAndRevisions<QQuick3DNode>("QtGraphs", 6);
    qmlRegisterTypesAndRevisions<QQuickGraphsColor>("QtGraphs", 6);
    qmlRegisterTypesAndRevisions<QQuickGraphsItem>("QtGraphs", 6);
    qmlRegisterTypesAndRevisions<QQuickGraphsNode>("QtGraphs", 6);
    qmlRegisterTypesAndRevisions<QQuickGraphsScatter>("QtGraphs", 6);
    qmlRegisterTypesAndRevisions<QQuickGraphsScatter3DSeries>("QtGraphs", 6);
    qmlRegisterTypesAndRevisions<QQuickGraphsScatterNode>("QtGraphs", 6);
    qmlRegisterTypesAndRevisions<QQuickGraphsSpline3DSeries>("QtGraphs", 6);
    qmlRegisterTypesAndRevisions<QQuickGraphsSurface>("QtGraphs", 6);
    qmlRegisterTypesAndRevisions<QQuickGraphsSurface3DSeries>("QtGraphs", 6);
    qmlRegisterTypesAndRevisions<QQuickGraphsSurfaceNode>("QtGraphs", 6);
    qmlRegisterTypesAndRevisions<QScatter3DSeries>("QtGraphs", 6);
    qmlRegisterTypesAndRevisions<QScatterDataProxy>("QtGraphs", 6);
    qmlRegisterTypesAndRevisions<QScatterSeries>("QtGraphs", 6);
    qmlRegisterTypesAndRevisions<QSpline3DSeries>("QtGraphs", 6);
    qmlRegisterTypesAndRevisions<QSplineControlAnimation>("QtGraphs", 6);
    qmlRegisterTypesAndRevisions<QSplineSeries>("QtGraphs", 6);
    qmlRegisterTypesAndRevisions<QSurface3DSeries>("QtGraphs", 6);
    qmlRegisterEnum<QSurface3DSeries::DrawFlag>("QSurface3DSeries::DrawFlag");
    qmlRegisterEnum<QSurface3DSeries::DrawFlags>("QSurface3DSeries::DrawFlags");
    qmlRegisterEnum<QSurface3DSeries::DrawFlag>("QSurface3DSeries::DrawFlag");
    qmlRegisterEnum<QSurface3DSeries::Shading>("QSurface3DSeries::Shading");
    qmlRegisterTypesAndRevisions<QSurfaceDataProxy>("QtGraphs", 6);
    qmlRegisterTypesAndRevisions<QValue3DAxis>("QtGraphs", 6);
    qmlRegisterTypesAndRevisions<QValue3DAxisFormatter>("QtGraphs", 6);
    qmlRegisterTypesAndRevisions<QValueAxis>("QtGraphs", 6);
    QMetaType::fromType<QVariantAnimation *>().id();
    qmlRegisterTypesAndRevisions<QXYModelMapper>("QtGraphs", 6);
    qmlRegisterTypesAndRevisions<QXYPoint>("QtGraphs", 6);
    qmlRegisterTypesAndRevisions<QXYSeries>("QtGraphs", 6);
    qmlRegisterEnum<QXYSeries::DeclarativePointHint>("QXYSeries::DeclarativePointHint");
    QMetaType::fromType<QXYSeriesAnimation *>().id();
    {
        Q_CONSTINIT static auto metaType = QQmlPrivate::metaTypeForNamespace(
            [](const QtPrivate::QMetaTypeInterface *) {return &QtGraphs3D::staticMetaObject;},
            "QtGraphs3D");
        QMetaType(&metaType).id();
    }
    qmlRegisterNamespaceAndRevisions(&QtGraphs3D::staticMetaObject, "QtGraphs", 6, nullptr, &QtGraphs3D::staticMetaObject, nullptr);
    qmlRegisterEnum<QtGraphs3D::SelectionFlag>("QtGraphs3D::SelectionFlag");
    qmlRegisterEnum<QtGraphs3D::SelectionFlags>("QtGraphs3D::SelectionFlags");
    qmlRegisterEnum<QtGraphs3D::SelectionFlag>("QtGraphs3D::SelectionFlag");
    qmlRegisterEnum<QtGraphs3D::SliceCaptureType>("QtGraphs3D::SliceCaptureType");
    qmlRegisterEnum<QtGraphs3D::ShadowQuality>("QtGraphs3D::ShadowQuality");
    qmlRegisterEnum<QtGraphs3D::ElementType>("QtGraphs3D::ElementType");
    qmlRegisterEnum<QtGraphs3D::OptimizationHint>("QtGraphs3D::OptimizationHint");
    qmlRegisterEnum<QtGraphs3D::RenderingMode>("QtGraphs3D::RenderingMode");
    qmlRegisterEnum<QtGraphs3D::CameraPreset>("QtGraphs3D::CameraPreset");
    qmlRegisterEnum<QtGraphs3D::GridLineType>("QtGraphs3D::GridLineType");
    qmlRegisterEnum<QtGraphs3D::TransparencyTechnique>("QtGraphs3D::TransparencyTechnique");
    QT_WARNING_POP
    qmlRegisterModule("QtGraphs", 6, 12);
}

static const QQmlModuleRegistration qtGraphsRegistration("QtGraphs", qml_register_types_QtGraphs);
