/****************************************************************************
** Meta object code from reading C++ file 'qgraphstheme.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../qtgraphs/src/common/theme/qgraphstheme.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qgraphstheme.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN11QGraphsLineE_t {};
} // unnamed namespace

template <> constexpr inline auto QGraphsLine::qt_create_metaobjectdata<qt_meta_tag_ZN11QGraphsLineE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QGraphsLine",
        "QML.Element",
        "graphsline",
        "QML.Creatable",
        "true",
        "QML.CreationMethod",
        "structured",
        "mainColor",
        "QColor",
        "subColor",
        "mainWidth",
        "subWidth",
        "labelTextColor"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
        // property 'mainColor'
        QtMocHelpers::PropertyData<QColor>(7, 0x80000000 | 8, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet | QMC::Final),
        // property 'subColor'
        QtMocHelpers::PropertyData<QColor>(9, 0x80000000 | 8, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet | QMC::Final),
        // property 'mainWidth'
        QtMocHelpers::PropertyData<qreal>(10, QMetaType::QReal, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final),
        // property 'subWidth'
        QtMocHelpers::PropertyData<qreal>(11, QMetaType::QReal, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final),
        // property 'labelTextColor'
        QtMocHelpers::PropertyData<QColor>(12, 0x80000000 | 8, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet | QMC::Final),
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
            {    5,    6 },
    });
    return QtMocHelpers::metaObjectData<QGraphsLine, void>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject QGraphsLine::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11QGraphsLineE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11QGraphsLineE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN11QGraphsLineE_t>.metaTypes,
    nullptr
} };

void QGraphsLine::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = reinterpret_cast<QGraphsLine *>(_o);
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QColor*>(_v) = _t->mainColor(); break;
        case 1: *reinterpret_cast<QColor*>(_v) = _t->subColor(); break;
        case 2: *reinterpret_cast<qreal*>(_v) = _t->mainWidth(); break;
        case 3: *reinterpret_cast<qreal*>(_v) = _t->subWidth(); break;
        case 4: *reinterpret_cast<QColor*>(_v) = _t->labelTextColor(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setMainColor(*reinterpret_cast<QColor*>(_v)); break;
        case 1: _t->setSubColor(*reinterpret_cast<QColor*>(_v)); break;
        case 2: _t->setMainWidth(*reinterpret_cast<qreal*>(_v)); break;
        case 3: _t->setSubWidth(*reinterpret_cast<qreal*>(_v)); break;
        case 4: _t->setLabelTextColor(*reinterpret_cast<QColor*>(_v)); break;
        default: break;
        }
    }
}
namespace {
struct qt_meta_tag_ZN12QGraphsThemeE_t {};
} // unnamed namespace

template <> constexpr inline auto QGraphsTheme::qt_create_metaobjectdata<qt_meta_tag_ZN12QGraphsThemeE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QGraphsTheme",
        "RegisterEnumClassesUnscoped",
        "false",
        "DefaultProperty",
        "themeChildren",
        "QML.Element",
        "GraphsTheme",
        "update",
        "",
        "colorSchemeChanged",
        "themeChanged",
        "QGraphsTheme::Theme",
        "theme",
        "colorStyleChanged",
        "QGraphsTheme::ColorStyle",
        "type",
        "backgroundColorChanged",
        "backgroundVisibleChanged",
        "plotAreaBackgroundColorChanged",
        "plotAreaBackgroundVisibleChanged",
        "gridVisibleChanged",
        "labelsVisibleChanged",
        "labelBackgroundColorChanged",
        "labelTextColorChanged",
        "singleHighlightColorChanged",
        "QColor",
        "color",
        "multiHighlightColorChanged",
        "singleHighlightGradientChanged",
        "QLinearGradient",
        "gradient",
        "multiHighlightGradientChanged",
        "labelFontChanged",
        "labelBackgroundVisibleChanged",
        "labelBorderVisibleChanged",
        "seriesColorsChanged",
        "QList<QColor>",
        "list",
        "seriesGradientsChanged",
        "QList<QLinearGradient>",
        "borderColorsChanged",
        "borderWidthChanged",
        "singleHighlightGradientQMLChanged",
        "multiHighlightGradientQMLChanged",
        "axisXLabelFontChanged",
        "axisYLabelFontChanged",
        "axisZLabelFontChanged",
        "gridChanged",
        "axisXChanged",
        "axisYChanged",
        "axisZChanged",
        "handleBaseColorUpdate",
        "handleBaseGradientUpdate",
        "colorScheme",
        "QGraphsTheme::ColorScheme",
        "colorStyle",
        "backgroundColor",
        "backgroundVisible",
        "plotAreaBackgroundColor",
        "plotAreaBackgroundVisible",
        "gridVisible",
        "axisXLabelFont",
        "QFont",
        "axisYLabelFont",
        "axisZLabelFont",
        "grid",
        "QGraphsLine",
        "axisX",
        "axisY",
        "axisZ",
        "labelFont",
        "labelsVisible",
        "labelBackgroundColor",
        "labelTextColor",
        "labelBackgroundVisible",
        "labelBorderVisible",
        "seriesColors",
        "borderColors",
        "borderWidth",
        "baseColors",
        "QQmlListProperty<QQuickGraphsColor>",
        "baseGradients",
        "QQmlListProperty<QQuickGradient>",
        "QQmlListProperty<QObject>",
        "singleHighlightColor",
        "multiHighlightColor",
        "singleHighlightGradient",
        "QQuickGradient*",
        "multiHighlightGradient",
        "Theme",
        "QtGreen",
        "QtGreenNeon",
        "MixSeries",
        "OrangeSeries",
        "YellowSeries",
        "BlueSeries",
        "PurpleSeries",
        "GreySeries",
        "UserDefined",
        "ColorStyle",
        "Uniform",
        "ObjectGradient",
        "RangeGradient",
        "ForceTheme",
        "No",
        "Yes",
        "ColorScheme",
        "Automatic",
        "Light",
        "Dark"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'update'
        QtMocHelpers::SignalData<void()>(7, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'colorSchemeChanged'
        QtMocHelpers::SignalData<void()>(9, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'themeChanged'
        QtMocHelpers::SignalData<void(QGraphsTheme::Theme)>(10, 8, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 11, 12 },
        }}),
        // Signal 'colorStyleChanged'
        QtMocHelpers::SignalData<void(QGraphsTheme::ColorStyle)>(13, 8, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 14, 15 },
        }}),
        // Signal 'backgroundColorChanged'
        QtMocHelpers::SignalData<void()>(16, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'backgroundVisibleChanged'
        QtMocHelpers::SignalData<void()>(17, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'plotAreaBackgroundColorChanged'
        QtMocHelpers::SignalData<void()>(18, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'plotAreaBackgroundVisibleChanged'
        QtMocHelpers::SignalData<void()>(19, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'gridVisibleChanged'
        QtMocHelpers::SignalData<void()>(20, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'labelsVisibleChanged'
        QtMocHelpers::SignalData<void()>(21, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'labelBackgroundColorChanged'
        QtMocHelpers::SignalData<void()>(22, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'labelTextColorChanged'
        QtMocHelpers::SignalData<void()>(23, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'singleHighlightColorChanged'
        QtMocHelpers::SignalData<void(QColor)>(24, 8, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 25, 26 },
        }}),
        // Signal 'multiHighlightColorChanged'
        QtMocHelpers::SignalData<void(QColor)>(27, 8, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 25, 26 },
        }}),
        // Signal 'singleHighlightGradientChanged'
        QtMocHelpers::SignalData<void(const QLinearGradient &)>(28, 8, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 29, 30 },
        }}),
        // Signal 'multiHighlightGradientChanged'
        QtMocHelpers::SignalData<void(const QLinearGradient &)>(31, 8, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 29, 30 },
        }}),
        // Signal 'labelFontChanged'
        QtMocHelpers::SignalData<void()>(32, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'labelBackgroundVisibleChanged'
        QtMocHelpers::SignalData<void()>(33, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'labelBorderVisibleChanged'
        QtMocHelpers::SignalData<void()>(34, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'seriesColorsChanged'
        QtMocHelpers::SignalData<void(const QList<QColor> &)>(35, 8, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 36, 37 },
        }}),
        // Signal 'seriesGradientsChanged'
        QtMocHelpers::SignalData<void(const QList<QLinearGradient> &)>(38, 8, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 39, 37 },
        }}),
        // Signal 'borderColorsChanged'
        QtMocHelpers::SignalData<void()>(40, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'borderWidthChanged'
        QtMocHelpers::SignalData<void()>(41, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'singleHighlightGradientQMLChanged'
        QtMocHelpers::SignalData<void()>(42, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'multiHighlightGradientQMLChanged'
        QtMocHelpers::SignalData<void()>(43, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'axisXLabelFontChanged'
        QtMocHelpers::SignalData<void()>(44, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'axisYLabelFontChanged'
        QtMocHelpers::SignalData<void()>(45, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'axisZLabelFontChanged'
        QtMocHelpers::SignalData<void()>(46, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'gridChanged'
        QtMocHelpers::SignalData<void()>(47, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'axisXChanged'
        QtMocHelpers::SignalData<void()>(48, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'axisYChanged'
        QtMocHelpers::SignalData<void()>(49, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'axisZChanged'
        QtMocHelpers::SignalData<void()>(50, 8, QMC::AccessPublic, QMetaType::Void),
        // Slot 'handleBaseColorUpdate'
        QtMocHelpers::SlotData<void()>(51, 8, QMC::AccessPublic, QMetaType::Void),
        // Slot 'handleBaseGradientUpdate'
        QtMocHelpers::SlotData<void()>(52, 8, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'colorScheme'
        QtMocHelpers::PropertyData<QGraphsTheme::ColorScheme>(53, 0x80000000 | 54, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet | QMC::Final, 1),
        // property 'theme'
        QtMocHelpers::PropertyData<QGraphsTheme::Theme>(12, 0x80000000 | 11, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet | QMC::Final, 2),
        // property 'colorStyle'
        QtMocHelpers::PropertyData<QGraphsTheme::ColorStyle>(55, 0x80000000 | 14, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet | QMC::Final, 3),
        // property 'backgroundColor'
        QtMocHelpers::PropertyData<QColor>(56, 0x80000000 | 25, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet | QMC::Final, 4),
        // property 'backgroundVisible'
        QtMocHelpers::PropertyData<bool>(57, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 5),
        // property 'plotAreaBackgroundColor'
        QtMocHelpers::PropertyData<QColor>(58, 0x80000000 | 25, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet | QMC::Final, 6),
        // property 'plotAreaBackgroundVisible'
        QtMocHelpers::PropertyData<bool>(59, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 7),
        // property 'gridVisible'
        QtMocHelpers::PropertyData<bool>(60, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 8),
        // property 'axisXLabelFont'
        QtMocHelpers::PropertyData<QFont>(61, 0x80000000 | 62, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet | QMC::Final, 25),
        // property 'axisYLabelFont'
        QtMocHelpers::PropertyData<QFont>(63, 0x80000000 | 62, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet | QMC::Final, 26),
        // property 'axisZLabelFont'
        QtMocHelpers::PropertyData<QFont>(64, 0x80000000 | 62, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet | QMC::Final, 27),
        // property 'grid'
        QtMocHelpers::PropertyData<QGraphsLine>(65, 0x80000000 | 66, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet | QMC::Final, 28),
        // property 'axisX'
        QtMocHelpers::PropertyData<QGraphsLine>(67, 0x80000000 | 66, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet | QMC::Final, 29),
        // property 'axisY'
        QtMocHelpers::PropertyData<QGraphsLine>(68, 0x80000000 | 66, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet | QMC::Final, 30),
        // property 'axisZ'
        QtMocHelpers::PropertyData<QGraphsLine>(69, 0x80000000 | 66, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet | QMC::Final, 31),
        // property 'labelFont'
        QtMocHelpers::PropertyData<QFont>(70, 0x80000000 | 62, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet | QMC::Final, 16),
        // property 'labelsVisible'
        QtMocHelpers::PropertyData<bool>(71, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 9),
        // property 'labelBackgroundColor'
        QtMocHelpers::PropertyData<QColor>(72, 0x80000000 | 25, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet | QMC::Final, 10),
        // property 'labelTextColor'
        QtMocHelpers::PropertyData<QColor>(73, 0x80000000 | 25, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet | QMC::Final, 11),
        // property 'labelBackgroundVisible'
        QtMocHelpers::PropertyData<bool>(74, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 17),
        // property 'labelBorderVisible'
        QtMocHelpers::PropertyData<bool>(75, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 18),
        // property 'seriesColors'
        QtMocHelpers::PropertyData<QList<QColor>>(76, 0x80000000 | 36, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet | QMC::Final, 19),
        // property 'borderColors'
        QtMocHelpers::PropertyData<QList<QColor>>(77, 0x80000000 | 36, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet | QMC::Final, 21),
        // property 'borderWidth'
        QtMocHelpers::PropertyData<qreal>(78, QMetaType::QReal, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 22),
        // property 'baseColors'
        QtMocHelpers::PropertyData<QQmlListProperty<QQuickGraphsColor>>(79, 0x80000000 | 80, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant | QMC::Final),
        // property 'baseGradients'
        QtMocHelpers::PropertyData<QQmlListProperty<QQuickGradient>>(81, 0x80000000 | 82, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant | QMC::Final),
        // property 'themeChildren'
        QtMocHelpers::PropertyData<QQmlListProperty<QObject>>(4, 0x80000000 | 83, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant | QMC::Final),
        // property 'singleHighlightColor'
        QtMocHelpers::PropertyData<QColor>(84, 0x80000000 | 25, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet | QMC::Final, 12),
        // property 'multiHighlightColor'
        QtMocHelpers::PropertyData<QColor>(85, 0x80000000 | 25, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet | QMC::Final, 13),
        // property 'singleHighlightGradient'
        QtMocHelpers::PropertyData<QQuickGradient*>(86, 0x80000000 | 87, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::Final, 23),
        // property 'multiHighlightGradient'
        QtMocHelpers::PropertyData<QQuickGradient*>(88, 0x80000000 | 87, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::Final, 24),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'Theme'
        QtMocHelpers::EnumData<enum Theme>(89, 89, QMC::EnumIsScoped).add({
            {   90, Theme::QtGreen },
            {   91, Theme::QtGreenNeon },
            {   92, Theme::MixSeries },
            {   93, Theme::OrangeSeries },
            {   94, Theme::YellowSeries },
            {   95, Theme::BlueSeries },
            {   96, Theme::PurpleSeries },
            {   97, Theme::GreySeries },
            {   98, Theme::UserDefined },
        }),
        // enum 'ColorStyle'
        QtMocHelpers::EnumData<enum ColorStyle>(99, 99, QMC::EnumIsScoped).add({
            {  100, ColorStyle::Uniform },
            {  101, ColorStyle::ObjectGradient },
            {  102, ColorStyle::RangeGradient },
        }),
        // enum 'ForceTheme'
        QtMocHelpers::EnumData<enum ForceTheme>(103, 103, QMC::EnumIsScoped).add({
            {  104, ForceTheme::No },
            {  105, ForceTheme::Yes },
        }),
        // enum 'ColorScheme'
        QtMocHelpers::EnumData<enum ColorScheme>(106, 106, QMC::EnumIsScoped).add({
            {  107, ColorScheme::Automatic },
            {  108, ColorScheme::Light },
            {  109, ColorScheme::Dark },
        }),
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
            {    5,    6 },
    });
    return QtMocHelpers::metaObjectData<QGraphsTheme, void>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject QGraphsTheme::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12QGraphsThemeE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12QGraphsThemeE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN12QGraphsThemeE_t>.metaTypes,
    nullptr
} };

void QGraphsTheme::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QGraphsTheme *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->update(); break;
        case 1: _t->colorSchemeChanged(); break;
        case 2: _t->themeChanged((*reinterpret_cast<std::add_pointer_t<QGraphsTheme::Theme>>(_a[1]))); break;
        case 3: _t->colorStyleChanged((*reinterpret_cast<std::add_pointer_t<QGraphsTheme::ColorStyle>>(_a[1]))); break;
        case 4: _t->backgroundColorChanged(); break;
        case 5: _t->backgroundVisibleChanged(); break;
        case 6: _t->plotAreaBackgroundColorChanged(); break;
        case 7: _t->plotAreaBackgroundVisibleChanged(); break;
        case 8: _t->gridVisibleChanged(); break;
        case 9: _t->labelsVisibleChanged(); break;
        case 10: _t->labelBackgroundColorChanged(); break;
        case 11: _t->labelTextColorChanged(); break;
        case 12: _t->singleHighlightColorChanged((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        case 13: _t->multiHighlightColorChanged((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        case 14: _t->singleHighlightGradientChanged((*reinterpret_cast<std::add_pointer_t<QLinearGradient>>(_a[1]))); break;
        case 15: _t->multiHighlightGradientChanged((*reinterpret_cast<std::add_pointer_t<QLinearGradient>>(_a[1]))); break;
        case 16: _t->labelFontChanged(); break;
        case 17: _t->labelBackgroundVisibleChanged(); break;
        case 18: _t->labelBorderVisibleChanged(); break;
        case 19: _t->seriesColorsChanged((*reinterpret_cast<std::add_pointer_t<QList<QColor>>>(_a[1]))); break;
        case 20: _t->seriesGradientsChanged((*reinterpret_cast<std::add_pointer_t<QList<QLinearGradient>>>(_a[1]))); break;
        case 21: _t->borderColorsChanged(); break;
        case 22: _t->borderWidthChanged(); break;
        case 23: _t->singleHighlightGradientQMLChanged(); break;
        case 24: _t->multiHighlightGradientQMLChanged(); break;
        case 25: _t->axisXLabelFontChanged(); break;
        case 26: _t->axisYLabelFontChanged(); break;
        case 27: _t->axisZLabelFontChanged(); break;
        case 28: _t->gridChanged(); break;
        case 29: _t->axisXChanged(); break;
        case 30: _t->axisYChanged(); break;
        case 31: _t->axisZChanged(); break;
        case 32: _t->handleBaseColorUpdate(); break;
        case 33: _t->handleBaseGradientUpdate(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (QGraphsTheme::*)()>(_a, &QGraphsTheme::update, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (QGraphsTheme::*)()>(_a, &QGraphsTheme::colorSchemeChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (QGraphsTheme::*)(QGraphsTheme::Theme )>(_a, &QGraphsTheme::themeChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (QGraphsTheme::*)(QGraphsTheme::ColorStyle )>(_a, &QGraphsTheme::colorStyleChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (QGraphsTheme::*)()>(_a, &QGraphsTheme::backgroundColorChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (QGraphsTheme::*)()>(_a, &QGraphsTheme::backgroundVisibleChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (QGraphsTheme::*)()>(_a, &QGraphsTheme::plotAreaBackgroundColorChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (QGraphsTheme::*)()>(_a, &QGraphsTheme::plotAreaBackgroundVisibleChanged, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (QGraphsTheme::*)()>(_a, &QGraphsTheme::gridVisibleChanged, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (QGraphsTheme::*)()>(_a, &QGraphsTheme::labelsVisibleChanged, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (QGraphsTheme::*)()>(_a, &QGraphsTheme::labelBackgroundColorChanged, 10))
            return;
        if (QtMocHelpers::indexOfMethod<void (QGraphsTheme::*)()>(_a, &QGraphsTheme::labelTextColorChanged, 11))
            return;
        if (QtMocHelpers::indexOfMethod<void (QGraphsTheme::*)(QColor )>(_a, &QGraphsTheme::singleHighlightColorChanged, 12))
            return;
        if (QtMocHelpers::indexOfMethod<void (QGraphsTheme::*)(QColor )>(_a, &QGraphsTheme::multiHighlightColorChanged, 13))
            return;
        if (QtMocHelpers::indexOfMethod<void (QGraphsTheme::*)(const QLinearGradient & )>(_a, &QGraphsTheme::singleHighlightGradientChanged, 14))
            return;
        if (QtMocHelpers::indexOfMethod<void (QGraphsTheme::*)(const QLinearGradient & )>(_a, &QGraphsTheme::multiHighlightGradientChanged, 15))
            return;
        if (QtMocHelpers::indexOfMethod<void (QGraphsTheme::*)()>(_a, &QGraphsTheme::labelFontChanged, 16))
            return;
        if (QtMocHelpers::indexOfMethod<void (QGraphsTheme::*)()>(_a, &QGraphsTheme::labelBackgroundVisibleChanged, 17))
            return;
        if (QtMocHelpers::indexOfMethod<void (QGraphsTheme::*)()>(_a, &QGraphsTheme::labelBorderVisibleChanged, 18))
            return;
        if (QtMocHelpers::indexOfMethod<void (QGraphsTheme::*)(const QList<QColor> & )>(_a, &QGraphsTheme::seriesColorsChanged, 19))
            return;
        if (QtMocHelpers::indexOfMethod<void (QGraphsTheme::*)(const QList<QLinearGradient> & )>(_a, &QGraphsTheme::seriesGradientsChanged, 20))
            return;
        if (QtMocHelpers::indexOfMethod<void (QGraphsTheme::*)()>(_a, &QGraphsTheme::borderColorsChanged, 21))
            return;
        if (QtMocHelpers::indexOfMethod<void (QGraphsTheme::*)()>(_a, &QGraphsTheme::borderWidthChanged, 22))
            return;
        if (QtMocHelpers::indexOfMethod<void (QGraphsTheme::*)()>(_a, &QGraphsTheme::singleHighlightGradientQMLChanged, 23))
            return;
        if (QtMocHelpers::indexOfMethod<void (QGraphsTheme::*)()>(_a, &QGraphsTheme::multiHighlightGradientQMLChanged, 24))
            return;
        if (QtMocHelpers::indexOfMethod<void (QGraphsTheme::*)()>(_a, &QGraphsTheme::axisXLabelFontChanged, 25))
            return;
        if (QtMocHelpers::indexOfMethod<void (QGraphsTheme::*)()>(_a, &QGraphsTheme::axisYLabelFontChanged, 26))
            return;
        if (QtMocHelpers::indexOfMethod<void (QGraphsTheme::*)()>(_a, &QGraphsTheme::axisZLabelFontChanged, 27))
            return;
        if (QtMocHelpers::indexOfMethod<void (QGraphsTheme::*)()>(_a, &QGraphsTheme::gridChanged, 28))
            return;
        if (QtMocHelpers::indexOfMethod<void (QGraphsTheme::*)()>(_a, &QGraphsTheme::axisXChanged, 29))
            return;
        if (QtMocHelpers::indexOfMethod<void (QGraphsTheme::*)()>(_a, &QGraphsTheme::axisYChanged, 30))
            return;
        if (QtMocHelpers::indexOfMethod<void (QGraphsTheme::*)()>(_a, &QGraphsTheme::axisZChanged, 31))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QGraphsTheme::ColorScheme*>(_v) = _t->colorScheme(); break;
        case 1: *reinterpret_cast<QGraphsTheme::Theme*>(_v) = _t->theme(); break;
        case 2: *reinterpret_cast<QGraphsTheme::ColorStyle*>(_v) = _t->colorStyle(); break;
        case 3: *reinterpret_cast<QColor*>(_v) = _t->backgroundColor(); break;
        case 4: *reinterpret_cast<bool*>(_v) = _t->isBackgroundVisible(); break;
        case 5: *reinterpret_cast<QColor*>(_v) = _t->plotAreaBackgroundColor(); break;
        case 6: *reinterpret_cast<bool*>(_v) = _t->isPlotAreaBackgroundVisible(); break;
        case 7: *reinterpret_cast<bool*>(_v) = _t->isGridVisible(); break;
        case 8: *reinterpret_cast<QFont*>(_v) = _t->axisXLabelFont(); break;
        case 9: *reinterpret_cast<QFont*>(_v) = _t->axisYLabelFont(); break;
        case 10: *reinterpret_cast<QFont*>(_v) = _t->axisZLabelFont(); break;
        case 11: *reinterpret_cast<QGraphsLine*>(_v) = _t->grid(); break;
        case 12: *reinterpret_cast<QGraphsLine*>(_v) = _t->axisX(); break;
        case 13: *reinterpret_cast<QGraphsLine*>(_v) = _t->axisY(); break;
        case 14: *reinterpret_cast<QGraphsLine*>(_v) = _t->axisZ(); break;
        case 15: *reinterpret_cast<QFont*>(_v) = _t->labelFont(); break;
        case 16: *reinterpret_cast<bool*>(_v) = _t->labelsVisible(); break;
        case 17: *reinterpret_cast<QColor*>(_v) = _t->labelBackgroundColor(); break;
        case 18: *reinterpret_cast<QColor*>(_v) = _t->labelTextColor(); break;
        case 19: *reinterpret_cast<bool*>(_v) = _t->isLabelBackgroundVisible(); break;
        case 20: *reinterpret_cast<bool*>(_v) = _t->isLabelBorderVisible(); break;
        case 21: *reinterpret_cast<QList<QColor>*>(_v) = _t->seriesColors(); break;
        case 22: *reinterpret_cast<QList<QColor>*>(_v) = _t->borderColors(); break;
        case 23: *reinterpret_cast<qreal*>(_v) = _t->borderWidth(); break;
        case 24: *reinterpret_cast<QQmlListProperty<QQuickGraphsColor>*>(_v) = _t->baseColorsQML(); break;
        case 25: *reinterpret_cast<QQmlListProperty<QQuickGradient>*>(_v) = _t->baseGradientsQML(); break;
        case 26: *reinterpret_cast<QQmlListProperty<QObject>*>(_v) = _t->themeChildren(); break;
        case 27: *reinterpret_cast<QColor*>(_v) = _t->singleHighlightColor(); break;
        case 28: *reinterpret_cast<QColor*>(_v) = _t->multiHighlightColor(); break;
        case 29: *reinterpret_cast<QQuickGradient**>(_v) = _t->singleHighlightGradientQML(); break;
        case 30: *reinterpret_cast<QQuickGradient**>(_v) = _t->multiHighlightGradientQML(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setColorScheme(*reinterpret_cast<QGraphsTheme::ColorScheme*>(_v)); break;
        case 1: _t->setTheme(*reinterpret_cast<QGraphsTheme::Theme*>(_v)); break;
        case 2: _t->setColorStyle(*reinterpret_cast<QGraphsTheme::ColorStyle*>(_v)); break;
        case 3: _t->setBackgroundColor(*reinterpret_cast<QColor*>(_v)); break;
        case 4: _t->setBackgroundVisible(*reinterpret_cast<bool*>(_v)); break;
        case 5: _t->setPlotAreaBackgroundColor(*reinterpret_cast<QColor*>(_v)); break;
        case 6: _t->setPlotAreaBackgroundVisible(*reinterpret_cast<bool*>(_v)); break;
        case 7: _t->setGridVisible(*reinterpret_cast<bool*>(_v)); break;
        case 8: _t->setAxisXLabelFont(*reinterpret_cast<QFont*>(_v)); break;
        case 9: _t->setAxisYLabelFont(*reinterpret_cast<QFont*>(_v)); break;
        case 10: _t->setAxisZLabelFont(*reinterpret_cast<QFont*>(_v)); break;
        case 11: _t->setGrid(*reinterpret_cast<QGraphsLine*>(_v)); break;
        case 12: _t->setAxisX(*reinterpret_cast<QGraphsLine*>(_v)); break;
        case 13: _t->setAxisY(*reinterpret_cast<QGraphsLine*>(_v)); break;
        case 14: _t->setAxisZ(*reinterpret_cast<QGraphsLine*>(_v)); break;
        case 15: _t->setLabelFont(*reinterpret_cast<QFont*>(_v)); break;
        case 16: _t->setLabelsVisible(*reinterpret_cast<bool*>(_v)); break;
        case 17: _t->setLabelBackgroundColor(*reinterpret_cast<QColor*>(_v)); break;
        case 18: _t->setLabelTextColor(*reinterpret_cast<QColor*>(_v)); break;
        case 19: _t->setLabelBackgroundVisible(*reinterpret_cast<bool*>(_v)); break;
        case 20: _t->setLabelBorderVisible(*reinterpret_cast<bool*>(_v)); break;
        case 21: _t->setSeriesColors(*reinterpret_cast<QList<QColor>*>(_v)); break;
        case 22: _t->setBorderColors(*reinterpret_cast<QList<QColor>*>(_v)); break;
        case 23: _t->setBorderWidth(*reinterpret_cast<qreal*>(_v)); break;
        case 27: _t->setSingleHighlightColor(*reinterpret_cast<QColor*>(_v)); break;
        case 28: _t->setMultiHighlightColor(*reinterpret_cast<QColor*>(_v)); break;
        case 29: _t->setSingleHighlightGradientQML(*reinterpret_cast<QQuickGradient**>(_v)); break;
        case 30: _t->setMultiHighlightGradientQML(*reinterpret_cast<QQuickGradient**>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *QGraphsTheme::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QGraphsTheme::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12QGraphsThemeE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "QQmlParserStatus"))
        return static_cast< QQmlParserStatus*>(this);
    if (!strcmp(_clname, "org.qt-project.Qt.QQmlParserStatus"))
        return static_cast< QQmlParserStatus*>(this);
    return QObject::qt_metacast(_clname);
}

int QGraphsTheme::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 34)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 34;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 34)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 34;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 31;
    }
    return _id;
}

// SIGNAL 0
void QGraphsTheme::update()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void QGraphsTheme::colorSchemeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void QGraphsTheme::themeChanged(QGraphsTheme::Theme _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void QGraphsTheme::colorStyleChanged(QGraphsTheme::ColorStyle _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void QGraphsTheme::backgroundColorChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void QGraphsTheme::backgroundVisibleChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void QGraphsTheme::plotAreaBackgroundColorChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void QGraphsTheme::plotAreaBackgroundVisibleChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void QGraphsTheme::gridVisibleChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 8, nullptr);
}

// SIGNAL 9
void QGraphsTheme::labelsVisibleChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 9, nullptr);
}

// SIGNAL 10
void QGraphsTheme::labelBackgroundColorChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 10, nullptr);
}

// SIGNAL 11
void QGraphsTheme::labelTextColorChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 11, nullptr);
}

// SIGNAL 12
void QGraphsTheme::singleHighlightColorChanged(QColor _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 12, nullptr, _t1);
}

// SIGNAL 13
void QGraphsTheme::multiHighlightColorChanged(QColor _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 13, nullptr, _t1);
}

// SIGNAL 14
void QGraphsTheme::singleHighlightGradientChanged(const QLinearGradient & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 14, nullptr, _t1);
}

// SIGNAL 15
void QGraphsTheme::multiHighlightGradientChanged(const QLinearGradient & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 15, nullptr, _t1);
}

// SIGNAL 16
void QGraphsTheme::labelFontChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 16, nullptr);
}

// SIGNAL 17
void QGraphsTheme::labelBackgroundVisibleChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 17, nullptr);
}

// SIGNAL 18
void QGraphsTheme::labelBorderVisibleChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 18, nullptr);
}

// SIGNAL 19
void QGraphsTheme::seriesColorsChanged(const QList<QColor> & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 19, nullptr, _t1);
}

// SIGNAL 20
void QGraphsTheme::seriesGradientsChanged(const QList<QLinearGradient> & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 20, nullptr, _t1);
}

// SIGNAL 21
void QGraphsTheme::borderColorsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 21, nullptr);
}

// SIGNAL 22
void QGraphsTheme::borderWidthChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 22, nullptr);
}

// SIGNAL 23
void QGraphsTheme::singleHighlightGradientQMLChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 23, nullptr);
}

// SIGNAL 24
void QGraphsTheme::multiHighlightGradientQMLChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 24, nullptr);
}

// SIGNAL 25
void QGraphsTheme::axisXLabelFontChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 25, nullptr);
}

// SIGNAL 26
void QGraphsTheme::axisYLabelFontChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 26, nullptr);
}

// SIGNAL 27
void QGraphsTheme::axisZLabelFontChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 27, nullptr);
}

// SIGNAL 28
void QGraphsTheme::gridChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 28, nullptr);
}

// SIGNAL 29
void QGraphsTheme::axisXChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 29, nullptr);
}

// SIGNAL 30
void QGraphsTheme::axisYChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 30, nullptr);
}

// SIGNAL 31
void QGraphsTheme::axisZChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 31, nullptr);
}
QT_WARNING_POP
