/****************************************************************************
** Meta object code from reading C++ file 'qgraphs3dnamespace.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../qtgraphs/src/graphs3d/utils/qgraphs3dnamespace.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qgraphs3dnamespace.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN10QtGraphs3DE_t {};
} // unnamed namespace

template <> constexpr inline auto QtGraphs3D::qt_create_metaobjectdata<qt_meta_tag_ZN10QtGraphs3DE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QtGraphs3D",
        "QML.Element",
        "Graphs3D",
        "SelectionFlag",
        "None",
        "Item",
        "Row",
        "ItemAndRow",
        "Column",
        "ItemAndColumn",
        "RowAndColumn",
        "ItemRowAndColumn",
        "Slice",
        "MultiSeries",
        "SelectionFlags",
        "SliceCaptureType",
        "NoImage",
        "RowImage",
        "ColumnImage",
        "ShadowQuality",
        "Low",
        "Medium",
        "High",
        "SoftLow",
        "SoftMedium",
        "SoftHigh",
        "ElementType",
        "Series",
        "AxisXLabel",
        "AxisYLabel",
        "AxisZLabel",
        "CustomItem",
        "OptimizationHint",
        "Default",
        "Legacy",
        "RenderingMode",
        "DirectToBackground",
        "Indirect",
        "CameraPreset",
        "NoPreset",
        "FrontLow",
        "Front",
        "FrontHigh",
        "LeftLow",
        "Left",
        "LeftHigh",
        "RightLow",
        "Right",
        "RightHigh",
        "BehindLow",
        "Behind",
        "BehindHigh",
        "IsometricLeft",
        "IsometricLeftHigh",
        "IsometricRight",
        "IsometricRightHigh",
        "DirectlyAbove",
        "DirectlyAboveCW45",
        "DirectlyAboveCCW45",
        "FrontBelow",
        "LeftBelow",
        "RightBelow",
        "BehindBelow",
        "DirectlyBelow",
        "GridLineType",
        "Shader",
        "Geometry",
        "TransparencyTechnique",
        "Approximate",
        "Accurate"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'SelectionFlag'
        QtMocHelpers::EnumData<SelectionFlag>(3, 3, QMC::EnumIsScoped).add({
            {    4, SelectionFlag::None },
            {    5, SelectionFlag::Item },
            {    6, SelectionFlag::Row },
            {    7, SelectionFlag::ItemAndRow },
            {    8, SelectionFlag::Column },
            {    9, SelectionFlag::ItemAndColumn },
            {   10, SelectionFlag::RowAndColumn },
            {   11, SelectionFlag::ItemRowAndColumn },
            {   12, SelectionFlag::Slice },
            {   13, SelectionFlag::MultiSeries },
        }),
        // flag 'SelectionFlags'
        QtMocHelpers::EnumData<SelectionFlags>(14, 3, QMC::EnumIsFlag | QMC::EnumIsScoped).add({
            {    4, SelectionFlag::None },
            {    5, SelectionFlag::Item },
            {    6, SelectionFlag::Row },
            {    7, SelectionFlag::ItemAndRow },
            {    8, SelectionFlag::Column },
            {    9, SelectionFlag::ItemAndColumn },
            {   10, SelectionFlag::RowAndColumn },
            {   11, SelectionFlag::ItemRowAndColumn },
            {   12, SelectionFlag::Slice },
            {   13, SelectionFlag::MultiSeries },
        }),
        // enum 'SliceCaptureType'
        QtMocHelpers::EnumData<SliceCaptureType>(15, 15, QMC::EnumIsScoped).add({
            {   16, SliceCaptureType::NoImage },
            {   17, SliceCaptureType::RowImage },
            {   18, SliceCaptureType::ColumnImage },
        }),
        // enum 'ShadowQuality'
        QtMocHelpers::EnumData<ShadowQuality>(19, 19, QMC::EnumIsScoped).add({
            {    4, ShadowQuality::None },
            {   20, ShadowQuality::Low },
            {   21, ShadowQuality::Medium },
            {   22, ShadowQuality::High },
            {   23, ShadowQuality::SoftLow },
            {   24, ShadowQuality::SoftMedium },
            {   25, ShadowQuality::SoftHigh },
        }),
        // enum 'ElementType'
        QtMocHelpers::EnumData<ElementType>(26, 26, QMC::EnumIsScoped).add({
            {    4, ElementType::None },
            {   27, ElementType::Series },
            {   28, ElementType::AxisXLabel },
            {   29, ElementType::AxisYLabel },
            {   30, ElementType::AxisZLabel },
            {   31, ElementType::CustomItem },
        }),
        // enum 'OptimizationHint'
        QtMocHelpers::EnumData<OptimizationHint>(32, 32, QMC::EnumIsScoped).add({
            {   33, OptimizationHint::Default },
            {   34, OptimizationHint::Legacy },
        }),
        // enum 'RenderingMode'
        QtMocHelpers::EnumData<RenderingMode>(35, 35, QMC::EnumIsScoped).add({
            {   36, RenderingMode::DirectToBackground },
            {   37, RenderingMode::Indirect },
        }),
        // enum 'CameraPreset'
        QtMocHelpers::EnumData<CameraPreset>(38, 38, QMC::EnumIsScoped).add({
            {   39, CameraPreset::NoPreset },
            {   40, CameraPreset::FrontLow },
            {   41, CameraPreset::Front },
            {   42, CameraPreset::FrontHigh },
            {   43, CameraPreset::LeftLow },
            {   44, CameraPreset::Left },
            {   45, CameraPreset::LeftHigh },
            {   46, CameraPreset::RightLow },
            {   47, CameraPreset::Right },
            {   48, CameraPreset::RightHigh },
            {   49, CameraPreset::BehindLow },
            {   50, CameraPreset::Behind },
            {   51, CameraPreset::BehindHigh },
            {   52, CameraPreset::IsometricLeft },
            {   53, CameraPreset::IsometricLeftHigh },
            {   54, CameraPreset::IsometricRight },
            {   55, CameraPreset::IsometricRightHigh },
            {   56, CameraPreset::DirectlyAbove },
            {   57, CameraPreset::DirectlyAboveCW45 },
            {   58, CameraPreset::DirectlyAboveCCW45 },
            {   59, CameraPreset::FrontBelow },
            {   60, CameraPreset::LeftBelow },
            {   61, CameraPreset::RightBelow },
            {   62, CameraPreset::BehindBelow },
            {   63, CameraPreset::DirectlyBelow },
        }),
        // enum 'GridLineType'
        QtMocHelpers::EnumData<GridLineType>(64, 64, QMC::EnumIsScoped).add({
            {   65, GridLineType::Shader },
            {   66, GridLineType::Geometry },
        }),
        // enum 'TransparencyTechnique'
        QtMocHelpers::EnumData<TransparencyTechnique>(67, 67, QMC::EnumIsScoped).add({
            {   33, TransparencyTechnique::Default },
            {   68, TransparencyTechnique::Approximate },
            {   69, TransparencyTechnique::Accurate },
        }),
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
    });
    return QtMocHelpers::metaObjectData<void, qt_meta_tag_ZN10QtGraphs3DE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}

static constexpr auto qt_staticMetaObjectContent_ZN10QtGraphs3DE =
    QtGraphs3D::qt_create_metaobjectdata<qt_meta_tag_ZN10QtGraphs3DE_t>();
static constexpr auto qt_staticMetaObjectStaticContent_ZN10QtGraphs3DE =
    qt_staticMetaObjectContent_ZN10QtGraphs3DE.staticData;
static constexpr auto qt_staticMetaObjectRelocatingContent_ZN10QtGraphs3DE =
    qt_staticMetaObjectContent_ZN10QtGraphs3DE.relocatingData;

Q_CONSTINIT const QMetaObject QtGraphs3D::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent_ZN10QtGraphs3DE.stringdata,
    qt_staticMetaObjectStaticContent_ZN10QtGraphs3DE.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent_ZN10QtGraphs3DE.metaTypes,
    nullptr
} };

QT_WARNING_POP
