/****************************************************************************
** Meta object code from reading C++ file 'q3dgraphswidgetitem.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../qtgraphs/src/graphs3d/widget/q3dgraphswidgetitem.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'q3dgraphswidgetitem.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN19Q3DGraphsWidgetItemE_t {};
} // unnamed namespace

template <> constexpr inline auto Q3DGraphsWidgetItem::qt_create_metaobjectdata<qt_meta_tag_ZN19Q3DGraphsWidgetItemE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Q3DGraphsWidgetItem",
        "RegisterEnumClassesUnscoped",
        "false",
        "activeThemeChanged",
        "",
        "QGraphsTheme*",
        "activeTheme",
        "shadowQualityChanged",
        "QtGraphs3D::ShadowQuality",
        "quality",
        "transparencyTechniqueChanged",
        "QtGraphs3D::TransparencyTechnique",
        "technique",
        "selectionModeChanged",
        "QtGraphs3D::SelectionFlags",
        "selectionMode",
        "selectedElementChanged",
        "QtGraphs3D::ElementType",
        "type",
        "measureFpsChanged",
        "enabled",
        "currentFpsChanged",
        "fps",
        "orthoProjectionChanged",
        "aspectRatioChanged",
        "ratio",
        "optimizationHintChanged",
        "QtGraphs3D::OptimizationHint",
        "hint",
        "polarChanged",
        "labelMarginChanged",
        "margin",
        "radialLabelOffsetChanged",
        "offset",
        "horizontalAspectRatioChanged",
        "localeChanged",
        "locale",
        "queriedGraphPositionChanged",
        "QVector3D",
        "data",
        "marginChanged",
        "cutoffMarginChanged",
        "newMargin",
        "cameraPresetChanged",
        "QtGraphs3D::CameraPreset",
        "preset",
        "cameraXRotationChanged",
        "rotation",
        "cameraYRotationChanged",
        "cameraZoomLevelChanged",
        "zoomLevel",
        "cameraTargetPositionChanged",
        "target",
        "minCameraZoomLevelChanged",
        "maxCameraZoomLevelChanged",
        "minCameraXRotationChanged",
        "minCameraYRotationChanged",
        "maxCameraXRotationChanged",
        "maxCameraYRotationChanged",
        "wrapCameraXRotationChanged",
        "wrap",
        "wrapCameraYRotationChanged",
        "msaaSamplesChanged",
        "samples",
        "tapped",
        "QEventPoint",
        "eventPoint",
        "Qt::MouseButton",
        "button",
        "doubleTapped",
        "longPressed",
        "dragged",
        "QVector2D",
        "delta",
        "wheel",
        "QWheelEvent*",
        "event",
        "pinch",
        "mouseMove",
        "QPoint",
        "mousePos",
        "zoomEnabledChanged",
        "enable",
        "zoomAtTargetEnabledChanged",
        "rotationEnabledChanged",
        "panEnabledChanged",
        "panModeEnabledChanged",
        "selectionEnabledChanged",
        "ambientLightStrengthChanged",
        "lightStrengthChanged",
        "shadowStrengthChanged",
        "lightColorChanged",
        "gridLineTypeChanged",
        "shadowQuality",
        "transparencyTechnique",
        "scene",
        "Q3DScene*",
        "measureFps",
        "currentFps",
        "orthoProjection",
        "selectedElement",
        "aspectRatio",
        "optimizationHint",
        "polar",
        "labelMargin",
        "radialLabelOffset",
        "horizontalAspectRatio",
        "queriedGraphPosition",
        "cutoffMargin",
        "cameraPreset",
        "cameraXRotation",
        "cameraYRotation",
        "cameraZoomLevel",
        "minCameraZoomLevel",
        "maxCameraZoomLevel",
        "wrapCameraXRotation",
        "wrapCameraYRotation",
        "minCameraXRotation",
        "maxCameraXRotation",
        "minCameraYRotation",
        "maxCameraYRotation",
        "cameraTargetPosition",
        "msaaSamples",
        "rotationEnabled",
        "panEnabled",
        "panModeEnabled",
        "zoomAtTargetEnabled",
        "selectionEnabled",
        "zoomEnabled",
        "lightColor",
        "QColor",
        "ambientLightStrength",
        "lightStrength",
        "shadowStrength",
        "gridLineType",
        "QtGraphs3D::GridLineType"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'activeThemeChanged'
        QtMocHelpers::SignalData<void(QGraphsTheme *)>(3, 4, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 6 },
        }}),
        // Signal 'shadowQualityChanged'
        QtMocHelpers::SignalData<void(QtGraphs3D::ShadowQuality)>(7, 4, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 8, 9 },
        }}),
        // Signal 'transparencyTechniqueChanged'
        QtMocHelpers::RevisionedSignalData<void(QtGraphs3D::TransparencyTechnique)>(10, 4, QMC::AccessPublic, 0x609, QMetaType::Void, {{
            { 0x80000000 | 11, 12 },
        }}),
        // Signal 'selectionModeChanged'
        QtMocHelpers::SignalData<void(const QtGraphs3D::SelectionFlags)>(13, 4, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 14, 15 },
        }}),
        // Signal 'selectedElementChanged'
        QtMocHelpers::SignalData<void(QtGraphs3D::ElementType)>(16, 4, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 17, 18 },
        }}),
        // Signal 'measureFpsChanged'
        QtMocHelpers::SignalData<void(bool)>(19, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 20 },
        }}),
        // Signal 'currentFpsChanged'
        QtMocHelpers::SignalData<void(int)>(21, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 22 },
        }}),
        // Signal 'orthoProjectionChanged'
        QtMocHelpers::SignalData<void(bool)>(23, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 20 },
        }}),
        // Signal 'aspectRatioChanged'
        QtMocHelpers::SignalData<void(qreal)>(24, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QReal, 25 },
        }}),
        // Signal 'optimizationHintChanged'
        QtMocHelpers::SignalData<void(QtGraphs3D::OptimizationHint)>(26, 4, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 27, 28 },
        }}),
        // Signal 'polarChanged'
        QtMocHelpers::SignalData<void(bool)>(29, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 20 },
        }}),
        // Signal 'labelMarginChanged'
        QtMocHelpers::SignalData<void(float)>(30, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Float, 31 },
        }}),
        // Signal 'radialLabelOffsetChanged'
        QtMocHelpers::SignalData<void(float)>(32, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Float, 33 },
        }}),
        // Signal 'horizontalAspectRatioChanged'
        QtMocHelpers::SignalData<void(qreal)>(34, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QReal, 25 },
        }}),
        // Signal 'localeChanged'
        QtMocHelpers::SignalData<void(const QLocale &)>(35, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QLocale, 36 },
        }}),
        // Signal 'queriedGraphPositionChanged'
        QtMocHelpers::SignalData<void(QVector3D)>(37, 4, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 38, 39 },
        }}),
        // Signal 'marginChanged'
        QtMocHelpers::SignalData<void(qreal)>(40, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QReal, 31 },
        }}),
        // Signal 'cutoffMarginChanged'
        QtMocHelpers::RevisionedSignalData<void(qreal)>(41, 4, QMC::AccessPublic, 0x60b, QMetaType::Void, {{
            { QMetaType::QReal, 42 },
        }}),
        // Signal 'cameraPresetChanged'
        QtMocHelpers::SignalData<void(QtGraphs3D::CameraPreset)>(43, 4, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 44, 45 },
        }}),
        // Signal 'cameraXRotationChanged'
        QtMocHelpers::SignalData<void(float)>(46, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Float, 47 },
        }}),
        // Signal 'cameraYRotationChanged'
        QtMocHelpers::SignalData<void(float)>(48, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Float, 47 },
        }}),
        // Signal 'cameraZoomLevelChanged'
        QtMocHelpers::SignalData<void(float)>(49, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Float, 50 },
        }}),
        // Signal 'cameraTargetPositionChanged'
        QtMocHelpers::SignalData<void(QVector3D)>(51, 4, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 38, 52 },
        }}),
        // Signal 'minCameraZoomLevelChanged'
        QtMocHelpers::SignalData<void(float)>(53, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Float, 50 },
        }}),
        // Signal 'maxCameraZoomLevelChanged'
        QtMocHelpers::SignalData<void(float)>(54, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Float, 50 },
        }}),
        // Signal 'minCameraXRotationChanged'
        QtMocHelpers::SignalData<void(float)>(55, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Float, 47 },
        }}),
        // Signal 'minCameraYRotationChanged'
        QtMocHelpers::SignalData<void(float)>(56, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Float, 47 },
        }}),
        // Signal 'maxCameraXRotationChanged'
        QtMocHelpers::SignalData<void(float)>(57, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Float, 47 },
        }}),
        // Signal 'maxCameraYRotationChanged'
        QtMocHelpers::SignalData<void(float)>(58, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Float, 47 },
        }}),
        // Signal 'wrapCameraXRotationChanged'
        QtMocHelpers::SignalData<void(bool)>(59, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 60 },
        }}),
        // Signal 'wrapCameraYRotationChanged'
        QtMocHelpers::SignalData<void(bool)>(61, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 60 },
        }}),
        // Signal 'msaaSamplesChanged'
        QtMocHelpers::SignalData<void(int)>(62, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 63 },
        }}),
        // Signal 'tapped'
        QtMocHelpers::SignalData<void(QEventPoint, Qt::MouseButton)>(64, 4, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 65, 66 }, { 0x80000000 | 67, 68 },
        }}),
        // Signal 'doubleTapped'
        QtMocHelpers::SignalData<void(QEventPoint, Qt::MouseButton)>(69, 4, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 65, 66 }, { 0x80000000 | 67, 68 },
        }}),
        // Signal 'longPressed'
        QtMocHelpers::SignalData<void()>(70, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'dragged'
        QtMocHelpers::SignalData<void(QVector2D)>(71, 4, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 72, 73 },
        }}),
        // Signal 'wheel'
        QtMocHelpers::SignalData<void(QWheelEvent *)>(74, 4, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 75, 76 },
        }}),
        // Signal 'pinch'
        QtMocHelpers::SignalData<void(qreal)>(77, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QReal, 73 },
        }}),
        // Signal 'mouseMove'
        QtMocHelpers::SignalData<void(QPoint)>(78, 4, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 79, 80 },
        }}),
        // Signal 'zoomEnabledChanged'
        QtMocHelpers::SignalData<void(bool)>(81, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 82 },
        }}),
        // Signal 'zoomAtTargetEnabledChanged'
        QtMocHelpers::SignalData<void(bool)>(83, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 82 },
        }}),
        // Signal 'rotationEnabledChanged'
        QtMocHelpers::SignalData<void(bool)>(84, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 82 },
        }}),
        // Signal 'panEnabledChanged'
        QtMocHelpers::RevisionedSignalData<void(bool)>(85, 4, QMC::AccessPublic, 0x60c, QMetaType::Void, {{
            { QMetaType::Bool, 82 },
        }}),
        // Signal 'panModeEnabledChanged'
        QtMocHelpers::RevisionedSignalData<void(bool)>(86, 4, QMC::AccessPublic, 0x60c, QMetaType::Void, {{
            { QMetaType::Bool, 82 },
        }}),
        // Signal 'selectionEnabledChanged'
        QtMocHelpers::SignalData<void(bool)>(87, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 82 },
        }}),
        // Signal 'ambientLightStrengthChanged'
        QtMocHelpers::SignalData<void()>(88, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'lightStrengthChanged'
        QtMocHelpers::SignalData<void()>(89, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'shadowStrengthChanged'
        QtMocHelpers::SignalData<void()>(90, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'lightColorChanged'
        QtMocHelpers::SignalData<void()>(91, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'gridLineTypeChanged'
        QtMocHelpers::SignalData<void()>(92, 4, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'activeTheme'
        QtMocHelpers::PropertyData<QGraphsTheme*>(6, 0x80000000 | 5, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 0),
        // property 'selectionMode'
        QtMocHelpers::PropertyData<QtGraphs3D::SelectionFlags>(15, 0x80000000 | 14, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 3),
        // property 'shadowQuality'
        QtMocHelpers::PropertyData<QtGraphs3D::ShadowQuality>(93, 0x80000000 | 8, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 1),
        // property 'transparencyTechnique'
        QtMocHelpers::PropertyData<QtGraphs3D::TransparencyTechnique>(94, 0x80000000 | 11, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 2, 0x609),
        // property 'scene'
        QtMocHelpers::PropertyData<Q3DScene*>(95, 0x80000000 | 96, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'measureFps'
        QtMocHelpers::PropertyData<bool>(97, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 5),
        // property 'currentFps'
        QtMocHelpers::PropertyData<int>(98, QMetaType::Int, QMC::DefaultPropertyFlags, 6),
        // property 'orthoProjection'
        QtMocHelpers::PropertyData<bool>(99, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 7),
        // property 'selectedElement'
        QtMocHelpers::PropertyData<QtGraphs3D::ElementType>(100, 0x80000000 | 17, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 4),
        // property 'aspectRatio'
        QtMocHelpers::PropertyData<qreal>(101, QMetaType::QReal, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 8),
        // property 'optimizationHint'
        QtMocHelpers::PropertyData<QtGraphs3D::OptimizationHint>(102, 0x80000000 | 27, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 9),
        // property 'polar'
        QtMocHelpers::PropertyData<bool>(103, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 10),
        // property 'labelMargin'
        QtMocHelpers::PropertyData<float>(104, QMetaType::Float, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 11),
        // property 'radialLabelOffset'
        QtMocHelpers::PropertyData<float>(105, QMetaType::Float, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 12),
        // property 'horizontalAspectRatio'
        QtMocHelpers::PropertyData<qreal>(106, QMetaType::QReal, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 13),
        // property 'locale'
        QtMocHelpers::PropertyData<QLocale>(36, QMetaType::QLocale, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 14),
        // property 'queriedGraphPosition'
        QtMocHelpers::PropertyData<QVector3D>(107, 0x80000000 | 38, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 15),
        // property 'margin'
        QtMocHelpers::PropertyData<qreal>(31, QMetaType::QReal, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 16),
        // property 'cutoffMargin'
        QtMocHelpers::PropertyData<qreal>(108, QMetaType::QReal, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 17, 0x60b),
        // property 'cameraPreset'
        QtMocHelpers::PropertyData<QtGraphs3D::CameraPreset>(109, 0x80000000 | 44, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 18),
        // property 'cameraXRotation'
        QtMocHelpers::PropertyData<float>(110, QMetaType::Float, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 19),
        // property 'cameraYRotation'
        QtMocHelpers::PropertyData<float>(111, QMetaType::Float, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 20),
        // property 'cameraZoomLevel'
        QtMocHelpers::PropertyData<float>(112, QMetaType::Float, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 21),
        // property 'minCameraZoomLevel'
        QtMocHelpers::PropertyData<float>(113, QMetaType::Float, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 23),
        // property 'maxCameraZoomLevel'
        QtMocHelpers::PropertyData<float>(114, QMetaType::Float, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 24),
        // property 'wrapCameraXRotation'
        QtMocHelpers::PropertyData<bool>(115, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 29),
        // property 'wrapCameraYRotation'
        QtMocHelpers::PropertyData<bool>(116, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 30),
        // property 'minCameraXRotation'
        QtMocHelpers::PropertyData<float>(117, QMetaType::Float, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 25),
        // property 'maxCameraXRotation'
        QtMocHelpers::PropertyData<float>(118, QMetaType::Float, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 27),
        // property 'minCameraYRotation'
        QtMocHelpers::PropertyData<float>(119, QMetaType::Float, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 26),
        // property 'maxCameraYRotation'
        QtMocHelpers::PropertyData<float>(120, QMetaType::Float, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 28),
        // property 'cameraTargetPosition'
        QtMocHelpers::PropertyData<QVector3D>(121, 0x80000000 | 38, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 22),
        // property 'msaaSamples'
        QtMocHelpers::PropertyData<int>(122, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 31),
        // property 'rotationEnabled'
        QtMocHelpers::PropertyData<bool>(123, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 41),
        // property 'panEnabled'
        QtMocHelpers::PropertyData<bool>(124, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 42, 0x60c),
        // property 'panModeEnabled'
        QtMocHelpers::PropertyData<bool>(125, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 43, 0x60c),
        // property 'zoomAtTargetEnabled'
        QtMocHelpers::PropertyData<bool>(126, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 40),
        // property 'selectionEnabled'
        QtMocHelpers::PropertyData<bool>(127, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 44),
        // property 'zoomEnabled'
        QtMocHelpers::PropertyData<bool>(128, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 39),
        // property 'lightColor'
        QtMocHelpers::PropertyData<QColor>(129, 0x80000000 | 130, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 48),
        // property 'ambientLightStrength'
        QtMocHelpers::PropertyData<float>(131, QMetaType::Float, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 45),
        // property 'lightStrength'
        QtMocHelpers::PropertyData<float>(132, QMetaType::Float, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 46),
        // property 'shadowStrength'
        QtMocHelpers::PropertyData<float>(133, QMetaType::Float, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 47),
        // property 'gridLineType'
        QtMocHelpers::PropertyData<QtGraphs3D::GridLineType>(134, 0x80000000 | 135, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet | QMC::Final, 49),
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
    });
    return QtMocHelpers::metaObjectData<Q3DGraphsWidgetItem, qt_meta_tag_ZN19Q3DGraphsWidgetItemE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT static const QMetaObject::SuperData qt_meta_extradata_ZN19Q3DGraphsWidgetItemE[] = {
    QMetaObject::SuperData::link<QtGraphs3D::staticMetaObject>(),
    nullptr
};

Q_CONSTINIT const QMetaObject Q3DGraphsWidgetItem::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN19Q3DGraphsWidgetItemE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN19Q3DGraphsWidgetItemE_t>.data,
    qt_static_metacall,
    qt_meta_extradata_ZN19Q3DGraphsWidgetItemE,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN19Q3DGraphsWidgetItemE_t>.metaTypes,
    nullptr
} };

void Q3DGraphsWidgetItem::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<Q3DGraphsWidgetItem *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->activeThemeChanged((*reinterpret_cast<std::add_pointer_t<QGraphsTheme*>>(_a[1]))); break;
        case 1: _t->shadowQualityChanged((*reinterpret_cast<std::add_pointer_t<QtGraphs3D::ShadowQuality>>(_a[1]))); break;
        case 2: _t->transparencyTechniqueChanged((*reinterpret_cast<std::add_pointer_t<QtGraphs3D::TransparencyTechnique>>(_a[1]))); break;
        case 3: _t->selectionModeChanged((*reinterpret_cast<std::add_pointer_t<QtGraphs3D::SelectionFlags>>(_a[1]))); break;
        case 4: _t->selectedElementChanged((*reinterpret_cast<std::add_pointer_t<QtGraphs3D::ElementType>>(_a[1]))); break;
        case 5: _t->measureFpsChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 6: _t->currentFpsChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 7: _t->orthoProjectionChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 8: _t->aspectRatioChanged((*reinterpret_cast<std::add_pointer_t<qreal>>(_a[1]))); break;
        case 9: _t->optimizationHintChanged((*reinterpret_cast<std::add_pointer_t<QtGraphs3D::OptimizationHint>>(_a[1]))); break;
        case 10: _t->polarChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 11: _t->labelMarginChanged((*reinterpret_cast<std::add_pointer_t<float>>(_a[1]))); break;
        case 12: _t->radialLabelOffsetChanged((*reinterpret_cast<std::add_pointer_t<float>>(_a[1]))); break;
        case 13: _t->horizontalAspectRatioChanged((*reinterpret_cast<std::add_pointer_t<qreal>>(_a[1]))); break;
        case 14: _t->localeChanged((*reinterpret_cast<std::add_pointer_t<QLocale>>(_a[1]))); break;
        case 15: _t->queriedGraphPositionChanged((*reinterpret_cast<std::add_pointer_t<QVector3D>>(_a[1]))); break;
        case 16: _t->marginChanged((*reinterpret_cast<std::add_pointer_t<qreal>>(_a[1]))); break;
        case 17: _t->cutoffMarginChanged((*reinterpret_cast<std::add_pointer_t<qreal>>(_a[1]))); break;
        case 18: _t->cameraPresetChanged((*reinterpret_cast<std::add_pointer_t<QtGraphs3D::CameraPreset>>(_a[1]))); break;
        case 19: _t->cameraXRotationChanged((*reinterpret_cast<std::add_pointer_t<float>>(_a[1]))); break;
        case 20: _t->cameraYRotationChanged((*reinterpret_cast<std::add_pointer_t<float>>(_a[1]))); break;
        case 21: _t->cameraZoomLevelChanged((*reinterpret_cast<std::add_pointer_t<float>>(_a[1]))); break;
        case 22: _t->cameraTargetPositionChanged((*reinterpret_cast<std::add_pointer_t<QVector3D>>(_a[1]))); break;
        case 23: _t->minCameraZoomLevelChanged((*reinterpret_cast<std::add_pointer_t<float>>(_a[1]))); break;
        case 24: _t->maxCameraZoomLevelChanged((*reinterpret_cast<std::add_pointer_t<float>>(_a[1]))); break;
        case 25: _t->minCameraXRotationChanged((*reinterpret_cast<std::add_pointer_t<float>>(_a[1]))); break;
        case 26: _t->minCameraYRotationChanged((*reinterpret_cast<std::add_pointer_t<float>>(_a[1]))); break;
        case 27: _t->maxCameraXRotationChanged((*reinterpret_cast<std::add_pointer_t<float>>(_a[1]))); break;
        case 28: _t->maxCameraYRotationChanged((*reinterpret_cast<std::add_pointer_t<float>>(_a[1]))); break;
        case 29: _t->wrapCameraXRotationChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 30: _t->wrapCameraYRotationChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 31: _t->msaaSamplesChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 32: _t->tapped((*reinterpret_cast<std::add_pointer_t<QEventPoint>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<Qt::MouseButton>>(_a[2]))); break;
        case 33: _t->doubleTapped((*reinterpret_cast<std::add_pointer_t<QEventPoint>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<Qt::MouseButton>>(_a[2]))); break;
        case 34: _t->longPressed(); break;
        case 35: _t->dragged((*reinterpret_cast<std::add_pointer_t<QVector2D>>(_a[1]))); break;
        case 36: _t->wheel((*reinterpret_cast<std::add_pointer_t<QWheelEvent*>>(_a[1]))); break;
        case 37: _t->pinch((*reinterpret_cast<std::add_pointer_t<qreal>>(_a[1]))); break;
        case 38: _t->mouseMove((*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[1]))); break;
        case 39: _t->zoomEnabledChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 40: _t->zoomAtTargetEnabledChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 41: _t->rotationEnabledChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 42: _t->panEnabledChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 43: _t->panModeEnabledChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 44: _t->selectionEnabledChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 45: _t->ambientLightStrengthChanged(); break;
        case 46: _t->lightStrengthChanged(); break;
        case 47: _t->shadowStrengthChanged(); break;
        case 48: _t->lightColorChanged(); break;
        case 49: _t->gridLineTypeChanged(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QGraphsTheme* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (Q3DGraphsWidgetItem::*)(QGraphsTheme * )>(_a, &Q3DGraphsWidgetItem::activeThemeChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (Q3DGraphsWidgetItem::*)(QtGraphs3D::ShadowQuality )>(_a, &Q3DGraphsWidgetItem::shadowQualityChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (Q3DGraphsWidgetItem::*)(QtGraphs3D::TransparencyTechnique )>(_a, &Q3DGraphsWidgetItem::transparencyTechniqueChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (Q3DGraphsWidgetItem::*)(const QtGraphs3D::SelectionFlags )>(_a, &Q3DGraphsWidgetItem::selectionModeChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (Q3DGraphsWidgetItem::*)(QtGraphs3D::ElementType )>(_a, &Q3DGraphsWidgetItem::selectedElementChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (Q3DGraphsWidgetItem::*)(bool )>(_a, &Q3DGraphsWidgetItem::measureFpsChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (Q3DGraphsWidgetItem::*)(int )>(_a, &Q3DGraphsWidgetItem::currentFpsChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (Q3DGraphsWidgetItem::*)(bool )>(_a, &Q3DGraphsWidgetItem::orthoProjectionChanged, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (Q3DGraphsWidgetItem::*)(qreal )>(_a, &Q3DGraphsWidgetItem::aspectRatioChanged, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (Q3DGraphsWidgetItem::*)(QtGraphs3D::OptimizationHint )>(_a, &Q3DGraphsWidgetItem::optimizationHintChanged, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (Q3DGraphsWidgetItem::*)(bool )>(_a, &Q3DGraphsWidgetItem::polarChanged, 10))
            return;
        if (QtMocHelpers::indexOfMethod<void (Q3DGraphsWidgetItem::*)(float )>(_a, &Q3DGraphsWidgetItem::labelMarginChanged, 11))
            return;
        if (QtMocHelpers::indexOfMethod<void (Q3DGraphsWidgetItem::*)(float )>(_a, &Q3DGraphsWidgetItem::radialLabelOffsetChanged, 12))
            return;
        if (QtMocHelpers::indexOfMethod<void (Q3DGraphsWidgetItem::*)(qreal )>(_a, &Q3DGraphsWidgetItem::horizontalAspectRatioChanged, 13))
            return;
        if (QtMocHelpers::indexOfMethod<void (Q3DGraphsWidgetItem::*)(const QLocale & )>(_a, &Q3DGraphsWidgetItem::localeChanged, 14))
            return;
        if (QtMocHelpers::indexOfMethod<void (Q3DGraphsWidgetItem::*)(QVector3D )>(_a, &Q3DGraphsWidgetItem::queriedGraphPositionChanged, 15))
            return;
        if (QtMocHelpers::indexOfMethod<void (Q3DGraphsWidgetItem::*)(qreal )>(_a, &Q3DGraphsWidgetItem::marginChanged, 16))
            return;
        if (QtMocHelpers::indexOfMethod<void (Q3DGraphsWidgetItem::*)(qreal )>(_a, &Q3DGraphsWidgetItem::cutoffMarginChanged, 17))
            return;
        if (QtMocHelpers::indexOfMethod<void (Q3DGraphsWidgetItem::*)(QtGraphs3D::CameraPreset )>(_a, &Q3DGraphsWidgetItem::cameraPresetChanged, 18))
            return;
        if (QtMocHelpers::indexOfMethod<void (Q3DGraphsWidgetItem::*)(float )>(_a, &Q3DGraphsWidgetItem::cameraXRotationChanged, 19))
            return;
        if (QtMocHelpers::indexOfMethod<void (Q3DGraphsWidgetItem::*)(float )>(_a, &Q3DGraphsWidgetItem::cameraYRotationChanged, 20))
            return;
        if (QtMocHelpers::indexOfMethod<void (Q3DGraphsWidgetItem::*)(float )>(_a, &Q3DGraphsWidgetItem::cameraZoomLevelChanged, 21))
            return;
        if (QtMocHelpers::indexOfMethod<void (Q3DGraphsWidgetItem::*)(QVector3D )>(_a, &Q3DGraphsWidgetItem::cameraTargetPositionChanged, 22))
            return;
        if (QtMocHelpers::indexOfMethod<void (Q3DGraphsWidgetItem::*)(float )>(_a, &Q3DGraphsWidgetItem::minCameraZoomLevelChanged, 23))
            return;
        if (QtMocHelpers::indexOfMethod<void (Q3DGraphsWidgetItem::*)(float )>(_a, &Q3DGraphsWidgetItem::maxCameraZoomLevelChanged, 24))
            return;
        if (QtMocHelpers::indexOfMethod<void (Q3DGraphsWidgetItem::*)(float )>(_a, &Q3DGraphsWidgetItem::minCameraXRotationChanged, 25))
            return;
        if (QtMocHelpers::indexOfMethod<void (Q3DGraphsWidgetItem::*)(float )>(_a, &Q3DGraphsWidgetItem::minCameraYRotationChanged, 26))
            return;
        if (QtMocHelpers::indexOfMethod<void (Q3DGraphsWidgetItem::*)(float )>(_a, &Q3DGraphsWidgetItem::maxCameraXRotationChanged, 27))
            return;
        if (QtMocHelpers::indexOfMethod<void (Q3DGraphsWidgetItem::*)(float )>(_a, &Q3DGraphsWidgetItem::maxCameraYRotationChanged, 28))
            return;
        if (QtMocHelpers::indexOfMethod<void (Q3DGraphsWidgetItem::*)(bool )>(_a, &Q3DGraphsWidgetItem::wrapCameraXRotationChanged, 29))
            return;
        if (QtMocHelpers::indexOfMethod<void (Q3DGraphsWidgetItem::*)(bool )>(_a, &Q3DGraphsWidgetItem::wrapCameraYRotationChanged, 30))
            return;
        if (QtMocHelpers::indexOfMethod<void (Q3DGraphsWidgetItem::*)(int )>(_a, &Q3DGraphsWidgetItem::msaaSamplesChanged, 31))
            return;
        if (QtMocHelpers::indexOfMethod<void (Q3DGraphsWidgetItem::*)(QEventPoint , Qt::MouseButton )>(_a, &Q3DGraphsWidgetItem::tapped, 32))
            return;
        if (QtMocHelpers::indexOfMethod<void (Q3DGraphsWidgetItem::*)(QEventPoint , Qt::MouseButton )>(_a, &Q3DGraphsWidgetItem::doubleTapped, 33))
            return;
        if (QtMocHelpers::indexOfMethod<void (Q3DGraphsWidgetItem::*)()>(_a, &Q3DGraphsWidgetItem::longPressed, 34))
            return;
        if (QtMocHelpers::indexOfMethod<void (Q3DGraphsWidgetItem::*)(QVector2D )>(_a, &Q3DGraphsWidgetItem::dragged, 35))
            return;
        if (QtMocHelpers::indexOfMethod<void (Q3DGraphsWidgetItem::*)(QWheelEvent * )>(_a, &Q3DGraphsWidgetItem::wheel, 36))
            return;
        if (QtMocHelpers::indexOfMethod<void (Q3DGraphsWidgetItem::*)(qreal )>(_a, &Q3DGraphsWidgetItem::pinch, 37))
            return;
        if (QtMocHelpers::indexOfMethod<void (Q3DGraphsWidgetItem::*)(QPoint )>(_a, &Q3DGraphsWidgetItem::mouseMove, 38))
            return;
        if (QtMocHelpers::indexOfMethod<void (Q3DGraphsWidgetItem::*)(bool )>(_a, &Q3DGraphsWidgetItem::zoomEnabledChanged, 39))
            return;
        if (QtMocHelpers::indexOfMethod<void (Q3DGraphsWidgetItem::*)(bool )>(_a, &Q3DGraphsWidgetItem::zoomAtTargetEnabledChanged, 40))
            return;
        if (QtMocHelpers::indexOfMethod<void (Q3DGraphsWidgetItem::*)(bool )>(_a, &Q3DGraphsWidgetItem::rotationEnabledChanged, 41))
            return;
        if (QtMocHelpers::indexOfMethod<void (Q3DGraphsWidgetItem::*)(bool )>(_a, &Q3DGraphsWidgetItem::panEnabledChanged, 42))
            return;
        if (QtMocHelpers::indexOfMethod<void (Q3DGraphsWidgetItem::*)(bool )>(_a, &Q3DGraphsWidgetItem::panModeEnabledChanged, 43))
            return;
        if (QtMocHelpers::indexOfMethod<void (Q3DGraphsWidgetItem::*)(bool )>(_a, &Q3DGraphsWidgetItem::selectionEnabledChanged, 44))
            return;
        if (QtMocHelpers::indexOfMethod<void (Q3DGraphsWidgetItem::*)()>(_a, &Q3DGraphsWidgetItem::ambientLightStrengthChanged, 45))
            return;
        if (QtMocHelpers::indexOfMethod<void (Q3DGraphsWidgetItem::*)()>(_a, &Q3DGraphsWidgetItem::lightStrengthChanged, 46))
            return;
        if (QtMocHelpers::indexOfMethod<void (Q3DGraphsWidgetItem::*)()>(_a, &Q3DGraphsWidgetItem::shadowStrengthChanged, 47))
            return;
        if (QtMocHelpers::indexOfMethod<void (Q3DGraphsWidgetItem::*)()>(_a, &Q3DGraphsWidgetItem::lightColorChanged, 48))
            return;
        if (QtMocHelpers::indexOfMethod<void (Q3DGraphsWidgetItem::*)()>(_a, &Q3DGraphsWidgetItem::gridLineTypeChanged, 49))
            return;
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 4:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< Q3DScene* >(); break;
        case 0:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QGraphsTheme* >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QGraphsTheme**>(_v) = _t->activeTheme(); break;
        case 1: *reinterpret_cast<QtGraphs3D::SelectionFlags*>(_v) = _t->selectionMode(); break;
        case 2: *reinterpret_cast<QtGraphs3D::ShadowQuality*>(_v) = _t->shadowQuality(); break;
        case 3: *reinterpret_cast<QtGraphs3D::TransparencyTechnique*>(_v) = _t->transparencyTechnique(); break;
        case 4: *reinterpret_cast<Q3DScene**>(_v) = _t->scene(); break;
        case 5: *reinterpret_cast<bool*>(_v) = _t->measureFps(); break;
        case 6: *reinterpret_cast<int*>(_v) = _t->currentFps(); break;
        case 7: *reinterpret_cast<bool*>(_v) = _t->isOrthoProjection(); break;
        case 8: *reinterpret_cast<QtGraphs3D::ElementType*>(_v) = _t->selectedElement(); break;
        case 9: *reinterpret_cast<qreal*>(_v) = _t->aspectRatio(); break;
        case 10: *reinterpret_cast<QtGraphs3D::OptimizationHint*>(_v) = _t->optimizationHint(); break;
        case 11: *reinterpret_cast<bool*>(_v) = _t->isPolar(); break;
        case 12: *reinterpret_cast<float*>(_v) = _t->labelMargin(); break;
        case 13: *reinterpret_cast<float*>(_v) = _t->radialLabelOffset(); break;
        case 14: *reinterpret_cast<qreal*>(_v) = _t->horizontalAspectRatio(); break;
        case 15: *reinterpret_cast<QLocale*>(_v) = _t->locale(); break;
        case 16: *reinterpret_cast<QVector3D*>(_v) = _t->queriedGraphPosition(); break;
        case 17: *reinterpret_cast<qreal*>(_v) = _t->margin(); break;
        case 18: *reinterpret_cast<qreal*>(_v) = _t->cutoffMargin(); break;
        case 19: *reinterpret_cast<QtGraphs3D::CameraPreset*>(_v) = _t->cameraPreset(); break;
        case 20: *reinterpret_cast<float*>(_v) = _t->cameraXRotation(); break;
        case 21: *reinterpret_cast<float*>(_v) = _t->cameraYRotation(); break;
        case 22: *reinterpret_cast<float*>(_v) = _t->cameraZoomLevel(); break;
        case 23: *reinterpret_cast<float*>(_v) = _t->minCameraZoomLevel(); break;
        case 24: *reinterpret_cast<float*>(_v) = _t->maxCameraZoomLevel(); break;
        case 25: *reinterpret_cast<bool*>(_v) = _t->wrapCameraXRotation(); break;
        case 26: *reinterpret_cast<bool*>(_v) = _t->wrapCameraYRotation(); break;
        case 27: *reinterpret_cast<float*>(_v) = _t->minCameraXRotation(); break;
        case 28: *reinterpret_cast<float*>(_v) = _t->maxCameraXRotation(); break;
        case 29: *reinterpret_cast<float*>(_v) = _t->minCameraYRotation(); break;
        case 30: *reinterpret_cast<float*>(_v) = _t->maxCameraYRotation(); break;
        case 31: *reinterpret_cast<QVector3D*>(_v) = _t->cameraTargetPosition(); break;
        case 32: *reinterpret_cast<int*>(_v) = _t->msaaSamples(); break;
        case 33: *reinterpret_cast<bool*>(_v) = _t->isRotationEnabled(); break;
        case 34: *reinterpret_cast<bool*>(_v) = _t->isPanEnabled(); break;
        case 35: *reinterpret_cast<bool*>(_v) = _t->isPanModeEnabled(); break;
        case 36: *reinterpret_cast<bool*>(_v) = _t->isZoomAtTargetEnabled(); break;
        case 37: *reinterpret_cast<bool*>(_v) = _t->isSelectionEnabled(); break;
        case 38: *reinterpret_cast<bool*>(_v) = _t->isZoomEnabled(); break;
        case 39: *reinterpret_cast<QColor*>(_v) = _t->lightColor(); break;
        case 40: *reinterpret_cast<float*>(_v) = _t->ambientLightStrength(); break;
        case 41: *reinterpret_cast<float*>(_v) = _t->lightStrength(); break;
        case 42: *reinterpret_cast<float*>(_v) = _t->shadowStrength(); break;
        case 43: *reinterpret_cast<QtGraphs3D::GridLineType*>(_v) = _t->gridLineType(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setActiveTheme(*reinterpret_cast<QGraphsTheme**>(_v)); break;
        case 1: _t->setSelectionMode(*reinterpret_cast<QtGraphs3D::SelectionFlags*>(_v)); break;
        case 2: _t->setShadowQuality(*reinterpret_cast<QtGraphs3D::ShadowQuality*>(_v)); break;
        case 3: _t->setTransparencyTechnique(*reinterpret_cast<QtGraphs3D::TransparencyTechnique*>(_v)); break;
        case 5: _t->setMeasureFps(*reinterpret_cast<bool*>(_v)); break;
        case 7: _t->setOrthoProjection(*reinterpret_cast<bool*>(_v)); break;
        case 9: _t->setAspectRatio(*reinterpret_cast<qreal*>(_v)); break;
        case 10: _t->setOptimizationHint(*reinterpret_cast<QtGraphs3D::OptimizationHint*>(_v)); break;
        case 11: _t->setPolar(*reinterpret_cast<bool*>(_v)); break;
        case 12: _t->setLabelMargin(*reinterpret_cast<float*>(_v)); break;
        case 13: _t->setRadialLabelOffset(*reinterpret_cast<float*>(_v)); break;
        case 14: _t->setHorizontalAspectRatio(*reinterpret_cast<qreal*>(_v)); break;
        case 15: _t->setLocale(*reinterpret_cast<QLocale*>(_v)); break;
        case 17: _t->setMargin(*reinterpret_cast<qreal*>(_v)); break;
        case 18: _t->setCutoffMargin(*reinterpret_cast<qreal*>(_v)); break;
        case 19: _t->setCameraPreset(*reinterpret_cast<QtGraphs3D::CameraPreset*>(_v)); break;
        case 20: _t->setCameraXRotation(*reinterpret_cast<float*>(_v)); break;
        case 21: _t->setCameraYRotation(*reinterpret_cast<float*>(_v)); break;
        case 22: _t->setCameraZoomLevel(*reinterpret_cast<float*>(_v)); break;
        case 23: _t->setMinCameraZoomLevel(*reinterpret_cast<float*>(_v)); break;
        case 24: _t->setMaxCameraZoomLevel(*reinterpret_cast<float*>(_v)); break;
        case 25: _t->setWrapCameraXRotation(*reinterpret_cast<bool*>(_v)); break;
        case 26: _t->setWrapCameraYRotation(*reinterpret_cast<bool*>(_v)); break;
        case 27: _t->setMinCameraXRotation(*reinterpret_cast<float*>(_v)); break;
        case 28: _t->setMaxCameraXRotation(*reinterpret_cast<float*>(_v)); break;
        case 29: _t->setMinCameraYRotation(*reinterpret_cast<float*>(_v)); break;
        case 30: _t->setMaxCameraYRotation(*reinterpret_cast<float*>(_v)); break;
        case 31: _t->setCameraTargetPosition(*reinterpret_cast<QVector3D*>(_v)); break;
        case 32: _t->setMsaaSamples(*reinterpret_cast<int*>(_v)); break;
        case 33: _t->setRotationEnabled(*reinterpret_cast<bool*>(_v)); break;
        case 34: _t->setPanEnabled(*reinterpret_cast<bool*>(_v)); break;
        case 35: _t->setPanModeEnabled(*reinterpret_cast<bool*>(_v)); break;
        case 36: _t->setZoomAtTargetEnabled(*reinterpret_cast<bool*>(_v)); break;
        case 37: _t->setSelectionEnabled(*reinterpret_cast<bool*>(_v)); break;
        case 38: _t->setZoomEnabled(*reinterpret_cast<bool*>(_v)); break;
        case 39: _t->setLightColor(*reinterpret_cast<QColor*>(_v)); break;
        case 40: _t->setAmbientLightStrength(*reinterpret_cast<float*>(_v)); break;
        case 41: _t->setLightStrength(*reinterpret_cast<float*>(_v)); break;
        case 42: _t->setShadowStrength(*reinterpret_cast<float*>(_v)); break;
        case 43: _t->setGridLineType(*reinterpret_cast<QtGraphs3D::GridLineType*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *Q3DGraphsWidgetItem::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Q3DGraphsWidgetItem::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN19Q3DGraphsWidgetItemE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int Q3DGraphsWidgetItem::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 50)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 50;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 50)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 50;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 44;
    }
    return _id;
}

// SIGNAL 0
void Q3DGraphsWidgetItem::activeThemeChanged(QGraphsTheme * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void Q3DGraphsWidgetItem::shadowQualityChanged(QtGraphs3D::ShadowQuality _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void Q3DGraphsWidgetItem::transparencyTechniqueChanged(QtGraphs3D::TransparencyTechnique _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void Q3DGraphsWidgetItem::selectionModeChanged(const QtGraphs3D::SelectionFlags _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void Q3DGraphsWidgetItem::selectedElementChanged(QtGraphs3D::ElementType _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void Q3DGraphsWidgetItem::measureFpsChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void Q3DGraphsWidgetItem::currentFpsChanged(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}

// SIGNAL 7
void Q3DGraphsWidgetItem::orthoProjectionChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1);
}

// SIGNAL 8
void Q3DGraphsWidgetItem::aspectRatioChanged(qreal _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 8, nullptr, _t1);
}

// SIGNAL 9
void Q3DGraphsWidgetItem::optimizationHintChanged(QtGraphs3D::OptimizationHint _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 9, nullptr, _t1);
}

// SIGNAL 10
void Q3DGraphsWidgetItem::polarChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 10, nullptr, _t1);
}

// SIGNAL 11
void Q3DGraphsWidgetItem::labelMarginChanged(float _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 11, nullptr, _t1);
}

// SIGNAL 12
void Q3DGraphsWidgetItem::radialLabelOffsetChanged(float _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 12, nullptr, _t1);
}

// SIGNAL 13
void Q3DGraphsWidgetItem::horizontalAspectRatioChanged(qreal _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 13, nullptr, _t1);
}

// SIGNAL 14
void Q3DGraphsWidgetItem::localeChanged(const QLocale & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 14, nullptr, _t1);
}

// SIGNAL 15
void Q3DGraphsWidgetItem::queriedGraphPositionChanged(QVector3D _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 15, nullptr, _t1);
}

// SIGNAL 16
void Q3DGraphsWidgetItem::marginChanged(qreal _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 16, nullptr, _t1);
}

// SIGNAL 17
void Q3DGraphsWidgetItem::cutoffMarginChanged(qreal _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 17, nullptr, _t1);
}

// SIGNAL 18
void Q3DGraphsWidgetItem::cameraPresetChanged(QtGraphs3D::CameraPreset _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 18, nullptr, _t1);
}

// SIGNAL 19
void Q3DGraphsWidgetItem::cameraXRotationChanged(float _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 19, nullptr, _t1);
}

// SIGNAL 20
void Q3DGraphsWidgetItem::cameraYRotationChanged(float _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 20, nullptr, _t1);
}

// SIGNAL 21
void Q3DGraphsWidgetItem::cameraZoomLevelChanged(float _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 21, nullptr, _t1);
}

// SIGNAL 22
void Q3DGraphsWidgetItem::cameraTargetPositionChanged(QVector3D _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 22, nullptr, _t1);
}

// SIGNAL 23
void Q3DGraphsWidgetItem::minCameraZoomLevelChanged(float _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 23, nullptr, _t1);
}

// SIGNAL 24
void Q3DGraphsWidgetItem::maxCameraZoomLevelChanged(float _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 24, nullptr, _t1);
}

// SIGNAL 25
void Q3DGraphsWidgetItem::minCameraXRotationChanged(float _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 25, nullptr, _t1);
}

// SIGNAL 26
void Q3DGraphsWidgetItem::minCameraYRotationChanged(float _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 26, nullptr, _t1);
}

// SIGNAL 27
void Q3DGraphsWidgetItem::maxCameraXRotationChanged(float _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 27, nullptr, _t1);
}

// SIGNAL 28
void Q3DGraphsWidgetItem::maxCameraYRotationChanged(float _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 28, nullptr, _t1);
}

// SIGNAL 29
void Q3DGraphsWidgetItem::wrapCameraXRotationChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 29, nullptr, _t1);
}

// SIGNAL 30
void Q3DGraphsWidgetItem::wrapCameraYRotationChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 30, nullptr, _t1);
}

// SIGNAL 31
void Q3DGraphsWidgetItem::msaaSamplesChanged(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 31, nullptr, _t1);
}

// SIGNAL 32
void Q3DGraphsWidgetItem::tapped(QEventPoint _t1, Qt::MouseButton _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 32, nullptr, _t1, _t2);
}

// SIGNAL 33
void Q3DGraphsWidgetItem::doubleTapped(QEventPoint _t1, Qt::MouseButton _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 33, nullptr, _t1, _t2);
}

// SIGNAL 34
void Q3DGraphsWidgetItem::longPressed()
{
    QMetaObject::activate(this, &staticMetaObject, 34, nullptr);
}

// SIGNAL 35
void Q3DGraphsWidgetItem::dragged(QVector2D _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 35, nullptr, _t1);
}

// SIGNAL 36
void Q3DGraphsWidgetItem::wheel(QWheelEvent * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 36, nullptr, _t1);
}

// SIGNAL 37
void Q3DGraphsWidgetItem::pinch(qreal _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 37, nullptr, _t1);
}

// SIGNAL 38
void Q3DGraphsWidgetItem::mouseMove(QPoint _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 38, nullptr, _t1);
}

// SIGNAL 39
void Q3DGraphsWidgetItem::zoomEnabledChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 39, nullptr, _t1);
}

// SIGNAL 40
void Q3DGraphsWidgetItem::zoomAtTargetEnabledChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 40, nullptr, _t1);
}

// SIGNAL 41
void Q3DGraphsWidgetItem::rotationEnabledChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 41, nullptr, _t1);
}

// SIGNAL 42
void Q3DGraphsWidgetItem::panEnabledChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 42, nullptr, _t1);
}

// SIGNAL 43
void Q3DGraphsWidgetItem::panModeEnabledChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 43, nullptr, _t1);
}

// SIGNAL 44
void Q3DGraphsWidgetItem::selectionEnabledChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 44, nullptr, _t1);
}

// SIGNAL 45
void Q3DGraphsWidgetItem::ambientLightStrengthChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 45, nullptr);
}

// SIGNAL 46
void Q3DGraphsWidgetItem::lightStrengthChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 46, nullptr);
}

// SIGNAL 47
void Q3DGraphsWidgetItem::shadowStrengthChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 47, nullptr);
}

// SIGNAL 48
void Q3DGraphsWidgetItem::lightColorChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 48, nullptr);
}

// SIGNAL 49
void Q3DGraphsWidgetItem::gridLineTypeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 49, nullptr);
}
QT_WARNING_POP
