// Copyright (C) 2025 The Qt Company Ltd.
// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
// Qt-Security score:significant reason:default


#include "qgraphs3dlogging_p.h"


QT_BEGIN_NAMESPACE

Q_LOGGING_CATEGORY(lcGraphs3D, "qt.graphs3d.general")
Q_LOGGING_CATEGORY(lcSeries3D, "qt.graphs3d.series")
Q_LOGGING_CATEGORY(lcProperties3D, "qt.graphs3d.properties")
Q_LOGGING_CATEGORY(lcEvents3D, "qt.graphs3d.events")
Q_LOGGING_CATEGORY(lcInput3D, "qt.graphs3d.inputEvents")
Q_LOGGING_CATEGORY(lcCritical3D, "qt.graphs3d.critical")
Q_LOGGING_CATEGORY(lcAxis3D, "qt.graphs3d.axis.general")
Q_LOGGING_CATEGORY(lcAProperties3D, "qt.graphs3d.axis.properties")
Q_LOGGING_CATEGORY(lcGraphsScene3D, "qt.graphs3d.scene")
Q_LOGGING_CATEGORY(lcGraphsBars3D, "qt.graphs3d.bars")
Q_LOGGING_CATEGORY(lcGraphsScatter3D, "qt.graphs3d.scatter")
Q_LOGGING_CATEGORY(lcGraphsSurface3D, "qt.graphs3d.surface")

QT_END_NAMESPACE

