/****************************************************************************
** Meta object code from reading C++ file 'qbar3dseries.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../qtgraphs/src/graphs3d/data/qbar3dseries.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qbar3dseries.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN12QBar3DSeriesE_t {};
} // unnamed namespace

template <> constexpr inline auto QBar3DSeries::qt_create_metaobjectdata<qt_meta_tag_ZN12QBar3DSeriesE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QBar3DSeries",
        "QML.Element",
        "auto",
        "QML.Creatable",
        "false",
        "QML.UncreatableReason",
        "Trying to create uncreatable: QBar3DSeries, use Bar3DSeries instea"
        "d.",
        "dataProxyChanged",
        "",
        "QBarDataProxy*",
        "proxy",
        "selectedBarChanged",
        "QPoint",
        "position",
        "meshAngleChanged",
        "angle",
        "rowColorsChanged",
        "QList<QColor>",
        "rowcolors",
        "rowLabelsChanged",
        "columnLabelsChanged",
        "dataArrayChanged",
        "QBarDataArray",
        "array",
        "valueColoringEnabledChanged",
        "enabled",
        "customRowLabelsChanged",
        "newCustomRowLabels",
        "customColumnLabelsChanged",
        "newCustomColumnLabels",
        "rowAxisChanged",
        "QCategory3DAxis*",
        "axis",
        "valueAxisChanged",
        "QValue3DAxis*",
        "columnAxisChanged",
        "dataProxy",
        "selectedBar",
        "meshAngle",
        "rowColors",
        "rowLabels",
        "columnLabels",
        "customRowLabels",
        "customColumnLabels",
        "dataArray",
        "valueColoringEnabled",
        "rowAxis",
        "valueAxis",
        "columnAxis"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'dataProxyChanged'
        QtMocHelpers::SignalData<void(QBarDataProxy *)>(7, 8, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 10 },
        }}),
        // Signal 'selectedBarChanged'
        QtMocHelpers::SignalData<void(QPoint)>(11, 8, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 12, 13 },
        }}),
        // Signal 'meshAngleChanged'
        QtMocHelpers::SignalData<void(float)>(14, 8, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Float, 15 },
        }}),
        // Signal 'rowColorsChanged'
        QtMocHelpers::SignalData<void(const QList<QColor> &)>(16, 8, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 17, 18 },
        }}),
        // Signal 'rowLabelsChanged'
        QtMocHelpers::SignalData<void()>(19, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'columnLabelsChanged'
        QtMocHelpers::SignalData<void()>(20, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'dataArrayChanged'
        QtMocHelpers::SignalData<void(const QBarDataArray &)>(21, 8, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 22, 23 },
        }}),
        // Signal 'valueColoringEnabledChanged'
        QtMocHelpers::RevisionedSignalData<void(bool)>(24, 8, QMC::AccessPublic, 0x609, QMetaType::Void, {{
            { QMetaType::Bool, 25 },
        }}),
        // Signal 'customRowLabelsChanged'
        QtMocHelpers::RevisionedSignalData<void(const QStringList &)>(26, 8, QMC::AccessPublic, 0x60b, QMetaType::Void, {{
            { QMetaType::QStringList, 27 },
        }}),
        // Signal 'customColumnLabelsChanged'
        QtMocHelpers::RevisionedSignalData<void(const QStringList &)>(28, 8, QMC::AccessPublic, 0x60b, QMetaType::Void, {{
            { QMetaType::QStringList, 29 },
        }}),
        // Signal 'rowAxisChanged'
        QtMocHelpers::RevisionedSignalData<void(QCategory3DAxis *)>(30, 8, QMC::AccessPublic, 0x60b, QMetaType::Void, {{
            { 0x80000000 | 31, 32 },
        }}),
        // Signal 'valueAxisChanged'
        QtMocHelpers::RevisionedSignalData<void(QValue3DAxis *)>(33, 8, QMC::AccessPublic, 0x60b, QMetaType::Void, {{
            { 0x80000000 | 34, 32 },
        }}),
        // Signal 'columnAxisChanged'
        QtMocHelpers::RevisionedSignalData<void(QCategory3DAxis *)>(35, 8, QMC::AccessPublic, 0x60b, QMetaType::Void, {{
            { 0x80000000 | 31, 32 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'dataProxy'
        QtMocHelpers::PropertyData<QBarDataProxy*>(36, 0x80000000 | 9, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet | QMC::Final, 0),
        // property 'selectedBar'
        QtMocHelpers::PropertyData<QPoint>(37, 0x80000000 | 12, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 1),
        // property 'meshAngle'
        QtMocHelpers::PropertyData<float>(38, QMetaType::Float, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 2),
        // property 'rowColors'
        QtMocHelpers::PropertyData<QList<QColor>>(39, 0x80000000 | 17, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 3),
        // property 'rowLabels'
        QtMocHelpers::PropertyData<QStringList>(40, QMetaType::QStringList, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 4),
        // property 'columnLabels'
        QtMocHelpers::PropertyData<QStringList>(41, QMetaType::QStringList, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 5),
        // property 'customRowLabels'
        QtMocHelpers::PropertyData<QStringList>(42, QMetaType::QStringList, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 8, 0x60b),
        // property 'customColumnLabels'
        QtMocHelpers::PropertyData<QStringList>(43, QMetaType::QStringList, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 9, 0x60b),
        // property 'dataArray'
        QtMocHelpers::PropertyData<QBarDataArray>(44, 0x80000000 | 22, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet | QMC::Final, 6),
        // property 'valueColoringEnabled'
        QtMocHelpers::PropertyData<bool>(45, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 7, 0x609),
        // property 'rowAxis'
        QtMocHelpers::PropertyData<QCategory3DAxis*>(46, 0x80000000 | 31, QMC::DefaultPropertyFlags | QMC::Writable | QMC::Resettable | QMC::EnumOrFlag | QMC::StdCppSet, 10, 0x60b),
        // property 'valueAxis'
        QtMocHelpers::PropertyData<QValue3DAxis*>(47, 0x80000000 | 34, QMC::DefaultPropertyFlags | QMC::Writable | QMC::Resettable | QMC::EnumOrFlag | QMC::StdCppSet, 11, 0x60b),
        // property 'columnAxis'
        QtMocHelpers::PropertyData<QCategory3DAxis*>(48, 0x80000000 | 31, QMC::DefaultPropertyFlags | QMC::Writable | QMC::Resettable | QMC::EnumOrFlag | QMC::StdCppSet, 12, 0x60b),
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
            {    5,    6 },
    });
    return QtMocHelpers::metaObjectData<QBar3DSeries, void>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject QBar3DSeries::staticMetaObject = { {
    QMetaObject::SuperData::link<QAbstract3DSeries::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12QBar3DSeriesE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12QBar3DSeriesE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN12QBar3DSeriesE_t>.metaTypes,
    nullptr
} };

void QBar3DSeries::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QBar3DSeries *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->dataProxyChanged((*reinterpret_cast<std::add_pointer_t<QBarDataProxy*>>(_a[1]))); break;
        case 1: _t->selectedBarChanged((*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[1]))); break;
        case 2: _t->meshAngleChanged((*reinterpret_cast<std::add_pointer_t<float>>(_a[1]))); break;
        case 3: _t->rowColorsChanged((*reinterpret_cast<std::add_pointer_t<QList<QColor>>>(_a[1]))); break;
        case 4: _t->rowLabelsChanged(); break;
        case 5: _t->columnLabelsChanged(); break;
        case 6: _t->dataArrayChanged((*reinterpret_cast<std::add_pointer_t<QBarDataArray>>(_a[1]))); break;
        case 7: _t->valueColoringEnabledChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 8: _t->customRowLabelsChanged((*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[1]))); break;
        case 9: _t->customColumnLabelsChanged((*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[1]))); break;
        case 10: _t->rowAxisChanged((*reinterpret_cast<std::add_pointer_t<QCategory3DAxis*>>(_a[1]))); break;
        case 11: _t->valueAxisChanged((*reinterpret_cast<std::add_pointer_t<QValue3DAxis*>>(_a[1]))); break;
        case 12: _t->columnAxisChanged((*reinterpret_cast<std::add_pointer_t<QCategory3DAxis*>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QBarDataProxy* >(); break;
            }
            break;
        case 10:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QCategory3DAxis* >(); break;
            }
            break;
        case 11:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QValue3DAxis* >(); break;
            }
            break;
        case 12:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QCategory3DAxis* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (QBar3DSeries::*)(QBarDataProxy * )>(_a, &QBar3DSeries::dataProxyChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (QBar3DSeries::*)(QPoint )>(_a, &QBar3DSeries::selectedBarChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (QBar3DSeries::*)(float )>(_a, &QBar3DSeries::meshAngleChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (QBar3DSeries::*)(const QList<QColor> & )>(_a, &QBar3DSeries::rowColorsChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (QBar3DSeries::*)()>(_a, &QBar3DSeries::rowLabelsChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (QBar3DSeries::*)()>(_a, &QBar3DSeries::columnLabelsChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (QBar3DSeries::*)(const QBarDataArray & )>(_a, &QBar3DSeries::dataArrayChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (QBar3DSeries::*)(bool )>(_a, &QBar3DSeries::valueColoringEnabledChanged, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (QBar3DSeries::*)(const QStringList & )>(_a, &QBar3DSeries::customRowLabelsChanged, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (QBar3DSeries::*)(const QStringList & )>(_a, &QBar3DSeries::customColumnLabelsChanged, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (QBar3DSeries::*)(QCategory3DAxis * )>(_a, &QBar3DSeries::rowAxisChanged, 10))
            return;
        if (QtMocHelpers::indexOfMethod<void (QBar3DSeries::*)(QValue3DAxis * )>(_a, &QBar3DSeries::valueAxisChanged, 11))
            return;
        if (QtMocHelpers::indexOfMethod<void (QBar3DSeries::*)(QCategory3DAxis * )>(_a, &QBar3DSeries::columnAxisChanged, 12))
            return;
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 0:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QBarDataProxy* >(); break;
        case 12:
        case 10:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QCategory3DAxis* >(); break;
        case 11:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QValue3DAxis* >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QBarDataProxy**>(_v) = _t->dataProxy(); break;
        case 1: *reinterpret_cast<QPoint*>(_v) = _t->selectedBar(); break;
        case 2: *reinterpret_cast<float*>(_v) = _t->meshAngle(); break;
        case 3: *reinterpret_cast<QList<QColor>*>(_v) = _t->rowColors(); break;
        case 4: *reinterpret_cast<QStringList*>(_v) = _t->rowLabels(); break;
        case 5: *reinterpret_cast<QStringList*>(_v) = _t->columnLabels(); break;
        case 6: *reinterpret_cast<QStringList*>(_v) = _t->customRowLabels(); break;
        case 7: *reinterpret_cast<QStringList*>(_v) = _t->customColumnLabels(); break;
        case 8: *reinterpret_cast<QBarDataArray*>(_v) = _t->dataArray(); break;
        case 9: *reinterpret_cast<bool*>(_v) = _t->isValueColoringEnabled(); break;
        case 10: *reinterpret_cast<QCategory3DAxis**>(_v) = _t->rowAxis(); break;
        case 11: *reinterpret_cast<QValue3DAxis**>(_v) = _t->valueAxis(); break;
        case 12: *reinterpret_cast<QCategory3DAxis**>(_v) = _t->columnAxis(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setDataProxy(*reinterpret_cast<QBarDataProxy**>(_v)); break;
        case 1: _t->setSelectedBar(*reinterpret_cast<QPoint*>(_v)); break;
        case 2: _t->setMeshAngle(*reinterpret_cast<float*>(_v)); break;
        case 3: _t->setRowColors(*reinterpret_cast<QList<QColor>*>(_v)); break;
        case 4: _t->setRowLabels(*reinterpret_cast<QStringList*>(_v)); break;
        case 5: _t->setColumnLabels(*reinterpret_cast<QStringList*>(_v)); break;
        case 6: _t->setCustomRowLabels(*reinterpret_cast<QStringList*>(_v)); break;
        case 7: _t->setCustomColumnLabels(*reinterpret_cast<QStringList*>(_v)); break;
        case 8: _t->setDataArray(*reinterpret_cast<QBarDataArray*>(_v)); break;
        case 9: _t->setValueColoringEnabled(*reinterpret_cast<bool*>(_v)); break;
        case 10: _t->setRowAxis(*reinterpret_cast<QCategory3DAxis**>(_v)); break;
        case 11: _t->setValueAxis(*reinterpret_cast<QValue3DAxis**>(_v)); break;
        case 12: _t->setColumnAxis(*reinterpret_cast<QCategory3DAxis**>(_v)); break;
        default: break;
        }
    }
    if (_c == QMetaObject::ResetProperty) {
        switch (_id) {
        case 10: _t->resetRowAxis(); break;
        case 11: _t->resetValueAxis(); break;
        case 12: _t->resetColumnAxis(); break;
        default: break;
        }
    }
}

const QMetaObject *QBar3DSeries::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QBar3DSeries::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12QBar3DSeriesE_t>.strings))
        return static_cast<void*>(this);
    return QAbstract3DSeries::qt_metacast(_clname);
}

int QBar3DSeries::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QAbstract3DSeries::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 13)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 13;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 13)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 13;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 13;
    }
    return _id;
}

// SIGNAL 0
void QBar3DSeries::dataProxyChanged(QBarDataProxy * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void QBar3DSeries::selectedBarChanged(QPoint _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void QBar3DSeries::meshAngleChanged(float _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void QBar3DSeries::rowColorsChanged(const QList<QColor> & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void QBar3DSeries::rowLabelsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void QBar3DSeries::columnLabelsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void QBar3DSeries::dataArrayChanged(const QBarDataArray & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}

// SIGNAL 7
void QBar3DSeries::valueColoringEnabledChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1);
}

// SIGNAL 8
void QBar3DSeries::customRowLabelsChanged(const QStringList & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 8, nullptr, _t1);
}

// SIGNAL 9
void QBar3DSeries::customColumnLabelsChanged(const QStringList & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 9, nullptr, _t1);
}

// SIGNAL 10
void QBar3DSeries::rowAxisChanged(QCategory3DAxis * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 10, nullptr, _t1);
}

// SIGNAL 11
void QBar3DSeries::valueAxisChanged(QValue3DAxis * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 11, nullptr, _t1);
}

// SIGNAL 12
void QBar3DSeries::columnAxisChanged(QCategory3DAxis * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 12, nullptr, _t1);
}
QT_WARNING_POP
