/****************************************************************************
** Meta object code from reading C++ file 'qquickgraphsbars_p.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../qtgraphs/src/graphs3d/qml/qquickgraphsbars_p.h"
#include <QtGui/qtextcursor.h>
#include <QtNetwork/QSslError>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qquickgraphsbars_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN16QQuickGraphsBarsE_t {};
} // unnamed namespace

template <> constexpr inline auto QQuickGraphsBars::qt_create_metaobjectdata<qt_meta_tag_ZN16QQuickGraphsBarsE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QQuickGraphsBars",
        "DefaultProperty",
        "seriesList",
        "QML.Element",
        "Bars3D",
        "rowAxisChanged",
        "",
        "QCategory3DAxis*",
        "axis",
        "valueAxisChanged",
        "QValue3DAxis*",
        "columnAxisChanged",
        "multiSeriesUniformChanged",
        "uniform",
        "barThicknessChanged",
        "thicknessRatio",
        "barSpacingChanged",
        "QSizeF",
        "spacing",
        "barSpacingRelativeChanged",
        "relative",
        "barSeriesMarginChanged",
        "margin",
        "meshFileNameChanged",
        "filename",
        "primarySeriesChanged",
        "QBar3DSeries*",
        "series",
        "selectedSeriesChanged",
        "floorLevelChanged",
        "level",
        "sliceImageChanged",
        "QImage",
        "image",
        "handleAxisXChanged",
        "QAbstract3DAxis*",
        "handleAxisYChanged",
        "handleAxisZChanged",
        "handleSeriesMeshChanged",
        "QAbstract3DSeries::Mesh",
        "mesh",
        "handleMeshSmoothChanged",
        "enable",
        "handleCameraRotationChanged",
        "handleArrayReset",
        "handleRowsAdded",
        "qsizetype",
        "startIndex",
        "count",
        "handleRowsChanged",
        "handleRowsRemoved",
        "handleRowsInserted",
        "handleItemChanged",
        "rowIndex",
        "columnIndex",
        "handleDataRowLabelsChanged",
        "handleDataColumnLabelsChanged",
        "handleRowColorsChanged",
        "handleValueColoringChanged",
        "addSeries",
        "removeSeries",
        "insertSeries",
        "index",
        "clearSelection",
        "renderSliceToImage",
        "requestedIndex",
        "QtGraphs3D::SliceCaptureType",
        "sliceType",
        "QUrl",
        "filePath",
        "rowAxis",
        "valueAxis",
        "columnAxis",
        "multiSeriesUniform",
        "barThickness",
        "barSpacing",
        "barSpacingRelative",
        "barSeriesMargin",
        "QQmlListProperty<QBar3DSeries>",
        "selectedSeries",
        "primarySeries",
        "floorLevel"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'rowAxisChanged'
        QtMocHelpers::SignalData<void(QCategory3DAxis *)>(5, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 8 },
        }}),
        // Signal 'valueAxisChanged'
        QtMocHelpers::SignalData<void(QValue3DAxis *)>(9, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 10, 8 },
        }}),
        // Signal 'columnAxisChanged'
        QtMocHelpers::SignalData<void(QCategory3DAxis *)>(11, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 8 },
        }}),
        // Signal 'multiSeriesUniformChanged'
        QtMocHelpers::SignalData<void(bool)>(12, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 13 },
        }}),
        // Signal 'barThicknessChanged'
        QtMocHelpers::SignalData<void(float)>(14, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Float, 15 },
        }}),
        // Signal 'barSpacingChanged'
        QtMocHelpers::SignalData<void(QSizeF)>(16, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 17, 18 },
        }}),
        // Signal 'barSpacingRelativeChanged'
        QtMocHelpers::SignalData<void(bool)>(19, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 20 },
        }}),
        // Signal 'barSeriesMarginChanged'
        QtMocHelpers::SignalData<void(QSizeF)>(21, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 17, 22 },
        }}),
        // Signal 'meshFileNameChanged'
        QtMocHelpers::SignalData<void(const QString &)>(23, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 24 },
        }}),
        // Signal 'primarySeriesChanged'
        QtMocHelpers::SignalData<void(QBar3DSeries *)>(25, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 26, 27 },
        }}),
        // Signal 'selectedSeriesChanged'
        QtMocHelpers::SignalData<void(QBar3DSeries *)>(28, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 26, 27 },
        }}),
        // Signal 'floorLevelChanged'
        QtMocHelpers::SignalData<void(float)>(29, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Float, 30 },
        }}),
        // Signal 'sliceImageChanged'
        QtMocHelpers::RevisionedSignalData<void(const QImage &)>(31, 6, QMC::AccessPublic, 0x60a, QMetaType::Void, {{
            { 0x80000000 | 32, 33 },
        }}),
        // Slot 'handleAxisXChanged'
        QtMocHelpers::SlotData<void(QAbstract3DAxis *)>(34, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 35, 8 },
        }}),
        // Slot 'handleAxisYChanged'
        QtMocHelpers::SlotData<void(QAbstract3DAxis *)>(36, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 35, 8 },
        }}),
        // Slot 'handleAxisZChanged'
        QtMocHelpers::SlotData<void(QAbstract3DAxis *)>(37, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 35, 8 },
        }}),
        // Slot 'handleSeriesMeshChanged'
        QtMocHelpers::SlotData<void(QAbstract3DSeries::Mesh)>(38, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 39, 40 },
        }}),
        // Slot 'handleMeshSmoothChanged'
        QtMocHelpers::SlotData<void(bool)>(41, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 42 },
        }}),
        // Slot 'handleCameraRotationChanged'
        QtMocHelpers::SlotData<void()>(43, 6, QMC::AccessPublic, QMetaType::Void),
        // Slot 'handleArrayReset'
        QtMocHelpers::SlotData<void()>(44, 6, QMC::AccessPublic, QMetaType::Void),
        // Slot 'handleRowsAdded'
        QtMocHelpers::SlotData<void(qsizetype, qsizetype)>(45, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 46, 47 }, { 0x80000000 | 46, 48 },
        }}),
        // Slot 'handleRowsChanged'
        QtMocHelpers::SlotData<void(qsizetype, qsizetype)>(49, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 46, 47 }, { 0x80000000 | 46, 48 },
        }}),
        // Slot 'handleRowsRemoved'
        QtMocHelpers::SlotData<void(qsizetype, qsizetype)>(50, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 46, 47 }, { 0x80000000 | 46, 48 },
        }}),
        // Slot 'handleRowsInserted'
        QtMocHelpers::SlotData<void(qsizetype, qsizetype)>(51, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 46, 47 }, { 0x80000000 | 46, 48 },
        }}),
        // Slot 'handleItemChanged'
        QtMocHelpers::SlotData<void(qsizetype, qsizetype)>(52, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 46, 53 }, { 0x80000000 | 46, 54 },
        }}),
        // Slot 'handleDataRowLabelsChanged'
        QtMocHelpers::SlotData<void()>(55, 6, QMC::AccessPublic, QMetaType::Void),
        // Slot 'handleDataColumnLabelsChanged'
        QtMocHelpers::SlotData<void()>(56, 6, QMC::AccessPublic, QMetaType::Void),
        // Slot 'handleRowColorsChanged'
        QtMocHelpers::SlotData<void()>(57, 6, QMC::AccessPublic, QMetaType::Void),
        // Slot 'handleValueColoringChanged'
        QtMocHelpers::SlotData<void()>(58, 6, QMC::AccessPublic, QMetaType::Void),
        // Method 'addSeries'
        QtMocHelpers::MethodData<void(QBar3DSeries *)>(59, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 26, 27 },
        }}),
        // Method 'removeSeries'
        QtMocHelpers::MethodData<void(QBar3DSeries *)>(60, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 26, 27 },
        }}),
        // Method 'insertSeries'
        QtMocHelpers::MethodData<void(qsizetype, QBar3DSeries *)>(61, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 46, 62 }, { 0x80000000 | 26, 27 },
        }}),
        // Method 'clearSelection'
        QtMocHelpers::MethodData<void()>(63, 6, QMC::AccessPublic, QMetaType::Void),
        // Method 'renderSliceToImage'
        QtMocHelpers::RevisionedMethodData<void(int, QtGraphs3D::SliceCaptureType, const QUrl &)>(64, 6, QMC::AccessPublic, 0x60a, QMetaType::Void, {{
            { QMetaType::Int, 65 }, { 0x80000000 | 66, 67 }, { 0x80000000 | 68, 69 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'rowAxis'
        QtMocHelpers::PropertyData<QCategory3DAxis*>(70, 0x80000000 | 7, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 0),
        // property 'valueAxis'
        QtMocHelpers::PropertyData<QValue3DAxis*>(71, 0x80000000 | 10, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 1),
        // property 'columnAxis'
        QtMocHelpers::PropertyData<QCategory3DAxis*>(72, 0x80000000 | 7, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 2),
        // property 'multiSeriesUniform'
        QtMocHelpers::PropertyData<bool>(73, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 3),
        // property 'barThickness'
        QtMocHelpers::PropertyData<float>(74, QMetaType::Float, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 4),
        // property 'barSpacing'
        QtMocHelpers::PropertyData<QSizeF>(75, 0x80000000 | 17, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 5),
        // property 'barSpacingRelative'
        QtMocHelpers::PropertyData<bool>(76, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 6),
        // property 'barSeriesMargin'
        QtMocHelpers::PropertyData<QSizeF>(77, 0x80000000 | 17, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 7),
        // property 'seriesList'
        QtMocHelpers::PropertyData<QQmlListProperty<QBar3DSeries>>(2, 0x80000000 | 78, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'selectedSeries'
        QtMocHelpers::PropertyData<QBar3DSeries*>(79, 0x80000000 | 26, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 10),
        // property 'primarySeries'
        QtMocHelpers::PropertyData<QBar3DSeries*>(80, 0x80000000 | 26, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 9),
        // property 'floorLevel'
        QtMocHelpers::PropertyData<float>(81, QMetaType::Float, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 11),
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
    });
    return QtMocHelpers::metaObjectData<QQuickGraphsBars, void>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject QQuickGraphsBars::staticMetaObject = { {
    QMetaObject::SuperData::link<QQuickGraphsItem::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16QQuickGraphsBarsE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16QQuickGraphsBarsE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN16QQuickGraphsBarsE_t>.metaTypes,
    nullptr
} };

void QQuickGraphsBars::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QQuickGraphsBars *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->rowAxisChanged((*reinterpret_cast<std::add_pointer_t<QCategory3DAxis*>>(_a[1]))); break;
        case 1: _t->valueAxisChanged((*reinterpret_cast<std::add_pointer_t<QValue3DAxis*>>(_a[1]))); break;
        case 2: _t->columnAxisChanged((*reinterpret_cast<std::add_pointer_t<QCategory3DAxis*>>(_a[1]))); break;
        case 3: _t->multiSeriesUniformChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 4: _t->barThicknessChanged((*reinterpret_cast<std::add_pointer_t<float>>(_a[1]))); break;
        case 5: _t->barSpacingChanged((*reinterpret_cast<std::add_pointer_t<QSizeF>>(_a[1]))); break;
        case 6: _t->barSpacingRelativeChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 7: _t->barSeriesMarginChanged((*reinterpret_cast<std::add_pointer_t<QSizeF>>(_a[1]))); break;
        case 8: _t->meshFileNameChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 9: _t->primarySeriesChanged((*reinterpret_cast<std::add_pointer_t<QBar3DSeries*>>(_a[1]))); break;
        case 10: _t->selectedSeriesChanged((*reinterpret_cast<std::add_pointer_t<QBar3DSeries*>>(_a[1]))); break;
        case 11: _t->floorLevelChanged((*reinterpret_cast<std::add_pointer_t<float>>(_a[1]))); break;
        case 12: _t->sliceImageChanged((*reinterpret_cast<std::add_pointer_t<QImage>>(_a[1]))); break;
        case 13: _t->handleAxisXChanged((*reinterpret_cast<std::add_pointer_t<QAbstract3DAxis*>>(_a[1]))); break;
        case 14: _t->handleAxisYChanged((*reinterpret_cast<std::add_pointer_t<QAbstract3DAxis*>>(_a[1]))); break;
        case 15: _t->handleAxisZChanged((*reinterpret_cast<std::add_pointer_t<QAbstract3DAxis*>>(_a[1]))); break;
        case 16: _t->handleSeriesMeshChanged((*reinterpret_cast<std::add_pointer_t<QAbstract3DSeries::Mesh>>(_a[1]))); break;
        case 17: _t->handleMeshSmoothChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 18: _t->handleCameraRotationChanged(); break;
        case 19: _t->handleArrayReset(); break;
        case 20: _t->handleRowsAdded((*reinterpret_cast<std::add_pointer_t<qsizetype>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<qsizetype>>(_a[2]))); break;
        case 21: _t->handleRowsChanged((*reinterpret_cast<std::add_pointer_t<qsizetype>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<qsizetype>>(_a[2]))); break;
        case 22: _t->handleRowsRemoved((*reinterpret_cast<std::add_pointer_t<qsizetype>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<qsizetype>>(_a[2]))); break;
        case 23: _t->handleRowsInserted((*reinterpret_cast<std::add_pointer_t<qsizetype>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<qsizetype>>(_a[2]))); break;
        case 24: _t->handleItemChanged((*reinterpret_cast<std::add_pointer_t<qsizetype>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<qsizetype>>(_a[2]))); break;
        case 25: _t->handleDataRowLabelsChanged(); break;
        case 26: _t->handleDataColumnLabelsChanged(); break;
        case 27: _t->handleRowColorsChanged(); break;
        case 28: _t->handleValueColoringChanged(); break;
        case 29: _t->addSeries((*reinterpret_cast<std::add_pointer_t<QBar3DSeries*>>(_a[1]))); break;
        case 30: _t->removeSeries((*reinterpret_cast<std::add_pointer_t<QBar3DSeries*>>(_a[1]))); break;
        case 31: _t->insertSeries((*reinterpret_cast<std::add_pointer_t<qsizetype>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QBar3DSeries*>>(_a[2]))); break;
        case 32: _t->clearSelection(); break;
        case 33: _t->renderSliceToImage((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QtGraphs3D::SliceCaptureType>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[3]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QCategory3DAxis* >(); break;
            }
            break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QValue3DAxis* >(); break;
            }
            break;
        case 2:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QCategory3DAxis* >(); break;
            }
            break;
        case 9:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QBar3DSeries* >(); break;
            }
            break;
        case 10:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QBar3DSeries* >(); break;
            }
            break;
        case 13:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QAbstract3DAxis* >(); break;
            }
            break;
        case 14:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QAbstract3DAxis* >(); break;
            }
            break;
        case 15:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QAbstract3DAxis* >(); break;
            }
            break;
        case 29:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QBar3DSeries* >(); break;
            }
            break;
        case 30:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QBar3DSeries* >(); break;
            }
            break;
        case 31:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QBar3DSeries* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (QQuickGraphsBars::*)(QCategory3DAxis * )>(_a, &QQuickGraphsBars::rowAxisChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (QQuickGraphsBars::*)(QValue3DAxis * )>(_a, &QQuickGraphsBars::valueAxisChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (QQuickGraphsBars::*)(QCategory3DAxis * )>(_a, &QQuickGraphsBars::columnAxisChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (QQuickGraphsBars::*)(bool )>(_a, &QQuickGraphsBars::multiSeriesUniformChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (QQuickGraphsBars::*)(float )>(_a, &QQuickGraphsBars::barThicknessChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (QQuickGraphsBars::*)(QSizeF )>(_a, &QQuickGraphsBars::barSpacingChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (QQuickGraphsBars::*)(bool )>(_a, &QQuickGraphsBars::barSpacingRelativeChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (QQuickGraphsBars::*)(QSizeF )>(_a, &QQuickGraphsBars::barSeriesMarginChanged, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (QQuickGraphsBars::*)(const QString & )>(_a, &QQuickGraphsBars::meshFileNameChanged, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (QQuickGraphsBars::*)(QBar3DSeries * )>(_a, &QQuickGraphsBars::primarySeriesChanged, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (QQuickGraphsBars::*)(QBar3DSeries * )>(_a, &QQuickGraphsBars::selectedSeriesChanged, 10))
            return;
        if (QtMocHelpers::indexOfMethod<void (QQuickGraphsBars::*)(float )>(_a, &QQuickGraphsBars::floorLevelChanged, 11))
            return;
        if (QtMocHelpers::indexOfMethod<void (QQuickGraphsBars::*)(const QImage & )>(_a, &QQuickGraphsBars::sliceImageChanged, 12))
            return;
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 10:
        case 9:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QBar3DSeries* >(); break;
        case 2:
        case 0:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QCategory3DAxis* >(); break;
        case 1:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QValue3DAxis* >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QCategory3DAxis**>(_v) = _t->rowAxis(); break;
        case 1: *reinterpret_cast<QValue3DAxis**>(_v) = _t->valueAxis(); break;
        case 2: *reinterpret_cast<QCategory3DAxis**>(_v) = _t->columnAxis(); break;
        case 3: *reinterpret_cast<bool*>(_v) = _t->isMultiSeriesUniform(); break;
        case 4: *reinterpret_cast<float*>(_v) = _t->barThickness(); break;
        case 5: *reinterpret_cast<QSizeF*>(_v) = _t->barSpacing(); break;
        case 6: *reinterpret_cast<bool*>(_v) = _t->isBarSpacingRelative(); break;
        case 7: *reinterpret_cast<QSizeF*>(_v) = _t->barSeriesMargin(); break;
        case 8: *reinterpret_cast<QQmlListProperty<QBar3DSeries>*>(_v) = _t->seriesList(); break;
        case 9: *reinterpret_cast<QBar3DSeries**>(_v) = _t->selectedSeries(); break;
        case 10: *reinterpret_cast<QBar3DSeries**>(_v) = _t->primarySeries(); break;
        case 11: *reinterpret_cast<float*>(_v) = _t->floorLevel(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setRowAxis(*reinterpret_cast<QCategory3DAxis**>(_v)); break;
        case 1: _t->setValueAxis(*reinterpret_cast<QValue3DAxis**>(_v)); break;
        case 2: _t->setColumnAxis(*reinterpret_cast<QCategory3DAxis**>(_v)); break;
        case 3: _t->setMultiSeriesUniform(*reinterpret_cast<bool*>(_v)); break;
        case 4: _t->setBarThickness(*reinterpret_cast<float*>(_v)); break;
        case 5: _t->setBarSpacing(*reinterpret_cast<QSizeF*>(_v)); break;
        case 6: _t->setBarSpacingRelative(*reinterpret_cast<bool*>(_v)); break;
        case 7: _t->setBarSeriesMargin(*reinterpret_cast<QSizeF*>(_v)); break;
        case 10: _t->setPrimarySeries(*reinterpret_cast<QBar3DSeries**>(_v)); break;
        case 11: _t->setFloorLevel(*reinterpret_cast<float*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *QQuickGraphsBars::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QQuickGraphsBars::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16QQuickGraphsBarsE_t>.strings))
        return static_cast<void*>(this);
    return QQuickGraphsItem::qt_metacast(_clname);
}

int QQuickGraphsBars::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QQuickGraphsItem::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 34)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 34;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 34)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 34;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 12;
    }
    return _id;
}

// SIGNAL 0
void QQuickGraphsBars::rowAxisChanged(QCategory3DAxis * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void QQuickGraphsBars::valueAxisChanged(QValue3DAxis * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void QQuickGraphsBars::columnAxisChanged(QCategory3DAxis * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void QQuickGraphsBars::multiSeriesUniformChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void QQuickGraphsBars::barThicknessChanged(float _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void QQuickGraphsBars::barSpacingChanged(QSizeF _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void QQuickGraphsBars::barSpacingRelativeChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}

// SIGNAL 7
void QQuickGraphsBars::barSeriesMarginChanged(QSizeF _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1);
}

// SIGNAL 8
void QQuickGraphsBars::meshFileNameChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 8, nullptr, _t1);
}

// SIGNAL 9
void QQuickGraphsBars::primarySeriesChanged(QBar3DSeries * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 9, nullptr, _t1);
}

// SIGNAL 10
void QQuickGraphsBars::selectedSeriesChanged(QBar3DSeries * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 10, nullptr, _t1);
}

// SIGNAL 11
void QQuickGraphsBars::floorLevelChanged(float _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 11, nullptr, _t1);
}

// SIGNAL 12
void QQuickGraphsBars::sliceImageChanged(const QImage & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 12, nullptr, _t1);
}
QT_WARNING_POP
