/****************************************************************************
** Meta object code from reading C++ file 'qabstractaxis.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../qtgraphs/src/graphs2d/axis/qabstractaxis.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qabstractaxis.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN13QAbstractAxisE_t {};
} // unnamed namespace

template <> constexpr inline auto QAbstractAxis::qt_create_metaobjectdata<qt_meta_tag_ZN13QAbstractAxisE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QAbstractAxis",
        "RegisterEnumClassesUnscoped",
        "false",
        "QML.Foreign",
        "QML.Creatable",
        "QML.UncreatableReason",
        "",
        "QML.Element",
        "AbstractAxis",
        "visibleChanged",
        "visible",
        "lineVisibleChanged",
        "labelsVisibleChanged",
        "labelsAngleChanged",
        "angle",
        "labelDelegateChanged",
        "gridVisibleChanged",
        "subGridVisibleChanged",
        "titleTextChanged",
        "title",
        "titleColorChanged",
        "QColor",
        "color",
        "titleVisibleChanged",
        "titleFontChanged",
        "QFont",
        "font",
        "alignmentChanged",
        "Qt::Alignment",
        "alignment",
        "textElideModeChanged",
        "Qt::TextElideMode",
        "elideMode",
        "subColorChanged",
        "newColor",
        "colorChanged",
        "update",
        "rangeChanged",
        "min",
        "max",
        "lineVisible",
        "labelsVisible",
        "labelsAngle",
        "labelDelegate",
        "QQmlComponent*",
        "gridVisible",
        "subGridVisible",
        "titleText",
        "titleColor",
        "titleVisible",
        "titleFont",
        "textElideMode",
        "subColor",
        "AxisType",
        "Value",
        "BarCategory",
        "DateTime",
        "LogValue"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'visibleChanged'
        QtMocHelpers::SignalData<void(bool)>(9, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 10 },
        }}),
        // Signal 'lineVisibleChanged'
        QtMocHelpers::SignalData<void(bool)>(11, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 10 },
        }}),
        // Signal 'labelsVisibleChanged'
        QtMocHelpers::SignalData<void(bool)>(12, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 10 },
        }}),
        // Signal 'labelsAngleChanged'
        QtMocHelpers::SignalData<void(qreal)>(13, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QReal, 14 },
        }}),
        // Signal 'labelDelegateChanged'
        QtMocHelpers::SignalData<void()>(15, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'gridVisibleChanged'
        QtMocHelpers::SignalData<void(bool)>(16, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 10 },
        }}),
        // Signal 'subGridVisibleChanged'
        QtMocHelpers::SignalData<void(bool)>(17, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 10 },
        }}),
        // Signal 'titleTextChanged'
        QtMocHelpers::SignalData<void(const QString &)>(18, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 19 },
        }}),
        // Signal 'titleColorChanged'
        QtMocHelpers::SignalData<void(QColor)>(20, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 21, 22 },
        }}),
        // Signal 'titleVisibleChanged'
        QtMocHelpers::SignalData<void(bool)>(23, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 10 },
        }}),
        // Signal 'titleFontChanged'
        QtMocHelpers::SignalData<void(const QFont &)>(24, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 25, 26 },
        }}),
        // Signal 'alignmentChanged'
        QtMocHelpers::RevisionedSignalData<void(Qt::Alignment)>(27, 6, QMC::AccessPublic, 0x609, QMetaType::Void, {{
            { 0x80000000 | 28, 29 },
        }}),
        // Signal 'textElideModeChanged'
        QtMocHelpers::RevisionedSignalData<void(Qt::TextElideMode)>(30, 6, QMC::AccessPublic, 0x60a, QMetaType::Void, {{
            { 0x80000000 | 31, 32 },
        }}),
        // Signal 'subColorChanged'
        QtMocHelpers::RevisionedSignalData<void(QColor)>(33, 6, QMC::AccessPublic, 0x60b, QMetaType::Void, {{
            { 0x80000000 | 21, 34 },
        }}),
        // Signal 'colorChanged'
        QtMocHelpers::RevisionedSignalData<void(QColor)>(35, 6, QMC::AccessPublic, 0x60b, QMetaType::Void, {{
            { 0x80000000 | 21, 34 },
        }}),
        // Signal 'update'
        QtMocHelpers::SignalData<void()>(36, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'rangeChanged'
        QtMocHelpers::SignalData<void(qreal, qreal)>(37, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QReal, 38 }, { QMetaType::QReal, 39 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'visible'
        QtMocHelpers::PropertyData<bool>(10, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 0),
        // property 'lineVisible'
        QtMocHelpers::PropertyData<bool>(40, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 1),
        // property 'labelsVisible'
        QtMocHelpers::PropertyData<bool>(41, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 2),
        // property 'labelsAngle'
        QtMocHelpers::PropertyData<qreal>(42, QMetaType::QReal, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 3),
        // property 'labelDelegate'
        QtMocHelpers::PropertyData<QQmlComponent*>(43, 0x80000000 | 44, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet | QMC::Final, 4),
        // property 'gridVisible'
        QtMocHelpers::PropertyData<bool>(45, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 5),
        // property 'subGridVisible'
        QtMocHelpers::PropertyData<bool>(46, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 6),
        // property 'titleText'
        QtMocHelpers::PropertyData<QString>(47, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 7),
        // property 'titleColor'
        QtMocHelpers::PropertyData<QColor>(48, 0x80000000 | 21, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet | QMC::Final, 8),
        // property 'titleVisible'
        QtMocHelpers::PropertyData<bool>(49, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 9),
        // property 'titleFont'
        QtMocHelpers::PropertyData<QFont>(50, 0x80000000 | 25, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet | QMC::Final, 10),
        // property 'alignment'
        QtMocHelpers::PropertyData<Qt::Alignment>(29, 0x80000000 | 28, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 11, 0x609),
        // property 'textElideMode'
        QtMocHelpers::PropertyData<Qt::TextElideMode>(51, 0x80000000 | 31, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 12, 0x60a),
        // property 'color'
        QtMocHelpers::PropertyData<QColor>(22, 0x80000000 | 21, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 14, 0x60b),
        // property 'subColor'
        QtMocHelpers::PropertyData<QColor>(52, 0x80000000 | 21, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 13, 0x60b),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'AxisType'
        QtMocHelpers::EnumData<enum AxisType>(53, 53, QMC::EnumIsScoped).add({
            {   54, AxisType::Value },
            {   55, AxisType::BarCategory },
            {   56, AxisType::DateTime },
            {   57, AxisType::LogValue },
        }),
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    0 },
            {    4,    2 },
            {    5,    6 },
            {    7,    8 },
    });
    return QtMocHelpers::metaObjectData<QAbstractAxis, void>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject QAbstractAxis::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13QAbstractAxisE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13QAbstractAxisE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN13QAbstractAxisE_t>.metaTypes,
    nullptr
} };

void QAbstractAxis::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QAbstractAxis *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->visibleChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 1: _t->lineVisibleChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 2: _t->labelsVisibleChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 3: _t->labelsAngleChanged((*reinterpret_cast<std::add_pointer_t<qreal>>(_a[1]))); break;
        case 4: _t->labelDelegateChanged(); break;
        case 5: _t->gridVisibleChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 6: _t->subGridVisibleChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 7: _t->titleTextChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 8: _t->titleColorChanged((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        case 9: _t->titleVisibleChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 10: _t->titleFontChanged((*reinterpret_cast<std::add_pointer_t<QFont>>(_a[1]))); break;
        case 11: _t->alignmentChanged((*reinterpret_cast<std::add_pointer_t<Qt::Alignment>>(_a[1]))); break;
        case 12: _t->textElideModeChanged((*reinterpret_cast<std::add_pointer_t<Qt::TextElideMode>>(_a[1]))); break;
        case 13: _t->subColorChanged((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        case 14: _t->colorChanged((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        case 15: _t->update(); break;
        case 16: _t->rangeChanged((*reinterpret_cast<std::add_pointer_t<qreal>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<qreal>>(_a[2]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (QAbstractAxis::*)(bool )>(_a, &QAbstractAxis::visibleChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAbstractAxis::*)(bool )>(_a, &QAbstractAxis::lineVisibleChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAbstractAxis::*)(bool )>(_a, &QAbstractAxis::labelsVisibleChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAbstractAxis::*)(qreal )>(_a, &QAbstractAxis::labelsAngleChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAbstractAxis::*)()>(_a, &QAbstractAxis::labelDelegateChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAbstractAxis::*)(bool )>(_a, &QAbstractAxis::gridVisibleChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAbstractAxis::*)(bool )>(_a, &QAbstractAxis::subGridVisibleChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAbstractAxis::*)(const QString & )>(_a, &QAbstractAxis::titleTextChanged, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAbstractAxis::*)(QColor )>(_a, &QAbstractAxis::titleColorChanged, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAbstractAxis::*)(bool )>(_a, &QAbstractAxis::titleVisibleChanged, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAbstractAxis::*)(const QFont & )>(_a, &QAbstractAxis::titleFontChanged, 10))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAbstractAxis::*)(Qt::Alignment )>(_a, &QAbstractAxis::alignmentChanged, 11))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAbstractAxis::*)(Qt::TextElideMode )>(_a, &QAbstractAxis::textElideModeChanged, 12))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAbstractAxis::*)(QColor )>(_a, &QAbstractAxis::subColorChanged, 13))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAbstractAxis::*)(QColor )>(_a, &QAbstractAxis::colorChanged, 14))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAbstractAxis::*)()>(_a, &QAbstractAxis::update, 15))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAbstractAxis::*)(qreal , qreal )>(_a, &QAbstractAxis::rangeChanged, 16))
            return;
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 4:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQmlComponent* >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<bool*>(_v) = _t->isVisible(); break;
        case 1: *reinterpret_cast<bool*>(_v) = _t->isLineVisible(); break;
        case 2: *reinterpret_cast<bool*>(_v) = _t->labelsVisible(); break;
        case 3: *reinterpret_cast<qreal*>(_v) = _t->labelsAngle(); break;
        case 4: *reinterpret_cast<QQmlComponent**>(_v) = _t->labelDelegate(); break;
        case 5: *reinterpret_cast<bool*>(_v) = _t->isGridVisible(); break;
        case 6: *reinterpret_cast<bool*>(_v) = _t->isSubGridVisible(); break;
        case 7: *reinterpret_cast<QString*>(_v) = _t->titleText(); break;
        case 8: *reinterpret_cast<QColor*>(_v) = _t->titleColor(); break;
        case 9: *reinterpret_cast<bool*>(_v) = _t->isTitleVisible(); break;
        case 10: *reinterpret_cast<QFont*>(_v) = _t->titleFont(); break;
        case 11: *reinterpret_cast<Qt::Alignment*>(_v) = _t->alignment(); break;
        case 12: *reinterpret_cast<Qt::TextElideMode*>(_v) = _t->textElideMode(); break;
        case 13: *reinterpret_cast<QColor*>(_v) = _t->color(); break;
        case 14: *reinterpret_cast<QColor*>(_v) = _t->subColor(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setVisible(*reinterpret_cast<bool*>(_v)); break;
        case 1: _t->setLineVisible(*reinterpret_cast<bool*>(_v)); break;
        case 2: _t->setLabelsVisible(*reinterpret_cast<bool*>(_v)); break;
        case 3: _t->setLabelsAngle(*reinterpret_cast<qreal*>(_v)); break;
        case 4: _t->setLabelDelegate(*reinterpret_cast<QQmlComponent**>(_v)); break;
        case 5: _t->setGridVisible(*reinterpret_cast<bool*>(_v)); break;
        case 6: _t->setSubGridVisible(*reinterpret_cast<bool*>(_v)); break;
        case 7: _t->setTitleText(*reinterpret_cast<QString*>(_v)); break;
        case 8: _t->setTitleColor(*reinterpret_cast<QColor*>(_v)); break;
        case 9: _t->setTitleVisible(*reinterpret_cast<bool*>(_v)); break;
        case 10: _t->setTitleFont(*reinterpret_cast<QFont*>(_v)); break;
        case 11: _t->setAlignment(*reinterpret_cast<Qt::Alignment*>(_v)); break;
        case 12: _t->setTextElideMode(*reinterpret_cast<Qt::TextElideMode*>(_v)); break;
        case 13: _t->setColor(*reinterpret_cast<QColor*>(_v)); break;
        case 14: _t->setSubColor(*reinterpret_cast<QColor*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *QAbstractAxis::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QAbstractAxis::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13QAbstractAxisE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int QAbstractAxis::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 17)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 17;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 17)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 17;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 15;
    }
    return _id;
}

// SIGNAL 0
void QAbstractAxis::visibleChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void QAbstractAxis::lineVisibleChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void QAbstractAxis::labelsVisibleChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void QAbstractAxis::labelsAngleChanged(qreal _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void QAbstractAxis::labelDelegateChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void QAbstractAxis::gridVisibleChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void QAbstractAxis::subGridVisibleChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}

// SIGNAL 7
void QAbstractAxis::titleTextChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1);
}

// SIGNAL 8
void QAbstractAxis::titleColorChanged(QColor _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 8, nullptr, _t1);
}

// SIGNAL 9
void QAbstractAxis::titleVisibleChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 9, nullptr, _t1);
}

// SIGNAL 10
void QAbstractAxis::titleFontChanged(const QFont & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 10, nullptr, _t1);
}

// SIGNAL 11
void QAbstractAxis::alignmentChanged(Qt::Alignment _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 11, nullptr, _t1);
}

// SIGNAL 12
void QAbstractAxis::textElideModeChanged(Qt::TextElideMode _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 12, nullptr, _t1);
}

// SIGNAL 13
void QAbstractAxis::subColorChanged(QColor _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 13, nullptr, _t1);
}

// SIGNAL 14
void QAbstractAxis::colorChanged(QColor _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 14, nullptr, _t1);
}

// SIGNAL 15
void QAbstractAxis::update()
{
    QMetaObject::activate(this, &staticMetaObject, 15, nullptr);
}

// SIGNAL 16
void QAbstractAxis::rangeChanged(qreal _t1, qreal _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 16, nullptr, _t1, _t2);
}
QT_WARNING_POP
