/****************************************************************************
** Meta object code from reading C++ file 'axisticker_p.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../qtgraphs/src/graphs2d/axis/axisticker_p.h"
#include <QtGui/qtextcursor.h>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'axisticker_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN10AxisTickerE_t {};
} // unnamed namespace

template <> constexpr inline auto AxisTicker::qt_create_metaobjectdata<qt_meta_tag_ZN10AxisTickerE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "AxisTicker",
        "iResolutionChanged",
        "",
        "smoothingChanged",
        "origoChanged",
        "subTicksVisibleChanged",
        "spacingChanged",
        "displacementChanged",
        "subTickColorChanged",
        "tickColorChanged",
        "subTickLineWidthChanged",
        "tickLineWidthChanged",
        "subTickScaleChanged",
        "subTickLengthChanged",
        "isHorizontalChanged",
        "flippedChanged",
        "logarithmicChanged",
        "baseChanged",
        "iResolution",
        "QVector3D",
        "smoothing",
        "origo",
        "subTicksVisible",
        "spacing",
        "displacement",
        "subTickColor",
        "QColor",
        "tickColor",
        "subTickLineWidth",
        "tickLineWidth",
        "subTickScale",
        "subTickLength",
        "isHorizontal",
        "flipped",
        "isLogarithmic",
        "base"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'iResolutionChanged'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'smoothingChanged'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'origoChanged'
        QtMocHelpers::SignalData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'subTicksVisibleChanged'
        QtMocHelpers::SignalData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'spacingChanged'
        QtMocHelpers::SignalData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'displacementChanged'
        QtMocHelpers::SignalData<void()>(7, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'subTickColorChanged'
        QtMocHelpers::SignalData<void()>(8, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'tickColorChanged'
        QtMocHelpers::SignalData<void()>(9, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'subTickLineWidthChanged'
        QtMocHelpers::SignalData<void()>(10, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'tickLineWidthChanged'
        QtMocHelpers::SignalData<void()>(11, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'subTickScaleChanged'
        QtMocHelpers::SignalData<void()>(12, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'subTickLengthChanged'
        QtMocHelpers::SignalData<void()>(13, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'isHorizontalChanged'
        QtMocHelpers::SignalData<void()>(14, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'flippedChanged'
        QtMocHelpers::SignalData<void()>(15, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'logarithmicChanged'
        QtMocHelpers::RevisionedSignalData<void()>(16, 2, QMC::AccessPublic, 0x60c, QMetaType::Void),
        // Signal 'baseChanged'
        QtMocHelpers::RevisionedSignalData<void()>(17, 2, QMC::AccessPublic, 0x60c, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'iResolution'
        QtMocHelpers::PropertyData<QVector3D>(18, 0x80000000 | 19, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Final, 0),
        // property 'smoothing'
        QtMocHelpers::PropertyData<qreal>(20, QMetaType::QReal, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 1),
        // property 'origo'
        QtMocHelpers::PropertyData<int>(21, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 2),
        // property 'subTicksVisible'
        QtMocHelpers::PropertyData<bool>(22, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 3),
        // property 'spacing'
        QtMocHelpers::PropertyData<qreal>(23, QMetaType::QReal, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 4),
        // property 'displacement'
        QtMocHelpers::PropertyData<qreal>(24, QMetaType::QReal, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 5),
        // property 'subTickColor'
        QtMocHelpers::PropertyData<QColor>(25, 0x80000000 | 26, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet | QMC::Final, 6),
        // property 'tickColor'
        QtMocHelpers::PropertyData<QColor>(27, 0x80000000 | 26, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet | QMC::Final, 7),
        // property 'subTickLineWidth'
        QtMocHelpers::PropertyData<qreal>(28, QMetaType::QReal, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 8),
        // property 'tickLineWidth'
        QtMocHelpers::PropertyData<qreal>(29, QMetaType::QReal, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 9),
        // property 'subTickScale'
        QtMocHelpers::PropertyData<qreal>(30, QMetaType::QReal, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 10),
        // property 'subTickLength'
        QtMocHelpers::PropertyData<qreal>(31, QMetaType::QReal, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 11),
        // property 'isHorizontal'
        QtMocHelpers::PropertyData<bool>(32, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 12),
        // property 'flipped'
        QtMocHelpers::PropertyData<bool>(33, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 13),
        // property 'isLogarithmic'
        QtMocHelpers::PropertyData<bool>(34, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable, 14, 0x60c),
        // property 'base'
        QtMocHelpers::PropertyData<qreal>(35, QMetaType::QReal, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 15, 0x60c),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<AxisTicker, qt_meta_tag_ZN10AxisTickerE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject AxisTicker::staticMetaObject = { {
    QMetaObject::SuperData::link<QQuickShaderEffect::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10AxisTickerE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10AxisTickerE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10AxisTickerE_t>.metaTypes,
    nullptr
} };

void AxisTicker::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<AxisTicker *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->iResolutionChanged(); break;
        case 1: _t->smoothingChanged(); break;
        case 2: _t->origoChanged(); break;
        case 3: _t->subTicksVisibleChanged(); break;
        case 4: _t->spacingChanged(); break;
        case 5: _t->displacementChanged(); break;
        case 6: _t->subTickColorChanged(); break;
        case 7: _t->tickColorChanged(); break;
        case 8: _t->subTickLineWidthChanged(); break;
        case 9: _t->tickLineWidthChanged(); break;
        case 10: _t->subTickScaleChanged(); break;
        case 11: _t->subTickLengthChanged(); break;
        case 12: _t->isHorizontalChanged(); break;
        case 13: _t->flippedChanged(); break;
        case 14: _t->logarithmicChanged(); break;
        case 15: _t->baseChanged(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (AxisTicker::*)()>(_a, &AxisTicker::iResolutionChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (AxisTicker::*)()>(_a, &AxisTicker::smoothingChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (AxisTicker::*)()>(_a, &AxisTicker::origoChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (AxisTicker::*)()>(_a, &AxisTicker::subTicksVisibleChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (AxisTicker::*)()>(_a, &AxisTicker::spacingChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (AxisTicker::*)()>(_a, &AxisTicker::displacementChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (AxisTicker::*)()>(_a, &AxisTicker::subTickColorChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (AxisTicker::*)()>(_a, &AxisTicker::tickColorChanged, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (AxisTicker::*)()>(_a, &AxisTicker::subTickLineWidthChanged, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (AxisTicker::*)()>(_a, &AxisTicker::tickLineWidthChanged, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (AxisTicker::*)()>(_a, &AxisTicker::subTickScaleChanged, 10))
            return;
        if (QtMocHelpers::indexOfMethod<void (AxisTicker::*)()>(_a, &AxisTicker::subTickLengthChanged, 11))
            return;
        if (QtMocHelpers::indexOfMethod<void (AxisTicker::*)()>(_a, &AxisTicker::isHorizontalChanged, 12))
            return;
        if (QtMocHelpers::indexOfMethod<void (AxisTicker::*)()>(_a, &AxisTicker::flippedChanged, 13))
            return;
        if (QtMocHelpers::indexOfMethod<void (AxisTicker::*)()>(_a, &AxisTicker::logarithmicChanged, 14))
            return;
        if (QtMocHelpers::indexOfMethod<void (AxisTicker::*)()>(_a, &AxisTicker::baseChanged, 15))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QVector3D*>(_v) = _t->iResolution(); break;
        case 1: *reinterpret_cast<qreal*>(_v) = _t->smoothing(); break;
        case 2: *reinterpret_cast<int*>(_v) = _t->origo(); break;
        case 3: *reinterpret_cast<bool*>(_v) = _t->subTicksVisible(); break;
        case 4: *reinterpret_cast<qreal*>(_v) = _t->spacing(); break;
        case 5: *reinterpret_cast<qreal*>(_v) = _t->displacement(); break;
        case 6: *reinterpret_cast<QColor*>(_v) = _t->subTickColor(); break;
        case 7: *reinterpret_cast<QColor*>(_v) = _t->tickColor(); break;
        case 8: *reinterpret_cast<qreal*>(_v) = _t->subTickLineWidth(); break;
        case 9: *reinterpret_cast<qreal*>(_v) = _t->tickLineWidth(); break;
        case 10: *reinterpret_cast<qreal*>(_v) = _t->subTickScale(); break;
        case 11: *reinterpret_cast<qreal*>(_v) = _t->subTickLength(); break;
        case 12: *reinterpret_cast<bool*>(_v) = _t->isHorizontal(); break;
        case 13: *reinterpret_cast<bool*>(_v) = _t->isFlipped(); break;
        case 14: *reinterpret_cast<bool*>(_v) = _t->isLogarithmic(); break;
        case 15: *reinterpret_cast<qreal*>(_v) = _t->base(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 1: _t->setSmoothing(*reinterpret_cast<qreal*>(_v)); break;
        case 2: _t->setOrigo(*reinterpret_cast<int*>(_v)); break;
        case 3: _t->setSubTicksVisible(*reinterpret_cast<bool*>(_v)); break;
        case 4: _t->setSpacing(*reinterpret_cast<qreal*>(_v)); break;
        case 5: _t->setDisplacement(*reinterpret_cast<qreal*>(_v)); break;
        case 6: _t->setSubTickColor(*reinterpret_cast<QColor*>(_v)); break;
        case 7: _t->setTickColor(*reinterpret_cast<QColor*>(_v)); break;
        case 8: _t->setSubTickLineWidth(*reinterpret_cast<qreal*>(_v)); break;
        case 9: _t->setTickLineWidth(*reinterpret_cast<qreal*>(_v)); break;
        case 10: _t->setSubTickScale(*reinterpret_cast<qreal*>(_v)); break;
        case 11: _t->setSubTickLength(*reinterpret_cast<qreal*>(_v)); break;
        case 12: _t->setIsHorizontal(*reinterpret_cast<bool*>(_v)); break;
        case 13: _t->setFlipped(*reinterpret_cast<bool*>(_v)); break;
        case 14: _t->setLogarithmic(*reinterpret_cast<bool*>(_v)); break;
        case 15: _t->setBase(*reinterpret_cast<qreal*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *AxisTicker::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *AxisTicker::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10AxisTickerE_t>.strings))
        return static_cast<void*>(this);
    return QQuickShaderEffect::qt_metacast(_clname);
}

int AxisTicker::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QQuickShaderEffect::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 16)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 16;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 16)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 16;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 16;
    }
    return _id;
}

// SIGNAL 0
void AxisTicker::iResolutionChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void AxisTicker::smoothingChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void AxisTicker::origoChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void AxisTicker::subTicksVisibleChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void AxisTicker::spacingChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void AxisTicker::displacementChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void AxisTicker::subTickColorChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void AxisTicker::tickColorChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void AxisTicker::subTickLineWidthChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 8, nullptr);
}

// SIGNAL 9
void AxisTicker::tickLineWidthChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 9, nullptr);
}

// SIGNAL 10
void AxisTicker::subTickScaleChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 10, nullptr);
}

// SIGNAL 11
void AxisTicker::subTickLengthChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 11, nullptr);
}

// SIGNAL 12
void AxisTicker::isHorizontalChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 12, nullptr);
}

// SIGNAL 13
void AxisTicker::flippedChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 13, nullptr);
}

// SIGNAL 14
void AxisTicker::logarithmicChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 14, nullptr);
}

// SIGNAL 15
void AxisTicker::baseChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 15, nullptr);
}
QT_WARNING_POP
