/****************************************************************************
** Meta object code from reading C++ file 'qlineseries.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../qtgraphs/src/graphs2d/linechart/qlineseries.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qlineseries.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN11QLineSeriesE_t {};
} // unnamed namespace

template <> constexpr inline auto QLineSeries::qt_create_metaobjectdata<qt_meta_tag_ZN11QLineSeriesE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QLineSeries",
        "RegisterEnumClassesUnscoped",
        "false",
        "QML.Element",
        "LineSeries",
        "widthChanged",
        "",
        "capStyleChanged",
        "joinStyleChanged",
        "Qt::PenJoinStyle",
        "newJoinStyle",
        "lineStyleChanged",
        "QLineSeries::LineStyle",
        "newLineStyle",
        "strokeStyleChanged",
        "QLineSeries::StrokeStyle",
        "newStrokeStyle",
        "dashOffsetChanged",
        "newDashOffset",
        "dashPatternChanged",
        "QList<qreal>",
        "newDashPattern",
        "dataPointCoordinatesAt",
        "QPointF",
        "x",
        "y",
        "width",
        "capStyle",
        "Qt::PenCapStyle",
        "joinStyle",
        "lineStyle",
        "strokeStyle",
        "dashOffset",
        "dashPattern",
        "LineStyle",
        "Straight",
        "StepLeft",
        "StepRight",
        "StepCenter",
        "StrokeStyle",
        "SolidLine",
        "DashLine"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'widthChanged'
        QtMocHelpers::SignalData<void()>(5, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'capStyleChanged'
        QtMocHelpers::SignalData<void()>(7, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'joinStyleChanged'
        QtMocHelpers::RevisionedSignalData<void(Qt::PenJoinStyle)>(8, 6, QMC::AccessPublic, 0x60b, QMetaType::Void, {{
            { 0x80000000 | 9, 10 },
        }}),
        // Signal 'lineStyleChanged'
        QtMocHelpers::RevisionedSignalData<void(QLineSeries::LineStyle)>(11, 6, QMC::AccessPublic, 0x60b, QMetaType::Void, {{
            { 0x80000000 | 12, 13 },
        }}),
        // Signal 'strokeStyleChanged'
        QtMocHelpers::RevisionedSignalData<void(QLineSeries::StrokeStyle)>(14, 6, QMC::AccessPublic, 0x60b, QMetaType::Void, {{
            { 0x80000000 | 15, 16 },
        }}),
        // Signal 'dashOffsetChanged'
        QtMocHelpers::RevisionedSignalData<void(qreal)>(17, 6, QMC::AccessPublic, 0x60b, QMetaType::Void, {{
            { QMetaType::QReal, 18 },
        }}),
        // Signal 'dashPatternChanged'
        QtMocHelpers::RevisionedSignalData<void(const QList<qreal> &)>(19, 6, QMC::AccessPublic, 0x60b, QMetaType::Void, {{
            { 0x80000000 | 20, 21 },
        }}),
        // Method 'dataPointCoordinatesAt'
        QtMocHelpers::RevisionedMethodData<QPointF(qreal, qreal)>(22, 6, QMC::AccessPublic, 0x60a, 0x80000000 | 23, {{
            { QMetaType::QReal, 24 }, { QMetaType::QReal, 25 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'width'
        QtMocHelpers::PropertyData<qreal>(26, QMetaType::QReal, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 0),
        // property 'capStyle'
        QtMocHelpers::PropertyData<Qt::PenCapStyle>(27, 0x80000000 | 28, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet | QMC::Final, 1),
        // property 'joinStyle'
        QtMocHelpers::PropertyData<Qt::PenJoinStyle>(29, 0x80000000 | 9, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 2, 0x60b),
        // property 'lineStyle'
        QtMocHelpers::PropertyData<QLineSeries::LineStyle>(30, 0x80000000 | 12, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 3, 0x60b),
        // property 'strokeStyle'
        QtMocHelpers::PropertyData<QLineSeries::StrokeStyle>(31, 0x80000000 | 15, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 4, 0x60b),
        // property 'dashOffset'
        QtMocHelpers::PropertyData<qreal>(32, QMetaType::QReal, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 5, 0x60b),
        // property 'dashPattern'
        QtMocHelpers::PropertyData<QList<qreal>>(33, 0x80000000 | 20, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 6, 0x60b),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'LineStyle'
        QtMocHelpers::EnumData<enum LineStyle>(34, 34, QMC::EnumIsScoped).add({
            {   35, LineStyle::Straight },
            {   36, LineStyle::StepLeft },
            {   37, LineStyle::StepRight },
            {   38, LineStyle::StepCenter },
        }),
        // enum 'StrokeStyle'
        QtMocHelpers::EnumData<enum StrokeStyle>(39, 39, QMC::EnumIsScoped).add({
            {   40, StrokeStyle::SolidLine },
            {   41, StrokeStyle::DashLine },
        }),
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
    });
    return QtMocHelpers::metaObjectData<QLineSeries, void>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject QLineSeries::staticMetaObject = { {
    QMetaObject::SuperData::link<QXYSeries::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11QLineSeriesE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11QLineSeriesE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN11QLineSeriesE_t>.metaTypes,
    nullptr
} };

void QLineSeries::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QLineSeries *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->widthChanged(); break;
        case 1: _t->capStyleChanged(); break;
        case 2: _t->joinStyleChanged((*reinterpret_cast<std::add_pointer_t<Qt::PenJoinStyle>>(_a[1]))); break;
        case 3: _t->lineStyleChanged((*reinterpret_cast<std::add_pointer_t<QLineSeries::LineStyle>>(_a[1]))); break;
        case 4: _t->strokeStyleChanged((*reinterpret_cast<std::add_pointer_t<QLineSeries::StrokeStyle>>(_a[1]))); break;
        case 5: _t->dashOffsetChanged((*reinterpret_cast<std::add_pointer_t<qreal>>(_a[1]))); break;
        case 6: _t->dashPatternChanged((*reinterpret_cast<std::add_pointer_t<QList<qreal>>>(_a[1]))); break;
        case 7: { QPointF _r = _t->dataPointCoordinatesAt((*reinterpret_cast<std::add_pointer_t<qreal>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<qreal>>(_a[2])));
            if (_a[0]) *reinterpret_cast<QPointF*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 6:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QList<qreal> >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (QLineSeries::*)()>(_a, &QLineSeries::widthChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (QLineSeries::*)()>(_a, &QLineSeries::capStyleChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (QLineSeries::*)(Qt::PenJoinStyle )>(_a, &QLineSeries::joinStyleChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (QLineSeries::*)(QLineSeries::LineStyle )>(_a, &QLineSeries::lineStyleChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (QLineSeries::*)(QLineSeries::StrokeStyle )>(_a, &QLineSeries::strokeStyleChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (QLineSeries::*)(qreal )>(_a, &QLineSeries::dashOffsetChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (QLineSeries::*)(const QList<qreal> & )>(_a, &QLineSeries::dashPatternChanged, 6))
            return;
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 6:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<qreal> >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<qreal*>(_v) = _t->width(); break;
        case 1: *reinterpret_cast<Qt::PenCapStyle*>(_v) = _t->capStyle(); break;
        case 2: *reinterpret_cast<Qt::PenJoinStyle*>(_v) = _t->joinStyle(); break;
        case 3: *reinterpret_cast<QLineSeries::LineStyle*>(_v) = _t->lineStyle(); break;
        case 4: *reinterpret_cast<QLineSeries::StrokeStyle*>(_v) = _t->strokeStyle(); break;
        case 5: *reinterpret_cast<qreal*>(_v) = _t->dashOffset(); break;
        case 6: *reinterpret_cast<QList<qreal>*>(_v) = _t->dashPattern(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setWidth(*reinterpret_cast<qreal*>(_v)); break;
        case 1: _t->setCapStyle(*reinterpret_cast<Qt::PenCapStyle*>(_v)); break;
        case 2: _t->setJoinStyle(*reinterpret_cast<Qt::PenJoinStyle*>(_v)); break;
        case 3: _t->setLineStyle(*reinterpret_cast<QLineSeries::LineStyle*>(_v)); break;
        case 4: _t->setStrokeStyle(*reinterpret_cast<QLineSeries::StrokeStyle*>(_v)); break;
        case 5: _t->setDashOffset(*reinterpret_cast<qreal*>(_v)); break;
        case 6: _t->setDashPattern(*reinterpret_cast<QList<qreal>*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *QLineSeries::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QLineSeries::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11QLineSeriesE_t>.strings))
        return static_cast<void*>(this);
    return QXYSeries::qt_metacast(_clname);
}

int QLineSeries::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QXYSeries::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 8)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 8)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    }
    return _id;
}

// SIGNAL 0
void QLineSeries::widthChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void QLineSeries::capStyleChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void QLineSeries::joinStyleChanged(Qt::PenJoinStyle _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void QLineSeries::lineStyleChanged(QLineSeries::LineStyle _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void QLineSeries::strokeStyleChanged(QLineSeries::StrokeStyle _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void QLineSeries::dashOffsetChanged(qreal _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void QLineSeries::dashPatternChanged(const QList<qreal> & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}
QT_WARNING_POP
