/****************************************************************************
** Meta object code from reading C++ file 'qabstract3dseries.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../qtgraphs/src/graphs3d/data/qabstract3dseries.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qabstract3dseries.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN17QAbstract3DSeriesE_t {};
} // unnamed namespace

template <> constexpr inline auto QAbstract3DSeries::qt_create_metaobjectdata<qt_meta_tag_ZN17QAbstract3DSeriesE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QAbstract3DSeries",
        "RegisterEnumClassesUnscoped",
        "false",
        "QML.Element",
        "Abstract3DSeries",
        "QML.Creatable",
        "QML.UncreatableReason",
        "Uncreatable base type",
        "itemLabelFormatChanged",
        "",
        "format",
        "visibleChanged",
        "visible",
        "meshChanged",
        "QAbstract3DSeries::Mesh",
        "mesh",
        "meshSmoothChanged",
        "enabled",
        "meshRotationChanged",
        "QQuaternion",
        "rotation",
        "userDefinedMeshChanged",
        "fileName",
        "colorStyleChanged",
        "QGraphsTheme::ColorStyle",
        "style",
        "baseColorChanged",
        "QColor",
        "color",
        "baseGradientChanged",
        "QLinearGradient",
        "gradient",
        "singleHighlightColorChanged",
        "singleHighlightGradientChanged",
        "multiHighlightColorChanged",
        "multiHighlightGradientChanged",
        "nameChanged",
        "name",
        "itemLabelChanged",
        "label",
        "itemLabelVisibleChanged",
        "lightingModeChanged",
        "QAbstract3DSeries::LightingMode",
        "lightingMode",
        "setMeshAxisAndAngle",
        "QVector3D",
        "axis",
        "angle",
        "type",
        "QAbstract3DSeries::SeriesType",
        "itemLabelFormat",
        "meshSmooth",
        "meshRotation",
        "userDefinedMesh",
        "colorStyle",
        "baseColor",
        "baseGradient",
        "singleHighlightColor",
        "singleHighlightGradient",
        "multiHighlightColor",
        "multiHighlightGradient",
        "itemLabel",
        "itemLabelVisible",
        "SeriesType",
        "None",
        "Bar",
        "Scatter",
        "Surface",
        "Mesh",
        "UserDefined",
        "Cube",
        "Pyramid",
        "Cone",
        "Cylinder",
        "BevelBar",
        "BevelCube",
        "Sphere",
        "Minimal",
        "Arrow",
        "Point",
        "LightingMode",
        "Shaded",
        "Unshaded"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'itemLabelFormatChanged'
        QtMocHelpers::SignalData<void(const QString &)>(8, 9, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 10 },
        }}),
        // Signal 'visibleChanged'
        QtMocHelpers::SignalData<void(bool)>(11, 9, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 12 },
        }}),
        // Signal 'meshChanged'
        QtMocHelpers::SignalData<void(QAbstract3DSeries::Mesh)>(13, 9, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 14, 15 },
        }}),
        // Signal 'meshSmoothChanged'
        QtMocHelpers::SignalData<void(bool)>(16, 9, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 17 },
        }}),
        // Signal 'meshRotationChanged'
        QtMocHelpers::SignalData<void(const QQuaternion &)>(18, 9, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 19, 20 },
        }}),
        // Signal 'userDefinedMeshChanged'
        QtMocHelpers::SignalData<void(const QString &)>(21, 9, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 22 },
        }}),
        // Signal 'colorStyleChanged'
        QtMocHelpers::SignalData<void(QGraphsTheme::ColorStyle)>(23, 9, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 24, 25 },
        }}),
        // Signal 'baseColorChanged'
        QtMocHelpers::SignalData<void(QColor)>(26, 9, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 27, 28 },
        }}),
        // Signal 'baseGradientChanged'
        QtMocHelpers::SignalData<void(const QLinearGradient &)>(29, 9, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 30, 31 },
        }}),
        // Signal 'singleHighlightColorChanged'
        QtMocHelpers::SignalData<void(QColor)>(32, 9, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 27, 28 },
        }}),
        // Signal 'singleHighlightGradientChanged'
        QtMocHelpers::SignalData<void(const QLinearGradient &)>(33, 9, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 30, 31 },
        }}),
        // Signal 'multiHighlightColorChanged'
        QtMocHelpers::SignalData<void(QColor)>(34, 9, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 27, 28 },
        }}),
        // Signal 'multiHighlightGradientChanged'
        QtMocHelpers::SignalData<void(const QLinearGradient &)>(35, 9, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 30, 31 },
        }}),
        // Signal 'nameChanged'
        QtMocHelpers::SignalData<void(const QString &)>(36, 9, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 37 },
        }}),
        // Signal 'itemLabelChanged'
        QtMocHelpers::SignalData<void(const QString &)>(38, 9, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 39 },
        }}),
        // Signal 'itemLabelVisibleChanged'
        QtMocHelpers::SignalData<void(bool)>(40, 9, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 12 },
        }}),
        // Signal 'lightingModeChanged'
        QtMocHelpers::SignalData<void(QAbstract3DSeries::LightingMode)>(41, 9, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 42, 43 },
        }}),
        // Method 'setMeshAxisAndAngle'
        QtMocHelpers::MethodData<void(QVector3D, float)>(44, 9, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 45, 46 }, { QMetaType::Float, 47 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'type'
        QtMocHelpers::PropertyData<QAbstract3DSeries::SeriesType>(48, 0x80000000 | 49, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'itemLabelFormat'
        QtMocHelpers::PropertyData<QString>(50, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 0),
        // property 'visible'
        QtMocHelpers::PropertyData<bool>(12, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 1),
        // property 'mesh'
        QtMocHelpers::PropertyData<QAbstract3DSeries::Mesh>(15, 0x80000000 | 14, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 2),
        // property 'meshSmooth'
        QtMocHelpers::PropertyData<bool>(51, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 3),
        // property 'meshRotation'
        QtMocHelpers::PropertyData<QQuaternion>(52, 0x80000000 | 19, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 4),
        // property 'userDefinedMesh'
        QtMocHelpers::PropertyData<QString>(53, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 5),
        // property 'colorStyle'
        QtMocHelpers::PropertyData<QGraphsTheme::ColorStyle>(54, 0x80000000 | 24, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 6),
        // property 'baseColor'
        QtMocHelpers::PropertyData<QColor>(55, 0x80000000 | 27, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 7),
        // property 'baseGradient'
        QtMocHelpers::PropertyData<QLinearGradient>(56, 0x80000000 | 30, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 8),
        // property 'singleHighlightColor'
        QtMocHelpers::PropertyData<QColor>(57, 0x80000000 | 27, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 9),
        // property 'singleHighlightGradient'
        QtMocHelpers::PropertyData<QLinearGradient>(58, 0x80000000 | 30, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 10),
        // property 'multiHighlightColor'
        QtMocHelpers::PropertyData<QColor>(59, 0x80000000 | 27, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 11),
        // property 'multiHighlightGradient'
        QtMocHelpers::PropertyData<QLinearGradient>(60, 0x80000000 | 30, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 12),
        // property 'name'
        QtMocHelpers::PropertyData<QString>(37, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 13),
        // property 'itemLabel'
        QtMocHelpers::PropertyData<QString>(61, QMetaType::QString, QMC::DefaultPropertyFlags, 14),
        // property 'itemLabelVisible'
        QtMocHelpers::PropertyData<bool>(62, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 15),
        // property 'lightingMode'
        QtMocHelpers::PropertyData<QAbstract3DSeries::LightingMode>(43, 0x80000000 | 42, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 16, 0x60a),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'SeriesType'
        QtMocHelpers::EnumData<enum SeriesType>(63, 63, QMC::EnumIsScoped).add({
            {   64, SeriesType::None },
            {   65, SeriesType::Bar },
            {   66, SeriesType::Scatter },
            {   67, SeriesType::Surface },
        }),
        // enum 'Mesh'
        QtMocHelpers::EnumData<enum Mesh>(68, 68, QMC::EnumIsScoped).add({
            {   69, Mesh::UserDefined },
            {   65, Mesh::Bar },
            {   70, Mesh::Cube },
            {   71, Mesh::Pyramid },
            {   72, Mesh::Cone },
            {   73, Mesh::Cylinder },
            {   74, Mesh::BevelBar },
            {   75, Mesh::BevelCube },
            {   76, Mesh::Sphere },
            {   77, Mesh::Minimal },
            {   78, Mesh::Arrow },
            {   79, Mesh::Point },
        }),
        // enum 'LightingMode'
        QtMocHelpers::EnumData<enum LightingMode>(80, 80, QMC::EnumIsScoped).add({
            {   81, LightingMode::Shaded },
            {   82, LightingMode::Unshaded },
        }),
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
            {    5,    2 },
            {    6,    7 },
    });
    return QtMocHelpers::metaObjectData<QAbstract3DSeries, void>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT static const QMetaObject::SuperData qt_meta_extradata_ZN17QAbstract3DSeriesE[] = {
    QMetaObject::SuperData::link<QGraphsTheme::staticMetaObject>(),
    nullptr
};

Q_CONSTINIT const QMetaObject QAbstract3DSeries::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17QAbstract3DSeriesE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17QAbstract3DSeriesE_t>.data,
    qt_static_metacall,
    qt_meta_extradata_ZN17QAbstract3DSeriesE,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN17QAbstract3DSeriesE_t>.metaTypes,
    nullptr
} };

void QAbstract3DSeries::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QAbstract3DSeries *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->itemLabelFormatChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 1: _t->visibleChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 2: _t->meshChanged((*reinterpret_cast<std::add_pointer_t<QAbstract3DSeries::Mesh>>(_a[1]))); break;
        case 3: _t->meshSmoothChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 4: _t->meshRotationChanged((*reinterpret_cast<std::add_pointer_t<QQuaternion>>(_a[1]))); break;
        case 5: _t->userDefinedMeshChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 6: _t->colorStyleChanged((*reinterpret_cast<std::add_pointer_t<QGraphsTheme::ColorStyle>>(_a[1]))); break;
        case 7: _t->baseColorChanged((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        case 8: _t->baseGradientChanged((*reinterpret_cast<std::add_pointer_t<QLinearGradient>>(_a[1]))); break;
        case 9: _t->singleHighlightColorChanged((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        case 10: _t->singleHighlightGradientChanged((*reinterpret_cast<std::add_pointer_t<QLinearGradient>>(_a[1]))); break;
        case 11: _t->multiHighlightColorChanged((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        case 12: _t->multiHighlightGradientChanged((*reinterpret_cast<std::add_pointer_t<QLinearGradient>>(_a[1]))); break;
        case 13: _t->nameChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 14: _t->itemLabelChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 15: _t->itemLabelVisibleChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 16: _t->lightingModeChanged((*reinterpret_cast<std::add_pointer_t<QAbstract3DSeries::LightingMode>>(_a[1]))); break;
        case 17: _t->setMeshAxisAndAngle((*reinterpret_cast<std::add_pointer_t<QVector3D>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[2]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (QAbstract3DSeries::*)(const QString & )>(_a, &QAbstract3DSeries::itemLabelFormatChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAbstract3DSeries::*)(bool )>(_a, &QAbstract3DSeries::visibleChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAbstract3DSeries::*)(QAbstract3DSeries::Mesh )>(_a, &QAbstract3DSeries::meshChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAbstract3DSeries::*)(bool )>(_a, &QAbstract3DSeries::meshSmoothChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAbstract3DSeries::*)(const QQuaternion & )>(_a, &QAbstract3DSeries::meshRotationChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAbstract3DSeries::*)(const QString & )>(_a, &QAbstract3DSeries::userDefinedMeshChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAbstract3DSeries::*)(QGraphsTheme::ColorStyle )>(_a, &QAbstract3DSeries::colorStyleChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAbstract3DSeries::*)(QColor )>(_a, &QAbstract3DSeries::baseColorChanged, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAbstract3DSeries::*)(const QLinearGradient & )>(_a, &QAbstract3DSeries::baseGradientChanged, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAbstract3DSeries::*)(QColor )>(_a, &QAbstract3DSeries::singleHighlightColorChanged, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAbstract3DSeries::*)(const QLinearGradient & )>(_a, &QAbstract3DSeries::singleHighlightGradientChanged, 10))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAbstract3DSeries::*)(QColor )>(_a, &QAbstract3DSeries::multiHighlightColorChanged, 11))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAbstract3DSeries::*)(const QLinearGradient & )>(_a, &QAbstract3DSeries::multiHighlightGradientChanged, 12))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAbstract3DSeries::*)(const QString & )>(_a, &QAbstract3DSeries::nameChanged, 13))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAbstract3DSeries::*)(const QString & )>(_a, &QAbstract3DSeries::itemLabelChanged, 14))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAbstract3DSeries::*)(bool )>(_a, &QAbstract3DSeries::itemLabelVisibleChanged, 15))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAbstract3DSeries::*)(QAbstract3DSeries::LightingMode )>(_a, &QAbstract3DSeries::lightingModeChanged, 16))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QAbstract3DSeries::SeriesType*>(_v) = _t->type(); break;
        case 1: *reinterpret_cast<QString*>(_v) = _t->itemLabelFormat(); break;
        case 2: *reinterpret_cast<bool*>(_v) = _t->isVisible(); break;
        case 3: *reinterpret_cast<QAbstract3DSeries::Mesh*>(_v) = _t->mesh(); break;
        case 4: *reinterpret_cast<bool*>(_v) = _t->isMeshSmooth(); break;
        case 5: *reinterpret_cast<QQuaternion*>(_v) = _t->meshRotation(); break;
        case 6: *reinterpret_cast<QString*>(_v) = _t->userDefinedMesh(); break;
        case 7: *reinterpret_cast<QGraphsTheme::ColorStyle*>(_v) = _t->colorStyle(); break;
        case 8: *reinterpret_cast<QColor*>(_v) = _t->baseColor(); break;
        case 9: *reinterpret_cast<QLinearGradient*>(_v) = _t->baseGradient(); break;
        case 10: *reinterpret_cast<QColor*>(_v) = _t->singleHighlightColor(); break;
        case 11: *reinterpret_cast<QLinearGradient*>(_v) = _t->singleHighlightGradient(); break;
        case 12: *reinterpret_cast<QColor*>(_v) = _t->multiHighlightColor(); break;
        case 13: *reinterpret_cast<QLinearGradient*>(_v) = _t->multiHighlightGradient(); break;
        case 14: *reinterpret_cast<QString*>(_v) = _t->name(); break;
        case 15: *reinterpret_cast<QString*>(_v) = _t->itemLabel(); break;
        case 16: *reinterpret_cast<bool*>(_v) = _t->isItemLabelVisible(); break;
        case 17: *reinterpret_cast<QAbstract3DSeries::LightingMode*>(_v) = _t->lightingMode(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 1: _t->setItemLabelFormat(*reinterpret_cast<QString*>(_v)); break;
        case 2: _t->setVisible(*reinterpret_cast<bool*>(_v)); break;
        case 3: _t->setMesh(*reinterpret_cast<QAbstract3DSeries::Mesh*>(_v)); break;
        case 4: _t->setMeshSmooth(*reinterpret_cast<bool*>(_v)); break;
        case 5: _t->setMeshRotation(*reinterpret_cast<QQuaternion*>(_v)); break;
        case 6: _t->setUserDefinedMesh(*reinterpret_cast<QString*>(_v)); break;
        case 7: _t->setColorStyle(*reinterpret_cast<QGraphsTheme::ColorStyle*>(_v)); break;
        case 8: _t->setBaseColor(*reinterpret_cast<QColor*>(_v)); break;
        case 9: _t->setBaseGradient(*reinterpret_cast<QLinearGradient*>(_v)); break;
        case 10: _t->setSingleHighlightColor(*reinterpret_cast<QColor*>(_v)); break;
        case 11: _t->setSingleHighlightGradient(*reinterpret_cast<QLinearGradient*>(_v)); break;
        case 12: _t->setMultiHighlightColor(*reinterpret_cast<QColor*>(_v)); break;
        case 13: _t->setMultiHighlightGradient(*reinterpret_cast<QLinearGradient*>(_v)); break;
        case 14: _t->setName(*reinterpret_cast<QString*>(_v)); break;
        case 16: _t->setItemLabelVisible(*reinterpret_cast<bool*>(_v)); break;
        case 17: _t->setLightingMode(*reinterpret_cast<QAbstract3DSeries::LightingMode*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *QAbstract3DSeries::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QAbstract3DSeries::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17QAbstract3DSeriesE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int QAbstract3DSeries::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 18)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 18;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 18)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 18;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 18;
    }
    return _id;
}

// SIGNAL 0
void QAbstract3DSeries::itemLabelFormatChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void QAbstract3DSeries::visibleChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void QAbstract3DSeries::meshChanged(QAbstract3DSeries::Mesh _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void QAbstract3DSeries::meshSmoothChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void QAbstract3DSeries::meshRotationChanged(const QQuaternion & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void QAbstract3DSeries::userDefinedMeshChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void QAbstract3DSeries::colorStyleChanged(QGraphsTheme::ColorStyle _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}

// SIGNAL 7
void QAbstract3DSeries::baseColorChanged(QColor _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1);
}

// SIGNAL 8
void QAbstract3DSeries::baseGradientChanged(const QLinearGradient & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 8, nullptr, _t1);
}

// SIGNAL 9
void QAbstract3DSeries::singleHighlightColorChanged(QColor _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 9, nullptr, _t1);
}

// SIGNAL 10
void QAbstract3DSeries::singleHighlightGradientChanged(const QLinearGradient & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 10, nullptr, _t1);
}

// SIGNAL 11
void QAbstract3DSeries::multiHighlightColorChanged(QColor _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 11, nullptr, _t1);
}

// SIGNAL 12
void QAbstract3DSeries::multiHighlightGradientChanged(const QLinearGradient & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 12, nullptr, _t1);
}

// SIGNAL 13
void QAbstract3DSeries::nameChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 13, nullptr, _t1);
}

// SIGNAL 14
void QAbstract3DSeries::itemLabelChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 14, nullptr, _t1);
}

// SIGNAL 15
void QAbstract3DSeries::itemLabelVisibleChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 15, nullptr, _t1);
}

// SIGNAL 16
void QAbstract3DSeries::lightingModeChanged(QAbstract3DSeries::LightingMode _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 16, nullptr, _t1);
}
QT_WARNING_POP
