/****************************************************************************
** Meta object code from reading C++ file 'q3dscene.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../qtgraphs/src/graphs3d/engine/q3dscene.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'q3dscene.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN8Q3DSceneE_t {};
} // unnamed namespace

template <> constexpr inline auto Q3DScene::qt_create_metaobjectdata<qt_meta_tag_ZN8Q3DSceneE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Q3DScene",
        "QML.Element",
        "Scene3D",
        "QML.Creatable",
        "false",
        "QML.UncreatableReason",
        "",
        "viewportChanged",
        "QRect",
        "viewport",
        "primarySubViewportChanged",
        "subViewport",
        "secondarySubViewportChanged",
        "secondarySubviewOnTopChanged",
        "isSecondaryOnTop",
        "slicingActiveChanged",
        "isSlicingActive",
        "devicePixelRatioChanged",
        "pixelRatio",
        "selectionQueryPositionChanged",
        "QPoint",
        "position",
        "graphPositionQueryChanged",
        "needRender",
        "primarySubViewport",
        "secondarySubViewport",
        "selectionQueryPosition",
        "secondarySubviewOnTop",
        "slicingActive",
        "devicePixelRatio",
        "graphPositionQuery",
        "invalidSelectionPoint"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'viewportChanged'
        QtMocHelpers::SignalData<void(QRect)>(7, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 8, 9 },
        }}),
        // Signal 'primarySubViewportChanged'
        QtMocHelpers::SignalData<void(QRect)>(10, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 8, 11 },
        }}),
        // Signal 'secondarySubViewportChanged'
        QtMocHelpers::SignalData<void(QRect)>(12, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 8, 11 },
        }}),
        // Signal 'secondarySubviewOnTopChanged'
        QtMocHelpers::SignalData<void(bool)>(13, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 14 },
        }}),
        // Signal 'slicingActiveChanged'
        QtMocHelpers::SignalData<void(bool)>(15, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 16 },
        }}),
        // Signal 'devicePixelRatioChanged'
        QtMocHelpers::SignalData<void(qreal)>(17, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QReal, 18 },
        }}),
        // Signal 'selectionQueryPositionChanged'
        QtMocHelpers::SignalData<void(QPoint)>(19, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 20, 21 },
        }}),
        // Signal 'graphPositionQueryChanged'
        QtMocHelpers::SignalData<void(QPoint)>(22, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 20, 21 },
        }}),
        // Signal 'needRender'
        QtMocHelpers::SignalData<void()>(23, 6, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'viewport'
        QtMocHelpers::PropertyData<QRect>(9, 0x80000000 | 8, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 0),
        // property 'primarySubViewport'
        QtMocHelpers::PropertyData<QRect>(24, 0x80000000 | 8, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 1),
        // property 'secondarySubViewport'
        QtMocHelpers::PropertyData<QRect>(25, 0x80000000 | 8, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 2),
        // property 'selectionQueryPosition'
        QtMocHelpers::PropertyData<QPoint>(26, 0x80000000 | 20, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 6),
        // property 'secondarySubviewOnTop'
        QtMocHelpers::PropertyData<bool>(27, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 3),
        // property 'slicingActive'
        QtMocHelpers::PropertyData<bool>(28, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 4),
        // property 'devicePixelRatio'
        QtMocHelpers::PropertyData<qreal>(29, QMetaType::QReal, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 5),
        // property 'graphPositionQuery'
        QtMocHelpers::PropertyData<QPoint>(30, 0x80000000 | 20, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 7),
        // property 'invalidSelectionPoint'
        QtMocHelpers::PropertyData<QPoint>(31, 0x80000000 | 20, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
            {    5,    6 },
    });
    return QtMocHelpers::metaObjectData<Q3DScene, void>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject Q3DScene::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8Q3DSceneE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8Q3DSceneE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN8Q3DSceneE_t>.metaTypes,
    nullptr
} };

void Q3DScene::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<Q3DScene *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->viewportChanged((*reinterpret_cast<std::add_pointer_t<QRect>>(_a[1]))); break;
        case 1: _t->primarySubViewportChanged((*reinterpret_cast<std::add_pointer_t<QRect>>(_a[1]))); break;
        case 2: _t->secondarySubViewportChanged((*reinterpret_cast<std::add_pointer_t<QRect>>(_a[1]))); break;
        case 3: _t->secondarySubviewOnTopChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 4: _t->slicingActiveChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 5: _t->devicePixelRatioChanged((*reinterpret_cast<std::add_pointer_t<qreal>>(_a[1]))); break;
        case 6: _t->selectionQueryPositionChanged((*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[1]))); break;
        case 7: _t->graphPositionQueryChanged((*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[1]))); break;
        case 8: _t->needRender(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (Q3DScene::*)(QRect )>(_a, &Q3DScene::viewportChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (Q3DScene::*)(QRect )>(_a, &Q3DScene::primarySubViewportChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (Q3DScene::*)(QRect )>(_a, &Q3DScene::secondarySubViewportChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (Q3DScene::*)(bool )>(_a, &Q3DScene::secondarySubviewOnTopChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (Q3DScene::*)(bool )>(_a, &Q3DScene::slicingActiveChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (Q3DScene::*)(qreal )>(_a, &Q3DScene::devicePixelRatioChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (Q3DScene::*)(QPoint )>(_a, &Q3DScene::selectionQueryPositionChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (Q3DScene::*)(QPoint )>(_a, &Q3DScene::graphPositionQueryChanged, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (Q3DScene::*)()>(_a, &Q3DScene::needRender, 8))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QRect*>(_v) = _t->viewport(); break;
        case 1: *reinterpret_cast<QRect*>(_v) = _t->primarySubViewport(); break;
        case 2: *reinterpret_cast<QRect*>(_v) = _t->secondarySubViewport(); break;
        case 3: *reinterpret_cast<QPoint*>(_v) = _t->selectionQueryPosition(); break;
        case 4: *reinterpret_cast<bool*>(_v) = _t->isSecondarySubviewOnTop(); break;
        case 5: *reinterpret_cast<bool*>(_v) = _t->isSlicingActive(); break;
        case 6: *reinterpret_cast<qreal*>(_v) = _t->devicePixelRatio(); break;
        case 7: *reinterpret_cast<QPoint*>(_v) = _t->graphPositionQuery(); break;
        case 8: *reinterpret_cast<QPoint*>(_v) = _t->invalidSelectionPoint(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 1: _t->setPrimarySubViewport(*reinterpret_cast<QRect*>(_v)); break;
        case 2: _t->setSecondarySubViewport(*reinterpret_cast<QRect*>(_v)); break;
        case 3: _t->setSelectionQueryPosition(*reinterpret_cast<QPoint*>(_v)); break;
        case 4: _t->setSecondarySubviewOnTop(*reinterpret_cast<bool*>(_v)); break;
        case 5: _t->setSlicingActive(*reinterpret_cast<bool*>(_v)); break;
        case 6: _t->setDevicePixelRatio(*reinterpret_cast<qreal*>(_v)); break;
        case 7: _t->setGraphPositionQuery(*reinterpret_cast<QPoint*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *Q3DScene::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Q3DScene::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8Q3DSceneE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int Q3DScene::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 9)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 9)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 9;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    }
    return _id;
}

// SIGNAL 0
void Q3DScene::viewportChanged(QRect _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void Q3DScene::primarySubViewportChanged(QRect _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void Q3DScene::secondarySubViewportChanged(QRect _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void Q3DScene::secondarySubviewOnTopChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void Q3DScene::slicingActiveChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void Q3DScene::devicePixelRatioChanged(qreal _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void Q3DScene::selectionQueryPositionChanged(QPoint _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}

// SIGNAL 7
void Q3DScene::graphPositionQueryChanged(QPoint _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1);
}

// SIGNAL 8
void Q3DScene::needRender()
{
    QMetaObject::activate(this, &staticMetaObject, 8, nullptr);
}
QT_WARNING_POP
