/****************************************************************************
** Meta object code from reading C++ file 'qsurfacedataproxy.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../qtgraphs/src/graphs3d/data/qsurfacedataproxy.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qsurfacedataproxy.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN17QSurfaceDataProxyE_t {};
} // unnamed namespace

template <> constexpr inline auto QSurfaceDataProxy::qt_create_metaobjectdata<qt_meta_tag_ZN17QSurfaceDataProxyE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QSurfaceDataProxy",
        "QML.Element",
        "SurfaceDataProxy",
        "QML.Creatable",
        "false",
        "QML.UncreatableReason",
        "",
        "arrayReset",
        "rowsAdded",
        "qsizetype",
        "startIndex",
        "count",
        "rowsChanged",
        "rowsRemoved",
        "rowsInserted",
        "itemChanged",
        "rowIndex",
        "columnIndex",
        "rowCountChanged",
        "columnCountChanged",
        "seriesChanged",
        "QSurface3DSeries*",
        "series",
        "rowCount",
        "columnCount"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'arrayReset'
        QtMocHelpers::SignalData<void()>(7, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'rowsAdded'
        QtMocHelpers::SignalData<void(qsizetype, qsizetype)>(8, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 10 }, { 0x80000000 | 9, 11 },
        }}),
        // Signal 'rowsChanged'
        QtMocHelpers::SignalData<void(qsizetype, qsizetype)>(12, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 10 }, { 0x80000000 | 9, 11 },
        }}),
        // Signal 'rowsRemoved'
        QtMocHelpers::SignalData<void(qsizetype, qsizetype)>(13, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 10 }, { 0x80000000 | 9, 11 },
        }}),
        // Signal 'rowsInserted'
        QtMocHelpers::SignalData<void(qsizetype, qsizetype)>(14, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 10 }, { 0x80000000 | 9, 11 },
        }}),
        // Signal 'itemChanged'
        QtMocHelpers::SignalData<void(qsizetype, qsizetype)>(15, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 16 }, { 0x80000000 | 9, 17 },
        }}),
        // Signal 'rowCountChanged'
        QtMocHelpers::SignalData<void(qsizetype)>(18, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 11 },
        }}),
        // Signal 'columnCountChanged'
        QtMocHelpers::SignalData<void(qsizetype)>(19, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 11 },
        }}),
        // Signal 'seriesChanged'
        QtMocHelpers::SignalData<void(QSurface3DSeries *)>(20, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 21, 22 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'rowCount'
        QtMocHelpers::PropertyData<qsizetype>(23, 0x80000000 | 9, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Final, 6),
        // property 'columnCount'
        QtMocHelpers::PropertyData<qsizetype>(24, 0x80000000 | 9, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Final, 7),
        // property 'series'
        QtMocHelpers::PropertyData<QSurface3DSeries*>(22, 0x80000000 | 21, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Final, 8),
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
            {    5,    6 },
    });
    return QtMocHelpers::metaObjectData<QSurfaceDataProxy, void>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject QSurfaceDataProxy::staticMetaObject = { {
    QMetaObject::SuperData::link<QAbstractDataProxy::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17QSurfaceDataProxyE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17QSurfaceDataProxyE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN17QSurfaceDataProxyE_t>.metaTypes,
    nullptr
} };

void QSurfaceDataProxy::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QSurfaceDataProxy *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->arrayReset(); break;
        case 1: _t->rowsAdded((*reinterpret_cast<std::add_pointer_t<qsizetype>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<qsizetype>>(_a[2]))); break;
        case 2: _t->rowsChanged((*reinterpret_cast<std::add_pointer_t<qsizetype>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<qsizetype>>(_a[2]))); break;
        case 3: _t->rowsRemoved((*reinterpret_cast<std::add_pointer_t<qsizetype>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<qsizetype>>(_a[2]))); break;
        case 4: _t->rowsInserted((*reinterpret_cast<std::add_pointer_t<qsizetype>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<qsizetype>>(_a[2]))); break;
        case 5: _t->itemChanged((*reinterpret_cast<std::add_pointer_t<qsizetype>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<qsizetype>>(_a[2]))); break;
        case 6: _t->rowCountChanged((*reinterpret_cast<std::add_pointer_t<qsizetype>>(_a[1]))); break;
        case 7: _t->columnCountChanged((*reinterpret_cast<std::add_pointer_t<qsizetype>>(_a[1]))); break;
        case 8: _t->seriesChanged((*reinterpret_cast<std::add_pointer_t<QSurface3DSeries*>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (QSurfaceDataProxy::*)()>(_a, &QSurfaceDataProxy::arrayReset, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (QSurfaceDataProxy::*)(qsizetype , qsizetype )>(_a, &QSurfaceDataProxy::rowsAdded, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (QSurfaceDataProxy::*)(qsizetype , qsizetype )>(_a, &QSurfaceDataProxy::rowsChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (QSurfaceDataProxy::*)(qsizetype , qsizetype )>(_a, &QSurfaceDataProxy::rowsRemoved, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (QSurfaceDataProxy::*)(qsizetype , qsizetype )>(_a, &QSurfaceDataProxy::rowsInserted, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (QSurfaceDataProxy::*)(qsizetype , qsizetype )>(_a, &QSurfaceDataProxy::itemChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (QSurfaceDataProxy::*)(qsizetype )>(_a, &QSurfaceDataProxy::rowCountChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (QSurfaceDataProxy::*)(qsizetype )>(_a, &QSurfaceDataProxy::columnCountChanged, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (QSurfaceDataProxy::*)(QSurface3DSeries * )>(_a, &QSurfaceDataProxy::seriesChanged, 8))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<qsizetype*>(_v) = _t->rowCount(); break;
        case 1: *reinterpret_cast<qsizetype*>(_v) = _t->columnCount(); break;
        case 2: *reinterpret_cast<QSurface3DSeries**>(_v) = _t->series(); break;
        default: break;
        }
    }
}

const QMetaObject *QSurfaceDataProxy::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QSurfaceDataProxy::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17QSurfaceDataProxyE_t>.strings))
        return static_cast<void*>(this);
    return QAbstractDataProxy::qt_metacast(_clname);
}

int QSurfaceDataProxy::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QAbstractDataProxy::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 9)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 9)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 9;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    }
    return _id;
}

// SIGNAL 0
void QSurfaceDataProxy::arrayReset()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void QSurfaceDataProxy::rowsAdded(qsizetype _t1, qsizetype _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2);
}

// SIGNAL 2
void QSurfaceDataProxy::rowsChanged(qsizetype _t1, qsizetype _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1, _t2);
}

// SIGNAL 3
void QSurfaceDataProxy::rowsRemoved(qsizetype _t1, qsizetype _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1, _t2);
}

// SIGNAL 4
void QSurfaceDataProxy::rowsInserted(qsizetype _t1, qsizetype _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1, _t2);
}

// SIGNAL 5
void QSurfaceDataProxy::itemChanged(qsizetype _t1, qsizetype _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1, _t2);
}

// SIGNAL 6
void QSurfaceDataProxy::rowCountChanged(qsizetype _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}

// SIGNAL 7
void QSurfaceDataProxy::columnCountChanged(qsizetype _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1);
}

// SIGNAL 8
void QSurfaceDataProxy::seriesChanged(QSurface3DSeries * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 8, nullptr, _t1);
}
QT_WARNING_POP
