/****************************************************************************
** Meta object code from reading C++ file 'qspline3dseries.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../qtgraphs/src/graphs3d/data/qspline3dseries.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qspline3dseries.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN15QSpline3DSeriesE_t {};
} // unnamed namespace

template <> constexpr inline auto QSpline3DSeries::qt_create_metaobjectdata<qt_meta_tag_ZN15QSpline3DSeriesE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QSpline3DSeries",
        "QML.AddedInVersion",
        "1545",
        "QML.Element",
        "auto",
        "QML.Creatable",
        "false",
        "QML.UncreatableReason",
        "Trying to create uncreatable: QSpline3DSeries, use Spline3DSeries "
        "instead.",
        "splineVisibilityChanged",
        "",
        "visible",
        "splineTensionChanged",
        "tension",
        "splineKnottingChanged",
        "knotting",
        "splineLoopingChanged",
        "looping",
        "splineColorChanged",
        "QColor",
        "color",
        "splineResolutionChanged",
        "resolution",
        "splineVisible",
        "splineTension",
        "splineKnotting",
        "splineLooping",
        "splineColor",
        "splineResolution"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'splineVisibilityChanged'
        QtMocHelpers::SignalData<void(bool)>(9, 10, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 11 },
        }}),
        // Signal 'splineTensionChanged'
        QtMocHelpers::SignalData<void(qreal)>(12, 10, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QReal, 13 },
        }}),
        // Signal 'splineKnottingChanged'
        QtMocHelpers::SignalData<void(qreal)>(14, 10, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QReal, 15 },
        }}),
        // Signal 'splineLoopingChanged'
        QtMocHelpers::SignalData<void(bool)>(16, 10, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 17 },
        }}),
        // Signal 'splineColorChanged'
        QtMocHelpers::SignalData<void(QColor)>(18, 10, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 19, 20 },
        }}),
        // Signal 'splineResolutionChanged'
        QtMocHelpers::SignalData<void(int)>(21, 10, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 22 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'splineVisible'
        QtMocHelpers::PropertyData<bool>(23, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 0),
        // property 'splineTension'
        QtMocHelpers::PropertyData<qreal>(24, QMetaType::QReal, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 1),
        // property 'splineKnotting'
        QtMocHelpers::PropertyData<qreal>(25, QMetaType::QReal, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 2),
        // property 'splineLooping'
        QtMocHelpers::PropertyData<bool>(26, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 3),
        // property 'splineColor'
        QtMocHelpers::PropertyData<QColor>(27, 0x80000000 | 19, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet | QMC::Final, 4),
        // property 'splineResolution'
        QtMocHelpers::PropertyData<int>(28, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 5),
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
            {    5,    6 },
            {    7,    8 },
    });
    return QtMocHelpers::metaObjectData<QSpline3DSeries, void>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject QSpline3DSeries::staticMetaObject = { {
    QMetaObject::SuperData::link<QScatter3DSeries::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15QSpline3DSeriesE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15QSpline3DSeriesE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN15QSpline3DSeriesE_t>.metaTypes,
    nullptr
} };

void QSpline3DSeries::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QSpline3DSeries *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->splineVisibilityChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 1: _t->splineTensionChanged((*reinterpret_cast<std::add_pointer_t<qreal>>(_a[1]))); break;
        case 2: _t->splineKnottingChanged((*reinterpret_cast<std::add_pointer_t<qreal>>(_a[1]))); break;
        case 3: _t->splineLoopingChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 4: _t->splineColorChanged((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        case 5: _t->splineResolutionChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (QSpline3DSeries::*)(bool )>(_a, &QSpline3DSeries::splineVisibilityChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (QSpline3DSeries::*)(qreal )>(_a, &QSpline3DSeries::splineTensionChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (QSpline3DSeries::*)(qreal )>(_a, &QSpline3DSeries::splineKnottingChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (QSpline3DSeries::*)(bool )>(_a, &QSpline3DSeries::splineLoopingChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (QSpline3DSeries::*)(QColor )>(_a, &QSpline3DSeries::splineColorChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (QSpline3DSeries::*)(int )>(_a, &QSpline3DSeries::splineResolutionChanged, 5))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<bool*>(_v) = _t->isSplineVisible(); break;
        case 1: *reinterpret_cast<qreal*>(_v) = _t->splineTension(); break;
        case 2: *reinterpret_cast<qreal*>(_v) = _t->splineKnotting(); break;
        case 3: *reinterpret_cast<bool*>(_v) = _t->isSplineLooping(); break;
        case 4: *reinterpret_cast<QColor*>(_v) = _t->splineColor(); break;
        case 5: *reinterpret_cast<int*>(_v) = _t->splineResolution(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setSplineVisible(*reinterpret_cast<bool*>(_v)); break;
        case 1: _t->setSplineTension(*reinterpret_cast<qreal*>(_v)); break;
        case 2: _t->setSplineKnotting(*reinterpret_cast<qreal*>(_v)); break;
        case 3: _t->setSplineLooping(*reinterpret_cast<bool*>(_v)); break;
        case 4: _t->setSplineColor(*reinterpret_cast<QColor*>(_v)); break;
        case 5: _t->setSplineResolution(*reinterpret_cast<int*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *QSpline3DSeries::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QSpline3DSeries::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15QSpline3DSeriesE_t>.strings))
        return static_cast<void*>(this);
    return QScatter3DSeries::qt_metacast(_clname);
}

int QSpline3DSeries::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QScatter3DSeries::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 6)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 6)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 6;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    }
    return _id;
}

// SIGNAL 0
void QSpline3DSeries::splineVisibilityChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void QSpline3DSeries::splineTensionChanged(qreal _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void QSpline3DSeries::splineKnottingChanged(qreal _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void QSpline3DSeries::splineLoopingChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void QSpline3DSeries::splineColorChanged(QColor _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void QSpline3DSeries::splineResolutionChanged(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}
QT_WARNING_POP
