// Copyright (C) 2025 The Qt Company Ltd.
// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
// Qt-Security score:significant reason:default


//
//  W A R N I N G
//  -------------
//
// This file is not part of the QtGraphs API.  It exists purely as an
// implementation detail.  This header file may change from version to
// version without notice, or even be removed.
//
// We mean it.

#ifndef QGRAPHS3DLOGGING
#define QGRAPHS3DLOGGING

#include <QtCore/qloggingcategory.h>

QT_BEGIN_NAMESPACE

Q_DECLARE_LOGGING_CATEGORY(lcGraphs3D)
Q_DECLARE_LOGGING_CATEGORY(lcSeries3D)
Q_DECLARE_LOGGING_CATEGORY(lcProperties3D)
Q_DECLARE_LOGGING_CATEGORY(lcEvents3D)
Q_DECLARE_LOGGING_CATEGORY(lcInput3D)
Q_DECLARE_LOGGING_CATEGORY(lcCritical3D)
Q_DECLARE_LOGGING_CATEGORY(lcAxis3D)
Q_DECLARE_LOGGING_CATEGORY(lcAProperties3D)
Q_DECLARE_LOGGING_CATEGORY(lcGraphsScene3D)
Q_DECLARE_LOGGING_CATEGORY(lcGraphsBars3D)
Q_DECLARE_LOGGING_CATEGORY(lcGraphsScatter3D)
Q_DECLARE_LOGGING_CATEGORY(lcGraphsSurface3D)

QT_END_NAMESPACE

#endif // QGRAPHS3DLOGGING

