/****************************************************************************
** Meta object code from reading C++ file 'qpieseries.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../qtgraphs/src/graphs2d/piechart/qpieseries.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qpieseries.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN10QPieSeriesE_t {};
} // unnamed namespace

template <> constexpr inline auto QPieSeries::qt_create_metaobjectdata<qt_meta_tag_ZN10QPieSeriesE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QPieSeries",
        "RegisterEnumClassesUnscoped",
        "false",
        "QML.Element",
        "PieSeries",
        "added",
        "",
        "QList<QPieSlice*>",
        "slices",
        "removed",
        "replaced",
        "countChanged",
        "sumChanged",
        "pieSizeChanged",
        "startAngleChanged",
        "endAngleChanged",
        "horizontalPositionChanged",
        "verticalPositionChanged",
        "holeSizeChanged",
        "clicked",
        "QPieSlice*",
        "slice",
        "doubleClicked",
        "pressed",
        "released",
        "angleSpanVisibleLimitChanged",
        "limit",
        "angleSpanLabelVisibilityChanged",
        "LabelVisibility",
        "visibility",
        "sliceDataChanged",
        "QList<qreal>",
        "sliceData",
        "sliceLabelsChanged",
        "sliceLabels",
        "handleSliceChange",
        "append",
        "insert",
        "qsizetype",
        "index",
        "remove",
        "clear",
        "label",
        "value",
        "at",
        "find",
        "replace",
        "removeMultiple",
        "count",
        "oldSlice",
        "newSlice",
        "take",
        "horizontalPosition",
        "verticalPosition",
        "pieSize",
        "startAngle",
        "endAngle",
        "sum",
        "holeSize",
        "angleSpanVisibleLimit",
        "angleSpanLabelVisibility",
        "None",
        "First",
        "Even",
        "Odd"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'added'
        QtMocHelpers::SignalData<void(const QList<QPieSlice*> &)>(5, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 8 },
        }}),
        // Signal 'removed'
        QtMocHelpers::SignalData<void(const QList<QPieSlice*> &)>(9, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 8 },
        }}),
        // Signal 'replaced'
        QtMocHelpers::SignalData<void(const QList<QPieSlice*> &)>(10, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 8 },
        }}),
        // Signal 'countChanged'
        QtMocHelpers::SignalData<void()>(11, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'sumChanged'
        QtMocHelpers::SignalData<void()>(12, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'pieSizeChanged'
        QtMocHelpers::SignalData<void()>(13, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'startAngleChanged'
        QtMocHelpers::SignalData<void()>(14, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'endAngleChanged'
        QtMocHelpers::SignalData<void()>(15, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'horizontalPositionChanged'
        QtMocHelpers::SignalData<void()>(16, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'verticalPositionChanged'
        QtMocHelpers::SignalData<void()>(17, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'holeSizeChanged'
        QtMocHelpers::SignalData<void()>(18, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'clicked'
        QtMocHelpers::RevisionedSignalData<void(QPieSlice *)>(19, 6, QMC::AccessPublic, 0x609, QMetaType::Void, {{
            { 0x80000000 | 20, 21 },
        }}),
        // Signal 'doubleClicked'
        QtMocHelpers::RevisionedSignalData<void(QPieSlice *)>(22, 6, QMC::AccessPublic, 0x609, QMetaType::Void, {{
            { 0x80000000 | 20, 21 },
        }}),
        // Signal 'pressed'
        QtMocHelpers::RevisionedSignalData<void(QPieSlice *)>(23, 6, QMC::AccessPublic, 0x609, QMetaType::Void, {{
            { 0x80000000 | 20, 21 },
        }}),
        // Signal 'released'
        QtMocHelpers::RevisionedSignalData<void(QPieSlice *)>(24, 6, QMC::AccessPublic, 0x609, QMetaType::Void, {{
            { 0x80000000 | 20, 21 },
        }}),
        // Signal 'angleSpanVisibleLimitChanged'
        QtMocHelpers::RevisionedSignalData<void(qreal)>(25, 6, QMC::AccessPublic, 0x60a, QMetaType::Void, {{
            { QMetaType::QReal, 26 },
        }}),
        // Signal 'angleSpanLabelVisibilityChanged'
        QtMocHelpers::RevisionedSignalData<void(enum LabelVisibility)>(27, 6, QMC::AccessPublic, 0x60a, QMetaType::Void, {{
            { 0x80000000 | 28, 29 },
        }}),
        // Signal 'sliceDataChanged'
        QtMocHelpers::RevisionedSignalData<void(const QList<qreal> &)>(30, 6, QMC::AccessPublic, 0x60c, QMetaType::Void, {{
            { 0x80000000 | 31, 32 },
        }}),
        // Signal 'sliceLabelsChanged'
        QtMocHelpers::RevisionedSignalData<void(const QStringList &)>(33, 6, QMC::AccessPublic, 0x60c, QMetaType::Void, {{
            { QMetaType::QStringList, 34 },
        }}),
        // Slot 'handleSliceChange'
        QtMocHelpers::SlotData<void()>(35, 6, QMC::AccessPublic, QMetaType::Void),
        // Method 'append'
        QtMocHelpers::MethodData<bool(QPieSlice *)>(36, 6, QMC::AccessPublic, QMetaType::Bool, {{
            { 0x80000000 | 20, 21 },
        }}),
        // Method 'append'
        QtMocHelpers::MethodData<bool(const QList<QPieSlice*> &)>(36, 6, QMC::AccessPublic, QMetaType::Bool, {{
            { 0x80000000 | 7, 8 },
        }}),
        // Method 'insert'
        QtMocHelpers::MethodData<bool(qsizetype, QPieSlice *)>(37, 6, QMC::AccessPublic, QMetaType::Bool, {{
            { 0x80000000 | 38, 39 }, { 0x80000000 | 20, 21 },
        }}),
        // Method 'remove'
        QtMocHelpers::MethodData<bool(QPieSlice *)>(40, 6, QMC::AccessPublic, QMetaType::Bool, {{
            { 0x80000000 | 20, 21 },
        }}),
        // Method 'clear'
        QtMocHelpers::MethodData<void()>(41, 6, QMC::AccessPublic, QMetaType::Void),
        // Method 'append'
        QtMocHelpers::MethodData<QPieSlice *(const QString &, qreal)>(36, 6, QMC::AccessPublic, 0x80000000 | 20, {{
            { QMetaType::QString, 42 }, { QMetaType::QReal, 43 },
        }}),
        // Method 'at'
        QtMocHelpers::MethodData<QPieSlice *(qsizetype)>(44, 6, QMC::AccessPublic, 0x80000000 | 20, {{
            { 0x80000000 | 38, 39 },
        }}),
        // Method 'find'
        QtMocHelpers::MethodData<QPieSlice *(const QString &)>(45, 6, QMC::AccessPublic, 0x80000000 | 20, {{
            { QMetaType::QString, 42 },
        }}),
        // Method 'replace'
        QtMocHelpers::MethodData<bool(qsizetype, QPieSlice *)>(46, 6, QMC::AccessPublic, QMetaType::Bool, {{
            { 0x80000000 | 38, 39 }, { 0x80000000 | 20, 21 },
        }}),
        // Method 'removeMultiple'
        QtMocHelpers::MethodData<void(qsizetype, int)>(47, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 38, 39 }, { QMetaType::Int, 48 },
        }}),
        // Method 'remove'
        QtMocHelpers::MethodData<bool(qsizetype)>(40, 6, QMC::AccessPublic, QMetaType::Bool, {{
            { 0x80000000 | 38, 39 },
        }}),
        // Method 'replace'
        QtMocHelpers::MethodData<bool(QPieSlice *, QPieSlice *)>(46, 6, QMC::AccessPublic, QMetaType::Bool, {{
            { 0x80000000 | 20, 49 }, { 0x80000000 | 20, 50 },
        }}),
        // Method 'replace'
        QtMocHelpers::MethodData<bool(const QList<QPieSlice*> &)>(46, 6, QMC::AccessPublic, QMetaType::Bool, {{
            { 0x80000000 | 7, 8 },
        }}),
        // Method 'take'
        QtMocHelpers::MethodData<bool(QPieSlice *)>(51, 6, QMC::AccessPublic, QMetaType::Bool, {{
            { 0x80000000 | 20, 21 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'horizontalPosition'
        QtMocHelpers::PropertyData<qreal>(52, QMetaType::QReal, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 8),
        // property 'verticalPosition'
        QtMocHelpers::PropertyData<qreal>(53, QMetaType::QReal, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 9),
        // property 'pieSize'
        QtMocHelpers::PropertyData<qreal>(54, QMetaType::QReal, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 5),
        // property 'startAngle'
        QtMocHelpers::PropertyData<qreal>(55, QMetaType::QReal, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 6),
        // property 'endAngle'
        QtMocHelpers::PropertyData<qreal>(56, QMetaType::QReal, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 7),
        // property 'count'
        QtMocHelpers::PropertyData<qsizetype>(48, 0x80000000 | 38, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Final, 3),
        // property 'sum'
        QtMocHelpers::PropertyData<qreal>(57, QMetaType::QReal, QMC::DefaultPropertyFlags | QMC::Final, 4),
        // property 'holeSize'
        QtMocHelpers::PropertyData<qreal>(58, QMetaType::QReal, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 10),
        // property 'angleSpanVisibleLimit'
        QtMocHelpers::PropertyData<qreal>(59, QMetaType::QReal, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 15, 0x60a),
        // property 'angleSpanLabelVisibility'
        QtMocHelpers::PropertyData<enum LabelVisibility>(60, 0x80000000 | 28, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 16, 0x60a),
        // property 'sliceData'
        QtMocHelpers::PropertyData<QList<qreal>>(32, 0x80000000 | 31, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 17, 0x60c),
        // property 'sliceLabels'
        QtMocHelpers::PropertyData<QStringList>(34, QMetaType::QStringList, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 18, 0x60c),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'LabelVisibility'
        QtMocHelpers::EnumData<enum LabelVisibility>(28, 28, QMC::EnumIsScoped).add({
            {   61, LabelVisibility::None },
            {   62, LabelVisibility::First },
            {   63, LabelVisibility::Even },
            {   64, LabelVisibility::Odd },
        }),
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
    });
    return QtMocHelpers::metaObjectData<QPieSeries, void>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject QPieSeries::staticMetaObject = { {
    QMetaObject::SuperData::link<QAbstractSeries::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10QPieSeriesE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10QPieSeriesE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10QPieSeriesE_t>.metaTypes,
    nullptr
} };

void QPieSeries::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QPieSeries *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->added((*reinterpret_cast<std::add_pointer_t<QList<QPieSlice*>>>(_a[1]))); break;
        case 1: _t->removed((*reinterpret_cast<std::add_pointer_t<QList<QPieSlice*>>>(_a[1]))); break;
        case 2: _t->replaced((*reinterpret_cast<std::add_pointer_t<QList<QPieSlice*>>>(_a[1]))); break;
        case 3: _t->countChanged(); break;
        case 4: _t->sumChanged(); break;
        case 5: _t->pieSizeChanged(); break;
        case 6: _t->startAngleChanged(); break;
        case 7: _t->endAngleChanged(); break;
        case 8: _t->horizontalPositionChanged(); break;
        case 9: _t->verticalPositionChanged(); break;
        case 10: _t->holeSizeChanged(); break;
        case 11: _t->clicked((*reinterpret_cast<std::add_pointer_t<QPieSlice*>>(_a[1]))); break;
        case 12: _t->doubleClicked((*reinterpret_cast<std::add_pointer_t<QPieSlice*>>(_a[1]))); break;
        case 13: _t->pressed((*reinterpret_cast<std::add_pointer_t<QPieSlice*>>(_a[1]))); break;
        case 14: _t->released((*reinterpret_cast<std::add_pointer_t<QPieSlice*>>(_a[1]))); break;
        case 15: _t->angleSpanVisibleLimitChanged((*reinterpret_cast<std::add_pointer_t<qreal>>(_a[1]))); break;
        case 16: _t->angleSpanLabelVisibilityChanged((*reinterpret_cast<std::add_pointer_t<enum LabelVisibility>>(_a[1]))); break;
        case 17: _t->sliceDataChanged((*reinterpret_cast<std::add_pointer_t<QList<qreal>>>(_a[1]))); break;
        case 18: _t->sliceLabelsChanged((*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[1]))); break;
        case 19: _t->handleSliceChange(); break;
        case 20: { bool _r = _t->append((*reinterpret_cast<std::add_pointer_t<QPieSlice*>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 21: { bool _r = _t->append((*reinterpret_cast<std::add_pointer_t<QList<QPieSlice*>>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 22: { bool _r = _t->insert((*reinterpret_cast<std::add_pointer_t<qsizetype>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QPieSlice*>>(_a[2])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 23: { bool _r = _t->remove((*reinterpret_cast<std::add_pointer_t<QPieSlice*>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 24: _t->clear(); break;
        case 25: { QPieSlice* _r = _t->append((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<qreal>>(_a[2])));
            if (_a[0]) *reinterpret_cast<QPieSlice**>(_a[0]) = std::move(_r); }  break;
        case 26: { QPieSlice* _r = _t->at((*reinterpret_cast<std::add_pointer_t<qsizetype>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QPieSlice**>(_a[0]) = std::move(_r); }  break;
        case 27: { QPieSlice* _r = _t->find((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QPieSlice**>(_a[0]) = std::move(_r); }  break;
        case 28: { bool _r = _t->replace((*reinterpret_cast<std::add_pointer_t<qsizetype>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QPieSlice*>>(_a[2])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 29: _t->removeMultiple((*reinterpret_cast<std::add_pointer_t<qsizetype>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 30: { bool _r = _t->remove((*reinterpret_cast<std::add_pointer_t<qsizetype>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 31: { bool _r = _t->replace((*reinterpret_cast<std::add_pointer_t<QPieSlice*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QPieSlice*>>(_a[2])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 32: { bool _r = _t->replace((*reinterpret_cast<std::add_pointer_t<QList<QPieSlice*>>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 33: { bool _r = _t->take((*reinterpret_cast<std::add_pointer_t<QPieSlice*>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QList<QPieSlice*> >(); break;
            }
            break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QList<QPieSlice*> >(); break;
            }
            break;
        case 2:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QList<QPieSlice*> >(); break;
            }
            break;
        case 11:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QPieSlice* >(); break;
            }
            break;
        case 12:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QPieSlice* >(); break;
            }
            break;
        case 13:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QPieSlice* >(); break;
            }
            break;
        case 14:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QPieSlice* >(); break;
            }
            break;
        case 17:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QList<qreal> >(); break;
            }
            break;
        case 20:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QPieSlice* >(); break;
            }
            break;
        case 21:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QList<QPieSlice*> >(); break;
            }
            break;
        case 22:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QPieSlice* >(); break;
            }
            break;
        case 23:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QPieSlice* >(); break;
            }
            break;
        case 28:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QPieSlice* >(); break;
            }
            break;
        case 31:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QPieSlice* >(); break;
            }
            break;
        case 32:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QList<QPieSlice*> >(); break;
            }
            break;
        case 33:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QPieSlice* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (QPieSeries::*)(const QList<QPieSlice*> & )>(_a, &QPieSeries::added, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (QPieSeries::*)(const QList<QPieSlice*> & )>(_a, &QPieSeries::removed, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (QPieSeries::*)(const QList<QPieSlice*> & )>(_a, &QPieSeries::replaced, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (QPieSeries::*)()>(_a, &QPieSeries::countChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (QPieSeries::*)()>(_a, &QPieSeries::sumChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (QPieSeries::*)()>(_a, &QPieSeries::pieSizeChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (QPieSeries::*)()>(_a, &QPieSeries::startAngleChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (QPieSeries::*)()>(_a, &QPieSeries::endAngleChanged, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (QPieSeries::*)()>(_a, &QPieSeries::horizontalPositionChanged, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (QPieSeries::*)()>(_a, &QPieSeries::verticalPositionChanged, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (QPieSeries::*)()>(_a, &QPieSeries::holeSizeChanged, 10))
            return;
        if (QtMocHelpers::indexOfMethod<void (QPieSeries::*)(QPieSlice * )>(_a, &QPieSeries::clicked, 11))
            return;
        if (QtMocHelpers::indexOfMethod<void (QPieSeries::*)(QPieSlice * )>(_a, &QPieSeries::doubleClicked, 12))
            return;
        if (QtMocHelpers::indexOfMethod<void (QPieSeries::*)(QPieSlice * )>(_a, &QPieSeries::pressed, 13))
            return;
        if (QtMocHelpers::indexOfMethod<void (QPieSeries::*)(QPieSlice * )>(_a, &QPieSeries::released, 14))
            return;
        if (QtMocHelpers::indexOfMethod<void (QPieSeries::*)(qreal )>(_a, &QPieSeries::angleSpanVisibleLimitChanged, 15))
            return;
        if (QtMocHelpers::indexOfMethod<void (QPieSeries::*)(LabelVisibility )>(_a, &QPieSeries::angleSpanLabelVisibilityChanged, 16))
            return;
        if (QtMocHelpers::indexOfMethod<void (QPieSeries::*)(const QList<qreal> & )>(_a, &QPieSeries::sliceDataChanged, 17))
            return;
        if (QtMocHelpers::indexOfMethod<void (QPieSeries::*)(const QStringList & )>(_a, &QPieSeries::sliceLabelsChanged, 18))
            return;
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 10:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<qreal> >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<qreal*>(_v) = _t->horizontalPosition(); break;
        case 1: *reinterpret_cast<qreal*>(_v) = _t->verticalPosition(); break;
        case 2: *reinterpret_cast<qreal*>(_v) = _t->pieSize(); break;
        case 3: *reinterpret_cast<qreal*>(_v) = _t->startAngle(); break;
        case 4: *reinterpret_cast<qreal*>(_v) = _t->endAngle(); break;
        case 5: *reinterpret_cast<qsizetype*>(_v) = _t->count(); break;
        case 6: *reinterpret_cast<qreal*>(_v) = _t->sum(); break;
        case 7: *reinterpret_cast<qreal*>(_v) = _t->holeSize(); break;
        case 8: *reinterpret_cast<qreal*>(_v) = _t->angleSpanVisibleLimit(); break;
        case 9: *reinterpret_cast<enum LabelVisibility*>(_v) = _t->angleSpanLabelVisibility(); break;
        case 10: *reinterpret_cast<QList<qreal>*>(_v) = _t->sliceData(); break;
        case 11: *reinterpret_cast<QStringList*>(_v) = _t->sliceLabels(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setHorizontalPosition(*reinterpret_cast<qreal*>(_v)); break;
        case 1: _t->setVerticalPosition(*reinterpret_cast<qreal*>(_v)); break;
        case 2: _t->setPieSize(*reinterpret_cast<qreal*>(_v)); break;
        case 3: _t->setStartAngle(*reinterpret_cast<qreal*>(_v)); break;
        case 4: _t->setEndAngle(*reinterpret_cast<qreal*>(_v)); break;
        case 7: _t->setHoleSize(*reinterpret_cast<qreal*>(_v)); break;
        case 8: _t->setAngleSpanVisibleLimit(*reinterpret_cast<qreal*>(_v)); break;
        case 9: _t->setAngleSpanLabelVisibility(*reinterpret_cast<enum LabelVisibility*>(_v)); break;
        case 10: _t->setSliceData(*reinterpret_cast<QList<qreal>*>(_v)); break;
        case 11: _t->setSliceLabels(*reinterpret_cast<QStringList*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *QPieSeries::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QPieSeries::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10QPieSeriesE_t>.strings))
        return static_cast<void*>(this);
    return QAbstractSeries::qt_metacast(_clname);
}

int QPieSeries::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QAbstractSeries::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 34)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 34;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 34)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 34;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 12;
    }
    return _id;
}

// SIGNAL 0
void QPieSeries::added(const QList<QPieSlice*> & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void QPieSeries::removed(const QList<QPieSlice*> & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void QPieSeries::replaced(const QList<QPieSlice*> & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void QPieSeries::countChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void QPieSeries::sumChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void QPieSeries::pieSizeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void QPieSeries::startAngleChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void QPieSeries::endAngleChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void QPieSeries::horizontalPositionChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 8, nullptr);
}

// SIGNAL 9
void QPieSeries::verticalPositionChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 9, nullptr);
}

// SIGNAL 10
void QPieSeries::holeSizeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 10, nullptr);
}

// SIGNAL 11
void QPieSeries::clicked(QPieSlice * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 11, nullptr, _t1);
}

// SIGNAL 12
void QPieSeries::doubleClicked(QPieSlice * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 12, nullptr, _t1);
}

// SIGNAL 13
void QPieSeries::pressed(QPieSlice * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 13, nullptr, _t1);
}

// SIGNAL 14
void QPieSeries::released(QPieSlice * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 14, nullptr, _t1);
}

// SIGNAL 15
void QPieSeries::angleSpanVisibleLimitChanged(qreal _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 15, nullptr, _t1);
}

// SIGNAL 16
void QPieSeries::angleSpanLabelVisibilityChanged(LabelVisibility _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 16, nullptr, _t1);
}

// SIGNAL 17
void QPieSeries::sliceDataChanged(const QList<qreal> & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 17, nullptr, _t1);
}

// SIGNAL 18
void QPieSeries::sliceLabelsChanged(const QStringList & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 18, nullptr, _t1);
}
QT_WARNING_POP
