/****************************************************************************
** Meta object code from reading C++ file 'qabstract3daxis.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../qtgraphs/src/graphs3d/axis/qabstract3daxis.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qabstract3daxis.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN15QAbstract3DAxisE_t {};
} // unnamed namespace

template <> constexpr inline auto QAbstract3DAxis::qt_create_metaobjectdata<qt_meta_tag_ZN15QAbstract3DAxisE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QAbstract3DAxis",
        "RegisterEnumClassesUnscoped",
        "false",
        "QML.Element",
        "Abstract3DAxis",
        "QML.Creatable",
        "QML.UncreatableReason",
        "",
        "titleChanged",
        "newTitle",
        "labelsChanged",
        "orientationChanged",
        "QAbstract3DAxis::AxisOrientation",
        "orientation",
        "minChanged",
        "value",
        "maxChanged",
        "rangeChanged",
        "min",
        "max",
        "autoAdjustRangeChanged",
        "autoAdjust",
        "scaleLabelsByCountChanged",
        "adjust",
        "labelSizeChanged",
        "size",
        "labelAutoAngleChanged",
        "angle",
        "titleVisibleChanged",
        "visible",
        "labelVisibleChanged",
        "titleFixedChanged",
        "fixed",
        "titleOffsetChanged",
        "offset",
        "title",
        "labels",
        "labelsVisible",
        "type",
        "QAbstract3DAxis::AxisType",
        "autoAdjustRange",
        "labelAutoAngle",
        "scaleLabelsByCount",
        "labelSize",
        "titleVisible",
        "titleFixed",
        "titleOffset",
        "AxisOrientation",
        "None",
        "X",
        "Y",
        "Z",
        "AxisType",
        "Category",
        "Value"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'titleChanged'
        QtMocHelpers::SignalData<void(const QString &)>(8, 7, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 9 },
        }}),
        // Signal 'labelsChanged'
        QtMocHelpers::SignalData<void()>(10, 7, QMC::AccessPublic, QMetaType::Void),
        // Signal 'orientationChanged'
        QtMocHelpers::SignalData<void(QAbstract3DAxis::AxisOrientation)>(11, 7, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 12, 13 },
        }}),
        // Signal 'minChanged'
        QtMocHelpers::SignalData<void(float)>(14, 7, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Float, 15 },
        }}),
        // Signal 'maxChanged'
        QtMocHelpers::SignalData<void(float)>(16, 7, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Float, 15 },
        }}),
        // Signal 'rangeChanged'
        QtMocHelpers::SignalData<void(float, float)>(17, 7, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Float, 18 }, { QMetaType::Float, 19 },
        }}),
        // Signal 'autoAdjustRangeChanged'
        QtMocHelpers::SignalData<void(bool)>(20, 7, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 21 },
        }}),
        // Signal 'scaleLabelsByCountChanged'
        QtMocHelpers::RevisionedSignalData<void(bool)>(22, 7, QMC::AccessPublic, 0x609, QMetaType::Void, {{
            { QMetaType::Bool, 23 },
        }}),
        // Signal 'labelSizeChanged'
        QtMocHelpers::RevisionedSignalData<void(qreal)>(24, 7, QMC::AccessPublic, 0x609, QMetaType::Void, {{
            { QMetaType::QReal, 25 },
        }}),
        // Signal 'labelAutoAngleChanged'
        QtMocHelpers::SignalData<void(float)>(26, 7, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Float, 27 },
        }}),
        // Signal 'titleVisibleChanged'
        QtMocHelpers::SignalData<void(bool)>(28, 7, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 29 },
        }}),
        // Signal 'labelVisibleChanged'
        QtMocHelpers::SignalData<void(bool)>(30, 7, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 29 },
        }}),
        // Signal 'titleFixedChanged'
        QtMocHelpers::SignalData<void(bool)>(31, 7, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 32 },
        }}),
        // Signal 'titleOffsetChanged'
        QtMocHelpers::SignalData<void(float)>(33, 7, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Float, 34 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'title'
        QtMocHelpers::PropertyData<QString>(35, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 0),
        // property 'labels'
        QtMocHelpers::PropertyData<QStringList>(36, QMetaType::QStringList, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 1),
        // property 'labelsVisible'
        QtMocHelpers::PropertyData<bool>(37, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 11),
        // property 'orientation'
        QtMocHelpers::PropertyData<QAbstract3DAxis::AxisOrientation>(13, 0x80000000 | 12, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Final, 2),
        // property 'type'
        QtMocHelpers::PropertyData<QAbstract3DAxis::AxisType>(38, 0x80000000 | 39, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'min'
        QtMocHelpers::PropertyData<float>(18, QMetaType::Float, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 3),
        // property 'max'
        QtMocHelpers::PropertyData<float>(19, QMetaType::Float, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 4),
        // property 'autoAdjustRange'
        QtMocHelpers::PropertyData<bool>(40, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 6),
        // property 'labelAutoAngle'
        QtMocHelpers::PropertyData<float>(41, QMetaType::Float, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 9),
        // property 'scaleLabelsByCount'
        QtMocHelpers::PropertyData<bool>(42, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 7, 0x609),
        // property 'labelSize'
        QtMocHelpers::PropertyData<qreal>(43, QMetaType::QReal, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 8, 0x609),
        // property 'titleVisible'
        QtMocHelpers::PropertyData<bool>(44, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 10),
        // property 'titleFixed'
        QtMocHelpers::PropertyData<bool>(45, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 12),
        // property 'titleOffset'
        QtMocHelpers::PropertyData<float>(46, QMetaType::Float, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 13),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'AxisOrientation'
        QtMocHelpers::EnumData<enum AxisOrientation>(47, 47, QMC::EnumIsScoped).add({
            {   48, AxisOrientation::None },
            {   49, AxisOrientation::X },
            {   50, AxisOrientation::Y },
            {   51, AxisOrientation::Z },
        }),
        // enum 'AxisType'
        QtMocHelpers::EnumData<enum AxisType>(52, 52, QMC::EnumIsScoped).add({
            {   48, AxisType::None },
            {   53, AxisType::Category },
            {   54, AxisType::Value },
        }),
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
            {    5,    2 },
            {    6,    7 },
    });
    return QtMocHelpers::metaObjectData<QAbstract3DAxis, void>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject QAbstract3DAxis::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15QAbstract3DAxisE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15QAbstract3DAxisE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN15QAbstract3DAxisE_t>.metaTypes,
    nullptr
} };

void QAbstract3DAxis::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QAbstract3DAxis *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->titleChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 1: _t->labelsChanged(); break;
        case 2: _t->orientationChanged((*reinterpret_cast<std::add_pointer_t<QAbstract3DAxis::AxisOrientation>>(_a[1]))); break;
        case 3: _t->minChanged((*reinterpret_cast<std::add_pointer_t<float>>(_a[1]))); break;
        case 4: _t->maxChanged((*reinterpret_cast<std::add_pointer_t<float>>(_a[1]))); break;
        case 5: _t->rangeChanged((*reinterpret_cast<std::add_pointer_t<float>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[2]))); break;
        case 6: _t->autoAdjustRangeChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 7: _t->scaleLabelsByCountChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 8: _t->labelSizeChanged((*reinterpret_cast<std::add_pointer_t<qreal>>(_a[1]))); break;
        case 9: _t->labelAutoAngleChanged((*reinterpret_cast<std::add_pointer_t<float>>(_a[1]))); break;
        case 10: _t->titleVisibleChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 11: _t->labelVisibleChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 12: _t->titleFixedChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 13: _t->titleOffsetChanged((*reinterpret_cast<std::add_pointer_t<float>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (QAbstract3DAxis::*)(const QString & )>(_a, &QAbstract3DAxis::titleChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAbstract3DAxis::*)()>(_a, &QAbstract3DAxis::labelsChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAbstract3DAxis::*)(QAbstract3DAxis::AxisOrientation )>(_a, &QAbstract3DAxis::orientationChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAbstract3DAxis::*)(float )>(_a, &QAbstract3DAxis::minChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAbstract3DAxis::*)(float )>(_a, &QAbstract3DAxis::maxChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAbstract3DAxis::*)(float , float )>(_a, &QAbstract3DAxis::rangeChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAbstract3DAxis::*)(bool )>(_a, &QAbstract3DAxis::autoAdjustRangeChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAbstract3DAxis::*)(bool )>(_a, &QAbstract3DAxis::scaleLabelsByCountChanged, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAbstract3DAxis::*)(qreal )>(_a, &QAbstract3DAxis::labelSizeChanged, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAbstract3DAxis::*)(float )>(_a, &QAbstract3DAxis::labelAutoAngleChanged, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAbstract3DAxis::*)(bool )>(_a, &QAbstract3DAxis::titleVisibleChanged, 10))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAbstract3DAxis::*)(bool )>(_a, &QAbstract3DAxis::labelVisibleChanged, 11))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAbstract3DAxis::*)(bool )>(_a, &QAbstract3DAxis::titleFixedChanged, 12))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAbstract3DAxis::*)(float )>(_a, &QAbstract3DAxis::titleOffsetChanged, 13))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QString*>(_v) = _t->title(); break;
        case 1: *reinterpret_cast<QStringList*>(_v) = _t->labels(); break;
        case 2: *reinterpret_cast<bool*>(_v) = _t->labelsVisible(); break;
        case 3: *reinterpret_cast<QAbstract3DAxis::AxisOrientation*>(_v) = _t->orientation(); break;
        case 4: *reinterpret_cast<QAbstract3DAxis::AxisType*>(_v) = _t->type(); break;
        case 5: *reinterpret_cast<float*>(_v) = _t->min(); break;
        case 6: *reinterpret_cast<float*>(_v) = _t->max(); break;
        case 7: *reinterpret_cast<bool*>(_v) = _t->isAutoAdjustRange(); break;
        case 8: *reinterpret_cast<float*>(_v) = _t->labelAutoAngle(); break;
        case 9: *reinterpret_cast<bool*>(_v) = _t->isScaleLabelsByCount(); break;
        case 10: *reinterpret_cast<qreal*>(_v) = _t->labelSize(); break;
        case 11: *reinterpret_cast<bool*>(_v) = _t->isTitleVisible(); break;
        case 12: *reinterpret_cast<bool*>(_v) = _t->isTitleFixed(); break;
        case 13: *reinterpret_cast<float*>(_v) = _t->titleOffset(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setTitle(*reinterpret_cast<QString*>(_v)); break;
        case 1: _t->setLabels(*reinterpret_cast<QStringList*>(_v)); break;
        case 2: _t->setLabelsVisible(*reinterpret_cast<bool*>(_v)); break;
        case 5: _t->setMin(*reinterpret_cast<float*>(_v)); break;
        case 6: _t->setMax(*reinterpret_cast<float*>(_v)); break;
        case 7: _t->setAutoAdjustRange(*reinterpret_cast<bool*>(_v)); break;
        case 8: _t->setLabelAutoAngle(*reinterpret_cast<float*>(_v)); break;
        case 9: _t->setScaleLabelsByCount(*reinterpret_cast<bool*>(_v)); break;
        case 10: _t->setLabelSize(*reinterpret_cast<qreal*>(_v)); break;
        case 11: _t->setTitleVisible(*reinterpret_cast<bool*>(_v)); break;
        case 12: _t->setTitleFixed(*reinterpret_cast<bool*>(_v)); break;
        case 13: _t->setTitleOffset(*reinterpret_cast<float*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *QAbstract3DAxis::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QAbstract3DAxis::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15QAbstract3DAxisE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int QAbstract3DAxis::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 14)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 14;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 14)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 14;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 14;
    }
    return _id;
}

// SIGNAL 0
void QAbstract3DAxis::titleChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void QAbstract3DAxis::labelsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void QAbstract3DAxis::orientationChanged(QAbstract3DAxis::AxisOrientation _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void QAbstract3DAxis::minChanged(float _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void QAbstract3DAxis::maxChanged(float _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void QAbstract3DAxis::rangeChanged(float _t1, float _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1, _t2);
}

// SIGNAL 6
void QAbstract3DAxis::autoAdjustRangeChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}

// SIGNAL 7
void QAbstract3DAxis::scaleLabelsByCountChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1);
}

// SIGNAL 8
void QAbstract3DAxis::labelSizeChanged(qreal _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 8, nullptr, _t1);
}

// SIGNAL 9
void QAbstract3DAxis::labelAutoAngleChanged(float _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 9, nullptr, _t1);
}

// SIGNAL 10
void QAbstract3DAxis::titleVisibleChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 10, nullptr, _t1);
}

// SIGNAL 11
void QAbstract3DAxis::labelVisibleChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 11, nullptr, _t1);
}

// SIGNAL 12
void QAbstract3DAxis::titleFixedChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 12, nullptr, _t1);
}

// SIGNAL 13
void QAbstract3DAxis::titleOffsetChanged(float _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 13, nullptr, _t1);
}
QT_WARNING_POP
