/****************************************************************************
** Meta object code from reading C++ file 'axisgrid_p.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../qtgraphs/src/graphs2d/axis/axisgrid_p.h"
#include <QtGui/qtextcursor.h>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'axisgrid_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN8AxisGridE_t {};
} // unnamed namespace

template <> constexpr inline auto AxisGrid::qt_create_metaobjectdata<qt_meta_tag_ZN8AxisGridE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "AxisGrid",
        "iResolutionChanged",
        "",
        "smoothingChanged",
        "origoChanged",
        "gridVisibilityChanged",
        "gridWidthChanged",
        "gridHeightChanged",
        "gridMovementChanged",
        "subGridColorChanged",
        "gridColorChanged",
        "plotAreaBackgroundColorChanged",
        "subGridLineWidthChanged",
        "gridLineWidthChanged",
        "verticalSubGridScaleChanged",
        "horizontalSubGridScaleChanged",
        "verticalLogarithmicChanged",
        "logarithmic",
        "horizontalLogarithmicChanged",
        "verticalBaseChanged",
        "base",
        "horizontalBaseChanged",
        "iResolution",
        "QVector3D",
        "smoothing",
        "origo",
        "gridVisibility",
        "QVector4D",
        "gridWidth",
        "gridHeight",
        "gridMovement",
        "QPointF",
        "subGridColor",
        "QColor",
        "gridColor",
        "plotAreaBackgroundColor",
        "subGridLineWidth",
        "gridLineWidth",
        "verticalSubGridScale",
        "horizontalSubGridScale",
        "verticalLogarithmic",
        "horizontalLogarithmic",
        "verticalBase",
        "horizontalBase"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'iResolutionChanged'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'smoothingChanged'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'origoChanged'
        QtMocHelpers::SignalData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'gridVisibilityChanged'
        QtMocHelpers::SignalData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'gridWidthChanged'
        QtMocHelpers::SignalData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'gridHeightChanged'
        QtMocHelpers::SignalData<void()>(7, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'gridMovementChanged'
        QtMocHelpers::SignalData<void()>(8, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'subGridColorChanged'
        QtMocHelpers::SignalData<void()>(9, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'gridColorChanged'
        QtMocHelpers::SignalData<void()>(10, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'plotAreaBackgroundColorChanged'
        QtMocHelpers::SignalData<void()>(11, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'subGridLineWidthChanged'
        QtMocHelpers::SignalData<void()>(12, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'gridLineWidthChanged'
        QtMocHelpers::SignalData<void()>(13, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'verticalSubGridScaleChanged'
        QtMocHelpers::SignalData<void()>(14, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'horizontalSubGridScaleChanged'
        QtMocHelpers::SignalData<void()>(15, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'verticalLogarithmicChanged'
        QtMocHelpers::RevisionedSignalData<void(bool)>(16, 2, QMC::AccessPublic, 0x60c, QMetaType::Void, {{
            { QMetaType::Bool, 17 },
        }}),
        // Signal 'horizontalLogarithmicChanged'
        QtMocHelpers::RevisionedSignalData<void(bool)>(18, 2, QMC::AccessPublic, 0x60c, QMetaType::Void, {{
            { QMetaType::Bool, 17 },
        }}),
        // Signal 'verticalBaseChanged'
        QtMocHelpers::RevisionedSignalData<void(qreal)>(19, 2, QMC::AccessPublic, 0x60c, QMetaType::Void, {{
            { QMetaType::QReal, 20 },
        }}),
        // Signal 'horizontalBaseChanged'
        QtMocHelpers::RevisionedSignalData<void(qreal)>(21, 2, QMC::AccessPublic, 0x60c, QMetaType::Void, {{
            { QMetaType::QReal, 20 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'iResolution'
        QtMocHelpers::PropertyData<QVector3D>(22, 0x80000000 | 23, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Final, 0),
        // property 'smoothing'
        QtMocHelpers::PropertyData<qreal>(24, QMetaType::QReal, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 1),
        // property 'origo'
        QtMocHelpers::PropertyData<int>(25, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 2),
        // property 'gridVisibility'
        QtMocHelpers::PropertyData<QVector4D>(26, 0x80000000 | 27, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet | QMC::Final, 3),
        // property 'gridWidth'
        QtMocHelpers::PropertyData<qreal>(28, QMetaType::QReal, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 4),
        // property 'gridHeight'
        QtMocHelpers::PropertyData<qreal>(29, QMetaType::QReal, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 5),
        // property 'gridMovement'
        QtMocHelpers::PropertyData<QPointF>(30, 0x80000000 | 31, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet | QMC::Final, 6),
        // property 'subGridColor'
        QtMocHelpers::PropertyData<QColor>(32, 0x80000000 | 33, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet | QMC::Final, 7),
        // property 'gridColor'
        QtMocHelpers::PropertyData<QColor>(34, 0x80000000 | 33, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet | QMC::Final, 8),
        // property 'plotAreaBackgroundColor'
        QtMocHelpers::PropertyData<QColor>(35, 0x80000000 | 33, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet | QMC::Final, 9),
        // property 'subGridLineWidth'
        QtMocHelpers::PropertyData<qreal>(36, QMetaType::QReal, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 10),
        // property 'gridLineWidth'
        QtMocHelpers::PropertyData<qreal>(37, QMetaType::QReal, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 11),
        // property 'verticalSubGridScale'
        QtMocHelpers::PropertyData<qreal>(38, QMetaType::QReal, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 12),
        // property 'horizontalSubGridScale'
        QtMocHelpers::PropertyData<qreal>(39, QMetaType::QReal, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 13),
        // property 'verticalLogarithmic'
        QtMocHelpers::PropertyData<bool>(40, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 14, 0x60c),
        // property 'horizontalLogarithmic'
        QtMocHelpers::PropertyData<bool>(41, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 15, 0x60c),
        // property 'verticalBase'
        QtMocHelpers::PropertyData<qreal>(42, QMetaType::QReal, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 16, 0x60c),
        // property 'horizontalBase'
        QtMocHelpers::PropertyData<qreal>(43, QMetaType::QReal, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 17, 0x60c),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<AxisGrid, qt_meta_tag_ZN8AxisGridE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject AxisGrid::staticMetaObject = { {
    QMetaObject::SuperData::link<QQuickShaderEffect::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8AxisGridE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8AxisGridE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN8AxisGridE_t>.metaTypes,
    nullptr
} };

void AxisGrid::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<AxisGrid *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->iResolutionChanged(); break;
        case 1: _t->smoothingChanged(); break;
        case 2: _t->origoChanged(); break;
        case 3: _t->gridVisibilityChanged(); break;
        case 4: _t->gridWidthChanged(); break;
        case 5: _t->gridHeightChanged(); break;
        case 6: _t->gridMovementChanged(); break;
        case 7: _t->subGridColorChanged(); break;
        case 8: _t->gridColorChanged(); break;
        case 9: _t->plotAreaBackgroundColorChanged(); break;
        case 10: _t->subGridLineWidthChanged(); break;
        case 11: _t->gridLineWidthChanged(); break;
        case 12: _t->verticalSubGridScaleChanged(); break;
        case 13: _t->horizontalSubGridScaleChanged(); break;
        case 14: _t->verticalLogarithmicChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 15: _t->horizontalLogarithmicChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 16: _t->verticalBaseChanged((*reinterpret_cast<std::add_pointer_t<qreal>>(_a[1]))); break;
        case 17: _t->horizontalBaseChanged((*reinterpret_cast<std::add_pointer_t<qreal>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (AxisGrid::*)()>(_a, &AxisGrid::iResolutionChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (AxisGrid::*)()>(_a, &AxisGrid::smoothingChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (AxisGrid::*)()>(_a, &AxisGrid::origoChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (AxisGrid::*)()>(_a, &AxisGrid::gridVisibilityChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (AxisGrid::*)()>(_a, &AxisGrid::gridWidthChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (AxisGrid::*)()>(_a, &AxisGrid::gridHeightChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (AxisGrid::*)()>(_a, &AxisGrid::gridMovementChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (AxisGrid::*)()>(_a, &AxisGrid::subGridColorChanged, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (AxisGrid::*)()>(_a, &AxisGrid::gridColorChanged, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (AxisGrid::*)()>(_a, &AxisGrid::plotAreaBackgroundColorChanged, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (AxisGrid::*)()>(_a, &AxisGrid::subGridLineWidthChanged, 10))
            return;
        if (QtMocHelpers::indexOfMethod<void (AxisGrid::*)()>(_a, &AxisGrid::gridLineWidthChanged, 11))
            return;
        if (QtMocHelpers::indexOfMethod<void (AxisGrid::*)()>(_a, &AxisGrid::verticalSubGridScaleChanged, 12))
            return;
        if (QtMocHelpers::indexOfMethod<void (AxisGrid::*)()>(_a, &AxisGrid::horizontalSubGridScaleChanged, 13))
            return;
        if (QtMocHelpers::indexOfMethod<void (AxisGrid::*)(bool )>(_a, &AxisGrid::verticalLogarithmicChanged, 14))
            return;
        if (QtMocHelpers::indexOfMethod<void (AxisGrid::*)(bool )>(_a, &AxisGrid::horizontalLogarithmicChanged, 15))
            return;
        if (QtMocHelpers::indexOfMethod<void (AxisGrid::*)(qreal )>(_a, &AxisGrid::verticalBaseChanged, 16))
            return;
        if (QtMocHelpers::indexOfMethod<void (AxisGrid::*)(qreal )>(_a, &AxisGrid::horizontalBaseChanged, 17))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QVector3D*>(_v) = _t->iResolution(); break;
        case 1: *reinterpret_cast<qreal*>(_v) = _t->smoothing(); break;
        case 2: *reinterpret_cast<int*>(_v) = _t->origo(); break;
        case 3: *reinterpret_cast<QVector4D*>(_v) = _t->gridVisibility(); break;
        case 4: *reinterpret_cast<qreal*>(_v) = _t->gridWidth(); break;
        case 5: *reinterpret_cast<qreal*>(_v) = _t->gridHeight(); break;
        case 6: *reinterpret_cast<QPointF*>(_v) = _t->gridMovement(); break;
        case 7: *reinterpret_cast<QColor*>(_v) = _t->subGridColor(); break;
        case 8: *reinterpret_cast<QColor*>(_v) = _t->gridColor(); break;
        case 9: *reinterpret_cast<QColor*>(_v) = _t->plotAreaBackgroundColor(); break;
        case 10: *reinterpret_cast<qreal*>(_v) = _t->subGridLineWidth(); break;
        case 11: *reinterpret_cast<qreal*>(_v) = _t->gridLineWidth(); break;
        case 12: *reinterpret_cast<qreal*>(_v) = _t->verticalSubGridScale(); break;
        case 13: *reinterpret_cast<qreal*>(_v) = _t->horizontalSubGridScale(); break;
        case 14: *reinterpret_cast<bool*>(_v) = _t->isVerticalLogarithmic(); break;
        case 15: *reinterpret_cast<bool*>(_v) = _t->isHorizontalLogarithmic(); break;
        case 16: *reinterpret_cast<qreal*>(_v) = _t->verticalBase(); break;
        case 17: *reinterpret_cast<qreal*>(_v) = _t->horizontalBase(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 1: _t->setSmoothing(*reinterpret_cast<qreal*>(_v)); break;
        case 2: _t->setOrigo(*reinterpret_cast<int*>(_v)); break;
        case 3: _t->setGridVisibility(*reinterpret_cast<QVector4D*>(_v)); break;
        case 4: _t->setGridWidth(*reinterpret_cast<qreal*>(_v)); break;
        case 5: _t->setGridHeight(*reinterpret_cast<qreal*>(_v)); break;
        case 6: _t->setGridMovement(*reinterpret_cast<QPointF*>(_v)); break;
        case 7: _t->setSubGridColor(*reinterpret_cast<QColor*>(_v)); break;
        case 8: _t->setGridColor(*reinterpret_cast<QColor*>(_v)); break;
        case 9: _t->setPlotAreaBackgroundColor(*reinterpret_cast<QColor*>(_v)); break;
        case 10: _t->setSubGridLineWidth(*reinterpret_cast<qreal*>(_v)); break;
        case 11: _t->setGridLineWidth(*reinterpret_cast<qreal*>(_v)); break;
        case 12: _t->setVerticalSubGridScale(*reinterpret_cast<qreal*>(_v)); break;
        case 13: _t->setHorizontalSubGridScale(*reinterpret_cast<qreal*>(_v)); break;
        case 14: _t->setVerticalLogarithmic(*reinterpret_cast<bool*>(_v)); break;
        case 15: _t->setHorizontalLogarithmic(*reinterpret_cast<bool*>(_v)); break;
        case 16: _t->setVerticalBase(*reinterpret_cast<qreal*>(_v)); break;
        case 17: _t->setHorizontalBase(*reinterpret_cast<qreal*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *AxisGrid::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *AxisGrid::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8AxisGridE_t>.strings))
        return static_cast<void*>(this);
    return QQuickShaderEffect::qt_metacast(_clname);
}

int AxisGrid::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QQuickShaderEffect::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 18)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 18;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 18)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 18;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 18;
    }
    return _id;
}

// SIGNAL 0
void AxisGrid::iResolutionChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void AxisGrid::smoothingChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void AxisGrid::origoChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void AxisGrid::gridVisibilityChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void AxisGrid::gridWidthChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void AxisGrid::gridHeightChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void AxisGrid::gridMovementChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void AxisGrid::subGridColorChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void AxisGrid::gridColorChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 8, nullptr);
}

// SIGNAL 9
void AxisGrid::plotAreaBackgroundColorChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 9, nullptr);
}

// SIGNAL 10
void AxisGrid::subGridLineWidthChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 10, nullptr);
}

// SIGNAL 11
void AxisGrid::gridLineWidthChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 11, nullptr);
}

// SIGNAL 12
void AxisGrid::verticalSubGridScaleChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 12, nullptr);
}

// SIGNAL 13
void AxisGrid::horizontalSubGridScaleChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 13, nullptr);
}

// SIGNAL 14
void AxisGrid::verticalLogarithmicChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 14, nullptr, _t1);
}

// SIGNAL 15
void AxisGrid::horizontalLogarithmicChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 15, nullptr, _t1);
}

// SIGNAL 16
void AxisGrid::verticalBaseChanged(qreal _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 16, nullptr, _t1);
}

// SIGNAL 17
void AxisGrid::horizontalBaseChanged(qreal _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 17, nullptr, _t1);
}
QT_WARNING_POP
