/****************************************************************************
** Meta object code from reading C++ file 'qquickgraphsnode_p.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../qtgraphs/src/graphs3d/qml/qquickgraphsnode_p.h"
#include <QtGui/qtextcursor.h>
#include <QtNetwork/QSslError>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qquickgraphsnode_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN16QQuickGraphsNodeE_t {};
} // unnamed namespace

template <> constexpr inline auto QQuickGraphsNode::qt_create_metaobjectdata<qt_meta_tag_ZN16QQuickGraphsNodeE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QQuickGraphsNode",
        "QML.Element",
        "GraphsNode",
        "QML.Creatable",
        "false",
        "QML.UncreatableReason",
        "",
        "queriedGraphPositionChanged",
        "selectedElementChanged",
        "optimizationHintChanged",
        "selectionModeChanged",
        "themeChanged",
        "aspectRatioChanged",
        "polarChanged",
        "labelMarginChanged",
        "radialLabelOffsetChanged",
        "horizontalAspectRatioChanged",
        "localeChanged",
        "marginChanged",
        "margin",
        "gridLineTypeChanged",
        "hasSeries",
        "QAbstract3DSeries*",
        "series",
        "clearSelection",
        "addCustomItem",
        "qsizetype",
        "QCustom3DItem*",
        "item",
        "removeCustomItems",
        "removeCustomItem",
        "removeCustomItemAt",
        "QVector3D",
        "position",
        "releaseCustomItem",
        "selectedLabelIndex",
        "selectedAxis",
        "QAbstract3DAxis*",
        "selectedCustomItemIndex",
        "selectedCustomItem",
        "doPicking",
        "QPointF",
        "point",
        "doRayPicking",
        "origin",
        "direction",
        "selectionMode",
        "QtGraphs3D::SelectionFlags",
        "theme",
        "QGraphsTheme*",
        "customItemList",
        "QQmlListProperty<QCustom3DItem>",
        "selectedElement",
        "QtGraphs3D::ElementType",
        "aspectRatio",
        "optimizationHint",
        "QtGraphs3D::OptimizationHint",
        "polar",
        "labelMargin",
        "radialLabelOffset",
        "horizontalAspectRatio",
        "locale",
        "queriedGraphPosition",
        "gridLineType",
        "QtGraphs3D::GridLineType"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'queriedGraphPositionChanged'
        QtMocHelpers::SignalData<void()>(7, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'selectedElementChanged'
        QtMocHelpers::SignalData<void()>(8, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'optimizationHintChanged'
        QtMocHelpers::SignalData<void()>(9, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'selectionModeChanged'
        QtMocHelpers::SignalData<void()>(10, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'themeChanged'
        QtMocHelpers::SignalData<void()>(11, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'aspectRatioChanged'
        QtMocHelpers::SignalData<void()>(12, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'polarChanged'
        QtMocHelpers::SignalData<void()>(13, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'labelMarginChanged'
        QtMocHelpers::SignalData<void()>(14, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'radialLabelOffsetChanged'
        QtMocHelpers::SignalData<void()>(15, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'horizontalAspectRatioChanged'
        QtMocHelpers::SignalData<void()>(16, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'localeChanged'
        QtMocHelpers::SignalData<void()>(17, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'marginChanged'
        QtMocHelpers::SignalData<void(qreal)>(18, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QReal, 19 },
        }}),
        // Signal 'gridLineTypeChanged'
        QtMocHelpers::SignalData<void()>(20, 6, QMC::AccessPublic, QMetaType::Void),
        // Method 'hasSeries'
        QtMocHelpers::MethodData<bool(QAbstract3DSeries *)>(21, 6, QMC::AccessPublic, QMetaType::Bool, {{
            { 0x80000000 | 22, 23 },
        }}),
        // Method 'clearSelection'
        QtMocHelpers::MethodData<void()>(24, 6, QMC::AccessPublic, QMetaType::Void),
        // Method 'addCustomItem'
        QtMocHelpers::MethodData<qsizetype(QCustom3DItem *)>(25, 6, QMC::AccessPublic, 0x80000000 | 26, {{
            { 0x80000000 | 27, 28 },
        }}),
        // Method 'removeCustomItems'
        QtMocHelpers::MethodData<void()>(29, 6, QMC::AccessPublic, QMetaType::Void),
        // Method 'removeCustomItem'
        QtMocHelpers::MethodData<void(QCustom3DItem *)>(30, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 27, 28 },
        }}),
        // Method 'removeCustomItemAt'
        QtMocHelpers::MethodData<void(QVector3D)>(31, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 32, 33 },
        }}),
        // Method 'releaseCustomItem'
        QtMocHelpers::MethodData<void(QCustom3DItem *)>(34, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 27, 28 },
        }}),
        // Method 'selectedLabelIndex'
        QtMocHelpers::MethodData<int() const>(35, 6, QMC::AccessPublic, QMetaType::Int),
        // Method 'selectedAxis'
        QtMocHelpers::MethodData<QAbstract3DAxis *() const>(36, 6, QMC::AccessPublic, 0x80000000 | 37),
        // Method 'selectedCustomItemIndex'
        QtMocHelpers::MethodData<qsizetype() const>(38, 6, QMC::AccessPublic, 0x80000000 | 26),
        // Method 'selectedCustomItem'
        QtMocHelpers::MethodData<QCustom3DItem *() const>(39, 6, QMC::AccessPublic, 0x80000000 | 27),
        // Method 'doPicking'
        QtMocHelpers::MethodData<bool(QPointF)>(40, 6, QMC::AccessPublic, QMetaType::Bool, {{
            { 0x80000000 | 41, 42 },
        }}),
        // Method 'doRayPicking'
        QtMocHelpers::MethodData<bool(const QVector3D &, const QVector3D &)>(43, 6, QMC::AccessPublic, QMetaType::Bool, {{
            { 0x80000000 | 32, 44 }, { 0x80000000 | 32, 45 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'selectionMode'
        QtMocHelpers::PropertyData<QtGraphs3D::SelectionFlags>(46, 0x80000000 | 47, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 3),
        // property 'theme'
        QtMocHelpers::PropertyData<QGraphsTheme*>(48, 0x80000000 | 49, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 4),
        // property 'customItemList'
        QtMocHelpers::PropertyData<QQmlListProperty<QCustom3DItem>>(50, 0x80000000 | 51, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'selectedElement'
        QtMocHelpers::PropertyData<QtGraphs3D::ElementType>(52, 0x80000000 | 53, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 1),
        // property 'aspectRatio'
        QtMocHelpers::PropertyData<qreal>(54, QMetaType::QReal, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 5),
        // property 'optimizationHint'
        QtMocHelpers::PropertyData<QtGraphs3D::OptimizationHint>(55, 0x80000000 | 56, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 2),
        // property 'polar'
        QtMocHelpers::PropertyData<bool>(57, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 6),
        // property 'labelMargin'
        QtMocHelpers::PropertyData<float>(58, QMetaType::Float, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 7),
        // property 'radialLabelOffset'
        QtMocHelpers::PropertyData<float>(59, QMetaType::Float, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 8),
        // property 'horizontalAspectRatio'
        QtMocHelpers::PropertyData<qreal>(60, QMetaType::QReal, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 9),
        // property 'locale'
        QtMocHelpers::PropertyData<QLocale>(61, QMetaType::QLocale, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 10),
        // property 'queriedGraphPosition'
        QtMocHelpers::PropertyData<QVector3D>(62, 0x80000000 | 32, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 0),
        // property 'margin'
        QtMocHelpers::PropertyData<qreal>(19, QMetaType::QReal, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 11),
        // property 'gridLineType'
        QtMocHelpers::PropertyData<QtGraphs3D::GridLineType>(63, 0x80000000 | 64, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet | QMC::Final, 12),
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
            {    5,    6 },
    });
    return QtMocHelpers::metaObjectData<QQuickGraphsNode, void>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT static const QMetaObject::SuperData qt_meta_extradata_ZN16QQuickGraphsNodeE[] = {
    QMetaObject::SuperData::link<QtGraphs3D::staticMetaObject>(),
    nullptr
};

Q_CONSTINIT const QMetaObject QQuickGraphsNode::staticMetaObject = { {
    QMetaObject::SuperData::link<QQuick3DNode::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16QQuickGraphsNodeE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16QQuickGraphsNodeE_t>.data,
    qt_static_metacall,
    qt_meta_extradata_ZN16QQuickGraphsNodeE,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN16QQuickGraphsNodeE_t>.metaTypes,
    nullptr
} };

void QQuickGraphsNode::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QQuickGraphsNode *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->queriedGraphPositionChanged(); break;
        case 1: _t->selectedElementChanged(); break;
        case 2: _t->optimizationHintChanged(); break;
        case 3: _t->selectionModeChanged(); break;
        case 4: _t->themeChanged(); break;
        case 5: _t->aspectRatioChanged(); break;
        case 6: _t->polarChanged(); break;
        case 7: _t->labelMarginChanged(); break;
        case 8: _t->radialLabelOffsetChanged(); break;
        case 9: _t->horizontalAspectRatioChanged(); break;
        case 10: _t->localeChanged(); break;
        case 11: _t->marginChanged((*reinterpret_cast<std::add_pointer_t<qreal>>(_a[1]))); break;
        case 12: _t->gridLineTypeChanged(); break;
        case 13: { bool _r = _t->hasSeries((*reinterpret_cast<std::add_pointer_t<QAbstract3DSeries*>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 14: _t->clearSelection(); break;
        case 15: { qsizetype _r = _t->addCustomItem((*reinterpret_cast<std::add_pointer_t<QCustom3DItem*>>(_a[1])));
            if (_a[0]) *reinterpret_cast<qsizetype*>(_a[0]) = std::move(_r); }  break;
        case 16: _t->removeCustomItems(); break;
        case 17: _t->removeCustomItem((*reinterpret_cast<std::add_pointer_t<QCustom3DItem*>>(_a[1]))); break;
        case 18: _t->removeCustomItemAt((*reinterpret_cast<std::add_pointer_t<QVector3D>>(_a[1]))); break;
        case 19: _t->releaseCustomItem((*reinterpret_cast<std::add_pointer_t<QCustom3DItem*>>(_a[1]))); break;
        case 20: { int _r = _t->selectedLabelIndex();
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 21: { QAbstract3DAxis* _r = _t->selectedAxis();
            if (_a[0]) *reinterpret_cast<QAbstract3DAxis**>(_a[0]) = std::move(_r); }  break;
        case 22: { qsizetype _r = _t->selectedCustomItemIndex();
            if (_a[0]) *reinterpret_cast<qsizetype*>(_a[0]) = std::move(_r); }  break;
        case 23: { QCustom3DItem* _r = _t->selectedCustomItem();
            if (_a[0]) *reinterpret_cast<QCustom3DItem**>(_a[0]) = std::move(_r); }  break;
        case 24: { bool _r = _t->doPicking((*reinterpret_cast<std::add_pointer_t<QPointF>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 25: { bool _r = _t->doRayPicking((*reinterpret_cast<std::add_pointer_t<QVector3D>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QVector3D>>(_a[2])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 13:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QAbstract3DSeries* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (QQuickGraphsNode::*)()>(_a, &QQuickGraphsNode::queriedGraphPositionChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (QQuickGraphsNode::*)()>(_a, &QQuickGraphsNode::selectedElementChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (QQuickGraphsNode::*)()>(_a, &QQuickGraphsNode::optimizationHintChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (QQuickGraphsNode::*)()>(_a, &QQuickGraphsNode::selectionModeChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (QQuickGraphsNode::*)()>(_a, &QQuickGraphsNode::themeChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (QQuickGraphsNode::*)()>(_a, &QQuickGraphsNode::aspectRatioChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (QQuickGraphsNode::*)()>(_a, &QQuickGraphsNode::polarChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (QQuickGraphsNode::*)()>(_a, &QQuickGraphsNode::labelMarginChanged, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (QQuickGraphsNode::*)()>(_a, &QQuickGraphsNode::radialLabelOffsetChanged, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (QQuickGraphsNode::*)()>(_a, &QQuickGraphsNode::horizontalAspectRatioChanged, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (QQuickGraphsNode::*)()>(_a, &QQuickGraphsNode::localeChanged, 10))
            return;
        if (QtMocHelpers::indexOfMethod<void (QQuickGraphsNode::*)(qreal )>(_a, &QQuickGraphsNode::marginChanged, 11))
            return;
        if (QtMocHelpers::indexOfMethod<void (QQuickGraphsNode::*)()>(_a, &QQuickGraphsNode::gridLineTypeChanged, 12))
            return;
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 1:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QGraphsTheme* >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QtGraphs3D::SelectionFlags*>(_v) = _t->selectionMode(); break;
        case 1: *reinterpret_cast<QGraphsTheme**>(_v) = _t->theme(); break;
        case 2: *reinterpret_cast<QQmlListProperty<QCustom3DItem>*>(_v) = _t->customItemList(); break;
        case 3: *reinterpret_cast<QtGraphs3D::ElementType*>(_v) = _t->selectedElement(); break;
        case 4: *reinterpret_cast<qreal*>(_v) = _t->aspectRatio(); break;
        case 5: *reinterpret_cast<QtGraphs3D::OptimizationHint*>(_v) = _t->optimizationHint(); break;
        case 6: *reinterpret_cast<bool*>(_v) = _t->isPolar(); break;
        case 7: *reinterpret_cast<float*>(_v) = _t->labelMargin(); break;
        case 8: *reinterpret_cast<float*>(_v) = _t->radialLabelOffset(); break;
        case 9: *reinterpret_cast<qreal*>(_v) = _t->horizontalAspectRatio(); break;
        case 10: *reinterpret_cast<QLocale*>(_v) = _t->locale(); break;
        case 11: *reinterpret_cast<QVector3D*>(_v) = _t->queriedGraphPosition(); break;
        case 12: *reinterpret_cast<qreal*>(_v) = _t->margin(); break;
        case 13: *reinterpret_cast<QtGraphs3D::GridLineType*>(_v) = _t->gridLineType(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setSelectionMode(*reinterpret_cast<QtGraphs3D::SelectionFlags*>(_v)); break;
        case 1: _t->setTheme(*reinterpret_cast<QGraphsTheme**>(_v)); break;
        case 4: _t->setAspectRatio(*reinterpret_cast<qreal*>(_v)); break;
        case 5: _t->setOptimizationHint(*reinterpret_cast<QtGraphs3D::OptimizationHint*>(_v)); break;
        case 6: _t->setPolar(*reinterpret_cast<bool*>(_v)); break;
        case 7: _t->setLabelMargin(*reinterpret_cast<float*>(_v)); break;
        case 8: _t->setRadialLabelOffset(*reinterpret_cast<float*>(_v)); break;
        case 9: _t->setHorizontalAspectRatio(*reinterpret_cast<qreal*>(_v)); break;
        case 10: _t->setLocale(*reinterpret_cast<QLocale*>(_v)); break;
        case 12: _t->setMargin(*reinterpret_cast<qreal*>(_v)); break;
        case 13: _t->setGridLineType(*reinterpret_cast<QtGraphs3D::GridLineType*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *QQuickGraphsNode::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QQuickGraphsNode::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16QQuickGraphsNodeE_t>.strings))
        return static_cast<void*>(this);
    return QQuick3DNode::qt_metacast(_clname);
}

int QQuickGraphsNode::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QQuick3DNode::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 26)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 26;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 26)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 26;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 14;
    }
    return _id;
}

// SIGNAL 0
void QQuickGraphsNode::queriedGraphPositionChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void QQuickGraphsNode::selectedElementChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void QQuickGraphsNode::optimizationHintChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void QQuickGraphsNode::selectionModeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void QQuickGraphsNode::themeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void QQuickGraphsNode::aspectRatioChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void QQuickGraphsNode::polarChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void QQuickGraphsNode::labelMarginChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void QQuickGraphsNode::radialLabelOffsetChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 8, nullptr);
}

// SIGNAL 9
void QQuickGraphsNode::horizontalAspectRatioChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 9, nullptr);
}

// SIGNAL 10
void QQuickGraphsNode::localeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 10, nullptr);
}

// SIGNAL 11
void QQuickGraphsNode::marginChanged(qreal _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 11, nullptr, _t1);
}

// SIGNAL 12
void QQuickGraphsNode::gridLineTypeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 12, nullptr);
}
QT_WARNING_POP
