/****************************************************************************
** Meta object code from reading C++ file 'qscatter3dseries.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../qtgraphs/src/graphs3d/data/qscatter3dseries.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qscatter3dseries.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN16QScatter3DSeriesE_t {};
} // unnamed namespace

template <> constexpr inline auto QScatter3DSeries::qt_create_metaobjectdata<qt_meta_tag_ZN16QScatter3DSeriesE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QScatter3DSeries",
        "QML.Element",
        "auto",
        "QML.Creatable",
        "false",
        "QML.UncreatableReason",
        "Trying to create uncreatable: QScatter3DSeries, use Scatter3DSerie"
        "s instead.",
        "dataProxyChanged",
        "",
        "QScatterDataProxy*",
        "proxy",
        "selectedItemChanged",
        "qsizetype",
        "index",
        "itemSizeChanged",
        "size",
        "dataArrayChanged",
        "QScatterDataArray",
        "array",
        "scaleArrayChanged",
        "QList<QVector3D>",
        "scaleArray",
        "axisXChanged",
        "QValue3DAxis*",
        "axis",
        "axisYChanged",
        "axisZChanged",
        "dataProxy",
        "selectedItem",
        "itemSize",
        "dataArray",
        "axisX",
        "axisY",
        "axisZ"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'dataProxyChanged'
        QtMocHelpers::SignalData<void(QScatterDataProxy *)>(7, 8, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 10 },
        }}),
        // Signal 'selectedItemChanged'
        QtMocHelpers::SignalData<void(qsizetype)>(11, 8, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 12, 13 },
        }}),
        // Signal 'itemSizeChanged'
        QtMocHelpers::SignalData<void(float)>(14, 8, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Float, 15 },
        }}),
        // Signal 'dataArrayChanged'
        QtMocHelpers::SignalData<void(const QScatterDataArray &)>(16, 8, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 17, 18 },
        }}),
        // Signal 'scaleArrayChanged'
        QtMocHelpers::RevisionedSignalData<void(const QList<QVector3D> &)>(19, 8, QMC::AccessPublic, 0x60a, QMetaType::Void, {{
            { 0x80000000 | 20, 21 },
        }}),
        // Signal 'axisXChanged'
        QtMocHelpers::RevisionedSignalData<void(QValue3DAxis *)>(22, 8, QMC::AccessPublic, 0x60b, QMetaType::Void, {{
            { 0x80000000 | 23, 24 },
        }}),
        // Signal 'axisYChanged'
        QtMocHelpers::RevisionedSignalData<void(QValue3DAxis *)>(25, 8, QMC::AccessPublic, 0x60b, QMetaType::Void, {{
            { 0x80000000 | 23, 24 },
        }}),
        // Signal 'axisZChanged'
        QtMocHelpers::RevisionedSignalData<void(QValue3DAxis *)>(26, 8, QMC::AccessPublic, 0x60b, QMetaType::Void, {{
            { 0x80000000 | 23, 24 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'dataProxy'
        QtMocHelpers::PropertyData<QScatterDataProxy*>(27, 0x80000000 | 9, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet | QMC::Final, 0),
        // property 'selectedItem'
        QtMocHelpers::PropertyData<qsizetype>(28, 0x80000000 | 12, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet | QMC::Final, 1),
        // property 'itemSize'
        QtMocHelpers::PropertyData<float>(29, QMetaType::Float, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 2),
        // property 'dataArray'
        QtMocHelpers::PropertyData<QScatterDataArray>(30, 0x80000000 | 17, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet | QMC::Final, 3),
        // property 'scaleArray'
        QtMocHelpers::PropertyData<QList<QVector3D>>(21, 0x80000000 | 20, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 4, 0x60a),
        // property 'axisX'
        QtMocHelpers::PropertyData<QValue3DAxis*>(31, 0x80000000 | 23, QMC::DefaultPropertyFlags | QMC::Writable | QMC::Resettable | QMC::EnumOrFlag | QMC::StdCppSet, 5, 0x60b),
        // property 'axisY'
        QtMocHelpers::PropertyData<QValue3DAxis*>(32, 0x80000000 | 23, QMC::DefaultPropertyFlags | QMC::Writable | QMC::Resettable | QMC::EnumOrFlag | QMC::StdCppSet, 6, 0x60b),
        // property 'axisZ'
        QtMocHelpers::PropertyData<QValue3DAxis*>(33, 0x80000000 | 23, QMC::DefaultPropertyFlags | QMC::Writable | QMC::Resettable | QMC::EnumOrFlag | QMC::StdCppSet, 7, 0x60b),
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
            {    5,    6 },
    });
    return QtMocHelpers::metaObjectData<QScatter3DSeries, void>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject QScatter3DSeries::staticMetaObject = { {
    QMetaObject::SuperData::link<QAbstract3DSeries::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16QScatter3DSeriesE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16QScatter3DSeriesE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN16QScatter3DSeriesE_t>.metaTypes,
    nullptr
} };

void QScatter3DSeries::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QScatter3DSeries *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->dataProxyChanged((*reinterpret_cast<std::add_pointer_t<QScatterDataProxy*>>(_a[1]))); break;
        case 1: _t->selectedItemChanged((*reinterpret_cast<std::add_pointer_t<qsizetype>>(_a[1]))); break;
        case 2: _t->itemSizeChanged((*reinterpret_cast<std::add_pointer_t<float>>(_a[1]))); break;
        case 3: _t->dataArrayChanged((*reinterpret_cast<std::add_pointer_t<QScatterDataArray>>(_a[1]))); break;
        case 4: _t->scaleArrayChanged((*reinterpret_cast<std::add_pointer_t<QList<QVector3D>>>(_a[1]))); break;
        case 5: _t->axisXChanged((*reinterpret_cast<std::add_pointer_t<QValue3DAxis*>>(_a[1]))); break;
        case 6: _t->axisYChanged((*reinterpret_cast<std::add_pointer_t<QValue3DAxis*>>(_a[1]))); break;
        case 7: _t->axisZChanged((*reinterpret_cast<std::add_pointer_t<QValue3DAxis*>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QScatterDataProxy* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (QScatter3DSeries::*)(QScatterDataProxy * )>(_a, &QScatter3DSeries::dataProxyChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (QScatter3DSeries::*)(qsizetype )>(_a, &QScatter3DSeries::selectedItemChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (QScatter3DSeries::*)(float )>(_a, &QScatter3DSeries::itemSizeChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (QScatter3DSeries::*)(const QScatterDataArray & )>(_a, &QScatter3DSeries::dataArrayChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (QScatter3DSeries::*)(const QList<QVector3D> & )>(_a, &QScatter3DSeries::scaleArrayChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (QScatter3DSeries::*)(QValue3DAxis * )>(_a, &QScatter3DSeries::axisXChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (QScatter3DSeries::*)(QValue3DAxis * )>(_a, &QScatter3DSeries::axisYChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (QScatter3DSeries::*)(QValue3DAxis * )>(_a, &QScatter3DSeries::axisZChanged, 7))
            return;
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 0:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QScatterDataProxy* >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QScatterDataProxy**>(_v) = _t->dataProxy(); break;
        case 1: *reinterpret_cast<qsizetype*>(_v) = _t->selectedItem(); break;
        case 2: *reinterpret_cast<float*>(_v) = _t->itemSize(); break;
        case 3: *reinterpret_cast<QScatterDataArray*>(_v) = _t->dataArray(); break;
        case 4: *reinterpret_cast<QList<QVector3D>*>(_v) = _t->scaleArray(); break;
        case 5: *reinterpret_cast<QValue3DAxis**>(_v) = _t->axisX(); break;
        case 6: *reinterpret_cast<QValue3DAxis**>(_v) = _t->axisY(); break;
        case 7: *reinterpret_cast<QValue3DAxis**>(_v) = _t->axisZ(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setDataProxy(*reinterpret_cast<QScatterDataProxy**>(_v)); break;
        case 1: _t->setSelectedItem(*reinterpret_cast<qsizetype*>(_v)); break;
        case 2: _t->setItemSize(*reinterpret_cast<float*>(_v)); break;
        case 3: _t->setDataArray(*reinterpret_cast<QScatterDataArray*>(_v)); break;
        case 4: _t->setScaleArray(*reinterpret_cast<QList<QVector3D>*>(_v)); break;
        case 5: _t->setAxisX(*reinterpret_cast<QValue3DAxis**>(_v)); break;
        case 6: _t->setAxisY(*reinterpret_cast<QValue3DAxis**>(_v)); break;
        case 7: _t->setAxisZ(*reinterpret_cast<QValue3DAxis**>(_v)); break;
        default: break;
        }
    }
    if (_c == QMetaObject::ResetProperty) {
        switch (_id) {
        case 5: _t->resetAxisX(); break;
        case 6: _t->resetAxisY(); break;
        case 7: _t->resetAxisZ(); break;
        default: break;
        }
    }
}

const QMetaObject *QScatter3DSeries::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QScatter3DSeries::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16QScatter3DSeriesE_t>.strings))
        return static_cast<void*>(this);
    return QAbstract3DSeries::qt_metacast(_clname);
}

int QScatter3DSeries::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QAbstract3DSeries::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 8)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 8)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    }
    return _id;
}

// SIGNAL 0
void QScatter3DSeries::dataProxyChanged(QScatterDataProxy * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void QScatter3DSeries::selectedItemChanged(qsizetype _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void QScatter3DSeries::itemSizeChanged(float _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void QScatter3DSeries::dataArrayChanged(const QScatterDataArray & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void QScatter3DSeries::scaleArrayChanged(const QList<QVector3D> & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void QScatter3DSeries::axisXChanged(QValue3DAxis * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void QScatter3DSeries::axisYChanged(QValue3DAxis * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}

// SIGNAL 7
void QScatter3DSeries::axisZChanged(QValue3DAxis * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1);
}
QT_WARNING_POP
