/****************************************************************************
** Meta object code from reading C++ file 'qbarset.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../qtgraphs/src/graphs2d/barchart/qbarset.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qbarset.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN7QBarSetE_t {};
} // unnamed namespace

template <> constexpr inline auto QBarSet::qt_create_metaobjectdata<qt_meta_tag_ZN7QBarSetE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QBarSet",
        "QML.Element",
        "BarSet",
        "update",
        "",
        "labelChanged",
        "colorChanged",
        "QColor",
        "color",
        "borderColorChanged",
        "labelColorChanged",
        "valuesChanged",
        "selectedColorChanged",
        "countChanged",
        "borderWidthChanged",
        "width",
        "valuesAdded",
        "qsizetype",
        "index",
        "count",
        "valuesRemoved",
        "valueChanged",
        "updatedBars",
        "valueAdded",
        "valueRemoved",
        "selectedBarsChanged",
        "QList<qsizetype>",
        "indexes",
        "append",
        "value",
        "QList<qreal>",
        "values",
        "insert",
        "remove",
        "replace",
        "at",
        "sum",
        "clear",
        "isBarSelected",
        "selectBar",
        "deselectBar",
        "setBarSelected",
        "selected",
        "selectAllBars",
        "deselectAllBars",
        "selectBars",
        "deselectBars",
        "toggleSelection",
        "label",
        "selectedColor",
        "borderColor",
        "labelColor",
        "QVariantList",
        "borderWidth",
        "selectedBars"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'update'
        QtMocHelpers::SignalData<void()>(3, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'labelChanged'
        QtMocHelpers::SignalData<void()>(5, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'colorChanged'
        QtMocHelpers::SignalData<void(QColor)>(6, 4, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 8 },
        }}),
        // Signal 'borderColorChanged'
        QtMocHelpers::SignalData<void(QColor)>(9, 4, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 8 },
        }}),
        // Signal 'labelColorChanged'
        QtMocHelpers::SignalData<void(QColor)>(10, 4, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 8 },
        }}),
        // Signal 'valuesChanged'
        QtMocHelpers::SignalData<void()>(11, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'selectedColorChanged'
        QtMocHelpers::SignalData<void(QColor)>(12, 4, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 8 },
        }}),
        // Signal 'countChanged'
        QtMocHelpers::SignalData<void()>(13, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'borderWidthChanged'
        QtMocHelpers::SignalData<void(qreal)>(14, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QReal, 15 },
        }}),
        // Signal 'valuesAdded'
        QtMocHelpers::SignalData<void(qsizetype, qsizetype)>(16, 4, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 17, 18 }, { 0x80000000 | 17, 19 },
        }}),
        // Signal 'valuesRemoved'
        QtMocHelpers::SignalData<void(qsizetype, qsizetype)>(20, 4, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 17, 18 }, { 0x80000000 | 17, 19 },
        }}),
        // Signal 'valueChanged'
        QtMocHelpers::SignalData<void(qsizetype)>(21, 4, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 17, 18 },
        }}),
        // Signal 'updatedBars'
        QtMocHelpers::SignalData<void()>(22, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'valueAdded'
        QtMocHelpers::SignalData<void(qsizetype, qsizetype)>(23, 4, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 17, 18 }, { 0x80000000 | 17, 19 },
        }}),
        // Signal 'valueRemoved'
        QtMocHelpers::SignalData<void(qsizetype, qsizetype)>(24, 4, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 17, 18 }, { 0x80000000 | 17, 19 },
        }}),
        // Signal 'selectedBarsChanged'
        QtMocHelpers::SignalData<void(const QList<qsizetype> &)>(25, 4, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 26, 27 },
        }}),
        // Method 'append'
        QtMocHelpers::MethodData<void(qreal)>(28, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QReal, 29 },
        }}),
        // Method 'append'
        QtMocHelpers::MethodData<void(const QList<qreal> &)>(28, 4, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 30, 31 },
        }}),
        // Method 'insert'
        QtMocHelpers::MethodData<void(qsizetype, qreal)>(32, 4, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 17, 18 }, { QMetaType::QReal, 29 },
        }}),
        // Method 'remove'
        QtMocHelpers::MethodData<void(qsizetype, qsizetype)>(33, 4, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 17, 18 }, { 0x80000000 | 17, 19 },
        }}),
        // Method 'remove'
        QtMocHelpers::MethodData<void(qsizetype)>(33, 4, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { 0x80000000 | 17, 18 },
        }}),
        // Method 'replace'
        QtMocHelpers::MethodData<void(qsizetype, qreal)>(34, 4, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 17, 18 }, { QMetaType::QReal, 29 },
        }}),
        // Method 'at'
        QtMocHelpers::MethodData<qreal(qsizetype) const>(35, 4, QMC::AccessPublic, QMetaType::QReal, {{
            { 0x80000000 | 17, 18 },
        }}),
        // Method 'count'
        QtMocHelpers::MethodData<qsizetype() const>(19, 4, QMC::AccessPublic, 0x80000000 | 17),
        // Method 'sum'
        QtMocHelpers::MethodData<qreal() const>(36, 4, QMC::AccessPublic, QMetaType::QReal),
        // Method 'clear'
        QtMocHelpers::MethodData<void()>(37, 4, QMC::AccessPublic, QMetaType::Void),
        // Method 'isBarSelected'
        QtMocHelpers::MethodData<bool(qsizetype) const>(38, 4, QMC::AccessPublic, QMetaType::Bool, {{
            { 0x80000000 | 17, 18 },
        }}),
        // Method 'selectBar'
        QtMocHelpers::MethodData<void(qsizetype)>(39, 4, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 17, 18 },
        }}),
        // Method 'deselectBar'
        QtMocHelpers::MethodData<void(qsizetype)>(40, 4, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 17, 18 },
        }}),
        // Method 'setBarSelected'
        QtMocHelpers::MethodData<void(qsizetype, bool)>(41, 4, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 17, 18 }, { QMetaType::Bool, 42 },
        }}),
        // Method 'selectAllBars'
        QtMocHelpers::MethodData<void()>(43, 4, QMC::AccessPublic, QMetaType::Void),
        // Method 'deselectAllBars'
        QtMocHelpers::MethodData<void()>(44, 4, QMC::AccessPublic, QMetaType::Void),
        // Method 'selectBars'
        QtMocHelpers::MethodData<void(const QList<qsizetype> &)>(45, 4, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 26, 27 },
        }}),
        // Method 'deselectBars'
        QtMocHelpers::MethodData<void(const QList<qsizetype> &)>(46, 4, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 26, 27 },
        }}),
        // Method 'toggleSelection'
        QtMocHelpers::MethodData<void(const QList<qsizetype> &)>(47, 4, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 26, 27 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'label'
        QtMocHelpers::PropertyData<QString>(48, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 1),
        // property 'color'
        QtMocHelpers::PropertyData<QColor>(8, 0x80000000 | 7, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet | QMC::Final, 2),
        // property 'selectedColor'
        QtMocHelpers::PropertyData<QColor>(49, 0x80000000 | 7, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet | QMC::Final, 6),
        // property 'borderColor'
        QtMocHelpers::PropertyData<QColor>(50, 0x80000000 | 7, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet | QMC::Final, 3),
        // property 'labelColor'
        QtMocHelpers::PropertyData<QColor>(51, 0x80000000 | 7, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet | QMC::Final, 4),
        // property 'values'
        QtMocHelpers::PropertyData<QVariantList>(31, 0x80000000 | 52, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet | QMC::Final, 5),
        // property 'borderWidth'
        QtMocHelpers::PropertyData<qreal>(53, QMetaType::QReal, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 8),
        // property 'count'
        QtMocHelpers::PropertyData<qsizetype>(19, 0x80000000 | 17, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Final, 7),
        // property 'selectedBars'
        QtMocHelpers::PropertyData<QList<qsizetype>>(54, 0x80000000 | 26, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Final, 15),
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
    });
    return QtMocHelpers::metaObjectData<QBarSet, void>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject QBarSet::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN7QBarSetE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN7QBarSetE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN7QBarSetE_t>.metaTypes,
    nullptr
} };

void QBarSet::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QBarSet *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->update(); break;
        case 1: _t->labelChanged(); break;
        case 2: _t->colorChanged((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        case 3: _t->borderColorChanged((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        case 4: _t->labelColorChanged((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        case 5: _t->valuesChanged(); break;
        case 6: _t->selectedColorChanged((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        case 7: _t->countChanged(); break;
        case 8: _t->borderWidthChanged((*reinterpret_cast<std::add_pointer_t<qreal>>(_a[1]))); break;
        case 9: _t->valuesAdded((*reinterpret_cast<std::add_pointer_t<qsizetype>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<qsizetype>>(_a[2]))); break;
        case 10: _t->valuesRemoved((*reinterpret_cast<std::add_pointer_t<qsizetype>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<qsizetype>>(_a[2]))); break;
        case 11: _t->valueChanged((*reinterpret_cast<std::add_pointer_t<qsizetype>>(_a[1]))); break;
        case 12: _t->updatedBars(); break;
        case 13: _t->valueAdded((*reinterpret_cast<std::add_pointer_t<qsizetype>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<qsizetype>>(_a[2]))); break;
        case 14: _t->valueRemoved((*reinterpret_cast<std::add_pointer_t<qsizetype>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<qsizetype>>(_a[2]))); break;
        case 15: _t->selectedBarsChanged((*reinterpret_cast<std::add_pointer_t<QList<qsizetype>>>(_a[1]))); break;
        case 16: _t->append((*reinterpret_cast<std::add_pointer_t<qreal>>(_a[1]))); break;
        case 17: _t->append((*reinterpret_cast<std::add_pointer_t<QList<qreal>>>(_a[1]))); break;
        case 18: _t->insert((*reinterpret_cast<std::add_pointer_t<qsizetype>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<qreal>>(_a[2]))); break;
        case 19: _t->remove((*reinterpret_cast<std::add_pointer_t<qsizetype>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<qsizetype>>(_a[2]))); break;
        case 20: _t->remove((*reinterpret_cast<std::add_pointer_t<qsizetype>>(_a[1]))); break;
        case 21: _t->replace((*reinterpret_cast<std::add_pointer_t<qsizetype>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<qreal>>(_a[2]))); break;
        case 22: { qreal _r = _t->at((*reinterpret_cast<std::add_pointer_t<qsizetype>>(_a[1])));
            if (_a[0]) *reinterpret_cast<qreal*>(_a[0]) = std::move(_r); }  break;
        case 23: { qsizetype _r = _t->count();
            if (_a[0]) *reinterpret_cast<qsizetype*>(_a[0]) = std::move(_r); }  break;
        case 24: { qreal _r = _t->sum();
            if (_a[0]) *reinterpret_cast<qreal*>(_a[0]) = std::move(_r); }  break;
        case 25: _t->clear(); break;
        case 26: { bool _r = _t->isBarSelected((*reinterpret_cast<std::add_pointer_t<qsizetype>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 27: _t->selectBar((*reinterpret_cast<std::add_pointer_t<qsizetype>>(_a[1]))); break;
        case 28: _t->deselectBar((*reinterpret_cast<std::add_pointer_t<qsizetype>>(_a[1]))); break;
        case 29: _t->setBarSelected((*reinterpret_cast<std::add_pointer_t<qsizetype>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 30: _t->selectAllBars(); break;
        case 31: _t->deselectAllBars(); break;
        case 32: _t->selectBars((*reinterpret_cast<std::add_pointer_t<QList<qsizetype>>>(_a[1]))); break;
        case 33: _t->deselectBars((*reinterpret_cast<std::add_pointer_t<QList<qsizetype>>>(_a[1]))); break;
        case 34: _t->toggleSelection((*reinterpret_cast<std::add_pointer_t<QList<qsizetype>>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 17:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QList<qreal> >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (QBarSet::*)()>(_a, &QBarSet::update, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (QBarSet::*)()>(_a, &QBarSet::labelChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (QBarSet::*)(QColor )>(_a, &QBarSet::colorChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (QBarSet::*)(QColor )>(_a, &QBarSet::borderColorChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (QBarSet::*)(QColor )>(_a, &QBarSet::labelColorChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (QBarSet::*)()>(_a, &QBarSet::valuesChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (QBarSet::*)(QColor )>(_a, &QBarSet::selectedColorChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (QBarSet::*)()>(_a, &QBarSet::countChanged, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (QBarSet::*)(qreal )>(_a, &QBarSet::borderWidthChanged, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (QBarSet::*)(qsizetype , qsizetype )>(_a, &QBarSet::valuesAdded, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (QBarSet::*)(qsizetype , qsizetype )>(_a, &QBarSet::valuesRemoved, 10))
            return;
        if (QtMocHelpers::indexOfMethod<void (QBarSet::*)(qsizetype )>(_a, &QBarSet::valueChanged, 11))
            return;
        if (QtMocHelpers::indexOfMethod<void (QBarSet::*)()>(_a, &QBarSet::updatedBars, 12))
            return;
        if (QtMocHelpers::indexOfMethod<void (QBarSet::*)(qsizetype , qsizetype )>(_a, &QBarSet::valueAdded, 13))
            return;
        if (QtMocHelpers::indexOfMethod<void (QBarSet::*)(qsizetype , qsizetype )>(_a, &QBarSet::valueRemoved, 14))
            return;
        if (QtMocHelpers::indexOfMethod<void (QBarSet::*)(const QList<qsizetype> & )>(_a, &QBarSet::selectedBarsChanged, 15))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QString*>(_v) = _t->label(); break;
        case 1: *reinterpret_cast<QColor*>(_v) = _t->color(); break;
        case 2: *reinterpret_cast<QColor*>(_v) = _t->selectedColor(); break;
        case 3: *reinterpret_cast<QColor*>(_v) = _t->borderColor(); break;
        case 4: *reinterpret_cast<QColor*>(_v) = _t->labelColor(); break;
        case 5: *reinterpret_cast<QVariantList*>(_v) = _t->values(); break;
        case 6: *reinterpret_cast<qreal*>(_v) = _t->borderWidth(); break;
        case 7: *reinterpret_cast<qsizetype*>(_v) = _t->count(); break;
        case 8: *reinterpret_cast<QList<qsizetype>*>(_v) = _t->selectedBars(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setLabel(*reinterpret_cast<QString*>(_v)); break;
        case 1: _t->setColor(*reinterpret_cast<QColor*>(_v)); break;
        case 2: _t->setSelectedColor(*reinterpret_cast<QColor*>(_v)); break;
        case 3: _t->setBorderColor(*reinterpret_cast<QColor*>(_v)); break;
        case 4: _t->setLabelColor(*reinterpret_cast<QColor*>(_v)); break;
        case 5: _t->setValues(*reinterpret_cast<QVariantList*>(_v)); break;
        case 6: _t->setBorderWidth(*reinterpret_cast<qreal*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *QBarSet::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QBarSet::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN7QBarSetE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int QBarSet::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 35)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 35;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 35)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 35;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    }
    return _id;
}

// SIGNAL 0
void QBarSet::update()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void QBarSet::labelChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void QBarSet::colorChanged(QColor _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void QBarSet::borderColorChanged(QColor _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void QBarSet::labelColorChanged(QColor _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void QBarSet::valuesChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void QBarSet::selectedColorChanged(QColor _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}

// SIGNAL 7
void QBarSet::countChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void QBarSet::borderWidthChanged(qreal _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 8, nullptr, _t1);
}

// SIGNAL 9
void QBarSet::valuesAdded(qsizetype _t1, qsizetype _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 9, nullptr, _t1, _t2);
}

// SIGNAL 10
void QBarSet::valuesRemoved(qsizetype _t1, qsizetype _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 10, nullptr, _t1, _t2);
}

// SIGNAL 11
void QBarSet::valueChanged(qsizetype _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 11, nullptr, _t1);
}

// SIGNAL 12
void QBarSet::updatedBars()
{
    QMetaObject::activate(this, &staticMetaObject, 12, nullptr);
}

// SIGNAL 13
void QBarSet::valueAdded(qsizetype _t1, qsizetype _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 13, nullptr, _t1, _t2);
}

// SIGNAL 14
void QBarSet::valueRemoved(qsizetype _t1, qsizetype _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 14, nullptr, _t1, _t2);
}

// SIGNAL 15
void QBarSet::selectedBarsChanged(const QList<qsizetype> & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 15, nullptr, _t1);
}
QT_WARNING_POP
