/****************************************************************************
** Meta object code from reading C++ file 'qcustom3ditem.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../qtgraphs/src/graphs3d/data/qcustom3ditem.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qcustom3ditem.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN13QCustom3DItemE_t {};
} // unnamed namespace

template <> constexpr inline auto QCustom3DItem::qt_create_metaobjectdata<qt_meta_tag_ZN13QCustom3DItemE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QCustom3DItem",
        "QML.Element",
        "Custom3DItem",
        "meshFileChanged",
        "",
        "meshFile",
        "textureFileChanged",
        "textureFile",
        "positionChanged",
        "QVector3D",
        "position",
        "positionAbsoluteChanged",
        "positionAbsolute",
        "scalingChanged",
        "scaling",
        "rotationChanged",
        "QQuaternion",
        "rotation",
        "rotationAbsoluteChanged",
        "newRotationAbsolute",
        "visibleChanged",
        "visible",
        "shadowCastingChanged",
        "shadowCasting",
        "scalingAbsoluteChanged",
        "scalingAbsolute",
        "needUpdate",
        "setRotationAxisAndAngle",
        "axis",
        "angle",
        "rotationAbsolute"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'meshFileChanged'
        QtMocHelpers::SignalData<void(const QString &)>(3, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 5 },
        }}),
        // Signal 'textureFileChanged'
        QtMocHelpers::SignalData<void(const QString &)>(6, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 7 },
        }}),
        // Signal 'positionChanged'
        QtMocHelpers::SignalData<void(QVector3D)>(8, 4, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 10 },
        }}),
        // Signal 'positionAbsoluteChanged'
        QtMocHelpers::SignalData<void(bool)>(11, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 12 },
        }}),
        // Signal 'scalingChanged'
        QtMocHelpers::SignalData<void(QVector3D)>(13, 4, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 14 },
        }}),
        // Signal 'rotationChanged'
        QtMocHelpers::SignalData<void(const QQuaternion &)>(15, 4, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 16, 17 },
        }}),
        // Signal 'rotationAbsoluteChanged'
        QtMocHelpers::RevisionedSignalData<void(bool)>(18, 4, QMC::AccessPublic, 0x60b, QMetaType::Void, {{
            { QMetaType::Bool, 19 },
        }}),
        // Signal 'visibleChanged'
        QtMocHelpers::SignalData<void(bool)>(20, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 21 },
        }}),
        // Signal 'shadowCastingChanged'
        QtMocHelpers::SignalData<void(bool)>(22, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 23 },
        }}),
        // Signal 'scalingAbsoluteChanged'
        QtMocHelpers::SignalData<void(bool)>(24, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 25 },
        }}),
        // Signal 'needUpdate'
        QtMocHelpers::SignalData<void()>(26, 4, QMC::AccessPublic, QMetaType::Void),
        // Method 'setRotationAxisAndAngle'
        QtMocHelpers::MethodData<void(QVector3D, float)>(27, 4, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 28 }, { QMetaType::Float, 29 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'meshFile'
        QtMocHelpers::PropertyData<QString>(5, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 0),
        // property 'textureFile'
        QtMocHelpers::PropertyData<QString>(7, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 1),
        // property 'position'
        QtMocHelpers::PropertyData<QVector3D>(10, 0x80000000 | 9, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet | QMC::Final, 2),
        // property 'positionAbsolute'
        QtMocHelpers::PropertyData<bool>(12, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 3),
        // property 'scaling'
        QtMocHelpers::PropertyData<QVector3D>(14, 0x80000000 | 9, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet | QMC::Final, 4),
        // property 'rotation'
        QtMocHelpers::PropertyData<QQuaternion>(17, 0x80000000 | 16, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet | QMC::Final, 5),
        // property 'visible'
        QtMocHelpers::PropertyData<bool>(21, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 7),
        // property 'shadowCasting'
        QtMocHelpers::PropertyData<bool>(23, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 8),
        // property 'scalingAbsolute'
        QtMocHelpers::PropertyData<bool>(25, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 9),
        // property 'rotationAbsolute'
        QtMocHelpers::PropertyData<bool>(30, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 6, 0x60b),
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
    });
    return QtMocHelpers::metaObjectData<QCustom3DItem, void>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject QCustom3DItem::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13QCustom3DItemE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13QCustom3DItemE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN13QCustom3DItemE_t>.metaTypes,
    nullptr
} };

void QCustom3DItem::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QCustom3DItem *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->meshFileChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 1: _t->textureFileChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 2: _t->positionChanged((*reinterpret_cast<std::add_pointer_t<QVector3D>>(_a[1]))); break;
        case 3: _t->positionAbsoluteChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 4: _t->scalingChanged((*reinterpret_cast<std::add_pointer_t<QVector3D>>(_a[1]))); break;
        case 5: _t->rotationChanged((*reinterpret_cast<std::add_pointer_t<QQuaternion>>(_a[1]))); break;
        case 6: _t->rotationAbsoluteChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 7: _t->visibleChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 8: _t->shadowCastingChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 9: _t->scalingAbsoluteChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 10: _t->needUpdate(); break;
        case 11: _t->setRotationAxisAndAngle((*reinterpret_cast<std::add_pointer_t<QVector3D>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[2]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (QCustom3DItem::*)(const QString & )>(_a, &QCustom3DItem::meshFileChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (QCustom3DItem::*)(const QString & )>(_a, &QCustom3DItem::textureFileChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (QCustom3DItem::*)(QVector3D )>(_a, &QCustom3DItem::positionChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (QCustom3DItem::*)(bool )>(_a, &QCustom3DItem::positionAbsoluteChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (QCustom3DItem::*)(QVector3D )>(_a, &QCustom3DItem::scalingChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (QCustom3DItem::*)(const QQuaternion & )>(_a, &QCustom3DItem::rotationChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (QCustom3DItem::*)(bool )>(_a, &QCustom3DItem::rotationAbsoluteChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (QCustom3DItem::*)(bool )>(_a, &QCustom3DItem::visibleChanged, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (QCustom3DItem::*)(bool )>(_a, &QCustom3DItem::shadowCastingChanged, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (QCustom3DItem::*)(bool )>(_a, &QCustom3DItem::scalingAbsoluteChanged, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (QCustom3DItem::*)()>(_a, &QCustom3DItem::needUpdate, 10))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QString*>(_v) = _t->meshFile(); break;
        case 1: *reinterpret_cast<QString*>(_v) = _t->textureFile(); break;
        case 2: *reinterpret_cast<QVector3D*>(_v) = _t->position(); break;
        case 3: *reinterpret_cast<bool*>(_v) = _t->isPositionAbsolute(); break;
        case 4: *reinterpret_cast<QVector3D*>(_v) = _t->scaling(); break;
        case 5: *reinterpret_cast<QQuaternion*>(_v) = _t->rotation(); break;
        case 6: *reinterpret_cast<bool*>(_v) = _t->isVisible(); break;
        case 7: *reinterpret_cast<bool*>(_v) = _t->isShadowCasting(); break;
        case 8: *reinterpret_cast<bool*>(_v) = _t->isScalingAbsolute(); break;
        case 9: *reinterpret_cast<bool*>(_v) = _t->isRotationAbsolute(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setMeshFile(*reinterpret_cast<QString*>(_v)); break;
        case 1: _t->setTextureFile(*reinterpret_cast<QString*>(_v)); break;
        case 2: _t->setPosition(*reinterpret_cast<QVector3D*>(_v)); break;
        case 3: _t->setPositionAbsolute(*reinterpret_cast<bool*>(_v)); break;
        case 4: _t->setScaling(*reinterpret_cast<QVector3D*>(_v)); break;
        case 5: _t->setRotation(*reinterpret_cast<QQuaternion*>(_v)); break;
        case 6: _t->setVisible(*reinterpret_cast<bool*>(_v)); break;
        case 7: _t->setShadowCasting(*reinterpret_cast<bool*>(_v)); break;
        case 8: _t->setScalingAbsolute(*reinterpret_cast<bool*>(_v)); break;
        case 9: _t->setRotationAbsolute(*reinterpret_cast<bool*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *QCustom3DItem::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QCustom3DItem::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13QCustom3DItemE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int QCustom3DItem::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 12)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 12;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 12)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 12;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    }
    return _id;
}

// SIGNAL 0
void QCustom3DItem::meshFileChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void QCustom3DItem::textureFileChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void QCustom3DItem::positionChanged(QVector3D _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void QCustom3DItem::positionAbsoluteChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void QCustom3DItem::scalingChanged(QVector3D _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void QCustom3DItem::rotationChanged(const QQuaternion & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void QCustom3DItem::rotationAbsoluteChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}

// SIGNAL 7
void QCustom3DItem::visibleChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1);
}

// SIGNAL 8
void QCustom3DItem::shadowCastingChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 8, nullptr, _t1);
}

// SIGNAL 9
void QCustom3DItem::scalingAbsoluteChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 9, nullptr, _t1);
}

// SIGNAL 10
void QCustom3DItem::needUpdate()
{
    QMetaObject::activate(this, &staticMetaObject, 10, nullptr);
}
QT_WARNING_POP
