/****************************************************************************
** Meta object code from reading C++ file 'qquickgraphssurfacenode_p.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../qtgraphs/src/graphs3d/qml/qquickgraphssurfacenode_p.h"
#include <QtGui/qtextcursor.h>
#include <QtNetwork/QSslError>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qquickgraphssurfacenode_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN23QQuickGraphsSurfaceNodeE_t {};
} // unnamed namespace

template <> constexpr inline auto QQuickGraphsSurfaceNode::qt_create_metaobjectdata<qt_meta_tag_ZN23QQuickGraphsSurfaceNodeE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QQuickGraphsSurfaceNode",
        "DefaultProperty",
        "seriesList",
        "QML.AddedInVersion",
        "1546",
        "QML.Element",
        "Surface3DNode",
        "axisXChanged",
        "",
        "QValue3DAxis*",
        "axis",
        "axisYChanged",
        "axisZChanged",
        "selectedSeriesChanged",
        "QSurface3DSeries*",
        "series",
        "flipHorizontalGridChanged",
        "flip",
        "addSeries",
        "removeSeries",
        "clearSelection",
        "doPicking",
        "QPointF",
        "point",
        "doRayPicking",
        "QVector3D",
        "origin",
        "direction",
        "axisX",
        "axisY",
        "axisZ",
        "selectedSeries",
        "QQmlListProperty<QSurface3DSeries>",
        "flipHorizontalGrid"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'axisXChanged'
        QtMocHelpers::SignalData<void(QValue3DAxis *)>(7, 8, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 10 },
        }}),
        // Signal 'axisYChanged'
        QtMocHelpers::SignalData<void(QValue3DAxis *)>(11, 8, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 10 },
        }}),
        // Signal 'axisZChanged'
        QtMocHelpers::SignalData<void(QValue3DAxis *)>(12, 8, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 10 },
        }}),
        // Signal 'selectedSeriesChanged'
        QtMocHelpers::SignalData<void(QSurface3DSeries *)>(13, 8, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 14, 15 },
        }}),
        // Signal 'flipHorizontalGridChanged'
        QtMocHelpers::SignalData<void(bool)>(16, 8, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 17 },
        }}),
        // Method 'addSeries'
        QtMocHelpers::MethodData<void(QSurface3DSeries *)>(18, 8, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 14, 15 },
        }}),
        // Method 'removeSeries'
        QtMocHelpers::MethodData<void(QSurface3DSeries *)>(19, 8, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 14, 15 },
        }}),
        // Method 'clearSelection'
        QtMocHelpers::MethodData<void()>(20, 8, QMC::AccessPublic, QMetaType::Void),
        // Method 'doPicking'
        QtMocHelpers::MethodData<bool(QPointF)>(21, 8, QMC::AccessPublic, QMetaType::Bool, {{
            { 0x80000000 | 22, 23 },
        }}),
        // Method 'doRayPicking'
        QtMocHelpers::MethodData<bool(const QVector3D &, const QVector3D &)>(24, 8, QMC::AccessPublic, QMetaType::Bool, {{
            { 0x80000000 | 25, 26 }, { 0x80000000 | 25, 27 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'axisX'
        QtMocHelpers::PropertyData<QValue3DAxis*>(28, 0x80000000 | 9, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 0),
        // property 'axisY'
        QtMocHelpers::PropertyData<QValue3DAxis*>(29, 0x80000000 | 9, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 1),
        // property 'axisZ'
        QtMocHelpers::PropertyData<QValue3DAxis*>(30, 0x80000000 | 9, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 2),
        // property 'selectedSeries'
        QtMocHelpers::PropertyData<QSurface3DSeries*>(31, 0x80000000 | 14, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 3),
        // property 'seriesList'
        QtMocHelpers::PropertyData<QQmlListProperty<QSurface3DSeries>>(2, 0x80000000 | 32, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'flipHorizontalGrid'
        QtMocHelpers::PropertyData<bool>(33, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 4),
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
            {    5,    6 },
    });
    return QtMocHelpers::metaObjectData<QQuickGraphsSurfaceNode, void>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject QQuickGraphsSurfaceNode::staticMetaObject = { {
    QMetaObject::SuperData::link<QQuickGraphsNode::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN23QQuickGraphsSurfaceNodeE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN23QQuickGraphsSurfaceNodeE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN23QQuickGraphsSurfaceNodeE_t>.metaTypes,
    nullptr
} };

void QQuickGraphsSurfaceNode::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QQuickGraphsSurfaceNode *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->axisXChanged((*reinterpret_cast<std::add_pointer_t<QValue3DAxis*>>(_a[1]))); break;
        case 1: _t->axisYChanged((*reinterpret_cast<std::add_pointer_t<QValue3DAxis*>>(_a[1]))); break;
        case 2: _t->axisZChanged((*reinterpret_cast<std::add_pointer_t<QValue3DAxis*>>(_a[1]))); break;
        case 3: _t->selectedSeriesChanged((*reinterpret_cast<std::add_pointer_t<QSurface3DSeries*>>(_a[1]))); break;
        case 4: _t->flipHorizontalGridChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 5: _t->addSeries((*reinterpret_cast<std::add_pointer_t<QSurface3DSeries*>>(_a[1]))); break;
        case 6: _t->removeSeries((*reinterpret_cast<std::add_pointer_t<QSurface3DSeries*>>(_a[1]))); break;
        case 7: _t->clearSelection(); break;
        case 8: { bool _r = _t->doPicking((*reinterpret_cast<std::add_pointer_t<QPointF>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 9: { bool _r = _t->doRayPicking((*reinterpret_cast<std::add_pointer_t<QVector3D>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QVector3D>>(_a[2])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QValue3DAxis* >(); break;
            }
            break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QValue3DAxis* >(); break;
            }
            break;
        case 2:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QValue3DAxis* >(); break;
            }
            break;
        case 3:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QSurface3DSeries* >(); break;
            }
            break;
        case 5:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QSurface3DSeries* >(); break;
            }
            break;
        case 6:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QSurface3DSeries* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (QQuickGraphsSurfaceNode::*)(QValue3DAxis * )>(_a, &QQuickGraphsSurfaceNode::axisXChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (QQuickGraphsSurfaceNode::*)(QValue3DAxis * )>(_a, &QQuickGraphsSurfaceNode::axisYChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (QQuickGraphsSurfaceNode::*)(QValue3DAxis * )>(_a, &QQuickGraphsSurfaceNode::axisZChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (QQuickGraphsSurfaceNode::*)(QSurface3DSeries * )>(_a, &QQuickGraphsSurfaceNode::selectedSeriesChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (QQuickGraphsSurfaceNode::*)(bool )>(_a, &QQuickGraphsSurfaceNode::flipHorizontalGridChanged, 4))
            return;
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 3:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QSurface3DSeries* >(); break;
        case 2:
        case 1:
        case 0:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QValue3DAxis* >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QValue3DAxis**>(_v) = _t->axisX(); break;
        case 1: *reinterpret_cast<QValue3DAxis**>(_v) = _t->axisY(); break;
        case 2: *reinterpret_cast<QValue3DAxis**>(_v) = _t->axisZ(); break;
        case 3: *reinterpret_cast<QSurface3DSeries**>(_v) = _t->selectedSeries(); break;
        case 4: *reinterpret_cast<QQmlListProperty<QSurface3DSeries>*>(_v) = _t->seriesList(); break;
        case 5: *reinterpret_cast<bool*>(_v) = _t->flipHorizontalGrid(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setAxisX(*reinterpret_cast<QValue3DAxis**>(_v)); break;
        case 1: _t->setAxisY(*reinterpret_cast<QValue3DAxis**>(_v)); break;
        case 2: _t->setAxisZ(*reinterpret_cast<QValue3DAxis**>(_v)); break;
        case 5: _t->setFlipHorizontalGrid(*reinterpret_cast<bool*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *QQuickGraphsSurfaceNode::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QQuickGraphsSurfaceNode::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN23QQuickGraphsSurfaceNodeE_t>.strings))
        return static_cast<void*>(this);
    return QQuickGraphsNode::qt_metacast(_clname);
}

int QQuickGraphsSurfaceNode::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QQuickGraphsNode::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 10)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 10)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    }
    return _id;
}

// SIGNAL 0
void QQuickGraphsSurfaceNode::axisXChanged(QValue3DAxis * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void QQuickGraphsSurfaceNode::axisYChanged(QValue3DAxis * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void QQuickGraphsSurfaceNode::axisZChanged(QValue3DAxis * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void QQuickGraphsSurfaceNode::selectedSeriesChanged(QSurface3DSeries * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void QQuickGraphsSurfaceNode::flipHorizontalGridChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}
QT_WARNING_POP
